/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsCaptureFilter.h>
#include <assert.h>
//------------------------------------------------------------------------------
fsCaptureFilter::fsCaptureFilter(IBaseFilter* filter, const mxString& friendlyName) :
fsSourceFilter(friendlyName, friendlyName)
{
	assert(filter != NULL);
	_filter			= filter;

	_capturePinIndex = _getCategoryPinIndex(true);
	_previewPinIndex = _getCategoryPinIndex(false);
	
	_isAnalogVideo = false;
	IAMAnalogVideoDecoder *analogVideo = NULL;
	_filter->QueryInterface(IID_IAMAnalogVideoDecoder, (void **)&analogVideo);
	if (analogVideo != NULL)
	{
		analogVideo->Release();
		analogVideo = NULL;
		_isAnalogVideo = true;
	}

	_isDigitalVideo = false;
	IAMExtDevice *extDevice = NULL;
	IAMExtTransport *extTransport = NULL;
	IAMTimecodeReader *timecodeReader = NULL;
	_filter->QueryInterface(IID_IAMExtDevice, (void **)&extDevice);
	_filter->QueryInterface(IID_IAMExtTransport, (void **)&extTransport);
	_filter->QueryInterface(IID_IAMTimecodeReader, (void **)&timecodeReader);
	if (extDevice != NULL && extTransport != NULL && timecodeReader != NULL)
	{
		_isDigitalVideo = true;
	}
	if (extDevice != NULL)
	{
		extDevice->Release();
		extDevice = NULL;
	}
	if (extTransport != NULL)
	{
		extTransport->Release();
		extTransport = NULL;
	}
	if (timecodeReader != NULL)
	{
		timecodeReader->Release();
		timecodeReader = NULL;
	}
}
//------------------------------------------------------------------------------
fsCaptureFilter::~fsCaptureFilter()
{
}
//------------------------------------------------------------------------------
mxInt
fsCaptureFilter::getCapturePinIndex() const
{
	return _capturePinIndex;
}
//------------------------------------------------------------------------------
mxInt
fsCaptureFilter::getPreviewPinIndex() const
{
	return _previewPinIndex;
}
//------------------------------------------------------------------------------
mxBool
fsCaptureFilter::isAnalogVideo() const
{
	return _isAnalogVideo;
}
//------------------------------------------------------------------------------
mxBool
fsCaptureFilter::isDigitalVideo() const
{
	return _isDigitalVideo;
}
//------------------------------------------------------------------------------
void
fsCaptureFilter::setAllocatorProperties(mxUInt outputPinIndex,
										mxLong numBuf,
										mxLong bufSize,
										mxLong bufAlign,
										mxLong bufPrefix)
{
	assert(outputPinIndex < getNumOutputPins());

	IPin* pin = getOutputPin(outputPinIndex);
	assert(pin != NULL);

	IAMBufferNegotiation* bufNego = NULL;

	pin->QueryInterface(IID_IAMBufferNegotiation, (void **)&bufNego);
	if (bufNego != NULL)
	{
		ALLOCATOR_PROPERTIES prop;
		ZeroMemory(&prop, sizeof(ALLOCATOR_PROPERTIES));
		prop.cBuffers = numBuf;
		prop.cbBuffer = bufSize;
		prop.cbAlign = bufAlign;
		prop.cbPrefix = bufPrefix;

		bufNego->SuggestAllocatorProperties(&prop);
		bufNego->Release();
	}

	pin->Release();
}
//------------------------------------------------------------------------------
void
fsCaptureFilter::getAllocatorProperties(mxUInt outputPinIndex,
										mxLong& numBuf,
										mxLong& bufSize,
										mxLong& bufAlign,
										mxLong& bufPrefix) const
{
	assert(outputPinIndex < getNumOutputPins());

	IPin* pin = getOutputPin(outputPinIndex);
	assert(pin != NULL);

	IAMBufferNegotiation* bufNego = NULL;

	pin->QueryInterface(IID_IAMBufferNegotiation, (void **)&bufNego);
	if (bufNego != NULL)
	{
		ALLOCATOR_PROPERTIES prop;
		ZeroMemory(&prop, sizeof(ALLOCATOR_PROPERTIES));

		bufNego->GetAllocatorProperties(&prop);
		bufNego->Release();

		numBuf = prop.cBuffers;
		bufSize = prop.cbBuffer;
		bufAlign = prop.cbAlign;
		bufPrefix = prop.cbPrefix;
	}

	pin->Release();
}
//------------------------------------------------------------------------------
mxInt
fsCaptureFilter::_getCategoryPinIndex(mxBool isCapturePin) const
{
	assert(_filter != NULL);

	mxUInt numOutputPin = getNumOutputPins();
	mxUInt i;

	for(i = 0; i < numOutputPin; i++)
	{
		IPin* pin = getOutputPin(i);
		assert(pin != NULL);

		GUID guid;
		DWORD ret;
		IKsPropertySet *propertySet = NULL;

		if (pin->QueryInterface(IID_IKsPropertySet, (void **)&propertySet) == S_OK)
		{
			propertySet->Get(AMPROPSETID_Pin, AMPROPERTY_PIN_CATEGORY,
											NULL, 0, &guid, sizeof(GUID), &ret);
			propertySet->Release();

			if ((isCapturePin && IsEqualGUID(PIN_CATEGORY_CAPTURE, guid)) ||
				(!isCapturePin && IsEqualGUID(PIN_CATEGORY_PREVIEW, guid)))
				break;
		}

		pin->Release();
	}

	if (i == numOutputPin)
		return -1;
	else
		return i;
}
//------------------------------------------------------------------------------
