/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsDigitalClockFilter.h>
#include <assert.h>
//------------------------------------------------------------------------------
fsDigitalClockFilter::fsDigitalClockFilter(const mxString& name) :
fsSourceFilter("Digital Clock", name)
{
	HRESULT hr;
	hr = CoCreateInstance(CLSID_DigitalClock, NULL, CLSCTX_INPROC,
						IID_IBaseFilter, (void **)&_filter);

	if (hr != S_OK)
		_filter		= NULL;

	_digitalClock = NULL;
	if (hr == S_OK)
	{
		hr = _filter->QueryInterface(IID_IDigitalClock, (void **)&_digitalClock);
		if (hr != S_OK)
		{
			_digitalClock = NULL;

			_filter->Release();
			_filter	= NULL;
		}
	}

	_headerString = "";
	_footerString = "";
	if (_digitalClock != NULL)
	{
		mxChar str[DC_MAXIMUM_HEADERFOOTER_LENGTH];

		_digitalClock->get_HeaderString(str);
		_headerString = mxString(str);

		_digitalClock->get_FooterString(str);
		_footerString = mxString(str);
	}
}
//------------------------------------------------------------------------------
fsDigitalClockFilter::~fsDigitalClockFilter()
{
	if (_digitalClock != NULL)
	{
		_digitalClock->Release();
		_digitalClock = NULL;
	}
}
//------------------------------------------------------------------------------
mxInt
fsDigitalClockFilter::getOutputVideoResolution() const
{
	mxInt resolution = 0;
	if (_digitalClock == NULL)
		return 0;
	_digitalClock->get_OutputVideoResolution(&resolution);
	return resolution;
}
//------------------------------------------------------------------------------
void
fsDigitalClockFilter::setOutputVideoResolution(mxInt resolution)
{
	if (_digitalClock == NULL)
		return ;
	_digitalClock->put_OutputVideoResolution(resolution);
}
//------------------------------------------------------------------------------
mxInt
fsDigitalClockFilter::getFrameRate() const
{
	mxInt frameRate = 0;
	if (_digitalClock == NULL)
		return frameRate;
	_digitalClock->get_FrameRate(&frameRate);
	return frameRate;
}
//------------------------------------------------------------------------------
void
fsDigitalClockFilter::setFrameRate(mxInt frameRate)
{
	if (_digitalClock == NULL)
		return ;
	_digitalClock->put_FrameRate(frameRate);
}
//------------------------------------------------------------------------------
mxInt
fsDigitalClockFilter::getFontSize() const
{
	mxInt fontSize = 0;
	if (_digitalClock == NULL)
		return fontSize;
	_digitalClock->get_FontSize(&fontSize);
	return fontSize;
}
//------------------------------------------------------------------------------
void
fsDigitalClockFilter::setFontSize(mxInt fontSize)
{
	if (_digitalClock == NULL)
		return ;
	_digitalClock->put_FontSize(fontSize);
}
//------------------------------------------------------------------------------
void
fsDigitalClockFilter::getFontColor(mxByte* color) const
{
	if (_digitalClock == NULL)
		return ;
	_digitalClock->get_FontColor(color);
}
//------------------------------------------------------------------------------
void
fsDigitalClockFilter::setFontColor(const mxByte* color)
{
	if (_digitalClock == NULL)
		return ;
	_digitalClock->put_FontColor(color);
}
//------------------------------------------------------------------------------
void
fsDigitalClockFilter::getBackgroundColor(mxByte* color) const
{
	if (_digitalClock == NULL)
		return ;
	_digitalClock->get_BackgroundColor(color);
}
//------------------------------------------------------------------------------
void
fsDigitalClockFilter::setBackgroundColor(const mxByte* color)
{
	if (_digitalClock == NULL)
		return ;
	_digitalClock->put_BackgroundColor(color);
}
//------------------------------------------------------------------------------
const mxString&
fsDigitalClockFilter::getHeaderString() const
{
	return _headerString;
}
//------------------------------------------------------------------------------
void
fsDigitalClockFilter::setHeaderString(const mxString& str)
{
	_headerString = str;
	if (_digitalClock == NULL)
		return ;
	_digitalClock->put_HeaderString(str.length() != 0 ? str.c_str() : "");
}
//------------------------------------------------------------------------------
const mxString&
fsDigitalClockFilter::getFooterString() const
{
	return _footerString;
}
//------------------------------------------------------------------------------
void
fsDigitalClockFilter::setFooterString(const mxString& str)
{
	_footerString = str;
	if (_digitalClock == NULL)
		return ;
	_digitalClock->put_FooterString(str.length() != 0 ? str.c_str() : "");
}
//------------------------------------------------------------------------------

