/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsFilter.h>
//------------------------------------------------------------------------------
fsFilter::fsFilter(const mxString& name) :
mxObject()
{
	_filter		= NULL;
	_name		= name;
}
//------------------------------------------------------------------------------
fsFilter::fsFilter(IBaseFilter* filter)
{
	assert(filter != NULL);

	_filter		= filter;
	_name		= "-";
}
//------------------------------------------------------------------------------
fsFilter::~fsFilter()
{
	if (_filter != NULL)
	{
		_filter->Release();
		_filter = NULL;
	}
}
//------------------------------------------------------------------------------
mxBool
fsFilter::isValid() const
{
	return (_filter != NULL);
}
//------------------------------------------------------------------------------
IBaseFilter*
fsFilter::getBaseFilter() const
{
	return _filter;
}
//------------------------------------------------------------------------------
const mxString&
fsFilter::getName() const
{
	return _name;
}
//------------------------------------------------------------------------------
void
fsFilter::setName(const mxString& name)
{
	_name = name;
}
//------------------------------------------------------------------------------
mxUInt
fsFilter::getNumInputPins() const
{
	return _getNumPins(true);
}
//------------------------------------------------------------------------------
mxUInt
fsFilter::getNumOutputPins() const
{
	return _getNumPins(false);
}
//------------------------------------------------------------------------------
fsFilterPtr
fsFilter::getUpStreamFilter(mxUInt pinInIndex) const
{
	IBaseFilter *filter = NULL;

	IPin* pinIn = getInputPin(pinInIndex);
	if (pinIn != NULL)
	{
		IPin* pinOut = NULL;
		pinIn->ConnectedTo(&pinOut);
		
		if (pinOut != NULL)
		{
			PIN_INFO pinInfo;
			if (pinOut->QueryPinInfo(&pinInfo) == NOERROR)
				filter = pinInfo.pFilter;
			pinOut->Release();
		}

		pinIn->Release();
	}

	return ((filter != NULL) ? new fsFilter(filter) : NULL);
}
//------------------------------------------------------------------------------
fsFilterPtr
fsFilter::getDownStreamFilter(mxUInt pinOutIndex) const
{
	IBaseFilter *filter = NULL;

	IPin* pinOut = getOutputPin(pinOutIndex);
	if (pinOut != NULL)
	{
		IPin* pinIn = NULL;
		pinOut->ConnectedTo(&pinIn);
		
		if (pinIn != NULL)
		{
			PIN_INFO pinInfo;
			if (pinIn->QueryPinInfo(&pinInfo) == NOERROR)
				filter = pinInfo.pFilter;
			pinIn->Release();
		}

		pinOut->Release();
	}

	return ((filter != NULL) ? new fsFilter(filter) : NULL);
}
//------------------------------------------------------------------------------
mxBool
fsFilter::isInputConnected(mxUInt pinInIndex) const
{
	IPin *inPin = getInputPin(pinInIndex);
	if (inPin == NULL)
		return false;

	IPin *outPin = NULL;
	HRESULT hr = inPin->ConnectedTo(&outPin);
	if (hr != VFW_E_NOT_CONNECTED)
		outPin->Release();

	inPin->Release();

	return (hr == S_OK);
}
//------------------------------------------------------------------------------
mxBool
fsFilter::isOutputConnected(mxUInt pinOutIndex) const
{
	IPin *outPin = getOutputPin(pinOutIndex);
	if (outPin == NULL)
		return false;

	IPin *inPin = NULL;
	HRESULT hr = outPin->ConnectedTo(&inPin);
	if (hr != VFW_E_NOT_CONNECTED)
		inPin->Release();

	outPin->Release();

	return (hr == S_OK);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
IPin*
fsFilter::getInputPin(mxUInt index) const
{
	return _getPin(true, index);
}
//------------------------------------------------------------------------------
IPin*
fsFilter::getOutputPin(mxUInt index) const
{
	return _getPin(false, index);
}
//------------------------------------------------------------------------------
mxUInt
fsFilter::_getNumPins(mxBool isInput) const
{
	if (_filter == NULL)
		return 0;

	HRESULT hr;
	ULONG fetched;
    PIN_DIRECTION pinDirection;
	IEnumPins* pins		= NULL;
	IPin* pin			= NULL;

	hr = _filter->EnumPins(&pins);
    pins->Reset();

	mxUInt count = 0;

	while(hr == NOERROR)
	{
		hr = pins->Next(1, &pin, &fetched);
		if (hr == S_OK && pin != NULL)
		{
			hr = pin->QueryDirection(&pinDirection);
			if (isInput && pinDirection == PINDIR_INPUT)
				count++;
			else if (!isInput && pinDirection == PINDIR_OUTPUT)
				count++;

			pin->Release();
			pin = NULL;
		}
	}

	if (pins != NULL)
		pins->Release();

	return count;
}
//------------------------------------------------------------------------------
IPin*
fsFilter::_getPin(mxBool isInput, mxUInt index) const
{
	HRESULT hr;
	ULONG fetched;

	PIN_DIRECTION pinDirection = (isInput ? PINDIR_INPUT : PINDIR_OUTPUT);
	PIN_DIRECTION enumPinDirection;

	IEnumPins* pins = NULL;
	IPin* pin = NULL;
	mxInt enumPinIndex = 0;

	hr = _filter->EnumPins(&pins);
    pins->Reset();

	while(hr == NOERROR)
	{
		hr = pins->Next(1, &pin, &fetched);
		if (hr == S_OK && pin != NULL)
		{
			hr = pin->QueryDirection(&enumPinDirection);
			if (enumPinDirection == pinDirection)
			{
				if (enumPinIndex == index)
					break;
				enumPinIndex++;
			}
			pin->Release();
			pin = NULL;
		}
	}

	pins->Release();
	pins = NULL;

    return pin;
}
//------------------------------------------------------------------------------
