/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsFilterEnumerator.h>
#include <atlstr.h>
//------------------------------------------------------------------------------
fsFilterEnumerator::fsFilterEnumerator(CategoryEnum category) :
mxObject(),
_category(category)
{
	GUID filterCategory;
	switch(category)
	{
	case C_VIDEO_CAPTURE:
		filterCategory = CLSID_VideoInputDeviceCategory;	break;
	case C_VIDEO_ENCODER:
		filterCategory = CLSID_VideoCompressorCategory;		break;
	case C_AUDIO_CAPTURE:
		filterCategory = CLSID_AudioInputDeviceCategory;	break;
	case C_AUDIO_ENCODER:
		filterCategory = CLSID_AudioCompressorCategory;		break;
	case C_AUDIO_RENDERER:
		filterCategory = CLSID_AudioRendererCategory;		break;
	default:
		return ;
	}

	HRESULT hr;

	// Create System Device Enumerator
	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC, 
							IID_ICreateDevEnum, (void **)&_systemDevice);
	if (hr != S_OK)
	{
		_systemDevice = NULL;
		_enumerator = NULL;
		return ;
	}

	// Enumerate System Device
	hr = _systemDevice->CreateClassEnumerator(filterCategory, &_enumerator, 0);
	if (hr != S_OK)
	{
		_systemDevice->Release();
		_systemDevice = NULL;
		_enumerator = NULL;
	}

}
//------------------------------------------------------------------------------
fsFilterEnumerator::~fsFilterEnumerator()
{
	if (_enumerator != NULL)
	{
		_enumerator->Release();
		_enumerator = NULL;
	}

	if (_systemDevice != NULL)
	{
		_systemDevice->Release();
		_systemDevice = NULL;
	}
}
//------------------------------------------------------------------------------
void
fsFilterEnumerator::reset()
{
	if (_enumerator == NULL)
		return ;
	_enumerator->Reset();
}
//------------------------------------------------------------------------------
mxBool
fsFilterEnumerator::nextFilter(fsFilterPtr& filter)
{
	filter = NULL;

	if (_enumerator == NULL)
		return false;

	IMoniker *pMoniker;
	ULONG cFetched;

TRYAGAIN_FORBUG:

	if (_enumerator->Next(1, &pMoniker, &cFetched) == S_OK)
	{
		IBaseFilter* filterObj = NULL;
		mxString friendlyName;

		// Get Friendly name
		IPropertyBag *pPropBag;
		pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);

		VARIANT varName;
		varName.vt = VT_BSTR;
		pPropBag->Read(L"FriendlyName", &varName, 0);

		CString	str(varName.bstrVal);
		friendlyName = mxString(str);
		SysFreeString(varName.bstrVal);

		pPropBag->Release();

		///
		/// For Windows Media Encoder filter Bug!
		///  ̃tB^폜Ƃɗ
		if (friendlyName == "Audio Source")
		{
			pMoniker->Release();
			goto TRYAGAIN_FORBUG;
		}
		/// For Windows Media Encoder filter Bug!
		///

		// Get Filter
		pMoniker->BindToObject(0, 0, IID_IBaseFilter, (void**)&filterObj);
		if (filterObj == NULL)
		{
			pMoniker->Release();
			return false;
		}

		// Release Moniker
		pMoniker->Release();

		switch(_category)
		{
		case C_VIDEO_CAPTURE:
		case C_AUDIO_CAPTURE:
			filter = new fsCaptureFilter(filterObj, friendlyName);
			break;
		case C_VIDEO_ENCODER:
		case C_AUDIO_ENCODER:
			filter = new fsEncoderFilter(filterObj, friendlyName);
			break;
		case C_AUDIO_RENDERER:
			filter = new fsAudioRenderFilter(filterObj, friendlyName);
			break;
		}
		return true;
	}

	return false;
}
//------------------------------------------------------------------------------
