/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsGraphManager.h>
#include <atlstr.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsGraphManager::fsGraphManager(GraphEventProc proc1, void* data1,
							   ProcessIntermediateProc proc2, void *data2) :
mxObject()
{
	// Callback
	_graphEventProc				= proc1;
	_graphEventData				= data1;
	_processIntermediateProc	= proc2;
	_processIntermediateData	= data2;

	// Graph Status
	_status						= GS_STOP;

	// FilterGraph Interfaces
	_graph						= NULL;
	_mediaControl				= NULL;
	_mediaSeeking				= NULL;
	_mediaEvent					= NULL;
	_videoWindow				= NULL;
	_basicVideo					= NULL;

	// GraphEvent
	_mediaEventThreadRun		= false;
	_mediaEventThread			= NULL;

	// For Graph Reflection
	_register					= 0;
}
//------------------------------------------------------------------------------
fsGraphManager::~fsGraphManager()
{
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Initialize / Shutdown
mxBool
fsGraphManager::initialize(mxBool enableAddToRot)
{
	HRESULT hr;

	_status		= GS_STOP;

	// Create Filter Graph instance
	hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC,
						IID_IGraphBuilder, (void **)&_graph);
	if (hr != NOERROR)
		return false;

	// Query Media control interface
	hr = _graph->QueryInterface(IID_IMediaControl, (void **)&_mediaControl);
	if (hr != NOERROR)
		return false;

	// Query Media seeking interface
	hr = _graph->QueryInterface(IID_IMediaSeeking, (void **)&_mediaSeeking);
	if (hr != NOERROR)
		return false;

	// Query Media Event interface
	hr = _graph->QueryInterface(IID_IMediaEventEx, (void **)&_mediaEvent);
	if (hr != NOERROR)
		return false;

	// Query Video Window interface
	hr = _graph->QueryInterface(IID_IVideoWindow, (void **)&_videoWindow);
	if (hr != NOERROR)
		return false;

	// Query Basic Video interface
	hr = _graph->QueryInterface(IID_IBasicVideo, (void **)&_basicVideo);
	if (hr != NOERROR)
		return false;

	// Xg[ICxgtB^OtnhÔLZ
	_mediaEvent->CancelDefaultHandling(EC_COMPLETE);

	// Create GraphEvent Thread
	_mediaEventThreadRun = true;
	_mediaEventThread = new mxThread(mediaEventLoopProc, this);

	// For Graph Reflection
	if (enableAddToRot)
		addToRot(_graph, &_register);

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsGraphManager::shutdown()
{
	if (_register)
		removeFromRot(_register);

	// Delete MediaEvent Thread
	_mediaEventThreadRun = false;
	_mediaEventThread->waitFor();
	delete _mediaEventThread;
	_mediaEventThread = NULL;

	// Release Basic Video
	if (_basicVideo != NULL)
	{
		_basicVideo->Release();
		_basicVideo = NULL;
	}
	// Release Video Window
	if (_videoWindow != NULL)
	{
		_videoWindow->Release();
		_videoWindow = NULL;
	}
	// Release Media Event
	if (_mediaEvent != NULL)
	{
		_mediaEvent->Release();
		_mediaEvent = NULL;
	}
	// Release Media seeking
	if (_mediaSeeking != NULL)
	{
		_mediaSeeking->Release();
		_mediaSeeking = NULL;
	}
	// Release Media control
	if (_mediaControl != NULL)
	{
		_mediaControl->Release();
		_mediaControl = NULL;
	}
	// Release Filter Graph
	if (_graph != NULL)
	{
		_graph->Release();
		_graph = NULL;
	}

	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Graph Control
mxInt
fsGraphManager::getStatus() const
{
	return _status;
}
//------------------------------------------------------------------------------
mxBool
fsGraphManager::start()
{
	if (_mediaControl == NULL || _status == GS_START)
		return false;
	_mediaControl->Run();

	FILTER_STATE state;
	_mediaControl->GetState(100000, (OAFilterState *)&state);

	 _status = GS_START;

	return (state == State_Running);
}
//------------------------------------------------------------------------------
mxBool
fsGraphManager::stop()
{
	if (_mediaControl == NULL ||  _status == GS_STOP)
		return false;
	_mediaControl->Stop();

	FILTER_STATE state;
	_mediaControl->GetState(100000, (OAFilterState *)&state);
#if 0
	mxInt64 pos = 0;
	_mediaSeeking->SetPositions(&pos, AM_SEEKING_AbsolutePositioning,
								NULL, AM_SEEKING_NoPositioning);
#endif
	_status = GS_STOP;

	return (state == State_Stopped);
}
//------------------------------------------------------------------------------
mxBool
fsGraphManager::pause()
{
	if (_mediaControl == NULL || _status == GS_PAUSE)
		return false;
	_mediaControl->Pause();

	FILTER_STATE state;
	_mediaControl->GetState(100000, (OAFilterState *)&state);

	_status = GS_PAUSE;

	return (state == State_Paused);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// VideoWindow Control
void
fsGraphManager::setVideoWindowPosition(mxInt x, mxInt y)
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	_videoWindow->put_Left((mxLong)x);
	_videoWindow->put_Top((mxLong)y);
}
//------------------------------------------------------------------------------
void
fsGraphManager::getVideoWindowPosition(mxInt& x, mxInt& y) const
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	x = 0;
	y = 0;

	_videoWindow->get_Left((mxLong *)&x);
	_videoWindow->get_Top((mxLong *)&y);
}
//------------------------------------------------------------------------------
void
fsGraphManager::setVideoWindowSize(mxUInt width, mxUInt height)
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	_videoWindow->put_Width((mxLong)width);
	_videoWindow->put_Height((mxLong)height);
}
//------------------------------------------------------------------------------
void
fsGraphManager::getVideoWindowSize(mxUInt& width, mxUInt& height) const
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	width = 1;
	height = 1;

	_videoWindow->get_Width((mxLong *)&width);
	_videoWindow->get_Height((mxLong *)&height);
}
//------------------------------------------------------------------------------
void
fsGraphManager::setVideoWindowOwner(HWND windowHandle)
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	_videoWindow->put_Owner((OAHWND)windowHandle);
}
//------------------------------------------------------------------------------
HWND
fsGraphManager::getVideoWindowOwner() const
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	HWND windowHandle = NULL;
	_videoWindow->get_Owner((OAHWND *)&windowHandle);
	return windowHandle;
}
//------------------------------------------------------------------------------
void
fsGraphManager::setVideoWindowStyle(mxLong style)
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	_videoWindow->put_WindowStyle(style);
}
//------------------------------------------------------------------------------
mxLong
fsGraphManager::getVideoWindowStyle() const
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	mxLong style = 0;

	_videoWindow->get_WindowStyle(&style);
	return style;
}
//------------------------------------------------------------------------------
void
fsGraphManager::setVideoWindowVisible(mxBool visible)
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	_videoWindow->put_Visible((visible ? OATRUE : OAFALSE));
}
//------------------------------------------------------------------------------
mxBool
fsGraphManager::getVideoWindowVisible() const
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	mxLong visible = OAFALSE;

	_videoWindow->get_Visible(&visible);
	return ((visible == OATRUE) ? true : false);
}
//------------------------------------------------------------------------------
void
fsGraphManager::setVideoWindowAutoShow(mxBool autoShow)
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	_videoWindow->put_AutoShow((autoShow ? OATRUE : OAFALSE));
}
//------------------------------------------------------------------------------
mxBool
fsGraphManager::getVideoWindowAutoShow() const
{
	assert(_graph != NULL);
	assert(_videoWindow != NULL);

	mxLong autoShow = OAFALSE;

	_videoWindow->get_AutoShow(&autoShow);
	return ((autoShow == OATRUE) ? true : false);
}
//------------------------------------------------------------------------------
void
fsGraphManager::getVideoWindowResolution(mxUInt& width, mxUInt& height) const
{
	assert(_basicVideo != NULL);

	HRESULT hr = _basicVideo->GetVideoSize((LONG *)&width, (LONG *)&height);
	if (hr != S_OK)
	{
		width = 0;
		height = 0;
	}
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Filter Operation
mxUInt
fsGraphManager::getNumFilters() const
{
	mxUInt count = 0;

	HRESULT hr;
	ULONG fetched;
	IEnumFilters* filters = NULL;
	IBaseFilter* filter = NULL;

	hr = _graph->EnumFilters(&filters);
	if (hr != S_OK)
		return 0;
	while(filters->Next(1, &filter, &fetched) == S_OK)
	{
		count++;

		filter->Release();
	}
	filters->Release();

	return count;
}
//------------------------------------------------------------------------------
void
fsGraphManager::addFilter(const fsFilter* filter)
{
	if (filter == NULL)
		return ;

	USES_CONVERSION;

	CString str = filter->getName().c_str();
	_graph->AddFilter(filter->getBaseFilter(), T2W(str));
}
//------------------------------------------------------------------------------
void
fsGraphManager::removeFilter(const fsFilter* filter)
{
	IBaseFilter* baseFilter = filter->getBaseFilter();

	IPin *pin, *toPin;

	HRESULT hr;
    ULONG fetched;

	IEnumPins *pins = NULL;
	hr = baseFilter->EnumPins(&pins);
    pins->Reset();

	while(hr == NOERROR)
	{
		hr = pins->Next(1, &pin, &fetched);
		if (hr == S_OK && pin != NULL)
		{
			pin->ConnectedTo(&toPin);
			if (toPin != NULL)
			{
				_graph->Disconnect(toPin);
				_graph->Disconnect(pin);
				toPin->Release();
			}
			pin->Release();
		}
	}

	if (pins != NULL)
		pins->Release();

	_graph->RemoveFilter(baseFilter);
}
//------------------------------------------------------------------------------
void
fsGraphManager::removeAllFilters()
{
	HRESULT hr;
	ULONG fetched;
	IEnumFilters* filters = NULL;
	IBaseFilter* filter = NULL;

	hr = _graph->EnumFilters(&filters);
	if (hr != S_OK)
		return ;

	while(filters->Next(1, &filter, &fetched) == S_OK)
	{
		hr = _graph->RemoveFilter(filter);
		if (hr != S_OK)
		{
			// ႤXbhŃOt~ꍇAsH
			assert(false);
			break;
		}
		filters->Reset();
		filter->Release();
	}
	filters->Release();
}
//------------------------------------------------------------------------------
// Connection
#define RELEASE_PINS													\
	if (pinOut != NULL)	{ pinOut->Release();	pinOut	= NULL; }		\
	if (pinIn != NULL)	{ pinIn->Release();		pinIn	= NULL; }
//------------------------------------------------------------------------------
mxBool
fsGraphManager::connectFilterIndirect(
				const fsFilter* filterOut, mxUInt pinOutIndex,
				const fsFilter* filterIn, mxUInt pinInIndex)
{
	if (filterOut == NULL || filterIn == NULL)
		return false;

	IPin* pinOut	= filterOut->getOutputPin(pinOutIndex);
	IPin* pinIn		= filterIn->getInputPin(pinInIndex);

	if (pinOut == NULL || pinIn == NULL)
	{
		RELEASE_PINS;
		return false;
	}

	// Connect Indirect
	HRESULT hr = _graph->Connect(pinOut, pinIn);

	RELEASE_PINS;


	// Process Intermediate Filters
	if (hr == S_OK || hr == VFW_S_PARTIAL_RENDER)
	{
		mxArray<fsFilterPtr>	intermediateFilters;

		fsFilterPtr downFilter = filterOut->getDownStreamFilter(pinOutIndex);
		while(downFilter->getBaseFilter() != filterIn->getBaseFilter())
		{
			if (downFilter == NULL)
			{
				assert(false);
				return (hr == S_OK);
			}

			intermediateFilters.add(downFilter);

			fsFilterPtr downDownFilter = NULL;

			mxUInt numPins = downFilter->getNumOutputPins();
			for(mxUInt i = 0; i < numPins; i++)
			{
				downDownFilter = downFilter->getDownStreamFilter(i);
				if (downDownFilter != NULL)
					break;
			}

			downFilter = downDownFilter;
		}

		if (intermediateFilters.getSize() != 0 && _processIntermediateProc != NULL)
		{
			_processIntermediateProc(intermediateFilters.getPointer(),
									 intermediateFilters.getSize(),
									 _processIntermediateData);
		}
	}

	return (hr == S_OK || hr == VFW_S_PARTIAL_RENDER);
}
//------------------------------------------------------------------------------
mxBool
fsGraphManager::connectFilterDirect(
				const fsFilter* filterOut, mxUInt pinOutIndex, 
				const fsFilter* filterIn, mxUInt pinInIndex)
{
	if (filterOut == NULL || filterIn == NULL)
		return false;

	IPin* pinOut	= filterOut->getOutputPin(pinOutIndex);
	IPin* pinIn		= filterIn->getInputPin(pinInIndex);

	if (pinOut == NULL || pinIn == NULL)
	{
		RELEASE_PINS;
		return false;
	}

	// Connect Direct
	HRESULT hr = _graph->ConnectDirect(pinOut, pinIn, NULL);

	RELEASE_PINS;

	return (hr == S_OK);
}
//------------------------------------------------------------------------------
#undef RELEASE_PINS
//------------------------------------------------------------------------------
mxBool
fsGraphManager::disconnectFilter(const fsFilter* filterOut, mxUInt& pinOutIndex,
							     const fsFilter* filterIn, mxUInt& pinInIndex)
{
	if (filterOut == NULL || filterIn == NULL)
		return false;

	mxUInt numOutPins = filterOut->getNumOutputPins();
	mxUInt numInPins = filterIn->getNumInputPins();

	IPin* pinOut		= NULL;
	IPin* pinIn			= NULL;
	IPin* pinOutToIn	= NULL;

	for(mxUInt i = 0; i < numOutPins; i++)
	{
		pinOut = filterOut->getOutputPin(i);
		pinOut->ConnectedTo(&pinOutToIn);

		if (pinOutToIn != NULL)
		{
			for(mxUInt j = 0; j < numInPins; j++)
			{
				pinIn = filterIn->getInputPin(j);

				if (pinOutToIn == pinIn)
				{
					_graph->Disconnect(pinOut);
					_graph->Disconnect(pinIn);

					pinOutIndex = i;
					pinInIndex = j;
				}

				pinIn->Release();
				pinIn = NULL;
			}

			pinOutToIn->Release();
			pinOutToIn = NULL;
		}

		pinOut->Release();
		pinOut = NULL;
	}

	return true;
}
//------------------------------------------------------------------------------

#include <flTypes/flEventCode.h>

//------------------------------------------------------------------------------
mxUInt
fsGraphManager::mediaEventLoop()
{
	HRESULT hr;
	HANDLE event;
	mxLong evCode, param1, param2;

	hr = _mediaEvent->GetEventHandle((OAEVENT *)&event);

	while(_mediaEventThreadRun)
	{
		if (WAIT_OBJECT_0 == WaitForSingleObject(event, 10))
		{
			while(hr = _mediaEvent->GetEvent(&evCode, &param1, &param2, 0), SUCCEEDED(hr))
			{
				_mediaEvent->FreeEventParams(evCode, param1, param2);

				if (_graphEventProc == NULL)
					continue;

				switch(evCode)
				{
				case EC_COMPLETE:
					_graphEventProc(ET_STREAMCOMPLETED, (void *)param1, (void *)param2, _graphEventData);
					break;
				case EC_MEDIATYPE_CHANGED:
					_graphEventProc(ET_MEDIATYPECHANGED, (void *)param1, (void *)param2, _graphEventData);
					break;
				}

			}
		}
	}

	return 0;
}
//------------------------------------------------------------------------------
mxUInt __stdcall
fsGraphManager::mediaEventLoopProc(void* data)
{
	fsGraphManager *manager = (fsGraphManager *)data;
	return manager->mediaEventLoop();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// For GraphEdit Reflection
HRESULT
fsGraphManager::addToRot(IUnknown *pUnkGraph, DWORD *pdwRegister)
{
	IMoniker * pMoniker;
	IRunningObjectTable *pROT;

	if (FAILED(GetRunningObjectTable(0, &pROT)))
	{
		return E_FAIL;
	}

	WCHAR wsz[256];
	wsprintfW(wsz, L"FilterGraph %08x pid %08x", (DWORD_PTR)pUnkGraph, GetCurrentProcessId());
	HRESULT hr = CreateItemMoniker(L"!", wsz, &pMoniker);

	if (SUCCEEDED(hr))
	{
		hr = pROT->Register(ROTFLAGS_REGISTRATIONKEEPSALIVE, pUnkGraph,
		pMoniker, pdwRegister);
		pMoniker->Release();
	}

	pROT->Release();

	return hr;
}
//------------------------------------------------------------------------------
void
fsGraphManager::removeFromRot(DWORD pdwRegister)
{
	IRunningObjectTable *pROT;
	if (SUCCEEDED(GetRunningObjectTable(0, &pROT)))
	{
		pROT->Revoke(pdwRegister);
		pROT->Release();
	}

	pdwRegister = 0;
}
//------------------------------------------------------------------------------
