/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsGraphManager_h__
#define __fsGraphManager_h__
//------------------------------------------------------------------------------
#include <fs/fsCommon/fsFilter.h>
#include <fs/fsCommon/fsCommon.h>
#include <mx/mxCore/mxThread.h>
#include <mx/mxCore/mxString.h>
#include <mx/mxCore/mxObject.h>
#include <Dshow.h>
//------------------------------------------------------------------------------
class fsGraphManager;
typedef mxObjectPtr<fsGraphManager>	fsGraphManagerPtr;
//------------------------------------------------------------------------------
class FSCOMMONCLASS fsGraphManager : public mxObject
{
	MXOBJECTHEADER(fsGraphManager, mxObject);

	typedef void (*GraphEventProc)(mxInt evType, void* param1, void* param2, void* data);
	typedef void (*ProcessIntermediateProc)(const fsFilterPtr* filters, mxUInt numFilters, void* data);

public:
	enum GraphEventTypeEnum {
		ET_STREAMCOMPLETED,
		ET_MEDIATYPECHANGED,
		NUM_GRAPHEVENTTYPE
	};

public:
	fsGraphManager(GraphEventProc proc1, void* data1,
				   ProcessIntermediateProc proc2, void *data2);

protected:
	virtual ~fsGraphManager();

public:
	void			setGraphEventProc(GraphEventProc proc, void* data);
	void			setProcessIntermediateProc(ProcessIntermediateProc proc, void *data);

public:
	// Initialize / Shutdown
	mxBool			initialize(mxBool enableAddToRot = false);
	mxBool			shutdown();

	// Graph Control
	enum GraphStatus {
		GS_START,
		GS_STOP,
		GS_PAUSE
	};
	mxInt			getStatus() const;
	mxBool			start();
	mxBool			stop();
	mxBool			pause();

	// VideoWindow Control
	void			setVideoWindowPosition(mxInt x, mxInt y);
	void			getVideoWindowPosition(mxInt& x, mxInt& y) const;
	void			setVideoWindowSize(mxUInt width, mxUInt height);
	void			getVideoWindowSize(mxUInt& width, mxUInt& height) const;
	void			setVideoWindowOwner(HWND windowHandle);
	HWND			getVideoWindowOwner() const;
	void			setVideoWindowStyle(mxLong style);
	mxLong			getVideoWindowStyle() const;
	void			setVideoWindowVisible(mxBool visible);
	mxBool			getVideoWindowVisible() const;
	void			setVideoWindowAutoShow(mxBool autoShow);
	mxBool			getVideoWindowAutoShow() const;
	void			getVideoWindowResolution(mxUInt& width, mxUInt& height) const;

public:
	// Filter Operation
	mxUInt			getNumFilters() const;
	void			addFilter(const fsFilter* filter);
	void			removeFilter(const fsFilter* filter);
	void			removeAllFilters();

	// Connection
	mxBool			connectFilterIndirect(
								const fsFilter* filterOut, mxUInt pinOutIndex,
								const fsFilter* filterIn, mxUInt pinInIndex);
	mxBool			connectFilterDirect(
								const fsFilter* filterOut, mxUInt pinOutIndex, 
								const fsFilter* filterIn, mxUInt pinInIndex);
	mxBool			disconnectFilter(
								const fsFilter* filterOut, mxUInt& pinOutIndex,
								const fsFilter* filterIn, mxUInt& pinInIndex);

protected:
	// Media Event
	mxUInt			mediaEventLoop();
	static mxUInt __stdcall	mediaEventLoopProc(void* data);

protected:
	// For Graph Reflection
	HRESULT			addToRot(IUnknown *pUnkGraph, DWORD *pdwRegister);
	void			removeFromRot(DWORD pdwRegister);

protected:
	// Callback
	GraphEventProc			_graphEventProc;
	void*					_graphEventData;
	ProcessIntermediateProc	_processIntermediateProc;
	void*					_processIntermediateData;

	// Graph Status
	mxInt					_status;

	// FilterGraph Interfaces
	IGraphBuilder*			_graph;
	IMediaControl*			_mediaControl;
	IMediaSeeking*			_mediaSeeking;
	IMediaEventEx*			_mediaEvent;
	IVideoWindow*			_videoWindow;
	IBasicVideo*			_basicVideo;

	// GraphEvent Thread
	mxBool					_mediaEventThreadRun;
	mxThread*				_mediaEventThread;

	// For Graph Reflection
	DWORD					_register;
};
//------------------------------------------------------------------------------
#endif // __fsGraphManager_h__
