/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsMulticastReceiverFilter.h>
//------------------------------------------------------------------------------
fsMulticastReceiverFilter::fsMulticastReceiverFilter(const mxString& name) :
fsFilter(name)
{
	HRESULT hr;
	hr = CoCreateInstance(CLSID_MulticastReceiver, NULL, CLSCTX_INPROC,
						IID_IBaseFilter, (void **)&_filter);

	_receiver = NULL;

	if (hr == S_OK)
	{
		hr = _filter->QueryInterface(IID_IMulticastReceiver, (void **)&_receiver);
		if (hr != S_OK)
		{
			_receiver	= NULL;

			_filter->Release();
			_filter	= NULL;
		}
	}

	_address		= "";
	_mediaTypeStr	= "";
	_senderAddress	= "";
	if (_receiver != NULL)
	{
		mxChar address[MR_MAXIMUM_ADDRESS_LENGTH];
		_receiver->get_Address(address);
		_address = mxString(address);

		mxChar mediaTypeStr[MR_MAXIMUM_MEDIATYPESTR_LENGTH];
		_receiver->get_MediaTypeString(mediaTypeStr);
		_mediaTypeStr = mxString(mediaTypeStr);

		mxChar senderAddress[MR_MAXIMUM_ADDRESS_LENGTH];
		_receiver->get_SenderAddress(senderAddress);
		_senderAddress = mxString(senderAddress);
	}

}
//------------------------------------------------------------------------------
fsMulticastReceiverFilter::~fsMulticastReceiverFilter()
{
	if (_receiver != NULL)
		_receiver->Release();
}
//------------------------------------------------------------------------------
const mxString&
fsMulticastReceiverFilter::getAddress() const
{
	return _address;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setAddress(const mxString& address)
{
	_address = address;
	if (_receiver == NULL)
		return ;
	_receiver->put_Address(address.c_str());
}
//------------------------------------------------------------------------------
mxUShort
fsMulticastReceiverFilter::getPort() const
{
	if (_receiver == NULL)
		return 0;
	mxUShort port;
	_receiver->get_Port(&port);
	return port;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setPort(mxUShort port)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_Port(port);
}
//------------------------------------------------------------------------------
mxULong
fsMulticastReceiverFilter::getTimeToLive() const
{
	if (_receiver == NULL)
		return 0;
	mxULong ttl;
	_receiver->get_TimeToLive(&ttl);
	return ttl;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setTimeToLive(mxULong ttl)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_TimeToLive(ttl);
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastReceiverFilter::getNumRejectionAddress() const
{
	if (_receiver == NULL)
		return 0;
	mxUInt numAddress;
	_receiver->get_NumRejectionAddress(&numAddress);
	return numAddress;
}
//------------------------------------------------------------------------------
mxString
fsMulticastReceiverFilter::getRejectionAddress(mxUInt index) const
{
	if (_receiver == NULL)
		return "";
	mxChar str[1024];
	_receiver->get_RejectionAddress(index, str);
	return mxString(str);
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::addRejectionAddress(const mxString& address)
{
	if (_receiver == NULL)
		return ;
	_receiver->add_RejectionAddress(address.c_str());
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::removeRejectionAddress(mxUInt index)
{
	if (_receiver == NULL)
		return ;
	_receiver->remove_RejectionAddress(index);
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::removeAllRejectionAddress()
{
	if (_receiver == NULL)
		return ;
	_receiver->remove_AllRejectionAddress();
}
//------------------------------------------------------------------------------
mxInt
fsMulticastReceiverFilter::getMediaType() const
{
	if (_receiver == NULL)
		return IMulticastReceiver::MRMT_UNKNOWN;
	mxInt mediaType;
	_receiver->get_MediaType(&mediaType);
	return mediaType;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setMediaType(mxInt mediaType)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_MediaType(mediaType);
}
//------------------------------------------------------------------------------
mxInt
fsMulticastReceiverFilter::getVideoFormat() const
{
	if (_receiver == NULL)
		return IMulticastReceiver::MRVF_UNKNOWN;
	mxInt videoFormat;
	_receiver->get_VideoFormat(&videoFormat);
	return videoFormat;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setVideoFormat(mxInt videoFormat)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_VideoFormat(videoFormat);
}
//------------------------------------------------------------------------------
mxInt
fsMulticastReceiverFilter::getVideoResolution() const
{
	if (_receiver == NULL)
		return IMulticastReceiver::MRVR_UNKNOWN;
	mxInt videoResolution;
	_receiver->get_VideoResolution(&videoResolution);
	return videoResolution;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setVideoResolution(mxInt videoResolution)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_VideoResolution(videoResolution);
}
//------------------------------------------------------------------------------
mxInt
fsMulticastReceiverFilter::getAudioFormat() const
{
	if (_receiver == NULL)
		return IMulticastReceiver::MRAF_UNKNOWN;
	mxInt audioFormat;
	_receiver->get_AudioFormat(&audioFormat);
	return audioFormat;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setAudioFormat(mxInt audioFormat)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_AudioFormat(audioFormat);
}
//------------------------------------------------------------------------------
mxInt
fsMulticastReceiverFilter::getAudioFrequency() const
{
	if (_receiver == NULL)
		return IMulticastReceiver::MRAQ_UNKNOWN;
	mxInt audioFrequency;
	_receiver->get_AudioFrequency(&audioFrequency);
	return audioFrequency;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setAudioFrequency(mxInt frequency)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_AudioFrequency(frequency);
}
//------------------------------------------------------------------------------
mxInt
fsMulticastReceiverFilter::getAudioStereo() const
{
	if (_receiver == NULL)
		return IMulticastReceiver::MRAS_UNKNOWN;
	mxInt audioStereo;
	_receiver->get_AudioStereo(&audioStereo);
	return audioStereo;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setAudioStereo(mxInt stereo)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_AudioStereo(stereo);
}
//------------------------------------------------------------------------------
mxInt
fsMulticastReceiverFilter::getAudioBitCount() const
{
	if (_receiver == NULL)
		return IMulticastReceiver::MRAB_UNKNOWN;
	mxInt audioBitCount;
	_receiver->get_AudioBitCount(&audioBitCount);
	return audioBitCount;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setAudioBitCount(mxInt bitCount)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_AudioBitCount(bitCount);
}
//------------------------------------------------------------------------------
mxBool
fsMulticastReceiverFilter::getEnableConnectReceiveMediaType() const
{
	if (_receiver == NULL)
		return false;
	mxBool flag;
	_receiver->get_EnableConnectReceiveMediaType(&flag);
	return flag;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::setEnableConnectReceiveMediaType(mxBool flag)
{
	if (_receiver == NULL)
		return ;
	_receiver->put_EnableConnectReceiveMediaType(flag);
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastReceiverFilter::getReceiveRate() const
{
	if (_receiver == NULL)
		return 0;
	mxUInt rate = 0;
	_receiver->get_ReceiveRate(&rate);
	return rate;
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastReceiverFilter::getFrameRate() const
{
	if (_receiver == NULL)
		return 0;
	mxUInt rate = 0;
	_receiver->get_FrameRate(&rate);
	return rate;
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastReceiverFilter::getNumDropPackets() const
{
	if (_receiver == NULL)
		return 0;
	mxUInt drops = 0;
	_receiver->get_NumDropPackets(&drops);
	return drops;
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastReceiverFilter::getNumDropFrames() const
{
	if (_receiver == NULL)
		return 0;
	mxUInt drops = 0;
	_receiver->get_NumDropFrames(&drops);
	return drops;
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastReceiverFilter::getPacketSize() const
{
	if (_receiver == NULL)
		return 0;
	mxUInt packetSize = 0;
	_receiver->get_PacketSize(&packetSize);
	return packetSize;
}
//------------------------------------------------------------------------------
void
fsMulticastReceiverFilter::getUserData(mxByte* userData, mxUInt& userDataLength) const
{
	if (_receiver == NULL)
	{
		userDataLength = 0;
		return ;
	}
	_receiver->get_UserData(userData, &userDataLength);
}
//------------------------------------------------------------------------------
const mxString&
fsMulticastReceiverFilter::getMediaTypeString()
{
	if (_receiver == NULL)
	{
		_mediaTypeStr = "<Unknown>";
		return _mediaTypeStr;
	}
	mxChar mediaTypeStr[MR_MAXIMUM_MEDIATYPESTR_LENGTH];
	_receiver->get_MediaTypeString(mediaTypeStr);
	_mediaTypeStr = mxString(mediaTypeStr);
	return _mediaTypeStr;
}
//------------------------------------------------------------------------------
const mxString&
fsMulticastReceiverFilter::getSenderAddress()
{
	if (_receiver == NULL)
	{
		_senderAddress = "";
		return _senderAddress;
	}
	mxChar senderAddress[MR_MAXIMUM_MEDIATYPESTR_LENGTH];
	_receiver->get_SenderAddress(senderAddress);
	_senderAddress = mxString(senderAddress);
	return _senderAddress;
}
//------------------------------------------------------------------------------
mxBool
fsMulticastReceiverFilter::isSocketOpen() const
{
	if (_receiver == NULL)
		return false;
	mxBool flag;
	_receiver->is_SocketOpen(&flag);
	return flag;
}
//------------------------------------------------------------------------------
mxBool
fsMulticastReceiverFilter::isStreaming() const
{
	if (_receiver == NULL)
		return false;
	mxBool flag;
	_receiver->is_Streaming(&flag);
	return flag;
}
//------------------------------------------------------------------------------
