/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsMulticastSenderFilter.h>
//------------------------------------------------------------------------------
fsMulticastSenderFilter::fsMulticastSenderFilter(const mxString& name) :
fsFilter(name)
{
	HRESULT hr;
	hr = CoCreateInstance(CLSID_MulticastSender, NULL, CLSCTX_INPROC,
						IID_IBaseFilter, (void **)&_filter);

	_sender = NULL;

	if (hr == S_OK)
	{
		hr = _filter->QueryInterface(IID_IMulticastSender, (void **)&_sender);
		if (hr != S_OK)
		{
			_sender	= NULL;

			_filter->Release();
			_filter	= NULL;
		}
	}

	_address = "";
	if (_sender != NULL)
	{
		mxChar address[MS_MAXIMUM_ADDRESS_LENGTH];
		_sender->get_Address(address); 
        _address = mxString(address);
	}
}
//------------------------------------------------------------------------------
fsMulticastSenderFilter::~fsMulticastSenderFilter()
{
	if (_sender != NULL)
		_sender->Release();
}
//------------------------------------------------------------------------------
mxBool
fsMulticastSenderFilter::getEnableTransmission() const
{
	if (_sender == NULL)
		return false;
	mxBool flag;
	_sender->get_EnableTransmit(&flag);
	return flag;
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::setEnableTransmission(mxBool flag)
{
	if (_sender == NULL)
		return ;
	_sender->put_EnableTransmit(flag);
}
//------------------------------------------------------------------------------
const mxString&
fsMulticastSenderFilter::getAddress() const
{
	return _address;
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::setAddress(const mxString& address)
{
	_address = address;
	if (_sender == NULL)
		return ;
	_sender->put_Address(address.c_str());
}
//------------------------------------------------------------------------------
mxUShort
fsMulticastSenderFilter::getPort() const
{
	if (_sender == NULL)
		return 0;
	mxUShort port;
	_sender->get_Port(&port);
	return port;
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::setPort(mxUShort port)
{
	if (_sender == NULL)
		return ;
	_sender->put_Port(port);
}
//------------------------------------------------------------------------------
mxULong
fsMulticastSenderFilter::getTimeToLive() const
{
	if (_sender == NULL)
		return 0;
	mxULong ttl;
	_sender->get_TimeToLive(&ttl);
	return ttl;
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::setTimeToLive(mxULong ttl)
{
	if (_sender == NULL)
		return ;
	_sender->put_TimeToLive(ttl);
}
//------------------------------------------------------------------------------
mxULong
fsMulticastSenderFilter::getIPv6InterfaceNo() const
{
	return 0;
	// Not implemented.
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::setIPv6InterfaceNo(mxULong ipv6InterfaceNo)
{
	// Not implemented.
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastSenderFilter::getMaximumPacketSize() const
{
	if (_sender == NULL)
		return 0;
	mxUInt packetSize;
	_sender->get_MaximumPacketSize(&packetSize);
	return packetSize;
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::setMaximumPacketSize(mxUInt size)
{
	if (_sender == NULL)
		return ;
	_sender->put_MaximumPacketSize(size);
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastSenderFilter::getMaximumTransmissionRate() const
{
	if (_sender == NULL)
		return 0;
	mxUInt rate;
	_sender->get_MaximumTransmissionRate(&rate);
	return rate;
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::setMaximumTransmissionRate(mxUInt rate)
{
	if (_sender == NULL)
		return ;
	_sender->put_MaximumTransmissionRate(rate);
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastSenderFilter::getTransmissionCount() const
{
	if (_sender == NULL)
		return 0;
	mxUInt count;
	_sender->get_TransmissionCount(&count);
	return count;
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::setTransmissionCount(mxUInt count)
{
	if (_sender == NULL)
		return ;
	_sender->put_TransmissionCount(count);
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::getUserData(mxByte* userData, mxUInt& userDataLength) const
{
	if (_sender == NULL)
	{
		userDataLength = 0;
		return ;
	}
	_sender->get_UserData(userData, &userDataLength);
}
//------------------------------------------------------------------------------
void
fsMulticastSenderFilter::setUserData(const mxByte* userData, mxUInt userDataLength)
{
	if (_sender == NULL)
		return ;
	_sender->put_UserData(userData, userDataLength);
}
//------------------------------------------------------------------------------
mxInt
fsMulticastSenderFilter::getInputMediaType() const
{
	if (_sender == NULL)
		return 0;
	mxInt mediaType;
	_sender->get_InputMediaType(&mediaType);
	return mediaType;
}
//------------------------------------------------------------------------------
mxInt
fsMulticastSenderFilter::getInputVideoFormat() const
{
	if (_sender == NULL)
		return 0;
	mxInt videoFormat;
    _sender->get_InputVideoFormat(&videoFormat);
	return videoFormat;
}
//------------------------------------------------------------------------------
mxInt
fsMulticastSenderFilter::getInputVideoResolution() const
{
	if (_sender == NULL)
		return 0;
	mxInt videoResolution;
	_sender->get_InputVideoResolution(&videoResolution);
	return videoResolution;
}
//------------------------------------------------------------------------------
mxInt
fsMulticastSenderFilter::getInputAudioFormat() const
{
	if (_sender == NULL)
		return 0;
	mxInt audioFormat;
	_sender->get_InputAudioFormat(&audioFormat);
	return audioFormat;
}
//------------------------------------------------------------------------------
mxInt
fsMulticastSenderFilter::getInputAudioFrequency() const
{
	if (_sender == NULL)
		return 0;
	mxInt audioFrequency;
	_sender->get_InputAudioFrequency(&audioFrequency);
	return audioFrequency;
}
//------------------------------------------------------------------------------
mxInt
fsMulticastSenderFilter::getInputAudioStereo() const
{
	if (_sender == NULL)
		return 0;
	mxInt audioStereo;
	_sender->get_InputAudioStereo(&audioStereo);
	return audioStereo;
}
//------------------------------------------------------------------------------
mxInt
fsMulticastSenderFilter::getInputAudioBitCount() const
{
	if (_sender == NULL)
		return 0;
	mxInt audioBitCount;
	_sender->get_InputAudioBitCount(&audioBitCount);
	return audioBitCount;
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastSenderFilter::getTransmissionRate() const
{
	if (_sender == NULL)
		return 0;
	mxUInt rate;
	_sender->get_TransmissionRate(&rate);
	return rate;
}
//------------------------------------------------------------------------------
mxUInt
fsMulticastSenderFilter::getFrameRate() const
{
	if (_sender == NULL)
		return 0;
	mxUInt rate;
	_sender->get_FrameRate(&rate);
	return rate;
}
//------------------------------------------------------------------------------
