/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsNamePriorityList.h>
//------------------------------------------------------------------------------
#define RECOMMENDED_STR		"Recommended"
#define GENERAL_STR			"General"
#define UNRECOMMENDED_STR	"Un-Recommended"
//------------------------------------------------------------------------------
fsNamePriorityList::fsNamePriorityList()
{
	_names.setGrowSize(64);
	_priorities.setGrowSize(64);
}
//------------------------------------------------------------------------------
fsNamePriorityList::~fsNamePriorityList()
{
}
//------------------------------------------------------------------------------
mxBool
fsNamePriorityList::load(const mxString& filename)
{
	FILE *fp = fopen(filename.c_str(), "r");
	if (fp == NULL)
		return false;

	mxChar str[1024];

	mxString name;
	PriorityEnum priority;

	_names.clear();
	_priorities.clear();
	_hash.clear();

	while(fgets(str, 1024, fp) != NULL)
	{
		mxChar *ptr = str;
		while((ptr < str + strlen(str)) && (*ptr == ' ' || *ptr == '\t'))
			ptr++;
		if (str + strlen(str) <= ptr || !strncmp(ptr, "//", 2))
			continue;

		mxChar* tok = strtok(str, ",\n");
		if (tok == NULL)
			continue;

		name = mxString(tok);

		tok = strtok(NULL, ",\n");
		if (tok == NULL)
			continue;

		if (!strcmpi(tok, GENERAL_STR))
			priority = P_GENERAL;
		else if (!strcmpi(tok, RECOMMENDED_STR))
			priority = P_RECOMMENDED;
		else if (!strcmpi(tok, UNRECOMMENDED_STR))
			priority = P_UNRECOMMENDED;
		else
			continue;

		if (_hash.find(name))
			continue;

		_names.add(name);
		_priorities.add(priority);

		_hash.insert(name, priority);
	}

	fclose(fp);

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsNamePriorityList::save(const mxString& filename)
{
	FILE *fp = fopen(filename.c_str(), "w");
	if (fp == NULL)
		return false;

	mxString priorityStr;

	for(mxUInt i = 0; i < _names.getSize(); i++)
	{
		switch(_priorities[i])
		{
		case P_RECOMMENDED:
			priorityStr = mxString(RECOMMENDED_STR);
			break;
		case P_GENERAL:
			priorityStr = mxString(GENERAL_STR);
			break;
		case P_UNRECOMMENDED:
			priorityStr = mxString(UNRECOMMENDED_STR);
			break;
		}

		fprintf(fp, "%s,%s\n", _names[i].c_str(), priorityStr);
	}

	return true;
}
//------------------------------------------------------------------------------
void
fsNamePriorityList::setPriority(const mxString& name, PriorityEnum priority)
{
	mxUInt index;
	
	if (_names.find(name, index))
	{
		_priorities[index] = priority;
		_hash.replace(name, priority);
	}
	else
	{
		_names.add(name);
		_priorities.add(priority);
		_hash.insert(name, priority);
	}
}
//------------------------------------------------------------------------------
fsNamePriorityList::PriorityEnum
fsNamePriorityList::getPriority(const mxString& name)
{
	PriorityEnum priority;
	if (_hash.get(name, priority))
		return priority;

	_names.add(name);
	_priorities.add(P_GENERAL);
	_hash.insert(name, P_GENERAL);

	return P_GENERAL;
}
//------------------------------------------------------------------------------
void
fsNamePriorityList::removePriority(const mxString& name)
{
	mxUInt index;
	if (_names.find(name, index))
	{
		_names.removeIndex(index);
		_priorities.removeIndex(index);

		_hash.remove(name);
	}
}
//------------------------------------------------------------------------------
void
fsNamePriorityList::removeAllPriorities()
{
	_names.clear();
	_priorities.clear();
	_hash.clear();
}
//------------------------------------------------------------------------------
