/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsPointerMixerFilter.h>
//------------------------------------------------------------------------------
fsPointerMixerFilter::fsPointerMixerFilter(const mxString& name) :
fsFilter(name)
{
	HRESULT hr;
	hr = CoCreateInstance(CLSID_PointerMixer, NULL, CLSCTX_INPROC,
						IID_IBaseFilter, (void **)&_filter);

	_pointerMixer = NULL;

	if (hr == S_OK)
	{
		hr = _filter->QueryInterface(IID_IPointerMixer, (void **)&_pointerMixer);
		if (hr != S_OK)
		{
			_pointerMixer	= NULL;

			_filter->Release();
			_filter	= NULL;
		}
	}

	_pointerShapeNames.setSize(0);
	_pointerShapeNames.setGrowSize(16);

	if (_pointerMixer == NULL)
		return ;

	mxUInt count = 0;
	_pointerMixer->get_NumPointerShape(&count);
	for(mxUInt i = 0; i < count; i++)
	{
		mxChar shapename[PM_MAXIMUM_POINTERSHAPENAME_LENGTH];
		_pointerMixer->get_PointerShapeName(i, shapename);
		_pointerShapeNames.add(mxString(shapename));
	}
}
//------------------------------------------------------------------------------
fsPointerMixerFilter::~fsPointerMixerFilter()
{
	if (_pointerMixer != NULL)
		_pointerMixer->Release();
}
//------------------------------------------------------------------------------
void
fsPointerMixerFilter::setWindowPosition(mxFloat x, mxFloat y)
{
	if (_pointerMixer == NULL)
		return ;
	_pointerMixer->put_WindowPosition(x, y);
}
//------------------------------------------------------------------------------
void
fsPointerMixerFilter::getWindowPosition(mxFloat& x, mxFloat& y) const
{
	x = 0;
	y = 0;
	if (_pointerMixer == NULL)
		return ;
	_pointerMixer->get_WindowPosition(&x, &y);
}
//------------------------------------------------------------------------------
void
fsPointerMixerFilter::setWindowSize(mxFloat width, mxFloat height)
{
	if (_pointerMixer == NULL)
		return ;
	_pointerMixer->put_WindowSize(width, height);
}
//------------------------------------------------------------------------------
void
fsPointerMixerFilter::getWindowSize(mxFloat& width, mxFloat& height) const
{
	width = 0;
	height = 0;
	if (_pointerMixer == NULL)
		return ;
	_pointerMixer->get_WindowSize(&width, &height);
}
//------------------------------------------------------------------------------
mxUInt
fsPointerMixerFilter::getNumPointerShape() const
{
	if (_pointerMixer == NULL)
		return 0;
	mxUInt count;
	_pointerMixer->get_NumPointerShape(&count);
	return count;
}
//------------------------------------------------------------------------------
mxBool
fsPointerMixerFilter::addPointerShape(const mxString& filename)
{
	if (_pointerMixer == NULL)
		return false;
	return (_pointerMixer->add_PointerShape(filename.c_str()) == NOERROR);
}
//------------------------------------------------------------------------------
void
fsPointerMixerFilter::removePointerShape(mxUInt index)
{
	if (_pointerMixer == NULL)
		return ;
	_pointerMixer->remove_PointerShape(index);
}
//------------------------------------------------------------------------------
const mxString&
fsPointerMixerFilter::getPointerShapeName(mxUInt index) const
{
	assert(index < _pointerShapeNames.getSize());
	return _pointerShapeNames[index];
}
//------------------------------------------------------------------------------
void
fsPointerMixerFilter::setPointerShapeName(mxUInt index, const mxString& name)
{
	assert(index < _pointerShapeNames.getSize());
	_pointerShapeNames[index] = name;
	if (_pointerMixer == NULL)
		return ;
	_pointerMixer->put_PointerShapeName(index, _pointerShapeNames[index].c_str());
}
//------------------------------------------------------------------------------
void
fsPointerMixerFilter::getPointerShapeOrigin(mxUInt index, mxInt& originX, mxInt& originY) const
{
	if (_pointerMixer == NULL)
		return ;
	_pointerMixer->get_PointerShapeOrigin(index, &originX, &originY);
}
//------------------------------------------------------------------------------
void
fsPointerMixerFilter::setPointerShapeOrigin(mxUInt index, mxInt originX, mxInt originY)
{
	if (_pointerMixer == NULL)
		return ;
	_pointerMixer->put_PointerShapeOrigin(index, originX, originY);
}
//------------------------------------------------------------------------------
