/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsPointerPositionFilter.h>
//------------------------------------------------------------------------------
fsPointerPositionFilter::fsPointerPositionFilter(const mxString& name) :
fsFilter(name)
{
	HRESULT hr;
	hr = CoCreateInstance(CLSID_PointerPosition, NULL, CLSCTX_INPROC,
						IID_IBaseFilter, (void **)&_filter);

	_pointerPosition = NULL;

	if (hr == S_OK)
	{
		hr = _filter->QueryInterface(IID_IPointerPosition, (void **)&_pointerPosition);
		if (hr != S_OK)
		{
			_pointerPosition	= NULL;

			_filter->Release();
			_filter	= NULL;
		}
	}


	_phrases.setSize(0);
	_phrases.setGrowSize(16);

	if (_pointerPosition == NULL)
		return ;

	mxUInt count = 0;
	_pointerPosition->get_NumPhrases(&count);
	for(mxUInt i = 0; i < count; i++)
	{
		mxChar phrase[PP_MAXIMUM_PHRASE_LENGTH];
		_pointerPosition->get_Phrase(i, phrase);
		_phrases.add(mxString(phrase));
	}
}
//------------------------------------------------------------------------------
fsPointerPositionFilter::~fsPointerPositionFilter()
{
	if (_pointerPosition != NULL)
		_pointerPosition->Release();
}
//------------------------------------------------------------------------------
// Monitor Functions
void
fsPointerPositionFilter::getScreenSize(mxUShort& width, mxUShort& height) const
{
	if (_pointerPosition == NULL)
	{
		width = 1;
		height = 1;
		return ;
	}
	_pointerPosition->get_ScreenSize(&width, &height);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::getPointerPosition(mxUShort& x, mxUShort& y) const
{
	if (_pointerPosition == NULL)
	{
		x = 0;
		y = 0;
		return ;
	}
	_pointerPosition->get_PointerPosition(&x, &y);
}
//------------------------------------------------------------------------------
mxUShort
fsPointerPositionFilter::getButtonStatus() const
{
	if (_pointerPosition == NULL)
		return 0;
	mxUShort status;
	_pointerPosition->get_ButtonStatus(&status);
	return status;
}
//------------------------------------------------------------------------------
mxUShort
fsPointerPositionFilter::getWheelStatus() const
{
	if (_pointerPosition == NULL)
		return 0;
	mxUShort status;
	_pointerPosition->get_WheelStatus(&status);
	return status;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::getNormalizedPointerPosition(mxFloat& x, mxFloat& y) const
{
	if (_pointerPosition == NULL)
	{
		x = 0;
		y = 0;
		return ;
	}
	_pointerPosition->get_NormalizedPointerPosition(&x, &y);
}
//------------------------------------------------------------------------------
// Standard Settings
//------------------------------------------------------------------------------
mxULong
fsPointerPositionFilter::getOutputMinimumPeriod() const
{
	if (_pointerPosition == NULL)
		return 0;
	mxULong period;
	_pointerPosition->get_OutputMinimumPeriod(&period);
	return period;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setOutputMinimumPeriod(mxULong period)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_OutputMinimumPeriod(period);
}
//------------------------------------------------------------------------------
mxShort
fsPointerPositionFilter::getPointerShape() const
{
	if (_pointerPosition == NULL)
		return 0;
	mxShort pointerShape;
	_pointerPosition->get_PointerShape(&pointerShape);
	return pointerShape;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setPointerShape(mxShort pointerShape)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_PointerShape(pointerShape);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::getPointerColor(mxByte* color) const
{
	if (_pointerPosition == NULL)
	{
		memset(color, 0, sizeof(mxByte) * 3);
		return ;
	}
	_pointerPosition->get_PointerColor(color);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setPointerColor(const mxByte* color)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_PointerColor(color);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::getNormalizedOrigin(mxFloat& x, mxFloat& y) const
{
	if (_pointerPosition == NULL)
	{
		x = 0;
		y = 0;
		return ;
	}
	_pointerPosition->get_NormalizedOrigin(&x, &y);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setNormalizedOrigin(mxFloat x, mxFloat y)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_NormalizedOrigin(x, y);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::getNormalizedSize(mxFloat& width, mxFloat& height) const
{
	if (_pointerPosition == NULL)
	{
		width = 0;
		height = 0;
		return ;
	}
	_pointerPosition->get_NormalizedSize(&width, &height);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setNormalizedSize(mxFloat width, mxFloat height)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_NormalizedSize(width, height);
}
//------------------------------------------------------------------------------
// Locus Settings
mxBool
fsPointerPositionFilter::getDrawLocusOn() const
{
	if (_pointerPosition == NULL)
		return false;
	mxBool flag;
	_pointerPosition->get_DrawLocusOn(&flag);
	return flag;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setDrawLocusOn(mxBool flag)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_DrawLocusOn(flag);
}
//------------------------------------------------------------------------------
mxUShort
fsPointerPositionFilter::getLocusDrawButton() const
{
	if (_pointerPosition == NULL)
		return 0;
	mxUShort buttonStatus;
	_pointerPosition->get_LocusDrawButton(&buttonStatus);
	return buttonStatus;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setLocusDrawButton(mxUShort buttonStatus)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_LocusDrawButton(buttonStatus);
}
//------------------------------------------------------------------------------
mxUShort
fsPointerPositionFilter::getLocusEraseButton() const
{
	if (_pointerPosition == NULL)
		return 0;
	mxUShort buttonStatus;
	_pointerPosition->get_LocusEraseButton(&buttonStatus);
	return buttonStatus;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setLocusEraseButton(mxUShort buttonStatus)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_LocusEraseButton(buttonStatus);
}
//------------------------------------------------------------------------------
mxShort
fsPointerPositionFilter::getLocusDrawPointerShape() const
{
	if (_pointerPosition == NULL)
		return 0;
	mxShort shape;
	_pointerPosition->get_LocusDrawPointerShape(&shape);
	return shape;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setLocusDrawPointerShape(mxShort shape)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_LocusDrawPointerShape(shape);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::getLocusDrawPointerColor(mxByte* color) const
{
	if (_pointerPosition == NULL)
	{
		memset(color, 0, sizeof(mxByte) * 3);
		return ;
	}
	_pointerPosition->get_LocusDrawPointerColor(color);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setLocusDrawPointerColor(const mxByte* color)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_LocusDrawPointerColor(color);
}
//------------------------------------------------------------------------------
mxBool
fsPointerPositionFilter::getLocusEdgingOn() const
{
	if (_pointerPosition == NULL)
		return false;
	mxBool flag;
	_pointerPosition->get_LocusEdgingOn(&flag);
	return flag;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setLocusEdgingOn(mxBool flag)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_LocusEdgingOn(flag);
}
//------------------------------------------------------------------------------
mxBool
fsPointerPositionFilter::getLocusBrokenLineOn() const
{
	if (_pointerPosition == NULL)
		return false;
	mxBool flag;
	_pointerPosition->get_LocusBrokenLineOn(&flag);
	return flag;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setLocusBrokenLineOn(mxBool flag)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_LocusBrokenLineOn(flag);
}
//------------------------------------------------------------------------------
mxUShort
fsPointerPositionFilter::getLocusWidth() const
{
	if (_pointerPosition == NULL)
		return 0;
	mxUShort width;
	_pointerPosition->get_LocusWidth(&width);
	return width;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setLocusWidth(mxUShort width)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_LocusWidth(width);
}
//------------------------------------------------------------------------------
// Phrase Settings
mxBool
fsPointerPositionFilter::getShowPhraseOn() const
{
	if (_pointerPosition == NULL)
		return false;
	mxBool flag;
	_pointerPosition->get_ShowPhraseOn(&flag);
	return flag;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setShowPhraseOn(mxBool flag)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_ShowPhraseOn(flag);
}
//------------------------------------------------------------------------------
mxUInt
fsPointerPositionFilter::getNumPhrases() const
{
	if (_pointerPosition == NULL)
		return 0;
	mxUInt count;
	_pointerPosition->get_NumPhrases(&count);
	return count;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setNumPhrases(mxUInt count)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_NumPhrases(count);

	_phrases.setSize(0);
	for(mxUInt i = 0; i < count; i++)
	{
		mxChar phrase[PP_MAXIMUM_PHRASE_LENGTH];
		_pointerPosition->get_Phrase(i, phrase);
		_phrases.add(mxString(phrase));
	}
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::addPhrase()
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->add_Phrase();
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::removePhrase(mxUInt index)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->remove_Phrase(index);
}
//------------------------------------------------------------------------------
const mxString&
fsPointerPositionFilter::getPhrase(mxUInt index) const
{
	assert(index < _phrases.getSize());
	return _phrases[index];
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setPhrase(mxUInt index, const mxString& phrase)
{
	assert(index < _phrases.getSize());
	_phrases[index] = phrase;
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_Phrase(index, _phrases[index].c_str());
}
//------------------------------------------------------------------------------
mxUShort
fsPointerPositionFilter::getPhraseShowButton(mxUInt index) const
{
	if (_pointerPosition == NULL)
		return 0;
	mxUShort buttonStatus;
	_pointerPosition->get_PhraseShowButton(index, &buttonStatus);
	return buttonStatus;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setPhraseShowButton(mxUInt index, mxUShort buttonStatus)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_PhraseShowButton(index, buttonStatus);
}
//------------------------------------------------------------------------------
mxShort
fsPointerPositionFilter::getPhraseShowPointerShape(mxUInt index) const
{
	if (_pointerPosition == NULL)
		return 0;
	mxShort shape;
	_pointerPosition->get_PhraseShowPointerShape(index, &shape);
	return shape;
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setPhraseShowPointerShape(mxUInt index, mxShort shape)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_PhraseShowPointerShape(index, shape);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::getPhraseShowPointerColor(mxUInt index, mxByte *color) const
{
	if (_pointerPosition == NULL)
	{
		memset(color, 0, sizeof(mxByte) * 3);
		return ;
	}
	_pointerPosition->get_PhraseShowPointerColor(index, color);
}
//------------------------------------------------------------------------------
void
fsPointerPositionFilter::setPhraseShowPointerColor(mxUInt index, const mxByte* color)
{
	if (_pointerPosition == NULL)
		return ;
	_pointerPosition->put_PhraseShowPointerColor(index, color);
}
//------------------------------------------------------------------------------
