/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsPointerPositionFilter_h__
#define __fsPointerPositionFilter_h__
//------------------------------------------------------------------------------
#include <fs/fsCommon/fsFilter.h>

#include <initguid.h>
#include <flTypes/flUIDs.h>
#include <flTypes/IPointerPosition.h>
//------------------------------------------------------------------------------
class fsPointerPositionFilter;
typedef mxObjectPtr<fsPointerPositionFilter>	fsPointerPositionFilterPtr;
//------------------------------------------------------------------------------
struct _Color
{
	mxByte rgb[3];
};
//------------------------------------------------------------------------------
class FSCOMMONCLASS fsPointerPositionFilter : public fsFilter
{
	MXOBJECTHEADER(fsPointerPositionFilter, fsFilter);

public:
	fsPointerPositionFilter(const mxString& name);

protected:
	virtual ~fsPointerPositionFilter();

public:
	// Monitor Functions
	void			getScreenSize(mxUShort& width, mxUShort& height) const;
	void			getPointerPosition(mxUShort& x, mxUShort& y) const;
	mxUShort		getButtonStatus() const;
	mxUShort		getWheelStatus() const;
	void			getNormalizedPointerPosition(mxFloat& x, mxFloat& y) const;
	
	// Standard Settings
	mxULong			getOutputMinimumPeriod() const;
	void			setOutputMinimumPeriod(mxULong period);

	mxShort			getPointerShape() const;
	void			setPointerShape(mxShort pointerShape);

	void			getPointerColor(mxByte* color) const;
	void			setPointerColor(const mxByte* color);

	void			getNormalizedOrigin(mxFloat& x, mxFloat& y) const;
	void			setNormalizedOrigin(mxFloat x, mxFloat y);

	void			getNormalizedSize(mxFloat& width, mxFloat& height) const;
	void			setNormalizedSize(mxFloat width, mxFloat height);

	// Locus Settings
	mxBool			getDrawLocusOn() const;
	void			setDrawLocusOn(mxBool flag);

	mxUShort		getLocusDrawButton() const;
	void			setLocusDrawButton(mxUShort buttonStatus);

	mxUShort		getLocusEraseButton() const;
	void			setLocusEraseButton(mxUShort buttonStatus);

	mxShort			getLocusDrawPointerShape() const;
	void			setLocusDrawPointerShape(mxShort shape);

	void			getLocusDrawPointerColor(mxByte *color) const;
	void			setLocusDrawPointerColor(const mxByte* color);

	mxBool			getLocusEdgingOn() const;
	void			setLocusEdgingOn(mxBool flag);

	mxBool			getLocusBrokenLineOn() const;
	void			setLocusBrokenLineOn(mxBool flag);

	mxUShort		getLocusWidth() const;
	void			setLocusWidth(mxUShort width);

	// Phrase Settings
	mxBool			getShowPhraseOn() const;
	void			setShowPhraseOn(mxBool flag);

	mxUInt			getNumPhrases() const;
	void			setNumPhrases(mxUInt count);
	void			addPhrase();
	void			removePhrase(mxUInt index);

	const mxString&	getPhrase(mxUInt index) const;
	void			setPhrase(mxUInt index, const mxString& phrase);

	mxUShort		getPhraseShowButton(mxUInt index) const;
	void			setPhraseShowButton(mxUInt index, mxUShort buttonStatus);

	mxShort			getPhraseShowPointerShape(mxUInt index) const;
	void			setPhraseShowPointerShape(mxUInt index, mxShort shape);

	void			getPhraseShowPointerColor(mxUInt index, mxByte* color) const;
	void			setPhraseShowPointerColor(mxUInt index, const mxByte* color);

protected:
	IPointerPosition*		_pointerPosition;
	mxArray<mxString>		_phrases;
};
//------------------------------------------------------------------------------
//DECLARE_SAFE_COPYFUNC(fsPointerPositionFilterPtr);
//DECLARE_SAFE_COMPFUNC(fsPointerPositionFilterPtr);
//------------------------------------------------------------------------------
#endif //__fsPointerPositionFilter_h__
