/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsStillPictureFilter.h>
#include <assert.h>
//------------------------------------------------------------------------------
fsStillPictureFilter::fsStillPictureFilter(const mxString& name) :
fsSourceFilter("Still Picture", name)
{
	HRESULT hr;
	hr = CoCreateInstance(CLSID_StillPicture, NULL, CLSCTX_INPROC,
						IID_IBaseFilter, (void **)&_filter);

	if (hr != S_OK)
		_filter		= NULL;

	_stillPicture = NULL;
	if (hr == S_OK)
	{
		hr = _filter->QueryInterface(IID_IStillPicture, (void **)&_stillPicture);
		if (hr != S_OK)
		{
			_stillPicture = NULL;

			_filter->Release();
			_filter	= NULL;
		}
	}

	_filename = "";
	if (_stillPicture != NULL)
	{
		mxChar filename[SP_MAXIMUM_FILENAME_LENGTH];
		_stillPicture->get_Filename(filename);
		_filename = mxString(filename);
	}
}
//------------------------------------------------------------------------------
fsStillPictureFilter::~fsStillPictureFilter()
{
	if (_stillPicture != NULL)
	{
		_stillPicture->Release();
		_stillPicture = NULL;
	}
}
//------------------------------------------------------------------------------
mxInt
fsStillPictureFilter::getOutputVideoResolution() const
{
	mxInt resolution = 0;
	if (_stillPicture == NULL)
		return 0;
	_stillPicture->get_OutputVideoResolution(&resolution);
	return resolution;
}
//------------------------------------------------------------------------------
void
fsStillPictureFilter::setOutputVideoResolution(mxInt resolution)
{
	if (_stillPicture == NULL)
		return ;
	_stillPicture->put_OutputVideoResolution(resolution);
}
//------------------------------------------------------------------------------
mxInt
fsStillPictureFilter::getOutputMode() const
{
	mxInt outputMode = 0;
	if (_stillPicture == NULL)
		return outputMode;
	_stillPicture->get_OutputMode(&outputMode);
	return outputMode;
}
//------------------------------------------------------------------------------
void
fsStillPictureFilter::setOutputMode(mxInt mode)
{
	if (_stillPicture == NULL)
		return ;
	_stillPicture->put_OutputMode(mode);
}
//------------------------------------------------------------------------------
const mxString&
fsStillPictureFilter::getFilename() const
{
	return _filename;
}
//------------------------------------------------------------------------------
void
fsStillPictureFilter::setFilename(const mxString& filename)
{
	_filename = filename;
	if (_stillPicture == NULL)
		return ;
	_stillPicture->put_Filename(filename.c_str());
}
//------------------------------------------------------------------------------
mxInt
fsStillPictureFilter::getFrameRate() const
{
	mxInt frameRate = 0;
	if (_stillPicture == NULL)
		return frameRate;
	_stillPicture->get_FrameRate(&frameRate);
	return frameRate;
}
//------------------------------------------------------------------------------
void
fsStillPictureFilter::setFrameRate(mxInt frameRate)
{
	if (_stillPicture == NULL)
		return ;
	_stillPicture->put_FrameRate(frameRate);
}
//------------------------------------------------------------------------------
mxBool
fsStillPictureFilter::getNoiseOn() const
{
	mxBool noiseOn = false;
	if (_stillPicture == NULL)
		return noiseOn;
	_stillPicture->get_NoiseOn(&noiseOn);
	return noiseOn;
}
//------------------------------------------------------------------------------
void
fsStillPictureFilter::setNoiseOn(mxBool flag)
{
	if (_stillPicture == NULL)
		return ;
	_stillPicture->put_NoiseOn(flag);
}
//------------------------------------------------------------------------------
