/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsCommon/fsVideoRenderFilter.h>
//------------------------------------------------------------------------------
fsVideoRenderFilter::fsVideoRenderFilter(const mxString& name) :
fsFilter(name)
{
	HRESULT hr;
	hr = CoCreateInstance(CLSID_VideoRendererDefault, NULL, CLSCTX_INPROC,
						IID_IBaseFilter, (void **)&_filter);

	if (hr != S_OK)
		hr = CoCreateInstance(CLSID_VideoRenderer, NULL, CLSCTX_INPROC,
						IID_IBaseFilter, (void **)&_filter);

	_videoWindow = NULL;

	if (hr == S_OK)
	{
		hr = _filter->QueryInterface(IID_IVideoWindow, (void **)&_videoWindow);
		if (hr != S_OK)
		{
			_videoWindow= NULL;

			_filter->Release();
			_filter		= NULL;
		}
	}
}
//------------------------------------------------------------------------------
fsVideoRenderFilter::~fsVideoRenderFilter()
{
	if (_videoWindow != NULL)
		_videoWindow->Release();
}
//------------------------------------------------------------------------------
void
fsVideoRenderFilter::getVideoWindowPosition(mxInt& x, mxInt& y) const
{
	if (_videoWindow == NULL)
	{
		x = 1;
		y = 1;
		return ;
	}

	_videoWindow->get_Left((mxLong *)&x);
	_videoWindow->get_Top((mxLong *)&y);
}
//------------------------------------------------------------------------------
void
fsVideoRenderFilter::setVideoWindowPosition(mxInt x, mxInt y)
{
	if (_videoWindow == NULL)
		return ;

	_videoWindow->put_Left((mxLong)x);
	_videoWindow->put_Top((mxLong)y);
}
//------------------------------------------------------------------------------
void
fsVideoRenderFilter::getVideoWindowSize(mxUInt& width, mxUInt& height) const
{
	if (_videoWindow == NULL)
	{
		width = 1;
		height = 1;
		return ;
	}

	_videoWindow->get_Width((mxLong *)&width);
	_videoWindow->get_Height((mxLong *)&height);
}
//------------------------------------------------------------------------------
void
fsVideoRenderFilter::setVideoWindowSize(mxUInt width, mxUInt height)
{
	if (_videoWindow == NULL)
		return ;

	_videoWindow->put_Width((mxLong)width);
	_videoWindow->put_Height((mxLong)height);
}
//------------------------------------------------------------------------------
mxBool
fsVideoRenderFilter::getVisible() const
{
	if (_videoWindow == NULL)
		return false;
	
	mxLong visible;
	_videoWindow->get_Visible(&visible);
	return (visible == OATRUE);
}
//------------------------------------------------------------------------------
void
fsVideoRenderFilter::setVisible(mxBool flag)
{
	if (_videoWindow == NULL)
		return ;
	_videoWindow->put_Visible(flag ? OATRUE : OAFALSE);	
}
//------------------------------------------------------------------------------
HWND
fsVideoRenderFilter::getWindowOwner() const
{
	if (_videoWindow == NULL)
		return NULL;
	HWND owner;
	_videoWindow->get_Owner((OAHWND *)&owner);
	return owner;
}
//------------------------------------------------------------------------------
void
fsVideoRenderFilter::setWindowOwner(HWND owner)
{
	if (_videoWindow == NULL)
		return ;
	_videoWindow->put_Owner((OAHWND)owner);
}
//------------------------------------------------------------------------------
mxLong
fsVideoRenderFilter::getWindowStyle() const
{
	if (_videoWindow == NULL)
		return 0;
	mxLong style;
	_videoWindow->get_WindowStyle(&style);
	return style;
}
//------------------------------------------------------------------------------
void
fsVideoRenderFilter::setWindowStyle(mxLong style)
{
	if (_videoWindow == NULL)
		return ;
	_videoWindow->put_WindowStyle(style);
}
//------------------------------------------------------------------------------
