/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsMultiReceiver/fsMediaReceiver.h>
//------------------------------------------------------------------------------
fsMediaReceiver::fsMediaReceiver(MediaTypeChangedProc proc, void* data) :
mxObject()
{
	_windowHandle			= NULL;;

	_graph					= NULL;

	_isValid				= false;
	_errorMessage			= "";

	// MediaTypeChanged Procedure
	_mediaTypeChangedProc	= proc;
	_mediaTypeChangedData	= data;
}
//------------------------------------------------------------------------------
fsMediaReceiver::~fsMediaReceiver()
{
}
//------------------------------------------------------------------------------
//
// Initialize / Shutdown
//
mxBool
fsMediaReceiver::initialize(HWND windowHandle)
{
	_windowHandle		= windowHandle;
	_isValid			= false;
	_errorMessage		= "";

	_graphCS.enter();

	// Create Graph manager
	_graph = new fsGraphManager(graphEventProc, this, processIntermediateProc, this);
	if (!_graph->initialize(true))
		return false;

	// Create Filters
	if (!createFilters())
	{
		deleteFilters();
		_graph->shutdown();
		return false;
	}

	// Initialize Filters
	initializeFilters();

	_graphCS.leave();

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsMediaReceiver::shutdown()
{
	_graphCS.enter();

	// Stop Graph
	_graph->stop();
	clearGraph();

	// Delete Filters
	deleteFilters();

	// Delete Graph manager
	_graph->shutdown();
	_graph		= NULL;

	_graphCS.leave();

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsMediaReceiver::standby()
{
	_graphCS.enter();

	_graph->stop();
	_graph->removeAllFilters();

	_graph->addFilter(*(_videoReceiver));

	mxBool flag = _graph->start();

	_graphCS.leave();

	return flag;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Streaming Control
//
//------------------------------------------------------------------------------
mxBool
fsMediaReceiver::getValid() const
{
	return _isValid;
}
//------------------------------------------------------------------------------
const mxString&
fsMediaReceiver::getErrorMessage() const
{
	return _errorMessage;
}
//------------------------------------------------------------------------------
mxBool
fsMediaReceiver::receiveVideo() const
{
	return _videoReceiver->isStreaming();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multicast Settings
//
//------------------------------------------------------------------------------
void
fsMediaReceiver::setAddress(const mxString& address)
{
	_videoReceiver->setAddress(address);
}
//------------------------------------------------------------------------------
const mxString&
fsMediaReceiver::getAddress() const
{
	return _videoReceiver->getAddress();
}
//------------------------------------------------------------------------------
// Video Receiver
mxUInt
fsMediaReceiver::getVideoReceiverReceiveRate() const
{
	return _videoReceiver->getReceiveRate();
}
//------------------------------------------------------------------------------
mxUInt
fsMediaReceiver::getVideoReceiverFrameRate() const
{
	return _videoReceiver->getFrameRate();
}
//------------------------------------------------------------------------------
mxUInt
fsMediaReceiver::getVideoReceiverNumDropPackets() const
{
	return _videoReceiver->getNumDropPackets();
}
//------------------------------------------------------------------------------
mxUInt
fsMediaReceiver::getVideoReceiverNumDropFrames() const
{
	return _videoReceiver->getNumDropFrames();
}
//------------------------------------------------------------------------------
mxUInt
fsMediaReceiver::getVideoReceiverPacketSize() const
{
	return _videoReceiver->getPacketSize();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Monitor
//
//------------------------------------------------------------------------------
mxBool
fsMediaReceiver::getSocketOpen() const
{
	return _videoReceiver->isSocketOpen();
}
//------------------------------------------------------------------------------
mxString captionStr;
//------------------------------------------------------------------------------
const mxString&
fsMediaReceiver::getCaption() const
{
	mxByte userData[MR_MAXIMUM_USERDATA_SIZE];
	mxUInt userDataSize;

	_videoReceiver->getUserData(userData, userDataSize);
	captionStr = (userDataSize != 0 ? mxString((const mxChar *)userData) : "");

	return captionStr;
}
//------------------------------------------------------------------------------
const mxString&
fsMediaReceiver::getMediaTypeString() const
{
	return _videoReceiver->getMediaTypeString();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Video Window Settings
//
//------------------------------------------------------------------------------
void
fsMediaReceiver::setVideoWindowPosition(mxInt x, mxInt y)
{
	_graph->setVideoWindowPosition(x, y);
}
//------------------------------------------------------------------------------
void
fsMediaReceiver::getVideoWindowPosition(mxInt& x, mxInt& y) const
{
	_graph->getVideoWindowPosition(x, y);
}
//------------------------------------------------------------------------------
void
fsMediaReceiver::setVideoWindowSize(mxInt width, mxInt height)
{
	_graph->setVideoWindowSize((mxUInt)width, (mxUInt)height);
}
//------------------------------------------------------------------------------
void
fsMediaReceiver::getVideoWindowSize(mxInt& width, mxInt& height) const
{
	_graph->getVideoWindowSize((mxUInt &)width, (mxUInt &)height);
}
//------------------------------------------------------------------------------
void
fsMediaReceiver::getVideoWindowResolution(mxInt& width, mxInt& height) const
{
	_graph->getVideoWindowResolution((mxUInt &)width, (mxUInt &)height);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsMediaReceiver::createFilters()
{
	//
	// Create Filters
	//
	_videoRender	= new fsVideoRenderFilter("VideoRender");
	_videoReceiver	= new fsMulticastReceiverFilter("VideoReceiver");

	//
	// CheckFilters
	//
	mxBool ret = true;
	mxString message("The following DirectShow Filter was not found.\n\n");

	if (!_videoRender->isValid())
	{
		message += "EVideo Renderer\n";
		ret &= false;
	}

	if (!_videoReceiver->isValid())
	{
		message += "EMulticast Receiver\n";
		ret &= false;
	}

	if (!ret)
		showMessageBox((LPTSTR)message.c_str());

	return ret;
}
//------------------------------------------------------------------------------
void
fsMediaReceiver::deleteFilters()
{
	_videoRender = NULL;
	_videoReceiver = NULL;
}
//------------------------------------------------------------------------------
void
fsMediaReceiver::initializeFilters()
{
	// VideoReceiver
	_videoReceiver->setAddress("224.0.1.0");//DEFAULT_ADDRESS);
	_videoReceiver->setPort(PORT_VIDEOMAIN);
	_videoReceiver->setTimeToLive(DEFAULT_TIMETOLIVE);
	_videoReceiver->setEnableConnectReceiveMediaType(true);
}
//------------------------------------------------------------------------------
mxBool
fsMediaReceiver::buildGraph()
{
	_isValid = true;

	_graph->addFilter(*(_videoReceiver));
	_graph->addFilter(*(_videoRender));
	
	if (!_graph->connectFilterIndirect(*(_videoReceiver), 0, *(_videoRender), 0))
	{
		_errorMessage = "Build Graph: VIDEO-DECODER filter is not found.";
		_isValid = false;
		return false;
	}

	// Window Setting
	_graph->setVideoWindowAutoShow(false);
	_graph->setVideoWindowOwner(_windowHandle);
	_graph->setVideoWindowVisible(true);
	_graph->setVideoWindowStyle(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);

	return true;
}
//------------------------------------------------------------------------------
void
fsMediaReceiver::clearGraph()
{
	_isValid = true;

	// Window Setting
	_graph->setVideoWindowVisible(false);
	_graph->setVideoWindowOwner(NULL);

	// Remove All Filters
	_graph->removeAllFilters();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsMediaReceiver::showMessageBox(LPSTR sz, ...)
{
    static TCHAR tach[2000];
    va_list va;

    va_start(va, sz);
    wvsprintf(tach, sz, va);
    va_end(va);

    MessageBox(_windowHandle, tach, TEXT("Error"), MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Graph Event
void
fsMediaReceiver::graphEvent(mxInt eventCode, void* param1, void* param2)
{
	if (eventCode != fsGraphManager::ET_MEDIATYPECHANGED)
		return ;

	_graphCS.enter();

	_graph->stop();
	clearGraph();
	if (buildGraph())
		_graph->start();

	_graphCS.leave();

	if (_mediaTypeChangedProc != NULL)
		_mediaTypeChangedProc(this, _mediaTypeChangedData);
}
//------------------------------------------------------------------------------
void
fsMediaReceiver::graphEventProc(mxInt eventCode, void *param1, void* param2, void *data)
{
	fsMediaReceiver *mediaReceiver = (fsMediaReceiver *)data;
	mediaReceiver->graphEvent(eventCode, param1, param2);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Process Intermediate
// ̊֐́AconnectFilterIndirect()ĂяoƂɁA
// ԃtB^ꂽꍇɌĂяo܂B
// ԃtB^ɑ΂A炩̃p[^ݒsꍇɎgp܂B
void
fsMediaReceiver::processIntermediate(const fsFilterPtr* filters, mxUInt numFilters)
{
	for(mxUInt i = 0; i < numFilters; i++)
	{
		IBaseFilter* filterObj = filters[i]->getBaseFilter();

		// DV Decoder Resolution Setting
		IIPDVDec* dvDecoder = NULL;
		HRESULT hr = filterObj->QueryInterface(IID_IIPDVDec, (void **)&dvDecoder);
		if (hr == S_OK)
		{
			dvDecoder->put_IPDisplay(DVDECODERRESOLUTION_360x240);
			dvDecoder->Release();
			dvDecoder = NULL;
			break;
		}
	}
}
//------------------------------------------------------------------------------
void
fsMediaReceiver::processIntermediateProc(const fsFilterPtr* filters, mxUInt numFilters, void* data)
{
	fsMediaReceiver *mediaReceiver = (fsMediaReceiver *)data;
	mediaReceiver->processIntermediate(filters, numFilters);
}
//------------------------------------------------------------------------------
