/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsMediaReceiver_h__
#define __fsMediaReceiver_h__
//------------------------------------------------------------------------------
#include <fs/fsCommon/fsVideoRenderFilter.h>
#include <fs/fsCommon/fsMulticastReceiverFilter.h>
#include <fs/fsCommon/fsGraphManager.h>
#include <mx/mxCore/mxCriticalSection.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------
class fsMediaReceiver;
typedef mxObjectPtr<fsMediaReceiver>	fsMediaReceiverPtr;
//------------------------------------------------------------------------------
class fsMediaReceiver : public mxObject
{
	MXOBJECTHEADER(fsMediaReceiver, mxObject);

	typedef void (*MediaTypeChangedProc)(fsMediaReceiver* receiver, void* param);

public:
	fsMediaReceiver(MediaTypeChangedProc proc, void* data);

protected:
	virtual ~fsMediaReceiver();

public:
	//
	// Initialize / Shutdown
	//
	mxBool				initialize(HWND windowHandle);
	mxBool				shutdown();
	mxBool				standby();

public:
	//
	// Streaming Control
	//
	mxBool				getValid() const;
	const mxString&		getErrorMessage() const;
	mxBool				receiveVideo() const;

	//
	// Multicast Settings
	//
	void				setAddress(const mxString& address);
	const mxString&		getAddress() const;

	// Video Receiver
	mxUInt				getVideoReceiverReceiveRate() const;
	mxUInt				getVideoReceiverFrameRate() const;
	mxUInt				getVideoReceiverNumDropPackets() const;
	mxUInt				getVideoReceiverNumDropFrames() const;
	mxUInt				getVideoReceiverPacketSize() const;

	//
	// Monitor
	//
	mxBool				getSocketOpen() const;
	const mxString&		getCaption() const;
	const mxString&		getMediaTypeString() const;

	//
	// Video Window Settings
	//
	void				setVideoWindowPosition(mxInt x, mxInt y);
	void				getVideoWindowPosition(mxInt& x, mxInt& y) const;
	void				setVideoWindowSize(mxInt width, mxInt height);
	void				getVideoWindowSize(mxInt& width, mxInt& height) const;
	void				getVideoWindowResolution(mxInt& width, mxInt& height) const;

protected:
	mxBool				createFilters();
	void				deleteFilters();

	void				initializeFilters();

	mxBool				buildGraph();
	void				clearGraph();

public:
	void				showMessageBox(LPSTR sz, ...);

protected:
	// Graph Event
	void				graphEvent(mxInt eventCode,
								void* param1,
								void* param2);
	static void			graphEventProc(mxInt eventCode, 
								void *param1,
								void* param2,
								void *data);

	// Process Intermediate
	void				processIntermediate(const fsFilterPtr* filters,
								mxUInt numFilters);
	static void			processIntermediateProc(const fsFilterPtr* filters, 
								mxUInt numFilters,
								void* data);


protected:
	HWND						_windowHandle;

	mxCriticalSection			_graphCS;
	fsGraphManagerPtr			_graph;

	mxBool						_isValid;
	mxString					_errorMessage;

	// Filters
	fsFilterPtr					_videoRender;
	fsMulticastReceiverFilterPtr	_videoReceiver;

	// MediaType Changed Procedure
	MediaTypeChangedProc		_mediaTypeChangedProc;
	void*						_mediaTypeChangedData;
};
//------------------------------------------------------------------------------
DECLARE_SAFE_COPYFUNC(fsMediaReceiverPtr);
DECLARE_SAFE_COMPFUNC(fsMediaReceiverPtr);
//------------------------------------------------------------------------------
#endif //__fsMediaReceiver_h__
