/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsMultiReceiver/fsMultiReceiver.h>

#include <olectl.h>
#if (1100 > _MSC_VER)
#include <olectlid.h>
#endif

#include <mx/mxCore/mxIniFile.h>
#include <mx/mxCore/mxRegistry.h>
//------------------------------------------------------------------------------
BOOL APIENTRY
DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
    return TRUE;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsMultiReceiver::fsMultiReceiver() :
mxObject()
{
	mxRegistry::setCompanyName(FS_REGISTRY_COMPANY_NAME);
	mxRegistry::setProductName(FS_REGISTRY_PRODUCT_NAME);

	_UICallbackFunction		= NULL;
	_UICallbackData			= NULL;

	_videoWindowMinimize	= false;
}
//------------------------------------------------------------------------------
fsMultiReceiver::~fsMultiReceiver()
{
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Initialize / Shutdown
//
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::initialize()
{
	HRESULT hr = CoInitialize(NULL);
	if (FAILED(hr))
		return false;

	readRegistry();

	_quitFlag = false;
	_addMediaReceiverAddresses.setGrowSize(16);
	_addMediaReceiverAddresses.clear();
	_removeMediaReceiverIndices.setGrowSize(16);
	_removeMediaReceiverIndices.clear();

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::shutdown()
{
	writeRegistry();

	for(mxUInt i = 0; i < _receivers.getSize(); i++)
	{
		fsMediaReceiverPtr receiver = _receivers[i];
		receiver->shutdown();
		_receivers.removeIndex(i--);
	}

	CoUninitialize();

	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multimedia Receiver
//
//------------------------------------------------------------------------------
mxInt
fsMultiReceiver::getNumMediaReceiver() const
{
	return _receivers.getSize();
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::addMediaReceiver(const mxString& address)
{
	_receiverCS.enter();

	// Get MediaReceiver Parent Window handle
	HWND windowHandle = NULL;
	if (_UICallbackFunction != NULL)
	{
		struct _Message
		{
			const mxChar*	address;
			HWND			windowHandle;
		};
		_Message mes;
		mes.address = address.c_str();
		mes.windowHandle = NULL;

		_UICallbackFunction("AddMediaReceiver", (void *)&mes, _UICallbackData);

		windowHandle = mes.windowHandle;
	}
	assert(windowHandle != NULL);

	// Create MediaReceiver
	fsMediaReceiverPtr receiver = new fsMediaReceiver(mediaTypeChangedProc, this);
	if (!receiver->initialize(windowHandle))
	{
		_receiverCS.leave();
		return false;
	}

	receiver->setAddress(address);

	// Standby MediaReceiver
	if (!receiver->standby())
	{
		_receiverCS.leave();
		return false;
	}

	_receivers.add(receiver);
	_receiverCS.leave();

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::removeMediaReceiver(mxInt index)
{
	_receiverCS.enter();

	if (_receivers.getSize() <= (mxUInt)index)
	{
		_receiverCS.leave();
		return false;
	}

	// Remove MediaReceiver
	fsMediaReceiverPtr receiver = _receivers[(mxUInt)index];
	receiver->shutdown();
	receiver = NULL;
	_receivers.removeIndex(index);

	// Remove MediaReceiver Parent Window
	if (_UICallbackFunction != NULL)
		_UICallbackFunction("RemoveMediaReceiver", (void *)index, _UICallbackData);

	_receiverCS.leave();

	return true;
}
//------------------------------------------------------------------------------
mxString str;
//------------------------------------------------------------------------------
// Streaming Control
mxBool
fsMultiReceiver::getMediaReceiverValid(mxInt index) const
{
	_receiverCS.enter();
	mxBool flag = false;
	if ((mxUInt)index < _receivers.getSize())
		flag = _receivers[(mxUInt)index]->getValid();
	_receiverCS.leave();
	return flag;
}
//------------------------------------------------------------------------------
const mxString&
fsMultiReceiver::getMediaReceiverErrorMessage(mxInt index) const
{
	_receiverCS.enter();
	str = "";
	if ((mxUInt)index < _receivers.getSize())
		str = _receivers[(mxUInt)index]->getErrorMessage();
	_receiverCS.leave();
	return str;
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::getMediaReceiverStreaming(mxInt index) const
{
	_receiverCS.enter();
	mxBool flag = false;
	if ((mxUInt)index < _receivers.getSize())
		flag = _receivers[(mxUInt)index]->receiveVideo();
	_receiverCS.leave();
	return flag;
}
//------------------------------------------------------------------------------
// Monitor
mxBool
fsMultiReceiver::getMediaReceiverSocketOpen(mxInt index) const
{
	_receiverCS.enter();
	mxBool flag = false;
	if ((mxUInt)index < _receivers.getSize())
		flag = _receivers[(mxUInt)index]->getSocketOpen();
	_receiverCS.leave();
	return flag;
}
//------------------------------------------------------------------------------
const mxString&
fsMultiReceiver::getMediaReceiverCaption(mxInt index) const
{
	_receiverCS.enter();
	str = "";
	if ((mxUInt)index < _receivers.getSize())
		str = _receivers[(mxUInt)index]->getCaption();
	_receiverCS.leave();
	return str;
}
//------------------------------------------------------------------------------
const mxString&
fsMultiReceiver::getMediaReceiverMediaTypeString(mxInt index) const
{
	_receiverCS.enter();
	str = "";
	if ((mxUInt)index < _receivers.getSize())
		str =  _receivers[(mxUInt)index]->getMediaTypeString();
	_receiverCS.leave();
	return str;
}
//------------------------------------------------------------------------------
// Multicast Settings
const mxString&
fsMultiReceiver::getMediaReceiverAddress(mxInt index) const
{
	_receiverCS.enter();
	str = "";
	if ((mxUInt)index < _receivers.getSize())
		str = _receivers[index]->getAddress();
	_receiverCS.leave();
	return str;
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiver::getMediaReceiverReceiveRate(mxInt index) const
{
	_receiverCS.enter();
	mxInt rate = 0;
	if ((mxUInt)index < _receivers.getSize())
		rate = _receivers[index]->getVideoReceiverReceiveRate();
	_receiverCS.leave();
	return rate;
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiver::getMediaReceiverFrameRate(mxInt index) const
{
	_receiverCS.enter();
	mxInt rate =0;
	if ((mxUInt)index < _receivers.getSize())
		rate = _receivers[index]->getVideoReceiverFrameRate();
	_receiverCS.leave();
	return rate;
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiver::getMediaReceiverNumDropPackets(mxInt index) const
{
	_receiverCS.enter();
	mxInt num = 0;
	if ((mxUInt)index < _receivers.getSize())
		num = _receivers[index]->getVideoReceiverNumDropPackets();
	_receiverCS.leave();
	return num;
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiver::getMediaReceiverNumDropFrames(mxInt index) const
{
	_receiverCS.enter();
	mxInt num = 0;
	if ((mxUInt)index < _receivers.getSize())
		num = _receivers[index]->getVideoReceiverNumDropFrames();
	_receiverCS.leave();
	return num;
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiver::getMediaReceiverPacketSize(mxInt index) const
{
	_receiverCS.enter();
	mxInt size = 0;
	if ((mxUInt)index < _receivers.getSize())
		size = _receivers[index]->getVideoReceiverPacketSize();
	_receiverCS.leave();
	return size;
}
//------------------------------------------------------------------------------
// Video Settings
void
fsMultiReceiver::setMediaReceiverVideoPosition(mxInt index, mxInt x, mxInt y)
{
	if (_receivers.getSize() <= (mxUInt)index)
		return ;
	_receivers[index]->setVideoWindowPosition(x, y);
}
//------------------------------------------------------------------------------
void
fsMultiReceiver::getMediaReceiverVideoPosition(mxInt index, mxInt& x, mxInt& y) const
{
	if (_receivers.getSize() <= (mxUInt)index)
		return ;
	_receivers[index]->getVideoWindowPosition(x, y);
}
//------------------------------------------------------------------------------
void
fsMultiReceiver::setMediaReceiverVideoSize(mxInt index, mxInt width, mxInt height)
{
	if (_receivers.getSize() <= (mxUInt)index)
		return ;
	_receivers[index]->setVideoWindowSize(width, height);
}
//------------------------------------------------------------------------------
void
fsMultiReceiver::getMediaReceiverVideoSize(mxInt index, mxInt& width, mxInt& height) const
{
	if (_receivers.getSize() <= (mxUInt)index)
		return ;
	_receivers[index]->getVideoWindowSize(width, height);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Video Window Settings
//
//------------------------------------------------------------------------------
void
fsMultiReceiver::setVideoWindowMinimize(mxBool flag)
{
	_receiverCS.enter();

	_videoWindowMinimize = flag;

	if (flag)
	{
		const mxInt W = 128;
		const mxInt H = 96;

		for(mxUInt i = 0; i < _receivers.getSize(); i++)
			_receivers[i]->setVideoWindowSize(W, H);
	}
	else
	{
		mxInt rw, rh;

		for(mxUInt i = 0; i < _receivers.getSize(); i++)
		{
			_receivers[i]->getVideoWindowResolution(rw, rh);
			_receivers[i]->setVideoWindowSize(rw, rh);
		}
	}

	_receiverCS.leave();

	// Resize All Video window
	if (_UICallbackFunction != NULL)
		_UICallbackFunction("ResizePanelAll", NULL, _UICallbackData);
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::getVideoWindowMinimize() const
{
	return _videoWindowMinimize;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Other
//
//------------------------------------------------------------------------------
void
fsMultiReceiver::quit()
{
	_quitFlag = true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::_requestAddMediaReceiver(const mxString& address)
{
	_receiverCS.enter();
	_addMediaReceiverAddresses.add(address);
	_receiverCS.leave();
	return true;
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::_requestRemoveMediaReceiver(mxInt index)
{
	_receiverCS.enter();
	_removeMediaReceiverIndices.add(index);
	_receiverCS.leave();
	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::checkQuit()
{
	return _quitFlag;
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::checkRequestAddMediaReceiver(mxString& address)
{
	mxBool flag;
	_receiverCS.enter();
	if (_addMediaReceiverAddresses.getSize() != 0)
	{
		address = _addMediaReceiverAddresses[0];
		_addMediaReceiverAddresses.removeIndex(0);
		flag = true;
	}
	else
	{
		flag = false;
	}
	_receiverCS.leave();
	return flag;
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::checkRequestRemoveMediaReceiver(mxInt& index)
{
	mxBool flag;
	_receiverCS.enter();
	if (_removeMediaReceiverIndices.getSize() != 0)
	{
		index = _removeMediaReceiverIndices[0];
		_removeMediaReceiverIndices.removeIndex(0);
		flag = true;
	}
	else
	{
		flag = false;
	}
	_receiverCS.leave();
	return flag;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// IniFile
mxBool
fsMultiReceiver::writeIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "a");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);
	mxString str;
	mxUInt num;

    num = getNumMediaReceiver();
	iniFile.writeUInt("MultiReceiver", "NumberOfMediaReceiver", num);

	for(mxUInt i = 0; i < num; i++)
	{
		str = getMediaReceiverAddress(i);
		iniFile.writeString("MultiReceiver", "MediaReceiver" + mxString((mxInt)i) + "Address", str);
	}

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiver::readIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "r");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);
	mxString def, str;
	mxUInt num;

	// Timer Stop
	if (_UICallbackFunction != NULL)
		_UICallbackFunction("TimerStop", NULL, _UICallbackData);

	// Remove Old Media Receivers
	for(mxInt i = 0; i < getNumMediaReceiver(); i++)
		removeMediaReceiver(i--);

	// Create New Media Receivers
	num = iniFile.readUInt("MultiReceiver", "NumberOfMediaReceiver", 0);
	for(mxUInt i = 0; i < num; i++)
	{
		def = "234.5.6.7";
		str = iniFile.readString("MultiReceiver", "MediaReceiver" + mxString((mxInt)i) + "Address", def);

		addMediaReceiver(str);
	}

	// Timer Start
	if (_UICallbackFunction != NULL)
		_UICallbackFunction("TimerStart", NULL, _UICallbackData);

	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Registry
void
fsMultiReceiver::writeRegistry()
{
	//// CURRENT_USER
	mxRegistry	registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);

	//
	// Video Window Minimize
	//
	registry.writeString("MultiReceiver", "VideoWindowMinimize", _videoWindowMinimize ? "True" : "False");

	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsMultiReceiver::readRegistry()
{
	//// CURRENT_USER
	mxRegistry	registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);

	mxString def, str;

	//
	// Video Window Minimize
	//
	def = _videoWindowMinimize ? "True" : "False";
	str = registry.readString("MultiReceiver", "VideoWindowMinimize", def);
	str.toLower();
	_videoWindowMinimize = (str == "true" ? true : false);

	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsMultiReceiver::writeRegistry(const mxString& section, const mxString& entry,
						const mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	registry.writeString(section, entry, value);
	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsMultiReceiver::readRegistry(const mxString& section, const mxString& entry,
						const mxString& defaultValue, mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	value = registry.readString(section, entry, defaultValue);
	registry.closeKey();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Callback
mxBool
fsMultiReceiver::setUICallbackFunction(fsMultiReceiverUICallbackFunction func, void* data)
{
	_UICallbackFunction = func;
	_UICallbackData = data;
	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// MediaType Changed
void
fsMultiReceiver::mediaTypeChanged(fsMediaReceiver* receiver)
{
	assert(receiver != NULL);

	_receiverCS.enter();

	mxUInt index = (mxUInt)-1;
	for(mxUInt i = 0; i < _receivers.getSize(); i++)
	{
		if (_receivers[i]->getAddress() == receiver->getAddress())
		{
			index = i;
			break;
		}
	}
	assert(i != _receivers.getSize());

	_receiverCS.leave();

	// Setup Video window size
	mxInt width, height;
	if (_videoWindowMinimize)
	{
		width = 128;
		height = 96;
	}
	else
	{
		_receivers[index]->getVideoWindowResolution(width, height);
	}
	_receivers[index]->setVideoWindowSize(width, height);

	// Resize Video window base panel
	if (_UICallbackFunction != NULL)
		_UICallbackFunction("ResizePanel", (void *)index, _UICallbackData);
}
//------------------------------------------------------------------------------
void
fsMultiReceiver::mediaTypeChangedProc(fsMediaReceiver* receiver, void* data)
{
	fsMultiReceiver *multiReceiver = (fsMultiReceiver *)data;
	multiReceiver->mediaTypeChanged(receiver);
}
//------------------------------------------------------------------------------
