/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsMultiReceiver_h__
#define __fsMultiReceiver_h__
//------------------------------------------------------------------------------
#include <fs/fsMultiReceiver/fsMediaReceiver.h>
#include <fs/fsMultiReceiverRMI/fsMultiReceiverIf.h>
#include <mx/mxCore/mxArray.h>
#include <mx/mxCore/mxCriticalSection.h>
#include <mx/mxCore/mxObject.h>
#include <mx/mxCore/mxCore.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------
class fsMultiReceiver;
typedef mxObjectPtr<fsMultiReceiver>	fsMultiReceiverPtr;
//------------------------------------------------------------------------------
class fsMultiReceiver : public mxObject, public fsMultiReceiverIf
{
	MXOBJECTHEADER(fsMultiReceiver, mxObject);

public:
	fsMultiReceiver();

protected:
	virtual ~fsMultiReceiver();

public:
	//
	// Initialize / Shutdown
	//
	mxBool				initialize();
	mxBool				shutdown();

public:
	//
	// Implements fsMultiReceiverIf
	//

	//
	// Multimedia Receiver
	//
	mxInt				getNumMediaReceiver() const;
	mxBool				addMediaReceiver(const mxString& address);
	mxBool				removeMediaReceiver(mxInt index);

	// Streaming Control
	mxBool				getMediaReceiverValid(mxInt index) const;
	const mxString& 	getMediaReceiverErrorMessage(mxInt index) const;
	mxBool				getMediaReceiverStreaming(mxInt index) const;

	// Monitor
	mxBool				getMediaReceiverSocketOpen(mxInt index) const;
	const mxString&		getMediaReceiverCaption(mxInt index) const;
	const mxString&		getMediaReceiverMediaTypeString(mxInt index) const;

	// Multicast Settings
	const mxString& 	getMediaReceiverAddress(mxInt index) const;
	mxInt				getMediaReceiverReceiveRate(mxInt index) const;
	mxInt				getMediaReceiverFrameRate(mxInt index) const;
	mxInt				getMediaReceiverNumDropPackets(mxInt index) const;
	mxInt				getMediaReceiverNumDropFrames(mxInt index) const;
	mxInt				getMediaReceiverPacketSize(mxInt index) const;

	// Video Settings
	void				setMediaReceiverVideoPosition(mxInt index, mxInt x, mxInt y);
	void				getMediaReceiverVideoPosition(mxInt index, mxInt& x, mxInt& y) const;
	void				setMediaReceiverVideoSize(mxInt index, mxInt width, mxInt height);
	void				getMediaReceiverVideoSize(mxInt index, mxInt& width, mxInt& height) const;

	//
	// Video Window Settings
	//
	void				setVideoWindowMinimize(mxBool flag);
	mxBool				getVideoWindowMinimize() const;

	//
	// Other
	//
	void				quit();

	// [gĂяoł́AaddMediaReceiver()g킸Å֐ĂяoB
	// WindowsbZ[WXbhȊÕXbhŌĂяoۂɁA
	// Ot}l[W[fbhbNNƂւ̑ΉB
	mxBool				_requestAddMediaReceiver(const mxString& address);
	mxBool				_requestRemoveMediaReceiver(mxInt index);

public:
	mxBool				checkQuit();
	mxBool				checkRequestAddMediaReceiver(mxString& address);
	mxBool				checkRequestRemoveMediaReceiver(mxInt& index);

public:
	// IniFile
	mxBool				writeIniFile(const mxString& filename);
	mxBool				readIniFile(const mxString& filename);

	// Registry
	void				writeRegistry();
	void				readRegistry();
	void				writeRegistry(const mxString& section, const mxString& entry,
							const mxString& value);
	void				readRegistry(const mxString& section, const mxString& entry,
							const mxString& defaultValue, mxString& value);

public:
	// Callback
	typedef int (*fsMultiReceiverUICallbackFunction)(const char*, void*, void*);
	mxBool				setUICallbackFunction(fsMultiReceiverUICallbackFunction func, void* data);

public:
	// Cast
	template<class T> T*		cast()			{ return static_cast<T*>(this); }
	template<class T> const T*	cast() const	{ return static_cast<const T*>(this); }

protected:
	// MediaType Changed
	void				mediaTypeChanged(fsMediaReceiver* receiver);
	static void			mediaTypeChangedProc(fsMediaReceiver* receiver, void* data);

protected:
	mxCriticalSection			_receiverCS;
	mxArray<fsMediaReceiverPtr>	_receivers;

	fsMultiReceiverUICallbackFunction	_UICallbackFunction;
	void*						_UICallbackData;

	mxBool						_videoWindowMinimize;

protected:
	mxBool						_quitFlag;
	mxStringArray				_addMediaReceiverAddresses;
	mxIntArray					_removeMediaReceiverIndices;
};
//------------------------------------------------------------------------------
#endif // __fsMultiReceiver_h__
