/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsMultiReceiverRMI/fsMultiReceiverSkel.h>
#include <fs/fsMultiReceiverRMI/fsMultiReceiverIf.h>
#include <fs/fsCommon/fsCommon.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------
fsMultiReceiverSkel::fsMultiReceiverSkel(fsMultiReceiverIf* multiReceiver) :
mxObject()
{
	_multiReceiver = multiReceiver;

	_rObject = new fsRemoteObject(invokeProc, this);

	fsRMIPropertyPtr property = new fsRMIProperty();
	property->setAddress(DEFAULT_ADDRESS);
	property->setPort(fsRMIProperty::PID_TRANSMIT, PORT_MULTIRECEIVER_REMOTEACK);
	property->setPort(fsRMIProperty::PID_RECEIVE, PORT_MULTIRECEIVER_REMOTE);
	property->setTimeToLive(1);
	property->setPacketSize(1024);
	property->setMaximumTransmissionRate(10 * 1024 * 1024);
	property->setTransmissionCount(2);
	_rObject->setProperty(property);
}
//------------------------------------------------------------------------------
fsMultiReceiverSkel::~fsMultiReceiverSkel()
{
	_rObject = NULL;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsMultiReceiverSkel::open()
{
	return _rObject->open();
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiverSkel::close()
{
	return _rObject->close();
}
//------------------------------------------------------------------------------
void
fsMultiReceiverSkel::setRMIProperty(const fsRMIPropertyPtr& property)
{
	_rObject->setProperty(property);
}
//------------------------------------------------------------------------------
const fsRMIPropertyPtr&
fsMultiReceiverSkel::getRMIProperty() const
{
	return _rObject->getProperty();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
const fsMultiReceiverIf*
fsMultiReceiverSkel::getMultiReceiver() const
{
	return _multiReceiver;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsMultiReceiverSkel::invoke(const mxString& method,
							 fsInvokeData& param,
							 fsInvokeData& retVal)
{
	//
	// Multimedia Receiver
	//
	if (method == "getNumMediaReceiver")
	{
		retVal.writeInt(_multiReceiver->getNumMediaReceiver());
	}
	else if (method == "addMediaReceiver")
	{
		retVal.writeBool(_multiReceiver->addMediaReceiver(param.readString()));
	}
	else if (method == "removeMediaReceiver")
	{
		retVal.writeBool(_multiReceiver->removeMediaReceiver(param.readInt()));
	}

	// Streaming Control
	else if (method == "getMediaReceiverValid")
	{
		retVal.writeBool(_multiReceiver->getMediaReceiverValid(param.readInt()));
	}
	else if (method == "getMediaReceiverErrorMessage")
	{
		retVal.writeString(_multiReceiver->getMediaReceiverErrorMessage(param.readInt()));
	}
	else if (method == "getMediaReceiverStreaming")
	{
		retVal.writeBool(_multiReceiver->getMediaReceiverStreaming(param.readInt()));
	}

	// Monitor
	else if (method == "getMediaReceiverSocketOpen")
	{
		retVal.writeBool(_multiReceiver->getMediaReceiverSocketOpen(param.readInt()));
	}
	else if (method == "getMediaReceiverCaption")
	{
		retVal.writeString(_multiReceiver->getMediaReceiverCaption(param.readInt()));
	}
	else if (method == "getMediaReceiverMediaTypeString")
	{
		retVal.writeString(_multiReceiver->getMediaReceiverMediaTypeString(param.readInt()));
	}

	// Multicast Settings
	else if (method == "getMediaReceiverAddress")
	{
		retVal.writeString(_multiReceiver->getMediaReceiverAddress(param.readInt()));
	}
	else if (method == "getMediaReceiverReceiveRate")
	{
		retVal.writeInt(_multiReceiver->getMediaReceiverReceiveRate(param.readInt()));
	}
	else if (method == "getMediaReceiverFrameRate")
	{
		retVal.writeInt(_multiReceiver->getMediaReceiverFrameRate(param.readInt()));
	}
	else if (method == "getMediaReceiverNumDropPackets")
	{
		retVal.writeInt(_multiReceiver->getMediaReceiverNumDropPackets(param.readInt()));
	}
	else if (method == "getMediaReceiverNumDropFrames")
	{
		retVal.writeInt(_multiReceiver->getMediaReceiverNumDropFrames(param.readInt()));
	}
	else if (method == "getMediaReceiverPacketSize")
	{
		retVal.writeInt(_multiReceiver->getMediaReceiverPacketSize(param.readInt()));
	}

	// Video Settings
	else if (method == "setMediaReceiverVideoPosition")
	{
		mxInt index = param.readInt();
		mxInt x = param.readInt();
		mxInt y = param.readInt();
		_multiReceiver->setMediaReceiverVideoPosition(index, x, y);
	}
	else if (method == "getMediaReceiverVideoPosition")
	{
		mxInt x, y;
		_multiReceiver->getMediaReceiverVideoPosition(param.readInt(), x, y);
		retVal.writeInt(x);
		retVal.writeInt(y);
	}
	else if (method == "setMediaReceiverVideoSize")
	{
		mxInt index = param.readInt();
		mxInt width = param.readInt();
		mxInt height = param.readInt();
		_multiReceiver->setMediaReceiverVideoSize(index, width, height);
	}
	else if (method == "getMediaReceiverVideoSize")
	{
		mxInt width, height;
		_multiReceiver->getMediaReceiverVideoSize(param.readInt(), width, height);
		retVal.writeInt(width);
		retVal.writeInt(height);
	}

	//
	// Video Window Settings
	//
	else if (method == "setVideoWindowMinimize")
	{
		_multiReceiver->setVideoWindowMinimize(param.readBool());
	}
	else if (method == "getVideoWindowMinimize")
	{
		retVal.writeBool(_multiReceiver->getVideoWindowMinimize());
	}

	//
	// Other
	//
	else if (method == "quit")
	{
		_multiReceiver->quit();
	}

	else if (method == "_requestAddMediaReceiver")
	{
		retVal.writeBool(_multiReceiver->_requestAddMediaReceiver(param.readString()));
	}
	else if (method == "_requestRemoveMediaReceiver")
	{
		retVal.writeBool(_multiReceiver->_requestRemoveMediaReceiver(param.readInt()));
	}
}
//------------------------------------------------------------------------------
void
fsMultiReceiverSkel::invokeProc(const mxString& method,
								 fsInvokeData& param,
								 fsInvokeData& retVal,
								 void *data)
{
	fsMultiReceiverSkel *remoteObject = (fsMultiReceiverSkel *)data;
	remoteObject->invoke(method, param, retVal);
}
//------------------------------------------------------------------------------
