/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsMultiReceiverRMI/fsMultiReceiverStub.h>
#include <fs/fsCommon/fsCommon.h>
//------------------------------------------------------------------------------
fsMultiReceiverStub::fsMultiReceiverStub() :
mxObject()
{
	_rInterface = new fsRemoteInterface(invokeNotifyProc, this);

	fsRMIPropertyPtr property = new fsRMIProperty();
	property->setAddress(DEFAULT_ADDRESS);
	property->setPort(fsRMIProperty::PID_TRANSMIT, PORT_MULTIRECEIVER_REMOTE);
	property->setPort(fsRMIProperty::PID_RECEIVE, PORT_MULTIRECEIVER_REMOTEACK);
	property->setTimeToLive(1);
	property->setPacketSize(1024);
	property->setMaximumTransmissionRate(10 * 1024 * 1024);
	property->setTransmissionCount(2);
	property->setInvokeTimeout(100);
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
fsMultiReceiverStub::~fsMultiReceiverStub()
{
	_rInterface = NULL;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsMultiReceiverStub::open()
{
	return _rInterface->open();
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiverStub::close()
{
	return _rInterface->close();
}
//------------------------------------------------------------------------------
void
fsMultiReceiverStub::setRMIProperty(const fsRMIPropertyPtr& property)
{
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
const fsRMIPropertyPtr&
fsMultiReceiverStub::getRMIProperty() const
{
	return _rInterface->getProperty();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Implements fsMultiReceiverIf
//
//------------------------------------------------------------------------------

fsInvokeData param;
fsInvokeData ret;
mxString str;

//------------------------------------------------------------------------------
//
// Multimedia Receiver
//
//------------------------------------------------------------------------------
mxInt
fsMultiReceiverStub::getNumMediaReceiver() const
{
	param.reset();
	_rInterface->invoke("getNumMediaReceiver", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiverStub::addMediaReceiver(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("addMediaReceiver", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiverStub::removeMediaReceiver(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("removeMediaReceiver", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
// Streaming Control
mxBool
fsMultiReceiverStub::getMediaReceiverValid(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverValid", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
const mxString&
fsMultiReceiverStub::getMediaReceiverErrorMessage(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverErrorMessage", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiverStub::getMediaReceiverStreaming(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverStreaming", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
// Monitor
mxBool
fsMultiReceiverStub::getMediaReceiverSocketOpen(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverSocketOpen", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
const mxString&
fsMultiReceiverStub::getMediaReceiverCaption(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverCaption", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
const mxString&
fsMultiReceiverStub::getMediaReceiverMediaTypeString(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverMediaTypeString", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
// Multicast Settings
const mxString&
fsMultiReceiverStub::getMediaReceiverAddress(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverAddress", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiverStub::getMediaReceiverReceiveRate(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverReceiveRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiverStub::getMediaReceiverFrameRate(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiverStub::getMediaReceiverNumDropPackets(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverNumDropPackets", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiverStub::getMediaReceiverNumDropFrames(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverNumDropFrames", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsMultiReceiverStub::getMediaReceiverPacketSize(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverPacketSize", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// Video Settings
void
fsMultiReceiverStub::setMediaReceiverVideoPosition(mxInt index, mxInt x, mxInt y)
{
	param.reset();
	param.writeInt(index);
	param.writeInt(x);
	param.writeInt(y);
	_rInterface->invoke("setMediaReceiverVideoPosition", param);
}
//------------------------------------------------------------------------------
void
fsMultiReceiverStub::getMediaReceiverVideoPosition(mxInt index, mxInt& x, mxInt& y) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverVideoPosition", param, ret);
	x = ret.readInt();
	y = ret.readInt();
}
//------------------------------------------------------------------------------
void
fsMultiReceiverStub::setMediaReceiverVideoSize(mxInt index, mxInt width, mxInt height)
{
	param.reset();
	param.writeInt(index);
	param.writeInt(width);
	param.writeInt(height);
	_rInterface->invoke("setMediaReceiverVideoSize", param);
}
//------------------------------------------------------------------------------
void
fsMultiReceiverStub::getMediaReceiverVideoSize(mxInt index, mxInt& width, mxInt& height) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getMediaReceiverVideoSize", param, ret);
	width = ret.readInt();
	height = ret.readInt();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Video Window Settings
//
//------------------------------------------------------------------------------
void
fsMultiReceiverStub::setVideoWindowMinimize(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setVideoWindowMinimize", param);
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiverStub::getVideoWindowMinimize() const
{
	param.reset();
	_rInterface->invoke("getVideoWindowMinimize", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Other
//
//------------------------------------------------------------------------------
void
fsMultiReceiverStub::quit()
{
	_rInterface->invoke("quit");
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsMultiReceiverStub::_requestAddMediaReceiver(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("_requestAddMediaReceiver", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiverStub::_requestRemoveMediaReceiver(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("_requestRemoveMediaReceiver", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsMultiReceiverStub::invokeNotifyProc(const mxString& method, mxBool result, void *data)
{
	fsMultiReceiverStub *multiReceiver = (fsMultiReceiverStub *)data;
	multiReceiver->invokeNotify(method, result);
}
//------------------------------------------------------------------------------
