/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsMultiReceiverRemote/fsMultiReceiverRemote.h>
#include <fs/fsMulticastTools/fsMulticastAddress.h>
#include <fs/fsCommon/fsCommon.h>
#include <mx/mxCore/mxRegistry.h>
#include <mx/mxCore/mxIniFile.h>
//------------------------------------------------------------------------------
#pragma comment(lib, "fsMultiReceiverRMI.lib")
#pragma comment(lib, "mxCore.lib")
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
BOOL APIENTRY
DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
    return TRUE;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsMultiReceiverRemote::fsMultiReceiverRemote() :
fsMultiReceiverStub()
{
	mxRegistry::setCompanyName(FS_REGISTRY_COMPANY_NAME);
	mxRegistry::setProductName(FS_REGISTRY_PRODUCT_NAME);

	_UICallbackFunction		= NULL;
	_UICallbackData			= NULL;
}
//------------------------------------------------------------------------------
fsMultiReceiverRemote::~fsMultiReceiverRemote()
{
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Callback
mxBool
fsMultiReceiverRemote::setUICallbackFunction(fsMultiReceiverUICallbackFunction func, void* data)
{
	_UICallbackFunction = func;
	_UICallbackData = data;
	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// IniFile
mxBool
fsMultiReceiverRemote::writeIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "a");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);
	mxString str;
	mxUInt num;

    num = getNumMediaReceiver();
	iniFile.writeUInt("MultiReceiver", "NumberOfMediaReceiver", num);

	for(mxUInt i = 0; i < num; i++)
	{
		str = getMediaReceiverAddress(i);
		iniFile.writeString("MultiReceiver", "MediaReceiver" + mxString((mxInt)i) + "Address", str);
	}

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsMultiReceiverRemote::readIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "r");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);
	mxString def, str;
	mxUInt num;

	// Remove Old Media Receivers
	mxUInt numReceiver = getNumMediaReceiver();
	for(mxUInt i = 0; i < numReceiver; i++)
	{
		_requestRemoveMediaReceiver(0);
	}

	// Create New Media Receivers
	num = iniFile.readUInt("MultiReceiver", "NumberOfMediaReceiver", 0);
	for(mxUInt i = 0; i < num; i++)
	{
		def = "234.5.6.7";
		str = iniFile.readString("MultiReceiver", "MediaReceiver" + mxString((mxInt)i) + "Address", def);

		_requestAddMediaReceiver(str);
	}

	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Registry
void
fsMultiReceiverRemote::writeRegistry(const mxString& section,
									   const mxString& entry,
									   const mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	registry.writeString(section, entry, value);
	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsMultiReceiverRemote::readRegistry(const mxString& section,
									  const mxString& entry,
									  const mxString& defaultValue,
									  mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	value = registry.readString(section, entry, defaultValue);
	registry.closeKey();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// RMI Multicast Setting
//
//------------------------------------------------------------------------------
void
fsMultiReceiverRemote::setAddress(const mxString& address)
{
	fsInvokeData param;
	param.reset();
	param.writeString(address);
	_rInterface->invoke("_property_Address", param);

	const fsRMIPropertyPtr& property = _rInterface->getProperty();
	property->setAddress(address);
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
const mxString&
fsMultiReceiverRemote::getAddress() const
{
	return _rInterface->getProperty()->getAddress();
}
//------------------------------------------------------------------------------
void
fsMultiReceiverRemote::setTimeToLive(mxULong ttl)
{
	fsInvokeData param;
	param.reset();
	param.writeInt((mxInt)ttl);
	_rInterface->invoke("_property_TimeToLive", param);

	const fsRMIPropertyPtr& property = _rInterface->getProperty();
	property->setTimeToLive(ttl);
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
mxULong
fsMultiReceiverRemote::getTimeToLive() const
{
	return _rInterface->getProperty()->getTimeToLive();
}
//------------------------------------------------------------------------------
// Create Address
mxString
fsMultiReceiverRemote::createIPv4Address()
{
	return fsMulticastAddress::createAddress(fsMulticastAddress::AF_IPv4);
}
//------------------------------------------------------------------------------
mxString
fsMultiReceiverRemote::createIPv6Address()
{
	return fsMulticastAddress::createAddress(fsMulticastAddress::AF_IPv6);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsMultiReceiverRemote::invokeNotify(const mxString& method, mxBool result)
{
	if (_UICallbackFunction == NULL)
		return ;

	const mxChar* cstr = method.c_str();
	_UICallbackFunction(result ? "InvokeSucceeded" : "InvokeFailed", (void *)cstr, _UICallbackData);
}
//------------------------------------------------------------------------------
