/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsMulticastTools/fsInvokeData.h>
//------------------------------------------------------------------------------
fsInvokeData::fsInvokeData()
{
	_buffer		= NULL;
	_bufferSize	= 0;
	_dataLength	= 0;
	_readPos	= 0;
}
//------------------------------------------------------------------------------
fsInvokeData::~fsInvokeData()
{
	delete _buffer;
}
//------------------------------------------------------------------------------
void
fsInvokeData::writeString(const mxString& str)
{
	mxUInt length = str.length();
	_write(&length, sizeof(mxUInt));

	if (length != 0)
		_write(str.c_str(), length);
}
//------------------------------------------------------------------------------
mxString
fsInvokeData::readString()
{
	mxUInt length;
	if (!_read(&length, sizeof(mxUInt)))
		return "";
	
	if (length == 0)
		return "";

	mxString str;
	str.length(length);
	_read(&str[0], length);
	return str;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsInvokeData&
fsInvokeData::operator =(const fsInvokeData& invokeData)
{
	_dataLength	= invokeData._dataLength;
	_readPos	= 0;

	delete _buffer;

	_bufferSize	= ((_dataLength + 127) & (~127));
	if (_bufferSize == 0)
		return (*this);

	_buffer		= new mxByte[_bufferSize];
	memcpy(_buffer, invokeData._buffer, _dataLength);

	return (*this);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsInvokeData::setDataLength(mxUInt length)
{
	_dataLength		= length;
	_readPos		= 0;

	while(_bufferSize < _dataLength)
	{
		_bufferSize += 128;
		mxByte *newBuffer = new mxByte[_bufferSize];

		if (_buffer != NULL)
			delete _buffer;

		_buffer = newBuffer;
	}
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsInvokeData::_write(const void *data, size_t length)
{
	if (length == 0)
		return false;

	while(_bufferSize < _dataLength + length)
	{
		_bufferSize += 128;
		mxByte * newBuffer = new mxByte[_bufferSize];

		if (_buffer != NULL)
		{
			memcpy(newBuffer, _buffer, _dataLength);
			delete _buffer;
		}
		_buffer = newBuffer;
	}

	memcpy(_buffer + _dataLength, data, length);

	_dataLength += length;

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsInvokeData::_read(void *data, size_t length)
{
	if (_buffer == NULL || _dataLength <= _readPos)
		return false;

	if (length == 0)
		return false;

	memcpy(data, _buffer + _readPos, length);
	_readPos += length;

	return true;
}
//------------------------------------------------------------------------------
