/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsMulticastTools/fsMulticastDetector.h>
#include <flBase/flString.h>
//------------------------------------------------------------------------------
fsMulticastDetector::fsMulticastDetector() :
mxObject()
{
	_isOpen			= false;
	_receiveTime	= 0;

	flNetwork::initialize();
	_receiver = new flFMTReceiver();
	_receiver->setReceiveProc(receiveProc, this);
}
//------------------------------------------------------------------------------
fsMulticastDetector::~fsMulticastDetector()
{
	_receiver->close();
	delete _receiver;
	flNetwork::shutdown();
}
//------------------------------------------------------------------------------
void
fsMulticastDetector::setAddress(const mxString& address)
{
	if (!strcmp(_receiver->getAddress().c_str(), address.c_str()))
		return ;

	_receiveTime = 0;

	_receiver->close();
	_receiver->setAddress(flString(address.c_str()));
	if (_isOpen)
		_receiver->open();
}
//------------------------------------------------------------------------------
mxString
fsMulticastDetector::getAddress() const
{
	return mxString(_receiver->getAddress().c_str());
}
//------------------------------------------------------------------------------
void
fsMulticastDetector::setPort(mxUShort port)
{
	if (_receiver->getPort() == port)
		return ;

	_receiveTime = 0;

	_receiver->close();
	_receiver->setPort(port);
	if (_isOpen)
		_receiver->open();
}
//------------------------------------------------------------------------------
mxUShort
fsMulticastDetector::getPort() const
{
	return _receiver->getPort();
}
//------------------------------------------------------------------------------
void
fsMulticastDetector::setTryUnicast(mxBool flag)
{
	if (_receiver->getTryUnicast() == flag)
		return ;

	_receiveTime = 0;

	_receiver->close();
	_receiver->setTryUnicast(flag);
	if (_isOpen)
		_receiver->open();
}
//------------------------------------------------------------------------------
mxBool
fsMulticastDetector::getTryUnicast() const
{
	return _receiver->getTryUnicast();
}
//------------------------------------------------------------------------------
mxBool
fsMulticastDetector::open()
{
	if (_isOpen)
		return true;

	_isOpen = true;
	_receiveTime = 0;

	return _receiver->open();
}
//------------------------------------------------------------------------------
mxBool
fsMulticastDetector::close()
{
	if (!_isOpen)
		return true;

	_isOpen = false;

	return _receiver->close();
}
//------------------------------------------------------------------------------
mxBool
fsMulticastDetector::isOpen() const
{
	return _receiver->isOpen();
}
//------------------------------------------------------------------------------
mxBool
fsMulticastDetector::detected(mxULong evalTime) const
{
	mxULong curTime = timeGetTime();

	if (curTime < _receiveTime)
		return true;

	return (curTime - _receiveTime < evalTime);
}
//------------------------------------------------------------------------------
void
fsMulticastDetector::receive(flChar *data, flUInt dataSize)
{
	_receiveTime = timeGetTime();
}
//------------------------------------------------------------------------------
void __stdcall
fsMulticastDetector::receiveProc(flChar *data, flUInt dataSize, void* receiveParam)
{
	fsMulticastDetector *receiver = (fsMulticastDetector *)receiveParam;
	receiver->receive(data, dataSize);
}
//------------------------------------------------------------------------------
