/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsMulticastTools/fsRemoteBase.h>
//------------------------------------------------------------------------------
#define FSREMOTEBASE_UNIQUEID		0x33334608
//------------------------------------------------------------------------------
fsRemoteBase::fsRemoteBase() :
mxObject()
{
	flNetwork::initialize();
	_sender = new flFMTSender(FSREMOTEBASE_UNIQUEID);
	_receiver = new flFMTReceiver(FSREMOTEBASE_UNIQUEID);
	_isOpen = false;

	_threadRun	= true;
	_thread		= new mxThread(threadLoopProc, this);
}
//------------------------------------------------------------------------------
fsRemoteBase::~fsRemoteBase()
{
	if (_isOpen)
		close();

	_threadRun = false;
	_thread->waitFor();
	delete _thread;

	delete _sender;
	delete _receiver;
	flNetwork::shutdown();
}
//------------------------------------------------------------------------------
//
// Open / Close
//
//------------------------------------------------------------------------------
mxBool
fsRemoteBase::open()
{
	if (_isOpen)
		return true;
	_isOpen = true;

	// Sender
	if (!_sender->open())
		return false;

	// Receiver
	if (!_receiver->open())
	{
		_sender->close();
		return false;
	}

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsRemoteBase::close()
{
	if (!_isOpen)
		return true;
	_isOpen = false;

	// Receiver
	_receiver->close();

	// Sender
	_sender->close();

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsRemoteBase::isOpen() const
{
	return (_sender->isOpen() && _receiver->isOpen());
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Property setting
//
//------------------------------------------------------------------------------
void
fsRemoteBase::setProperty(const fsRMIPropertyPtr& property)
{
	_property = property;
	updateProperty();
}
//------------------------------------------------------------------------------
const fsRMIPropertyPtr&
fsRemoteBase::getProperty() const
{
	return _property;
}
//------------------------------------------------------------------------------
void
fsRemoteBase::updateProperty()
{
	mxBool isOpen = _isOpen;
	if (isOpen)
		close();

	// Address
	const mxString& address = _property->getAddress();
	if (strcmp(_sender->getAddress().c_str(), address.c_str()))
	{
		_sender->setAddress(flString(address.c_str()));
		_receiver->setAddress(flString(address.c_str()));
	}

	// Port
	mxUShort port = _property->getPort(fsRMIProperty::PID_TRANSMIT);
	if (_sender->getPort() != port)
		_sender->setPort(port);

	port = _property->getPort(fsRMIProperty::PID_RECEIVE);
	if (_receiver->getPort() != port)
		_receiver->setPort(port);

	// Time to Live
	mxULong	ttl = _property->getTimeToLive();
	if (_sender->getTimeToLive() != ttl)
		_sender->setTimeToLive(ttl);

	// Packet Size
	mxUInt packetSize = _property->getPacketSize();
	if (_sender->getPacketSize() != packetSize)
		_sender->setPacketSize(packetSize);

	// Maximum Transmission Rate
	mxUInt rate = _property->getMaximumTransmissionRate();
	if (_sender->getMaximumTransmissionRate() != rate)
		_sender->setMaximumTransmissionRate(rate);

	// Transmission Count
	mxUInt count = _property->getTransmissionCount();
	if (_sender->getTransmissionCount() != count)
		_sender->setTransmissionCount(count);

	if (isOpen)
		open();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxUInt
fsRemoteBase::threadLoop()
{
	while(_threadRun)
	{
		flChar *data;
		flUInt dataSize;

		if (_receiver->receive(data, dataSize))
			receive(data, dataSize);
		else
			Sleep(10);

		Sleep(0);
	}
	return 0;
}
//------------------------------------------------------------------------------
mxUInt __stdcall
fsRemoteBase::threadLoopProc(void* data)
{
	fsRemoteBase *remoteBase = (fsRemoteBase *)data;
	return remoteBase->threadLoop();
}
//------------------------------------------------------------------------------
