/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsMulticastTools/fsRemoteInterface.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsRemoteInterface::fsRemoteInterface(InvokeNotifyProc invokeNotifyProc, void* invokeNotifyData) :
fsRemoteBase()
{
	_invokeNotifyProc	= invokeNotifyProc;
	_invokeNotifyData	= invokeNotifyData;

	_invokeCount		= 0;

	_return.InvokeCount	= (mxUInt)-1;
}
//------------------------------------------------------------------------------
fsRemoteInterface::~fsRemoteInterface()
{
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsRemoteInterface::invoke(const mxString& method, const fsInvokeData& param, fsInvokeData& retVal)
{
	if (method.length() == 0)
		return false;

#ifdef _DEBUG
	mxNotify::post(mxNotify::INFO, "Invoke (Ret,Param) [%s]", method.c_str());
#endif

	// Reset Return Buffer
	retVal.reset();

	// Create Byte Stream data
	_oStream.reset();
	_oStream.writeUInt(ID_INVOKE);
	_oStream.writeUInt(_invokeCount);
	_oStream.writeLong((mxLong)this);
	_oStream.writeUInt(method.length() + 1);
	_oStream.writeChar((const flChar *)method.c_str(), method.length() + 1);
	_oStream.writeUInt(param.getDataLength());
	if (param.getDataLength() != 0)
		_oStream.writeChar((const flChar *)param.getData(), param.getDataLength());

	// Send Invoke
	const flChar *data;
	flUInt size;
	_oStream.get(data, size);
	_sender->send(data, size);

	// Wait for invoke Completion
	mxBool invokeResult = false;
	mxULong beginTime = GetTickCount();
	while(GetTickCount() - beginTime < _property->getInvokeTimeout())
	{
		Sleep(1);

		_returnCS.enter();
		if (_return.InvokeCount == _invokeCount)
		{
			invokeResult = true;
			retVal = _return.Data;
			_returnCS.leave();
			break;
		}
		_returnCS.leave();
	}

	// Increment Invoke count
	_invokeCount++;

	// Notify Invoke Result
	if (_invokeNotifyProc != NULL)
		_invokeNotifyProc(method, invokeResult, _invokeNotifyData);

#ifdef _DEBUG
	if (!invokeResult)	mxNotify::post(mxNotify::INFO, " Timeout.");
#endif

	return invokeResult;
}
//------------------------------------------------------------------------------
mxBool
fsRemoteInterface::invoke(const mxString& method, const fsInvokeData& param)
{
	if (method.length() == 0)
		return false;

#ifdef _DEBUG
	mxNotify::post(mxNotify::INFO, "Invoke (Param)     [%s]", method.c_str());
#endif

	// Create Byte Stream data
	_oStream.reset();
	_oStream.writeUInt(ID_INVOKE);
	_oStream.writeUInt(_invokeCount);
	_oStream.writeLong((mxLong)this);
	_oStream.writeUInt(method.length() + 1);
	_oStream.writeChar((const flChar *)method.c_str(), method.length() + 1);
	_oStream.writeUInt(param.getDataLength());
	if (param.getDataLength() != 0)
		_oStream.writeChar((const flChar *)param.getData(), param.getDataLength());

	// Send Invoke
	const flChar *data;
	flUInt size;
	_oStream.get(data, size);
	_sender->send(data, size);

	// Increment Invoke count
	_invokeCount++;

	// Notify Invoke Result
	if (_invokeNotifyProc != NULL)
		_invokeNotifyProc(method, true, _invokeNotifyData);

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsRemoteInterface::invoke(const mxString& method)
{
	if (method.length() == 0)
		return false;

#ifdef _DEBUG
	mxNotify::post(mxNotify::INFO, "Invoke (Void)      [%s]", method.c_str());
#endif

	// Create Byte Stream data
	_oStream.reset();
	_oStream.writeUInt(ID_INVOKE);
	_oStream.writeUInt(_invokeCount);
	_oStream.writeLong((mxLong)this);
	_oStream.writeUInt(method.length() + 1);
	_oStream.writeChar((const flChar *)method.c_str(), method.length() + 1);
	_oStream.writeUInt(0); // data length

	// Send Invoke
	const flChar *data;
	flUInt size;
	_oStream.get(data, size);
	_sender->send(data, size);

	// Increment Invoke count
	_invokeCount++;

	// Notify Invoke Result
	if (_invokeNotifyProc != NULL)
		_invokeNotifyProc(method, true, _invokeNotifyData);

	return true;
}
//------------------------------------------------------------------------------
void
fsRemoteInterface::receive(flChar *data, flUInt dataSize)
{
	// Receive Return
	_iStream.reset();
	_iStream.set(data, dataSize);

	mxUInt id;
	_iStream.readUInt(id);
	if (id != ID_RETURN)
		return ;

	mxUInt invokeCount;
	_iStream.readUInt(invokeCount);
	if (invokeCount != _invokeCount)
		return ;

	mxLong instance;
	_iStream.readLong(instance);
	if (instance != (mxLong)this)
		return ;

	// Set Return Value
	mxUInt length;
	_iStream.readUInt(length);

	_returnCS.enter();
	_return.InvokeCount = invokeCount;
	_return.Data.setDataLength(length);
	_iStream.readChar((flChar *)_return.Data.getData(), length);
	_returnCS.leave();
}
//------------------------------------------------------------------------------
