/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsMulticastTools/fsValueMediator.h>
#include <time.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------
#define FSVALUEMEDIATOR_UNIQUEID	0x2d170e64
//------------------------------------------------------------------------------
mxBool	fsValueMediator::_isInitRand = false;
//------------------------------------------------------------------------------
fsValueMediator::fsValueMediator(EventProc eventProc, void *eventData) :
mxObject()
{
	// Notify
	_notifyInterval			= 2500;

	// Receive
	_receiveLostInterval	= 5000;
	_receiveValues.setGrowSize(128);
	_receiveTimes.setGrowSize(128);
	_eventProc				= eventProc;
	_eventData				= eventData;
	_valueDetectedChange	= new fsStatusChange(1000);
	_valueLostChange		= new fsStatusChange(10000);

	// Multicast Objects
	flNetwork::initialize();
	_sender					= new flFMTSender(FSVALUEMEDIATOR_UNIQUEID);
	_receiver					= new flFMTReceiver(FSVALUEMEDIATOR_UNIQUEID);
	_isOpen					= false;

	// Thred
	_threadRun				= false;
	_thread					= NULL;
}
//------------------------------------------------------------------------------
fsValueMediator::~fsValueMediator()
{
	if (_thread != NULL)
	{
		_threadRun = false;
		_thread->waitFor();
		delete _thread;
		_thread = NULL;
	}

	_sender->close();
	_receiver->close();

	delete _sender;
	delete _receiver;
	flNetwork::shutdown();
}
//------------------------------------------------------------------------------
//
// Open / Close
//
mxBool
fsValueMediator::open()
{
	if (_isOpen)
		return true;
	_isOpen = true;

	// Setup OwnKey
	_ownKeyValue.Key = fsKey::createNewKey();

	// Sender
	if (!_sender->open())
		return false;

	// Receiver
	_receiver->setReceiveProc(receiveProc, this);
	if (!_receiver->open())
	{
		_sender->close();
		return false;
	}

	// Init Status Change
	_valueDetectedChange->init();
	_valueLostChange->init();

	// Thread
	_threadRun = true;
	_thread = new mxThread(threadLoopProc, this);

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsValueMediator::close()
{
	if (!_isOpen)
		return true;
	_isOpen = false;

	// Thread
	if (_thread != NULL)
	{
		_threadRun = false;
		_thread->waitFor();
		delete _thread;
		_thread = NULL;
	}

	// Receiver
	_receiver->close();

	// Sender
	_sender->close();
	return true;
}
//------------------------------------------------------------------------------
mxBool
fsValueMediator::isOpen() const
{
	return (_sender->isOpen() && _receiver->isOpen());
}
//------------------------------------------------------------------------------
//
// Setting Multicast
//
void
fsValueMediator::setAddress(const mxString& address)
{
	if (!strcmp(_sender->getAddress().c_str(), address.c_str()))
		return ;

	// Sender
	_sender->close();
	_sender->setAddress(flString(address.c_str()));
	if (_isOpen)
		_sender->open();

	// Receiver
	_receiver->close();
	_receiver->setAddress(flString(address.c_str()));
	if (_isOpen)
		_receiver->open();
}
//------------------------------------------------------------------------------
mxString
fsValueMediator::getAddress() const
{
	return mxString(_sender->getAddress().c_str());
}
//------------------------------------------------------------------------------
void
fsValueMediator::setPort(mxUShort port)
{
	if (_sender->getPort() == port)
		return ;

	// Sender
	_sender->close();
	_sender->setPort(port);
	if (_isOpen)
		_sender->open();

	// Receiver
	_receiver->close();
	_receiver->setPort(port);
	if (_isOpen)
		_receiver->open();
}
//------------------------------------------------------------------------------
mxUShort
fsValueMediator::getPort() const
{
	return _sender->getPort();
}
//------------------------------------------------------------------------------
void
fsValueMediator::setTimeToLive(mxULong ttl)
{
	if (_sender->getTimeToLive() == ttl)
		return ;

	// Sender
	_sender->close();
	_sender->setTimeToLive(ttl);
	if (_isOpen)
		_sender->open();
}
//------------------------------------------------------------------------------
mxULong
fsValueMediator::getTimeToLive() const
{
	return _sender->getTimeToLive();
}
//------------------------------------------------------------------------------
mxUInt
fsValueMediator::getNumRejectionAddress() const
{
	return _receiver->getNumRejectionAddress();
}
//------------------------------------------------------------------------------
mxString
fsValueMediator::getRejectionAddress(mxUInt index) const
{
	return mxString(_receiver->getRejectionAddress(index).c_str());
}
//------------------------------------------------------------------------------
void
fsValueMediator::addRejectionAddress(const mxString& address)
{
	_receiver->addRejectionAddress(flString(address.c_str()));
}
//------------------------------------------------------------------------------
void
fsValueMediator::removeRejectionAddress(mxUInt index)
{
	_receiver->removeRejectionAddress(index);
}
//------------------------------------------------------------------------------
void
fsValueMediator::removeAllRejectionAddress()
{
	_receiver->removeAllRejectionAddress();
}
//------------------------------------------------------------------------------
void
fsValueMediator::setPacketSize(mxUInt size)
{
	if (_sender->getPacketSize() == size)
		return ;

	// Sender
	_sender->setPacketSize(size);
}
//------------------------------------------------------------------------------
mxUInt
fsValueMediator::getPacketSize() const
{
	return _sender->getPacketSize();
}
//------------------------------------------------------------------------------
//
// Setting Value
//
void
fsValueMediator::setValue(mxInt value)
{
	_ownKeyValue.Value = value;
}
//------------------------------------------------------------------------------
mxInt
fsValueMediator::getValue() const
{
	return _ownKeyValue.Value;
}
//------------------------------------------------------------------------------
mxInt
fsValueMediator::getFreeValue(mxInt minValue, mxInt maxValue)
{
	if (!_isInitRand)
		initRand();

	mxInt len = maxValue - minValue;
	mxInt value = (rand() % len) + minValue;

	return value;
}
//------------------------------------------------------------------------------
//
// Notify
//
void
fsValueMediator::setNotifyInterval(mxULong interval)
{
	_notifyInterval = interval;
}
//------------------------------------------------------------------------------
mxULong
fsValueMediator::getNotifyInterval() const
{
	return _notifyInterval;
}
//------------------------------------------------------------------------------
//
// Receive
//
//------------------------------------------------------------------------------
void
fsValueMediator::setReceiveLostInterval(mxULong interval)
{
	_receiveLostInterval = interval;
}
//------------------------------------------------------------------------------
mxULong
fsValueMediator::getReceiveLostInterval() const
{
	return _receiveLostInterval;
}
//------------------------------------------------------------------------------
void
fsValueMediator::setEventBlankTimeInValueDetected(mxULong time)
{
	_valueDetectedChange->setChangedInterval(time);
}
//------------------------------------------------------------------------------
mxULong
fsValueMediator::getEventBlankTimeInValueDetected() const
{
	return _valueDetectedChange->getChangedInterval();
}
//------------------------------------------------------------------------------
void
fsValueMediator::setEventBlankTimeInValueLost(mxULong time)
{
	_valueLostChange->setChangedInterval(time);
}
//------------------------------------------------------------------------------
mxULong
fsValueMediator::getEventBlankTimeInValueLost() const
{
	return _valueLostChange->getChangedInterval();
}
//------------------------------------------------------------------------------
void
fsValueMediator::receive(flChar *data, flUInt dataSize)
{
	if (dataSize != sizeof(fsKeyValueData))
		return ;

	fsKeyValueData *keyValue = (fsKeyValueData *)data;

	// Disregard Own KeyValue
	if (keyValue->Key == _ownKeyValue.Key)
		return ;

	// Fire Event If value collides with.
	if (keyValue->Value == _ownKeyValue.Value)
	{
		if (_eventProc != NULL)
			_eventProc(E_COLLIDEVALUE, NULL, _eventData);
	}


	// Update Receive Values
	_cs.enter();

    mxUInt index;
	if (_receiveValues.find(keyValue->Value, index))
	{
		_receiveTimes[index] = GetTickCount();
	}
	else
	{
		_receiveValues.add(keyValue->Value);
		_receiveTimes.add(GetTickCount());

#ifdef _DEBUG
mxNotify::post(mxNotify::INFO, "Changed : Detected.");
#endif
		_valueDetectedChange->changed();

		if (_eventProc != NULL)
			_eventProc(E_VALUEDETECTED, (void *)keyValue->Value, _eventData);
	}

	_cs.leave();
}
//------------------------------------------------------------------------------
void __stdcall
fsValueMediator::receiveProc(flChar *data, flUInt dataSize, void* receiveParam)
{
	fsValueMediator *mediator = (fsValueMediator *)receiveParam;
	mediator->receive(data, dataSize);
}
//------------------------------------------------------------------------------
mxUInt
fsValueMediator::threadLoop()
{
	mxULong curTime = 0;
	mxULong prevNotifyTime = 0;

	while(_threadRun)
	{
		curTime = GetTickCount();

		// Notify PortNumber
		if (_notifyInterval < curTime - prevNotifyTime)
		{
			prevNotifyTime = curTime;

			_sender->send((const flChar *)&_ownKeyValue, sizeof(_ownKeyValue));
		}
		Sleep(5);

		// Check Receive Values
		_cs.enter();
		{
			for(mxUInt i = 0; i < _receiveValues.getSize(); i++)
			{
				mxInt interval = mxInt(curTime) - mxInt(_receiveTimes[i]);

				if ((mxInt)_receiveLostInterval < interval)
				{
#ifdef _DEBUG
mxNotify::post(mxNotify::INFO, "Changed : Lost. (Interval:%d)", interval);
#endif
					_valueLostChange->changed();

					if (_eventProc != NULL)
						_eventProc(E_VALUELOST, (void *)_receiveValues[i], _eventData);

					_receiveValues.removeIndex(i);
					_receiveTimes.removeIndex(i);

					i--;
				}
			}
		}
		_cs.leave();
		Sleep(5);

		// Fire Event
		_cs.enter();
		{
			mxBool notify;

			notify = _valueDetectedChange->notifyChanged();
			notify |= _valueLostChange->notifyChanged();

			if (notify)
			{
				if (_eventProc != NULL)
				{
					rsReceiveValueStatus status;
					status.NumValues = _receiveValues.getSize();
					status.Values = _receiveValues.getPointer();
					_eventProc(E_RECEIVEVALUESTATUS, (void *)&status, _eventData);
				}
			}
		}
		_cs.leave();
		Sleep(5);
	}

	return 0;
}
//------------------------------------------------------------------------------
mxUInt __stdcall
fsValueMediator::threadLoopProc(void* data)
{
	fsValueMediator *mediator = (fsValueMediator *)data;
	return mediator->threadLoop();
}
//------------------------------------------------------------------------------
void
fsValueMediator::initRand()
{
	_isInitRand = true;

	// Time
	time_t t;
	time(&t);

	// IP Address
	const flString& address = flNetwork::getLocalhost();

	// Create Seed
	srand((mxUInt)t + (mxUInt)address.hash());
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsStatusChange::fsStatusChange(mxULong changedInterval)
{
	_changedInterval = changedInterval;

	_changed = false;
	_changedTime = 0;
}
//------------------------------------------------------------------------------
fsStatusChange::~fsStatusChange()
{
}
//------------------------------------------------------------------------------
void
fsStatusChange::setChangedInterval(mxULong time)
{
	_changedInterval = time;
}
//------------------------------------------------------------------------------
mxULong
fsStatusChange::getChangedInterval() const
{
	return _changedInterval;
}
//------------------------------------------------------------------------------
void
fsStatusChange::init()
{
	_changed = false;
	_changedTime = GetTickCount();
}
//------------------------------------------------------------------------------
void
fsStatusChange::changed()
{
	if (!_changed)
		_changedTime = GetTickCount();
	_changed = true;
}
//------------------------------------------------------------------------------
mxBool
fsStatusChange::notifyChanged()
{
	if (!_changed)
		return false;

	if (_changedInterval < GetTickCount() - _changedTime)
	{
		_changed = false;
		return true;
	}

	return false;
}
//------------------------------------------------------------------------------
