/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsValueMediator_h__
#define __fsValueMediator_h__
//------------------------------------------------------------------------------
#include <fs/fsMulticastTools/fsKey.h>
#include <fs/fsMulticastTools/fsMulticastTools.h>
#include <mx/mxCore/mxCriticalSection.h>
#include <mx/mxCore/mxArray.h>
#include <mx/mxCore/mxThread.h>
#include <mx/mxCore/mxString.h>
#include <mx/mxCore/mxObject.h>
#include <flNetwork/flFMTSender.h>
#include <flNetwork/flFMTReceiver.h>
//------------------------------------------------------------------------------
struct fsKeyValueData
{
	fsKeyValueData()
	{
		Value	= 0;
	}

	fsKey		Key;
	mxInt		Value;
};
//------------------------------------------------------------------------------
struct rsReceiveValueStatus
{
	rsReceiveValueStatus()
	{
		NumValues	= 0;
		Values		= NULL;
	};

	mxUInt			NumValues;
	const mxInt*	Values;
};
//------------------------------------------------------------------------------
class fsStatusChange;
class fsValueMediator;
typedef mxObjectPtr<fsValueMediator>	fsValueMediatorPtr;
//------------------------------------------------------------------------------
class FSMULTICASTTOOLSCLASS fsValueMediator : public mxObject
{
	MXOBJECTHEADER(fsValueMediator, mxObject);

public:
	enum EventTypeEnum 
	{
		E_RECEIVEVALUESTATUS,
		E_COLLIDEVALUE,
		E_VALUEDETECTED,
		E_VALUELOST,

		NUM_EVENT
	};
	typedef void (*EventProc)(EventTypeEnum eventType, void* eventParam, void* data);

public:
	fsValueMediator(EventProc eventProc, void *eventData);

protected:
	virtual ~fsValueMediator();

public:
	//
	// Open / Close
	//
	mxBool			open();
	mxBool			close();
	mxBool			isOpen() const;

	//
	// Setting Multicast
	//
	void			setAddress(const mxString& address);
	mxString		getAddress() const;
	void			setPort(mxUShort port);
	mxUShort		getPort() const;
	void			setTimeToLive(mxULong ttl);
	mxULong			getTimeToLive() const;
	void			setPacketSize(mxUInt size);
	mxUInt			getPacketSize() const;

	mxUInt			getNumRejectionAddress() const;
	mxString		getRejectionAddress(mxUInt index) const;
	void			addRejectionAddress(const mxString& address);
	void			removeRejectionAddress(mxUInt index);
	void			removeAllRejectionAddress();

	//
	// Setting Value
	//
	void			setValue(mxInt value);
	mxInt			getValue() const;
	static mxInt	getFreeValue(mxInt minValue, mxInt maxValue);

	//
	// Notify
	//
	void			setNotifyInterval(mxULong interval);
	mxULong			getNotifyInterval() const;

	//
	// Receive
	//
	void			setReceiveLostInterval(mxULong interval);
	mxULong			getReceiveLostInterval() const;
	void			setEventBlankTimeInValueDetected(mxULong time);
	mxULong			getEventBlankTimeInValueDetected() const;
	void			setEventBlankTimeInValueLost(mxULong time);
	mxULong			getEventBlankTimeInValueLost() const;

protected:
	void			receive(flChar *data, flUInt dataSize);
	static void __stdcall	receiveProc(flChar *data, flUInt dataSize, void* receiveParam);

	mxUInt			threadLoop();
	static mxUInt __stdcall	threadLoopProc(void* data);

private:
	static void		initRand();

protected:
	// Own KeyValue
	fsKeyValueData			_ownKeyValue;

	// Notify
	mxULong					_notifyInterval;

	// Receive
	mxULong					_receiveLostInterval;
	mxArray<mxInt>			_receiveValues;
	mxArray<mxULong>		_receiveTimes;
	EventProc				_eventProc;
	void*					_eventData;
	fsStatusChange*			_valueDetectedChange;
	fsStatusChange*			_valueLostChange;

	// Multicast Objects
	flFMTSender*			_sender;
	flFMTReceiver*			_receiver;
	mxBool					_isOpen;

	// Thred
	mxBool					_threadRun;
	mxThread*				_thread;

	// Critical Section
	mxCriticalSection		_cs;

private:
	static mxBool			_isInitRand;
};
//------------------------------------------------------------------------------
class fsStatusChange
{
public:
	fsStatusChange(mxULong changedInterval);
	~fsStatusChange();

public:
	void		setChangedInterval(mxULong time);
	mxULong		getChangedInterval() const;

	void		init();
	void		changed();
	mxBool		notifyChanged();

protected:
	mxULong		_changedInterval;

	mxBool		_changed;
	mxULong		_changedTime;
};
//------------------------------------------------------------------------------
#endif //__fsValueMediator_h__
