/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsReceiver/fsReceiver.h>
#include <fs/fsMulticastTools/fsMulticastAddress.h>
#include <mx/mxCore/mxFile.h>
#include <mx/mxCore/mxIniFile.h>
#include <mx/mxCore/mxRegistry.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
#include <olectl.h>
#if (1100 > _MSC_VER)
#include <olectlid.h>
#endif
//------------------------------------------------------------------------------
#pragma comment(lib, "fsCommon.lib")
#pragma comment(lib, "fsStatus.lib")
#pragma comment(lib, "fsMulticastTools.lib")
#pragma comment(lib, "fsReceiverRMI.lib")
#pragma comment(lib, "mxCore.lib")
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#define ClearGraph				\
	if (!_freezeFlag)			\
	{							\
		_graph->stop();			\
		clearGraph();			\
	}
#define BuildGraph											\
	if (!_freezeFlag)										\
	{														\
		if (buildGraph())									\
			_isStarted ? _graph->start() : _graph->stop();	\
		else												\
			_graph->stop();									\
	}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
BOOL APIENTRY
DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
    return TRUE;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsReceiver::fsReceiver() :
mxObject()
{
	mxRegistry::setCompanyName(FS_REGISTRY_COMPANY_NAME);
	mxRegistry::setProductName(FS_REGISTRY_PRODUCT_NAME);

	_pointerReceivers.setGrowSize(16);
}
//------------------------------------------------------------------------------
fsReceiver::~fsReceiver()
{
}
//------------------------------------------------------------------------------
//
// Initialize / Shutdown
//
//------------------------------------------------------------------------------
mxBool
fsReceiver::initialize(HWND windowHandle)
{
	// Initialize Variables
	_windowHandle				= windowHandle;
	_isValid					= true;
	_isStarted					= true;
	_statusTime					= 0;

	_freezeFlag					= false;
	_quitFlag					= false;

	_UICallbackFunction			= NULL;
	_UICallbackData				= NULL;

	_synchronizeSenderAddressOn	= false;

	_focusDisplay				= FSFD_NONE;
	_directVideoModeOn			= false;
	_dvInterleavedModeOn		= false;
	_pointerCoordinate			= FSC_VIDEOWINDOW;

	_pointerAutomaticSetupOn	= false;
	pointerDefaultSetup();

	_videoWindowPolicy			= FSVWP_ORIGINALSIZE;
	_keepVideoAspect			= true;
	_fitWindowToVideo			= false;


	// Initialize COM
	HRESULT hr = CoInitialize(NULL);
	if (FAILED(hr))
		return false;

	// Create graph manager
	_graph = new fsGraphManager(graphEventProc, this, processIntermediateProc, this);
	if (!_graph->initialize(true))
	{
		_graph = NULL;
		return false;
	}

	// Create Multicast Detector
	_multicastDetectors = new fsMulticastDetectors();

	// Create Filters
	if (!createFilters())
	{
		deleteFilters();

		_graph->shutdown();
		_graph = NULL;

		return false;
	}

	// Initialize Filters
	initializeFilters();

	// Read Registry
	readRegistry();

	// Port mediator
	_portMediator	= new fsValueMediator(mediatorEventProc, this);
	_portMediator->setAddress(DEFAULT_ADDRESS);
	_portMediator->setTimeToLive(DEFAULT_TIMETOLIVE);
	_portMediator->setPort(PORT_POINTER_MEDIATE);
	_portMediator->setValue(fsValueMediator::getFreeValue(PORT_POINTER_MIN, PORT_POINTER_MAX));
	_portMediator->open();

	// Create Status receiver
	_statusReceiver = new fsStatusReceiver(statusProc, this);
	_statusReceiver->setAddress(DEFAULT_ADDRESS);
	_statusReceiver->setTimeToLive(DEFAULT_TIMETOLIVE);
	_statusReceiver->setPort(PORT_STATUS);
	_statusReceiver->open();


	// Setup Pointer port
	_graphCS.enter();

	mxUShort port = (mxUShort)_portMediator->getValue();

	createPointerReceivers(1);
	_pointerReceivers[0]->setPort(port);
	_pointerSender->setPort(port);

	pointerAutomaticSetup(port);
	updatePointer();

	_graphCS.leave();

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::shutdown()
{
	if (_graph == NULL)
		return true;

	// Delete Status receiver
	_statusReceiver->close();
	_statusReceiver		= NULL;

	// Delete Port mediator
	_portMediator->close();
	_portMediator		= NULL;

	// Delete Multicast Detector
	_multicastDetectors	= NULL;

	// Stop Graph
	_graph->stop();
	clearGraph();

	// Write Registry
	writeRegistry();

	// Delete Filters
	deleteFilters();

	// Delete Graph manager
	_graph->shutdown();
	_graph		= NULL;

	// Shutdown COM
	CoUninitialize();

	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Streaming Control
//
//------------------------------------------------------------------------------
mxBool
fsReceiver::getValid() const
{
	return _isValid;
}
//------------------------------------------------------------------------------
void
fsReceiver::getErrorMessage(mxInt& id, mxInt& type, mxString& message) const
{
	id		= _errorMessage.Id;
	type	= _errorMessage.Type;
	message	= _errorMessage.Message;
}
//------------------------------------------------------------------------------
void
fsReceiver::setStart(mxBool flag)
{
	assert(_graph != NULL);

	if (_isStarted == flag)
		return ;

	_graphCS.enter();

	_isStarted = flag;
	if (_isValid)
		_isStarted ? _graph->start() : _graph->stop();
	else
		_graph->stop();

	_graphCS.leave();
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::getStart() const
{
	return _isStarted;
}
//------------------------------------------------------------------------------
void
fsReceiver::setPointerTransmissionOn(mxBool flag)
{
	_pointerSender->setEnableTransmission(flag);
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::getPointerTransmissionOn() const
{
	return _pointerSender->getEnableTransmission();
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::receiveSenderData() const
{
	return (GetTickCount() - _statusTime < 3000);
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::receiveVideo() const
{
	return _videoReceiverMain->isStreaming() || _videoReceiverSub->isStreaming();
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::receiveAudio() const
{
	return _audioReceiver->isStreaming();
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::receivePointerFocus() const
{
	return _pointerFocusReceiver->isStreaming();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multicast Settings
//
//------------------------------------------------------------------------------
void
fsReceiver::setAddress(const mxString& address)
{
	_videoReceiverMain->setAddress(address);
	_videoReceiverSub->setAddress(address);
	_audioReceiver->setAddress(address);
	_pointerFocusReceiver->setAddress(address);
	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
		_pointerReceivers[i]->setAddress(address);
	_pointerSender->setAddress(address);

	_statusReceiver->setAddress(address);
	_portMediator->setAddress(address);
}
//------------------------------------------------------------------------------
const mxString&
fsReceiver::getAddress() const
{
	return _videoReceiverMain->getAddress();
}
//------------------------------------------------------------------------------
void
fsReceiver::setTimeToLive(mxULong ttl)
{
	_videoReceiverMain->setTimeToLive(ttl);
	_videoReceiverSub->setTimeToLive(ttl);
	_audioReceiver->setTimeToLive(ttl);
	_pointerFocusReceiver->setTimeToLive(ttl);
	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
		_pointerReceivers[i]->setTimeToLive(ttl);
	_pointerSender->setTimeToLive(ttl);

	_statusReceiver->setTimeToLive(ttl);
	_portMediator->setTimeToLive(ttl);
}
//------------------------------------------------------------------------------
mxULong
fsReceiver::getTimeToLive() const
{
	return _videoReceiverMain->getTimeToLive();
}
//------------------------------------------------------------------------------
void
fsReceiver::setSynchronizeSenderAddressOn(mxBool flag)
{
	_synchronizeSenderAddressOn = flag;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::getSynchronizeSenderAddressOn() const
{
	return _synchronizeSenderAddressOn;
}
//------------------------------------------------------------------------------
// Create Address
mxString
fsReceiver::createIPv4Address()
{
	return fsMulticastAddress::createAddress(fsMulticastAddress::AF_IPv4);
}
//------------------------------------------------------------------------------
mxString
fsReceiver::createIPv6Address()
{
	return fsMulticastAddress::createAddress(fsMulticastAddress::AF_IPv6);
}
//------------------------------------------------------------------------------
// Rejection Address
const mxString&
fsReceiver::getCurrentReceiveAddress() const
{
	return _videoReceiverMain->getSenderAddress();
}
//------------------------------------------------------------------------------
mxUInt
fsReceiver::getNumRejectionAddress() const
{
	return _videoReceiverMain->getNumRejectionAddress();
}
//------------------------------------------------------------------------------
mxString
fsReceiver::getRejectionAddress(mxUInt index) const
{
	return _videoReceiverMain->getRejectionAddress(index);
}
//------------------------------------------------------------------------------
void
fsReceiver::addRejectionAddress(const mxString& address)
{
	_videoReceiverMain->addRejectionAddress(address);
	_videoReceiverSub->addRejectionAddress(address);
	_audioReceiver->addRejectionAddress(address);
	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
		_pointerReceivers[i]->addRejectionAddress(address);

	_statusReceiver->addRejectionAddress(address);
	_portMediator->addRejectionAddress(address);
}
//------------------------------------------------------------------------------
void
fsReceiver::removeRejectionAddress(mxUInt index)
{
	_videoReceiverMain->removeRejectionAddress(index);
	_videoReceiverSub->removeRejectionAddress(index);
	_audioReceiver->removeRejectionAddress(index);
	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
		_pointerReceivers[i]->removeRejectionAddress(index);

	_statusReceiver->removeRejectionAddress(index);
	_portMediator->removeRejectionAddress(index);
}
//------------------------------------------------------------------------------
// VideoReceiver (Main)
mxInt
fsReceiver::getVideoReceiverMainReceiveRate() const
{
	return _videoReceiverMain->getReceiveRate();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVideoReceiverMainFrameRate() const
{
	return _videoReceiverMain->getFrameRate();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVideoReceiverMainNumDropPackets() const
{
	return _videoReceiverMain->getNumDropPackets();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVideoReceiverMainNumDropFrames() const
{
	return _videoReceiverMain->getNumDropFrames();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVideoReceiverMainPacketSize() const
{
	return _videoReceiverMain->getPacketSize();
}
//------------------------------------------------------------------------------
// VideoReceiver (Sub)
mxInt
fsReceiver::getVideoReceiverSubReceiveRate() const
{
	return _videoReceiverSub->getReceiveRate();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVideoReceiverSubFrameRate() const
{
	return _videoReceiverSub->getFrameRate();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVideoReceiverSubNumDropPackets() const
{
	return _videoReceiverSub->getNumDropPackets();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVideoReceiverSubNumDropFrames() const
{
	return _videoReceiverSub->getNumDropFrames();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVideoReceiverSubPacketSize() const
{
	return _videoReceiverSub->getPacketSize();
}
//------------------------------------------------------------------------------
// AudioReceiver
mxInt
fsReceiver::getAudioReceiverReceiveRate() const
{
	return _audioReceiver->getReceiveRate();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getAudioReceiverFrameRate() const
{
	return _audioReceiver->getFrameRate();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getAudioReceiverNumDropPackets() const
{
	return _audioReceiver->getNumDropPackets();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getAudioReceiverNumDropFrames() const
{
	return _audioReceiver->getNumDropFrames();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getAudioReceiverPacketSize() const
{
	return _audioReceiver->getPacketSize();
}
//------------------------------------------------------------------------------
// PointerReceiver (Focus)
mxInt
fsReceiver::getPointerFocusReceiverReceiveRate() const
{
	return _pointerFocusReceiver->getReceiveRate();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerFocusReceiverFrameRate() const
{
	return _pointerFocusReceiver->getFrameRate();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerFocusReceiverNumDropPackets() const
{
	return _pointerFocusReceiver->getNumDropPackets();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerFocusReceiverNumDropFrames() const
{
	return _pointerFocusReceiver->getNumDropFrames();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerFocusReceiverPacketSize() const
{
	return _pointerFocusReceiver->getPacketSize();
}
//------------------------------------------------------------------------------
// PointerReceiver[s]
mxInt
fsReceiver::getPointerReceiverReceiveRate() const
{
	mxUInt rate = 0;
	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
		rate += _pointerReceivers[i]->getReceiveRate();

	return (mxInt)rate;
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerReceiverFrameRate() const
{
	mxUInt rate = 0;
	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
		rate += _pointerReceivers[i]->getFrameRate();

	return (mxInt)rate;
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerReceiverNumDropPackets() const
{
	mxUInt dropPackets = 0;
	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
		dropPackets += _pointerReceivers[i]->getNumDropPackets();

	return dropPackets;
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerReceiverNumDropFrames() const
{
	mxUInt dropFrames = 0;
	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
		dropFrames += _pointerReceivers[i]->getNumDropFrames();

	return dropFrames;
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerReceiverPacketSize() const
{
	if (_pointerReceivers.getSize() == 0)
		return 0;
	return _pointerReceivers[0]->getPacketSize();
}
//------------------------------------------------------------------------------
// PointerSender
void
fsReceiver::setPointerSenderMaxTransmissionRate(mxInt rate)
{
	_pointerSender->setMaximumTransmissionRate((mxUInt)rate);
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerSenderMaxTransmissionRate() const
{
	return _pointerSender->getMaximumTransmissionRate();
}
//------------------------------------------------------------------------------
void
fsReceiver::setPointerSenderTransmissionCount(mxInt count)
{
	_pointerSender->setTransmissionCount((mxUInt)count);
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerSenderTransmissionCount() const
{
	return _pointerSender->getTransmissionCount();
}
//------------------------------------------------------------------------------
void
fsReceiver::setPointerSenderPacketSize(mxInt packetSize)
{
	_pointerSender->setMaximumPacketSize((mxUInt)packetSize);
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerSenderPacketSize() const
{
	return  _pointerSender->getMaximumPacketSize();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multicast Detector Settings
//
//------------------------------------------------------------------------------
mxBool
fsReceiver::addMulticastDetector(const mxString& address)
{
	return _multicastDetectors->add(address, PORT_STATUS);
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::removeMulticastDetector(const mxString& address)
{
	return _multicastDetectors->remove(address, PORT_STATUS);
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::wasDetectedMulticast(const mxString& address)
{
	return _multicastDetectors->detected(address, PORT_STATUS);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Pointer Settings
//
//------------------------------------------------------------------------------
void
fsReceiver::setPointerAutomaticSetupOn(mxBool flag)
{
	_pointerAutomaticSetupOn = flag;

	pointerAutomaticSetup(_portMediator->getValue());
	updatePointer();
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::getPointerAutomaticSetupOn() const
{
	return _pointerAutomaticSetupOn;
}
//------------------------------------------------------------------------------
// Basic Settings
void
fsReceiver::setGeneralPointer(mxInt pointer)
{
	_generalPointer = pointer;
	updatePointer();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getGeneralPointer() const
{
	return _generalPointer;
}
//------------------------------------------------------------------------------
void
fsReceiver::setRaiseOn(mxBool flag)
{
	_raiseOn = flag;
	updatePointer();
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::getRaiseOn() const
{
	return _raiseOn;
}
//------------------------------------------------------------------------------
void
fsReceiver::setRaisePointer(mxInt pointer)
{
	_raisePointer = pointer;
	updatePointer();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getRaisePointer() const
{
	return _raisePointer;
}
//------------------------------------------------------------------------------
void
fsReceiver::setDrawLocusOn(mxBool flag)
{
	_drawLocusOn = flag;
	updatePointer();
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::getDrawLocusOn() const
{
	return _drawLocusOn;
}
//------------------------------------------------------------------------------
void
fsReceiver::setLocusPointer(mxInt pointer)
{
	_locusPointer = pointer;
	updatePointer();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getLocusPointer() const
{
	return _locusPointer;
}
//------------------------------------------------------------------------------
// Detail Settings
void
fsReceiver::setPointer(mxInt entryNo, mxInt pointer)
{
	_pointers[entryNo] = pointer;
	updatePointer();
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointer(mxInt entryNo) const
{
	return _pointers[entryNo];
}
//------------------------------------------------------------------------------
void
fsReceiver::setPointerPhrase(mxInt entryNo, const mxString& phrase)
{
	_pointerPhrases[entryNo] = phrase;
	updatePointer();
}
//------------------------------------------------------------------------------
const mxString&
fsReceiver::getPointerPhrase(mxInt entryNo) const
{
	return _pointerPhrases[entryNo];
}
//------------------------------------------------------------------------------
void
fsReceiver::setPointerColor(mxInt entryNo, const mxByte* color)
{
	memcpy(_pointerColors[entryNo], color, sizeof(_pointerColors[entryNo]));
	updatePointer();
}
//------------------------------------------------------------------------------
void
fsReceiver::getPointerColor(mxInt entryNo, mxByte* color) const
{
	memcpy(color, _pointerColors[entryNo], sizeof(_pointerColors[entryNo]));
}
//------------------------------------------------------------------------------
void
fsReceiver::setPointerCoordinate(mxInt coordinate)
{
	_pointerCoordinate = coordinate;
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getPointerCoordinate() const
{
	return _pointerCoordinate;
}
//------------------------------------------------------------------------------
void
fsReceiver::computePointerRegion()
{
	switch(_pointerCoordinate)
	{
	case FSC_DESKTOPWINDOW:
	default:
		{
			// Get Window States
			mxUInt rw, rh;

			_graph->getVideoWindowResolution(rw, rh);
			RECT rect;
			GetWindowRect(GetDesktopWindow(), &rect);

			// Setup Pointer Mixer
			_pointerMixer->setWindowPosition(0.0f, 0.0f);
			_pointerMixer->setWindowSize((mxFloat)rw, (mxFloat)rh);

			// Setup Pointer Position
			_pointerPosition->setNormalizedOrigin(0.0f, 0.0f);
			_pointerPosition->setNormalizedSize((mxFloat)rw, (mxFloat)rh);
		}
		break;

	case FSC_VIDEOWINDOW:
		{
			// Get Window States
			mxInt x, y;
			mxUInt w, h;
			mxUInt rw, rh;
			mxFloat rateW, rateH;

			_graph->getVideoWindowPosition(x, y);
			_graph->getVideoWindowSize(w, h);
			_graph->getVideoWindowResolution(rw, rh);
			RECT rect;
			GetWindowRect(GetDesktopWindow(), &rect);

			// Setup Pointer Mixer
			_pointerMixer->setWindowPosition(0.0f, 0.0f);
			_pointerMixer->setWindowSize((mxFloat)rw, (mxFloat)rh);

			if (w == 0)	w = 1;
			if (h == 0)	h = 1;
			rateW = mxFloat(rw) / mxFloat(w);
			rateH = mxFloat(rh) / mxFloat(h);

			// Setup Pointer Position
			_pointerPosition->setNormalizedOrigin((mxFloat)-x * rateW, (mxFloat)-y * rateH);
			_pointerPosition->setNormalizedSize((mxFloat)(rect.right - rect.left) * rateW,
												(mxFloat)(rect.bottom - rect.top) * rateH);

		}
		break;
	}
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Caption Settings
//
//------------------------------------------------------------------------------
void
fsReceiver::updateCaption()
{
	mxByte userData[MR_MAXIMUM_USERDATA_SIZE];
	mxUInt userDataSize;

	_videoReceiverMain->getUserData(userData, userDataSize);
	if (userDataSize == 0)
		return ;

	flUInt width, height;
	_graph->getVideoWindowResolution(width, height);

	const flUInt NUMRESOLUTIONS = 11;
	const flUInt widths [] =   { 128, 176, 320, 352, 360, 480, 640, 720, 1080, 1280, 1600 };
	const flUInt fontSizes [] = { 10,  10,  16,  16,  16,  16,  18,  18,   20,   20,   22 };

	flUInt fontSize = 10;
	for(flUInt i = 0; i < NUMRESOLUTIONS; i++)
		if (widths[i] <= width)
			fontSize = fontSizes[i];

	_drawCaption->setFontSize(fontSize);
	_drawCaption->setCaption(mxString((const mxChar *)userData));
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Video Window Settings
//
//------------------------------------------------------------------------------
void
fsReceiver::setVideoWindowPolicy(mxInt policy)
{
	_videoWindowPolicy = policy;
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVideoWindowPolicy() const
{
	return _videoWindowPolicy;
}
//------------------------------------------------------------------------------
void
fsReceiver::setKeepVideoAspect(mxBool flag)
{
	_keepVideoAspect = flag;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::getKeepVideoAspect() const
{
	return _keepVideoAspect;
}
//------------------------------------------------------------------------------
void
fsReceiver::setFitWindowToVideo(mxBool flag)
{
	_fitWindowToVideo = flag;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::getFitWindowToVideo() const
{
	return _fitWindowToVideo;
}
//------------------------------------------------------------------------------
void
fsReceiver::computeVideoWindow()
{
	if (_windowHandle == NULL)
		return ;

	// Get ClientWindow size
	mxUInt cWidth, cHeight;
	RECT rect;
	GetWindowRect(_windowHandle, &rect);
	cWidth = (mxUInt)(rect.right - rect.left);
	cHeight = (mxUInt)(rect.bottom - rect.top);

	// Get Video Renderer Resolution
	mxUInt videoWidth, videoHeight;
	_graph->getVideoWindowResolution(videoWidth, videoHeight);


	mxInt x, y;
	mxUInt width, height;

	x = 0;
	y = 0;
	width = 1;
	height= 1;

	switch(_videoWindowPolicy)
	{
	case FSVWP_FITTOWINDOW:
		{
			if (_keepVideoAspect)
			{
				mxFloat videoAspect, cAspect;

				if (videoHeight == 0)
					break;
				videoAspect = mxFloat(videoWidth) / mxFloat(videoHeight);

				if (cHeight == 0)
					break;
				cAspect = mxFloat(cWidth) / mxFloat(cHeight);

				if (cAspect < videoAspect)
				{
					width = cWidth;
					height = (mxUInt)(mxFloat(width) / videoAspect);

					x = 0;
					y = (cHeight - height) / 2;
				}
				else
				{
					height = cHeight;
					width = (mxUInt)(mxFloat(height) * videoAspect);

					y = 0;
					x = (cWidth - width) / 2;
				}
			}
			else
			{
				x = 0;
				y = 0;

				width = cWidth;
				height = cHeight;
			}
		}
		break;
	case FSVWP_ORIGINALSIZE:
		{
			if (videoWidth == 0 || videoHeight == 0)
				break;

			if (_fitWindowToVideo)
			{
				const mxInt videoSize[2] = { (mxInt)videoWidth, (mxInt)videoHeight };
				if (_UICallbackFunction != NULL)
					_UICallbackFunction("ResizeForm", (void *)videoSize, _UICallbackData);

				GetWindowRect(_windowHandle, &rect);
				cWidth = (mxUInt)(rect.right - rect.left);
				cHeight = (mxUInt)(rect.bottom - rect.top);
			}

			x = mxInt((cWidth < videoWidth) ? 0 : ((cWidth - videoWidth) / 2));
			y = mxInt((cHeight < videoHeight) ? 0 : ((cHeight - videoHeight) / 2));

			width = videoWidth;
			height = videoHeight;

		}
		break;
	}

	_graph->setVideoWindowSize(width, height);
	_graph->setVideoWindowPosition(x, y);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Volume
//
//------------------------------------------------------------------------------
void
fsReceiver::getVolume(mxInt& left, mxInt& right) const
{
	_volumeIndicator->getVolume(left, right);
}
//------------------------------------------------------------------------------
void
fsReceiver::setVolumeEvaluationTime(mxInt evalTime)
{
	_volumeIndicator->setEvaluationTime((mxULong)evalTime);
}
//------------------------------------------------------------------------------
mxInt
fsReceiver::getVolumeEvaluationTime() const
{
	return (mxInt)_volumeIndicator->getEvaluationTime();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Other
//
//------------------------------------------------------------------------------
void
fsReceiver::quit()
{
	_quitFlag = true;
}
//------------------------------------------------------------------------------
void
fsReceiver::freeze(mxBool flag)
{
	if (_freezeFlag && flag || !_freezeFlag && !flag)
		return ;

	if (flag)
	{
		_statusReceiver->setReceiveOn(false);
		ClearGraph;
		_freezeFlag = true;
	}
	else
	{
		_freezeFlag = false;
		BuildGraph;
		_statusReceiver->setReceiveOn(true);
	}
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Quit
mxBool
fsReceiver::checkQuit()
{
	return _quitFlag;
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// IniFile
mxBool
fsReceiver::writeIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "a");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);
	mxString str;

	//
	// Multicast Settings
	//
	iniFile.writeString("Receiver", "MulticastAddress", getAddress());
	iniFile.writeUInt("Receiver", "MulticastTimeToLive", (mxUInt)getTimeToLive());

	str = getSynchronizeSenderAddressOn() ? "true" : "false";
	iniFile.writeString("Receiver", "SynchronizeSenderAddressOn", str);

	// Rejection Address
	mxUInt numReject = getNumRejectionAddress();
	iniFile.writeUInt("Receiver", "NumberOfRejectionAddress", numReject);
	for(mxUInt i = 0; i < numReject; i++)
		iniFile.writeString("Receiver", "RejectionAddress" + mxString((mxInt)i), getRejectionAddress(i));

	//
	// Pointer Sender
	//
	iniFile.writeUInt("Receiver", "PointerSenderMaxTransmissionRate", (mxUInt)getPointerSenderMaxTransmissionRate());
	iniFile.writeUInt("Receiver", "PointerSenderTransmissionCount", (mxUInt)getPointerSenderTransmissionCount());
	iniFile.writeUInt("Receiver", "PointerSenderPacketSize", (mxUInt)getPointerSenderPacketSize());

	//
	// Pointer Settings
	//
	FS_GENERALPOINTER2STR(getGeneralPointer(), str);
	iniFile.writeString("Receiver", "PointerGeneralPointer", str);

	str = getRaiseOn() ? "true" : "false";
	iniFile.writeString("Receiver", "PointerRaiseOn", str);

	FS_RAISEPOINTER2STR(getRaisePointer(), str);
	iniFile.writeString("Receiver", "PointerRaisePointer", str);

	str = getDrawLocusOn() ? "true" : "false";
	iniFile.writeString("Receiver", "PointerDrawLocusOn", str);

	FS_LOCUSPOINTER2STR(getLocusPointer(), str);
	iniFile.writeString("Receiver", "PointerLocusPointer", str);

	mxString DetailEntry [NUM_POINTERDETAILENTRY] = {
		"General", "Locus", "Raise1", "Raise2", "Raise3", "Raise4", "Raise5" };
	for(mxUInt i = 0; i < NUM_POINTERDETAILENTRY; i ++)
	{
		str = mxString(getPointer(i));
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"Pointer", str);

		str = getPointerPhrase(i);
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerPhrase", str);

		mxByte color[3];
		getPointerColor(i, color);
		
		str = mxString((mxInt)color[0]);
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorR", str);

		str = mxString((mxInt)color[1]);
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorG", str);

		str = mxString((mxInt)color[2]);
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorB", str);
	}

	str = getPointerAutomaticSetupOn() ? "true" : "false";
	iniFile.writeString("Receiver", "PointerAutomaticSetupOn", str);

	FS_COORDINATE2STR(getPointerCoordinate(), str);
	iniFile.writeString("Receiver", "PointerCoordinate", str);

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::readIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "r");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);

	mxString str, def;
	mxUInt val;
	mxInt index;

	// Stop Graph and StatusReceiver
	_graphCS.enter();
	freeze(true);

	//
	// Multicast Settins
	//
	str = iniFile.readString("Receiver", "MulticastAddress", getAddress());
	val = iniFile.readUInt("Receiver", "MulticastTimeToLive", getTimeToLive());

	setAddress(str);
	setTimeToLive((mxULong)val);

	def = getSynchronizeSenderAddressOn() ? "true" : "false";
	str = iniFile.readString("Receiver", "SynchronizeSenderAddressOn", def);
	str.toLower();
	setSynchronizeSenderAddressOn((str == "true"));

	// Rejection Address
	mxUInt numReject = getNumRejectionAddress();
	for(mxUInt i = 0; i < numReject; i++)
		removeRejectionAddress(0);
	numReject = iniFile.readUInt("Receiver", "NumberOfRejectionAddress", 0);
	for(mxUInt i = 0; i < numReject; i++)
	{
		str = iniFile.readString("Receiver", "RejectionAddress" + mxString((mxInt)i), "");
		if (str.length() != 0)
			addRejectionAddress(str);
	}

	//
	// Pointer Sender
	//
	val = iniFile.readUInt("Receiver", "PointerSenderMaxTransmissionRate", (mxUInt)getPointerSenderMaxTransmissionRate());
	setPointerSenderMaxTransmissionRate((mxInt)val);
	val = iniFile.readUInt("Receiver", "PointerSenderTransmissionCount", (mxUInt)getPointerSenderTransmissionCount());
	setPointerSenderTransmissionCount((mxInt)val);
	val = iniFile.readUInt("Receiver", "PointerSenderPacketSize", (mxUInt)getPointerSenderPacketSize());
	setPointerSenderPacketSize((mxInt)val);

	//
	// Pointer Settings
	//
	FS_GENERALPOINTER2STR(getGeneralPointer(), def);
	str = iniFile.readString("Receiver", "PointerGeneralPointer", def);
	FS_STR2GENERALPOINTER(str, index);
	setGeneralPointer(index);

	def = getRaiseOn() ? "true" : "false";
	str = iniFile.readString("Receiver", "PointerRaiseOn", def);
	str.toLower();
	setRaiseOn((str == "true"));

	FS_RAISEPOINTER2STR(getRaisePointer(), def);
	str = iniFile.readString("Receiver", "PointerRaisePointer", def);
	FS_STR2RAISEPOINTER(str, index);
	setRaisePointer(index);

	def = getDrawLocusOn() ? "true" : "false";
	str = iniFile.readString("Receiver", "PointerDrawLocusOn", def);
	str.toLower();
	setDrawLocusOn((str == "true"));

	FS_LOCUSPOINTER2STR(getLocusPointer(), def);
	str = iniFile.readString("Receiver", "PointerLocusPointer", def);
	FS_STR2LOCUSPOINTER(str, index);
	setLocusPointer(index);

	mxString DetailEntry [NUM_POINTERDETAILENTRY] = {
		"General", "Locus", "Raise1", "Raise2", "Raise3", "Raise4", "Raise5" };
	for(mxUInt i = 0; i < NUM_POINTERDETAILENTRY; i ++)
	{
		def = mxString(getPointer(i));
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"Pointer", def);
		if (str.length() != 0)
			setPointer(i, atoi(str.c_str()));
		else
			setPointer(i, 0);

		def = getPointerPhrase(i);
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerPhrase", def);
		setPointerPhrase(i, str);

		mxByte color[3];
		getPointerColor(i, color);
		
		def = mxString((mxInt)color[0]);
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorR", def);
		if (str.length() != 0)
			color[0] = atoi(str.c_str());
		else
			color[0] = 0;

		def = mxString((mxInt)color[1]);
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorG", def);
		if (str.length() != 0)
			color[1] = atoi(str.c_str());
		else
			color[1] = 0;

		def = mxString((mxInt)color[2]);
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorB", def);
		if (str.length() != 0)
			color[2] = atoi(str.c_str());
		else
			color[2] = 0;

		setPointerColor(i, color);
	}

	def = getPointerAutomaticSetupOn() ? "true" : "false";
	str = iniFile.readString("Receiver", "PointerAutomaticSetupOn", def);
	str.toLower();
	setPointerAutomaticSetupOn((str == "true"));

	FS_COORDINATE2STR(getPointerCoordinate(), def);
	str = iniFile.readString("Receiver", "PointerCoordinate", def);
	FS_STR2COORDINATE(str, index);
	setPointerCoordinate(index);

	// Start Graph and StatusReceiver
	freeze(false);
	_graphCS.leave();

	return true;
}
//------------------------------------------------------------------------------
// Registry
void
fsReceiver::writeRegistry()
{
	//// CURRENT_USER
	mxRegistry	registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);

	mxString str;

	//
	// Multicast Settings
	//
	str = getSynchronizeSenderAddressOn() ? "True" : "False";
	registry.writeString("Receiver", "SynchronizeSenderAddress", str);

	//
	// Video Window Settings
	//
	// Video Window Policy
	FS_VIDEOWINDOWPOLICY2STR(getVideoWindowPolicy(), str);
	registry.writeString("Receiver", "VideoWindowPolicy", str);

	// Keep Video Aspect
	str = (getKeepVideoAspect() ? "True" : "False");
	registry.writeString("Receiver", "KeepVideoAspect", str);
	
	// Fit Window to Video
	str = (getFitWindowToVideo() ? "True" : "False");
	registry.writeString("Receiver", "FitWindowToVideo", str);


	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsReceiver::readRegistry()
{
	//// CURRENT_USER
	mxRegistry	registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);

	mxString def, str;
	mxInt index;

	// Stop Graph
	_freezeFlag = true;

	//
	// Multicast Settings
	//
	def = getSynchronizeSenderAddressOn() ? "True" : "False";
	str = registry.readString("Receiver", "SynchronizeSenderAddress", def);
	str.toLower();
	setSynchronizeSenderAddressOn((str == "true"));

	//
	// Video Window Settings
	//
	// Video Window Policy
	FS_VIDEOWINDOWPOLICY2STR(getVideoWindowPolicy(), def);
	str = registry.readString("Receiver", "VideoWindowPolicy", def);
	FS_STR2VIDEOWINDOWPOLICY(str, index);
	setVideoWindowPolicy(index);

	// Keep Video Aspect
	def = (getKeepVideoAspect() ? "True" : "False");
	str = registry.readString("Receiver", "KeepVideoAspect", def);
	str.toLower();
	setKeepVideoAspect((str == "true"));
	
	// Fit Window to Video
	def = (getFitWindowToVideo() ? "True" : "False");
	str = registry.readString("Receiver", "FitWindowToVideo", def);
	str.toLower();
	setFitWindowToVideo((str == "true"));

	// Start Graph
	_freezeFlag = false;

	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsReceiver::writeRegistry(const mxString& section, const mxString& entry,
						  const mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	registry.writeString(section, entry, value);
	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsReceiver::readRegistry(const mxString& section, const mxString& entry,
						 const mxString& defaultValue, mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	value = registry.readString(section, entry, defaultValue);
	registry.closeKey();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Callback
mxBool
fsReceiver::setUICallbackFunction(fsReceiverUICallbackFunction func, void* data)
{
	_UICallbackFunction = func;
	_UICallbackData = data;
	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsReceiver::createFilters()
{
	fsFilterPtr filter = NULL;
	fsFilterEnumeratorPtr enumerator;

	//
	// Create Filters
	//

	// Renderer Filters
	_videoRender		= new fsVideoRenderFilter("VideoRender");

	_audioRender = NULL;
	enumerator = new fsFilterEnumerator(fsFilterEnumerator::C_AUDIO_RENDERER);
	fsFilterPtr tmpAudioRender = NULL;
	while(enumerator->nextFilter(filter))
	{
		tmpAudioRender = filter;

		const mxString& friendlyName = filter->cast<fsAudioRenderFilter>()->getFriendlyName();
		if (friendlyName.subString(0, 7) != "Default" &&
			friendlyName.subString(0, 6) != "Direct")
		{
			_audioRender = filter;
			break;
		}
	}
	if (_audioRender == NULL)
		_audioRender = tmpAudioRender;

	// Multicast Receiver Filters
	_videoReceiverMain		= new fsMulticastReceiverFilter("VideoReceiver(Main)");
	_videoReceiverSub		= new fsMulticastReceiverFilter("VideoReceiver(Sub)");
	_audioReceiver			= new fsMulticastReceiverFilter("AudioReceiver");
	_pointerFocusReceiver	= new fsMulticastReceiverFilter("PointerFocusReceiver");

	// Multicast Sender Filters
	_pointerSender			= new fsMulticastSenderFilter("PointerSender");

	// Other Filters
	_formatTransform1		= new fsFormatTransFilter("FormatTrans1");
	_formatTransform2		= new fsFormatTransFilter("FormatTrans2");
	_videoOverlayMixer		= new fsVideoOverlayMixerFilter("VideoOverlayMixer");
	_nonLinearZoom			= new fsNonLinearZoomFilter("NonLinearZoom");
	_pointerMixer			= new fsPointerMixerFilter("PointerMixer");
	_pointerPosition		= new fsPointerPositionFilter("PointerPosition");
	_dvSplitter				= new fsDVSplitterFilter("DVSplitter");
	_volumeIndicator		= new fsVolumeIndicatorFilter("VolumeIndicator");
	_drawCaption			= new fsDrawCaptionFilter("DrawCaption");

	//
	// Check Filters
	//
	// ȉ̃tB^̐Ɏsꍇ́AAvP[V͑sB
	//  AudioRender

	mxBool ret = true;
	mxString message("The following DirectShow Filter was not found.\n\n");

	if (!_videoRender->isValid())
	{
		message += "EVideo Renderer\n";
		ret &= false;
	}
	if (!_audioRender->isValid())
	{
		message += "EAudio Renderer\n";
		_audioRender = NULL;
	}
	if (!_videoReceiverMain->isValid() ||
		!_videoReceiverSub->isValid() ||
		!_audioReceiver->isValid() ||
		!_pointerFocusReceiver->isValid())
	{
		message += "EMulticast Receiver\n";
		ret &= false;
	}
	if (!_pointerSender->isValid())
	{
		message += "EMulticast Sender\n";
		ret &= false;
	}
	if (!_formatTransform1->isValid() ||
		!_formatTransform2->isValid())
	{
		message += "EFormat Transform\n";
		ret &= false;
	}
	if (!_videoOverlayMixer->isValid())
	{
		message += "EImage Compound\n";
		ret &= false;
	}
	if (!_nonLinearZoom->isValid())
	{
		message += "ENon-Linear Zoom\n";
		ret &= false;
	}
	if (!_pointerMixer->isValid())
	{
		message += "EPointer Mixer\n";
		ret &= false;
	}
	if (!_pointerPosition->isValid())
	{
		message += "EPointer Position\n";
		ret &= false;
	}
	if (!_dvSplitter->isValid())
	{
		message += "EDV-Splitter\n";
		_dvSplitter = NULL;
	}
	if (!_volumeIndicator->isValid())
	{
		message += "EVolume Indicator\n";
		ret &= false;
	}
	if (!_drawCaption->isValid())
	{
		message += "EDraw Caption\n";
		ret &= false;
	}

	if (!ret)
		showMessageBox((LPTSTR)message.c_str());

	return ret;
}
//------------------------------------------------------------------------------
void
fsReceiver::deleteFilters()
{
	_audioRender			= NULL;
	_videoRender			= NULL;
	_videoReceiverMain		= NULL;
	_videoReceiverSub		= NULL;
	_audioReceiver			= NULL;
	_pointerFocusReceiver	= NULL;
	_pointerReceivers.clear();
	_pointerSender			= NULL;
	_formatTransform1		= NULL;
	_formatTransform2		= NULL;
	_videoOverlayMixer		= NULL;
	_nonLinearZoom			= NULL;
	_pointerMixer			= NULL;
	_pointerPosition		= NULL;
	_dvSplitter				= NULL;
	_volumeIndicator		= NULL;
	_drawCaption			= NULL;
}
//------------------------------------------------------------------------------
void
fsReceiver::initializeFilters()
{
	// VideoReceiver (Main)
	_videoReceiverMain->setAddress(DEFAULT_ADDRESS);
	_videoReceiverMain->setPort(PORT_VIDEOMAIN);
	_videoReceiverMain->setTimeToLive(DEFAULT_TIMETOLIVE);
	_videoReceiverMain->setEnableConnectReceiveMediaType(true);

	// VideoReceiver (Sub)
	_videoReceiverSub->setAddress(DEFAULT_ADDRESS);
	_videoReceiverSub->setPort(PORT_VIDEOFOCUS);
	_videoReceiverSub->setTimeToLive(DEFAULT_TIMETOLIVE);
	_videoReceiverSub->setEnableConnectReceiveMediaType(true);

	// AudioReceiver
	_audioReceiver->setAddress(DEFAULT_ADDRESS);
	_audioReceiver->setPort(PORT_AUDIO);
	_audioReceiver->setTimeToLive(DEFAULT_TIMETOLIVE);
	_audioReceiver->setEnableConnectReceiveMediaType(true);

	// PointerReceiver (Focus)
	_pointerFocusReceiver->setAddress(DEFAULT_ADDRESS);
	_pointerFocusReceiver->setPort(PORT_FOCUS_POINTER);
	_pointerFocusReceiver->setTimeToLive(DEFAULT_TIMETOLIVE);
	_pointerFocusReceiver->setEnableConnectReceiveMediaType(false);
	_pointerFocusReceiver->setMediaType(IMulticastReceiver::MRMT_POINTER);

	// PointerSender
	_pointerSender->setAddress(DEFAULT_ADDRESS);
	_pointerSender->setPort(PORT_POINTER_MAX + 1);
	_pointerSender->setTimeToLive(DEFAULT_TIMETOLIVE);
	_pointerSender->setMaximumTransmissionRate(DEFAULT_MAXTRANSMISSIONRATE);
	_pointerSender->setTransmissionCount(DEFAULT_TRANSMISSIONCOUNT);
	_pointerSender->setMaximumPacketSize(DEFAULT_PACKETSIZE);

	// FormatTransform
	_formatTransform1->setOutputVideoResolution(IFormatTrans::VR_SQCIF);
	_formatTransform1->setInputVideoColor(IFormatTrans::VC_RGB24);
	_formatTransform2->setOutputVideoResolution(IFormatTrans::VR_SQCIF);
	_formatTransform2->setInputVideoColor(IFormatTrans::VC_RGB24);

	// VideoOverlayMixer
	_videoOverlayMixer->setFrameRate(25);

	// PointerMixer
	mxString modulePath = mxFile::getModulePath();
	_pointerMixer->addPointerShape(modulePath + "\\pencil.bmp");
	_pointerMixer->addPointerShape(modulePath + "\\fountainpen.bmp");
	_pointerMixer->addPointerShape(modulePath + "\\brush.bmp");
	_pointerMixer->addPointerShape(modulePath + "\\paa.bmp");
	_pointerMixer->addPointerShape(modulePath + "\\tyoki.bmp");
	_pointerMixer->addPointerShape(modulePath + "\\guu.bmp");
	_pointerMixer->setPointerShapeOrigin(6, 0, 32);
	_pointerMixer->setPointerShapeOrigin(7, 5, 32);
	_pointerMixer->setPointerShapeOrigin(8, 0, 29);
	_pointerMixer->setPointerShapeOrigin(9, -16, 0);
	_pointerMixer->setPointerShapeOrigin(10, -16, 0);
	_pointerMixer->setPointerShapeOrigin(11, -16, 0);

	// PointerPosition
	_pointerPosition->setDrawLocusOn(false);
	_pointerPosition->setShowPhraseOn(true);
	_pointerPosition->setNumPhrases(NUM_POINTERDETAILENTRY);
	_pointerPosition->setOutputMinimumPeriod(50);
	_pointerPosition->setLocusWidth(1);
	_pointerPosition->setLocusEdgingOn(true);

	// DrawCaption
	_drawCaption->setFontSize(16);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Graph Event
void
fsReceiver::graphEvent(mxInt eventCode, void* param1, void* param2)
{
	if (eventCode != fsGraphManager::ET_MEDIATYPECHANGED)
		return ;

#ifdef _DEBUG
	mxNotify::post(mxNotify::INFO, "Media Type Changed.");
#endif

	if (_UICallbackFunction != NULL)
		_UICallbackFunction("TimerStop", NULL, _UICallbackData);

	_graphCS.enter();
	ClearGraph;
	BuildGraph;
	_graphCS.leave();

	computeVideoWindow();

	if (_UICallbackFunction != NULL)
		_UICallbackFunction("TimerStart", NULL, _UICallbackData);
}
//------------------------------------------------------------------------------
void
fsReceiver::graphEventProc(mxInt eventCode, void *param1, void* param2, void *data)
{
	fsReceiver* receiver = (fsReceiver *)data;
	receiver->graphEvent(eventCode, param1, param2);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Process Intermediate
// ̊֐́AconnectFilterIndirect()ĂяoƂɁA
// ԃtB^ꂽꍇɌĂяo܂B
// ԃtB^ɑ΂A炩̃p[^ݒsꍇɎgp܂B
void
fsReceiver::processIntermediate(const fsFilterPtr* filters, mxUInt numFilters)
{
	for(mxUInt i = 0; i < numFilters; i++)
	{
		IBaseFilter* filterObj = filters[i]->getBaseFilter();

		// DV Decoder Resolution Setting
		IIPDVDec* dvDecoder = NULL;
		HRESULT hr = filterObj->QueryInterface(IID_IIPDVDec, (void **)&dvDecoder);
		if (hr == S_OK)
		{
			dvDecoder->put_IPDisplay(DVDECODERRESOLUTION_720x480);
			dvDecoder->Release();
			dvDecoder = NULL;
			break;
		}
	}
}
//------------------------------------------------------------------------------
void
fsReceiver::processIntermediateProc(const fsFilterPtr* filters, mxUInt numFilters, void* data)
{
	fsReceiver*	receiver = (fsReceiver *)data;
	receiver->processIntermediate(filters, numFilters);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Status
void
fsReceiver::status(mxInt changed, const fsStatus& status)
{
	_statusTime = GetTickCount();

	if (changed == 0)
		return ;

#ifdef _DEBUG
	mxNotify::post(mxNotify::INFO, "Status.");
#endif

	if (_UICallbackFunction != NULL)
		_UICallbackFunction("TimerStop", NULL, _UICallbackData);

	// Ot̍č\z
	if ((changed & S_DIRECTVIDEOMODEON) != 0 ||
		(changed & S_DVINTERLEAVEDMODEON) != 0 ||
		(changed & S_FOCUSDISPLAY) != 0 ||
		(changed & S_PARTIALZOOMOUTPUTVIDEORESOLUTION) != 0 ||
		(changed & S_NONLINEARZOOMOUTPUTVIDEORESOLUTION) != 0 ||
		(changed & S_IMAGECOMPOUNDOUTPUTVIDEORESOLUTION) != 0)
	{
		_directVideoModeOn = status.DirectVideoModeOn;
		_dvInterleavedModeOn = status.DvInterleavedModeOn;
		_focusDisplay = status.FocusDisplay;
		_formatTransform1->setOutputVideoResolution(status.PartialZoomOutputVideoResolution);
		_nonLinearZoom->setOutputVideoResolution(status.NonLinearZoomOutputVideoResolution);
		_formatTransform2->setOutputVideoResolution(status.ImageCompoundOutputVideoResolution);

		_graphCS.enter();
		ClearGraph;
		BuildGraph;
		_graphCS.leave();
	}

	// Ot̍č\zsv
	if ((changed & S_NONLINEARZOOMMASKTYPE) != 0 ||
		(changed & S_NONLINEARZOOMMASKWIDTH) != 0 ||
		(changed & S_NONLINEARZOOMMASKHEIGHT) != 0 ||
		(changed & S_IMAGECOMPOUNDMASKTYPE) != 0 ||
		(changed & S_IMAGECOMPOUNDBLENDINGTYPE) != 0)
	{
		_nonLinearZoom->setMask(status.NonLinearZoomMaskType);
		_nonLinearZoom->setMaskSize(status.NonLinearZoomMaskWidth,
									status.NonLinearZoomMaskHeight);
		_videoOverlayMixer->setMask(status.ImageCompoundMaskType);
		_videoOverlayMixer->setBlending(status.ImageCompoundBlendingType);
	}

	// AhX̓
	if ((changed & S_MULTICASTADDRESS) != 0)
	{
		if (_synchronizeSenderAddressOn && getAddress() != status.MulticastAddress)
			setAddress(status.MulticastAddress);
	}

	computeVideoWindow();

	if (_UICallbackFunction != NULL)
		_UICallbackFunction("TimerStart", NULL, _UICallbackData);
}
//------------------------------------------------------------------------------
void
fsReceiver::statusProc(mxInt changed, const fsStatus& status, void *data)
{
	fsReceiver *receiver = (fsReceiver *)data;
	receiver->status(changed, status);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Mediator Event
void
fsReceiver::mediatorEvent(fsValueMediator::EventTypeEnum eventType, void* eventParam)
{
#ifdef _DEBUG
	mxNotify::post(mxNotify::INFO, "Mediator Event.");
#endif

	if (_UICallbackFunction != NULL)
		_UICallbackFunction("TimerStop", NULL, _UICallbackData);

	switch(eventType)
	{
	case fsValueMediator::E_RECEIVEVALUESTATUS:
		{
			rsReceiveValueStatus *status = (rsReceiveValueStatus *)eventParam;

			if (status->NumValues == _pointerReceivers.getSize() - 1)
			{
				_pointerReceivers[0]->setPort((mxUShort)_portMediator->getValue());
				for(mxUInt i = 0; i < status->NumValues; i++)
					_pointerReceivers[i + 1]->setPort((mxUShort)status->Values[i]);
			}
			else
			{
				_graphCS.enter();
				ClearGraph;

				createPointerReceivers(status->NumValues + 1);

				_pointerReceivers[0]->setPort((mxUShort)_portMediator->getValue());
				for(mxUInt i = 0; i < status->NumValues; i++)
					_pointerReceivers[i + 1]->setPort((mxUShort)status->Values[i]);

				BuildGraph;
				_graphCS.leave();
			}
		}
		break;

	case fsValueMediator::E_COLLIDEVALUE:
		{
			// Create New Value
			mxInt  value = fsValueMediator::getFreeValue(PORT_POINTER_MIN, PORT_POINTER_MAX);
			_portMediator->setValue(value);

			// Setup Port
			_pointerSender->setPort((mxUShort)value);
			_pointerReceivers[0]->setPort((mxUShort)value);

			// Update Pointer
			pointerAutomaticSetup((mxUShort)value);
			updatePointer();
		}
		break;

	case fsValueMediator::E_VALUEDETECTED:
	case fsValueMediator::E_VALUELOST:
		break;
	}

	computeVideoWindow();

	if (_UICallbackFunction != NULL)
		_UICallbackFunction("TimerStart", NULL, _UICallbackData);
}
//------------------------------------------------------------------------------
void
fsReceiver::mediatorEventProc(fsValueMediator::EventTypeEnum eventType, void* eventParam, void* data)
{
	fsReceiver *receiver = (fsReceiver *)data;
	receiver->mediatorEvent(eventType, eventParam);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsReceiver::showMessageBox(LPSTR sz, ...)
{
    static TCHAR tach[2000];
    va_list va;

    va_start(va, sz);
    wvsprintf(tach, sz, va);
    va_end(va);

    MessageBox(_windowHandle, tach, TEXT("Error"), MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL);
}
//------------------------------------------------------------------------------
void
fsReceiver::createPointerReceivers(mxUInt num)
{
	const mxString& address = _videoReceiverMain->getAddress();
	mxULong ttl = _videoReceiverMain->getTimeToLive();

	_pointerReceivers.clear();
	_pointerReceivers.setSize(num);

	for(mxUInt i = 0; i < num; i++)
	{
		mxString name = "PointerReceiver" + mxString((mxInt)i);
		fsMulticastReceiverFilterPtr mc = new fsMulticastReceiverFilter(name);
		mc->setAddress(address);
		mc->setPort(PORT_POINTER_MAX + 1);
		mc->setTimeToLive(ttl);
		mc->setEnableConnectReceiveMediaType(false);
		mc->setMediaType(IMulticastReceiver::MRMT_POINTER);

		_pointerReceivers[i] = mc;
	}
}
//------------------------------------------------------------------------------
void
fsReceiver::pointerDefaultSetup()
{
	_raiseOn				= true;
	_drawLocusOn			= false;
	_generalPointer			= FSGP_STANDARD;
	_raisePointer			= FSRP_POINTER1;
	_locusPointer			= FSLP_POINTER1;

	//
	// General Pointer
	//
	_pointers[FSDP_GENERAL]			= DEFAULT_GENERALPOINTER;
	_pointerPhrases[FSDP_GENERAL]	= DEFAULT_GENERALPHRASE;
	memset(_pointerColors[FSDP_GENERAL], 0xff, 3);

	//
	// Locus Pointer
	//
	_pointers[FSDP_LOCUS]			= DEFAULT_LOCUSPOINTER;
	_pointerPhrases[FSDP_LOCUS]	= DEFAULT_LOCUSPHRASE;
	memset(_pointerColors[FSDP_LOCUS], 0xff, 3);

	//
	// Raise Pointer
	//
	_pointers[FSDP_RAISE1]			= DEFAULT_RAISE1POINTER;
	_pointerPhrases[FSDP_RAISE1]	= DEFAULT_RAISE1PHRASE;
	memset(_pointerColors[FSDP_RAISE1], 0xff, 3);

	_pointers[FSDP_RAISE2]			= DEFAULT_RAISE2POINTER;
	_pointerPhrases[FSDP_RAISE2]	= DEFAULT_RAISE2PHRASE;
	memset(_pointerColors[FSDP_RAISE2], 0xff, 3);

	_pointers[FSDP_RAISE3]			= DEFAULT_RAISE3POINTER;
	_pointerPhrases[FSDP_RAISE3]	= DEFAULT_RAISE3PHRASE;
	memset(_pointerColors[FSDP_RAISE3], 0xff, 3);

	_pointers[FSDP_RAISE4]			= DEFAULT_RAISE4POINTER;
	_pointerPhrases[FSDP_RAISE4]	= DEFAULT_RAISE4PHRASE;
	memset(_pointerColors[FSDP_RAISE4], 0xff, 3);

	_pointers[FSDP_RAISE5]			= DEFAULT_RAISE5POINTER;
	_pointerPhrases[FSDP_RAISE5]	= DEFAULT_RAISE5PHRASE;
	memset(_pointerColors[FSDP_RAISE5], 0xff, 3);
}
//------------------------------------------------------------------------------
void
fsReceiver::pointerAutomaticSetup(mxUShort portNumber)
{
	if (!_pointerAutomaticSetupOn)
		return ;

	portNumber -= PORT_POINTER_MIN;

	mxInt shape;
	mxByte color[3];
	mxFloat r, g, b;

	//
	// General Pointer
	//
	_generalPointer = FSGP_DETAILPOINTER;

	// Shape
	shape = portNumber % 6;

	// Color
	r = mxFloat(((portNumber          ) % 2) + 2) / 3.0f;
	g = mxFloat(((portNumber /      2 ) % 2) + 2) / 3.0f;
	b = mxFloat(((portNumber / (2 * 2)) % 2) + 2) / 3.0f;

	color[0] = mxByte(r * 255.0f);
	color[1] = mxByte(g * 255.0f);
	color[2] = mxByte(b * 255.0f);

	_pointers[FSDP_GENERAL]			= shape;
	memcpy(_pointerColors[FSDP_GENERAL], color, sizeof(color));

	//
	// Locus Pointer
	//
	_locusPointer = FSLP_DETAILPOINTER;

	// Shape
	shape = portNumber % 3 + FSLP_POINTER1;

	// Color
	g = mxFloat(((portNumber          ) % 2) + 2) / 3.0f;
	b = mxFloat(((portNumber /      2 ) % 2) + 2) / 3.0f;
	r = mxFloat(((portNumber / (2 * 2)) % 2) + 2) / 3.0f;

	color[0] = mxByte(r * 255.0f);
	color[1] = mxByte(g * 255.0f);
	color[2] = mxByte(b * 255.0f);

	_pointers[FSDP_LOCUS]			= shape;
	memcpy(_pointerColors[FSDP_LOCUS], color, sizeof(color));

	//
	// Raise Pointer
	//
	_raisePointer = FSRP_DETAILPOINTER;
	for(mxUInt i = FSDP_RAISE1; i <= FSDP_RAISE5; i++)
	{
		shape = (portNumber + i) % 3 + FSRP_POINTER1;

		// Color
		b = mxFloat(((portNumber          ) % 2) + 2) / 3.0f;
		r = mxFloat(((portNumber /      2 ) % 2) + 2) / 3.0f;
		g = mxFloat(((portNumber / (2 * 2)) % 2) + 2) / 3.0f;

		color[0] = mxByte(r * 255.0f);
		color[1] = mxByte(g * 255.0f);
		color[2] = mxByte(b * 255.0f);

		_pointers[i]			= shape;
		memcpy(_pointerColors[i], color, sizeof(color));
	}
}
//------------------------------------------------------------------------------
void
fsReceiver::updatePointer()
{
	const mxUShort IGNOREALL = 0xffff;
	const mxByte WHITE [] = { 0xff, 0xff, 0xff };

	// update General
	_pointerPosition->setPhraseShowButton(FSDP_GENERAL, 0x0000);

	if (_generalPointer == FSGP_DETAILPOINTER)
	{
		_pointerPosition->setPhrase(FSDP_GENERAL, _pointerPhrases[FSDP_GENERAL]);
		_pointerPosition->setPhraseShowPointerShape(FSDP_GENERAL, _pointers[FSDP_GENERAL]);
		_pointerPosition->setPhraseShowPointerColor(FSDP_GENERAL, _pointerColors[FSDP_GENERAL]);
	}
	else
	{
		_pointerPosition->setPhrase(FSDP_GENERAL, "");
		_pointerPosition->setPhraseShowPointerShape(FSDP_GENERAL, _generalPointer);
		_pointerPosition->setPhraseShowPointerColor(FSDP_GENERAL, WHITE);
	}

	// update Locus
	if (_drawLocusOn)
	{
		_pointerPosition->setDrawLocusOn(true);
		_pointerPosition->setPhraseShowButton(FSDP_LOCUS, 0x0001);

		if (_locusPointer == FSLP_DETAILPOINTER)
		{
			_pointerPosition->setPhrase(FSDP_LOCUS, _pointerPhrases[FSDP_LOCUS]);
			_pointerPosition->setLocusDrawPointerShape(_pointers[FSDP_LOCUS]);
			_pointerPosition->setLocusDrawPointerColor(_pointerColors[FSDP_LOCUS]);
		}
		else
		{
			_pointerPosition->setPhrase(FSDP_LOCUS, "");
			_pointerPosition->setLocusDrawPointerShape(_locusPointer);
			_pointerPosition->setLocusDrawPointerColor(WHITE);
		}
	}
	else
	{
		_pointerPosition->setDrawLocusOn(false);
		_pointerPosition->setPhraseShowButton(FSDP_LOCUS, IGNOREALL);
	}

	// update Phrase
	if (_raiseOn)
	{
		for(mxUInt i = FSDP_RAISE1, ii = 0; i <= FSDP_RAISE5; i++, ii++)
		{
			_pointerPosition->setPhraseShowButton(i, 0x0001 << ii);
			if (_raisePointer == FSRP_DETAILPOINTER)
			{
				_pointerPosition->setPhrase(i, _pointerPhrases[i]);
				_pointerPosition->setPhraseShowPointerShape(i, _pointers[i]);
				_pointerPosition->setPhraseShowPointerColor(i, _pointerColors[i]);
			}
			else
			{
				_pointerPosition->setPhrase(i, "");
				_pointerPosition->setPhraseShowPointerShape(i, _raisePointer);
				_pointerPosition->setPhraseShowPointerColor(i, WHITE);
			}
		}
	}
	else
	{
		_pointerPosition->setPhraseShowButton(FSDP_RAISE1, IGNOREALL);
		_pointerPosition->setPhraseShowButton(FSDP_RAISE2, IGNOREALL);
		_pointerPosition->setPhraseShowButton(FSDP_RAISE3, IGNOREALL);
		_pointerPosition->setPhraseShowButton(FSDP_RAISE4, IGNOREALL);
		_pointerPosition->setPhraseShowButton(FSDP_RAISE5, IGNOREALL);
	}
}
//------------------------------------------------------------------------------
