/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsReceiver_h__
#define __fsReceiver_h__
//------------------------------------------------------------------------------
#include <fs/fsReceiverRMI/fsReceiverIf.h>
#include <fs/fsCommon/fsFilterEnumerator.h>
#include <fs/fsCommon/fsMulticastSenderFilter.h>
#include <fs/fsCommon/fsMulticastReceiverFilter.h>
#include <fs/fsCommon/fsVideoRenderFilter.h>
#include <fs/fsCommon/fsAudioRenderFilter.h>
#include <fs/fsCommon/fsFormatTransFilter.h>
#include <fs/fsCommon/fsVideoOverlayMixerFilter.h>
#include <fs/fsCommon/fsNonLinearZoomFilter.h>
#include <fs/fsCommon/fsPointerMixerFilter.h>
#include <fs/fsCommon/fsPointerPositionFilter.h>
#include <fs/fsCommon/fsDVSplitterFilter.h>
#include <fs/fsCommon/fsVolumeIndicatorFilter.h>
#include <fs/fsCommon/fsDrawCaptionFilter.h>
#include <fs/fsCommon/fsGraphManager.h>
#include <fs/fsCommon/fsErrorMessage.h>
#include <fs/fsMulticastTools/fsMulticastDetectors.h>
#include <fs/fsMulticastTools/fsValueMediator.h>
#include <fs/fsStatus/fsStatusReceiver.h>
#include <fs/fsStatus/fsStatus.h>
#include <mx/mxCore/mxCriticalSection.h>
#include <mx/mxCore/mxObject.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------
class fsReceiver;
typedef mxObjectPtr<fsReceiver>	fsReceiverPtr;
//------------------------------------------------------------------------------
class fsReceiver : public mxObject, public fsReceiverIf
{
	MXOBJECTHEADER(fsReceiver, mxObject);

public:
	fsReceiver();

protected:
	virtual ~fsReceiver();

public:
	// Initialize / Shutdown
	mxBool				initialize(HWND windowHandle);
	mxBool				shutdown();

public:
	//
	// Implements fsReceiverIf
	//

	//
	// Streaming Control
	//
	mxBool				getValid() const;
	void				getErrorMessage(mxInt& id, mxInt& type, mxString& message) const;
	void				setStart(mxBool flag);
	mxBool				getStart() const;
	void				setPointerTransmissionOn(mxBool flag);
	mxBool				getPointerTransmissionOn() const;
	mxBool				receiveSenderData() const;
	mxBool				receiveVideo() const;
	mxBool				receiveAudio() const;
	mxBool				receivePointerFocus() const;


	//
	// Multicast Settings
	//
	void				setAddress(const mxString& address);
	const mxString&		getAddress() const;
	void				setTimeToLive(mxULong ttl);
	mxULong				getTimeToLive() const;
	void				setSynchronizeSenderAddressOn(mxBool flag);
	mxBool				getSynchronizeSenderAddressOn() const;

	// Create Address
	mxString			createIPv4Address();
	mxString			createIPv6Address();

	// Rejection Address
	const mxString&		getCurrentReceiveAddress() const;
	mxUInt				getNumRejectionAddress() const;
	mxString			getRejectionAddress(mxUInt index) const;
	void				addRejectionAddress(const mxString& address);
    void				removeRejectionAddress(mxUInt index);

	// VideoReceiver (Main)
	mxInt				getVideoReceiverMainReceiveRate() const;
	mxInt				getVideoReceiverMainFrameRate() const;
	mxInt				getVideoReceiverMainNumDropPackets() const;
	mxInt				getVideoReceiverMainNumDropFrames() const;
	mxInt				getVideoReceiverMainPacketSize() const;

	// VideoReceiver (Sub)
	mxInt				getVideoReceiverSubReceiveRate() const;
	mxInt				getVideoReceiverSubFrameRate() const;
	mxInt				getVideoReceiverSubNumDropPackets() const;
	mxInt				getVideoReceiverSubNumDropFrames() const;
	mxInt				getVideoReceiverSubPacketSize() const;

	// AudioReceiver
	mxInt				getAudioReceiverReceiveRate() const;
	mxInt				getAudioReceiverFrameRate() const;
	mxInt				getAudioReceiverNumDropPackets() const;
	mxInt				getAudioReceiverNumDropFrames() const;
	mxInt				getAudioReceiverPacketSize() const;

	// PointerReceiver (Focus)
	mxInt				getPointerFocusReceiverReceiveRate() const;
	mxInt				getPointerFocusReceiverFrameRate() const;
	mxInt				getPointerFocusReceiverNumDropPackets() const;
	mxInt				getPointerFocusReceiverNumDropFrames() const;
	mxInt				getPointerFocusReceiverPacketSize() const;

	// PointerReceiver[s]
	mxInt				getPointerReceiverReceiveRate() const;
	mxInt				getPointerReceiverFrameRate() const;
	mxInt				getPointerReceiverNumDropPackets() const;
	mxInt				getPointerReceiverNumDropFrames() const;
	mxInt				getPointerReceiverPacketSize() const;

	// PointerSender
	void				setPointerSenderMaxTransmissionRate(mxInt rate);
	mxInt				getPointerSenderMaxTransmissionRate() const;
	void				setPointerSenderTransmissionCount(mxInt count);
	mxInt				getPointerSenderTransmissionCount() const;
	void				setPointerSenderPacketSize(mxInt packetSize);
	mxInt				getPointerSenderPacketSize() const;


	//
	// Multicast Detector Settings
	//
	mxBool				addMulticastDetector(const mxString& address);
	mxBool				removeMulticastDetector(const mxString& address);
	mxBool				wasDetectedMulticast(const mxString& address);


	//
	// Pointer Settings
	//
	void				setPointerAutomaticSetupOn(mxBool flag);
	mxBool				getPointerAutomaticSetupOn() const;

	// Basic Settings
	void				setGeneralPointer(mxInt pointer);
	mxInt				getGeneralPointer() const;
	void				setRaiseOn(mxBool flag);
	mxBool				getRaiseOn() const;
	void				setRaisePointer(mxInt pointer);
	mxInt				getRaisePointer() const;
	void				setDrawLocusOn(mxBool flag);
	mxBool				getDrawLocusOn() const;
	void				setLocusPointer(mxInt pointer);
	mxInt				getLocusPointer() const;

	// Detail Settings
	void				setPointer(mxInt entryNo, mxInt pointer);
	mxInt				getPointer(mxInt entryNo) const;
	void				setPointerPhrase(mxInt entryNo, const mxString& phrase);
	const mxString&		getPointerPhrase(mxInt entryNo) const;
	void				setPointerColor(mxInt entryNo, const mxByte* color);
	void				getPointerColor(mxInt entryNo, mxByte* color) const;

	// Other Settings
	void				setPointerCoordinate(mxInt coordinate);
	mxInt				getPointerCoordinate() const;
	void				computePointerRegion();


	//
	// Caption Settings
	//
	void				updateCaption();


	//
	// Video Window Settings
	//
	void				setVideoWindowPolicy(mxInt policy);
	mxInt				getVideoWindowPolicy() const;
	void				setKeepVideoAspect(mxBool flag);
	mxBool				getKeepVideoAspect() const;
	void				setFitWindowToVideo(mxBool flag);
	mxBool				getFitWindowToVideo() const;
	void				computeVideoWindow();


	//
	// Volume Settings
	//
	void				getVolume(mxInt& left, mxInt& right) const;
	void				setVolumeEvaluationTime(mxInt evalTime);
	mxInt				getVolumeEvaluationTime() const;


	//
	// Other
	//
	void				quit();
	void				freeze(mxBool flag);

public:
	// Quit
	mxBool				checkQuit();

	// IniFile
	mxBool				writeIniFile(const mxString& filename);
	mxBool				readIniFile(const mxString& filename);

	// Registry
	void				writeRegistry();
	void				readRegistry();
	void				writeRegistry(const mxString& section,
									const mxString& entry,
									const mxString& value);
	void				readRegistry(const mxString& section,
									const mxString& entry,
									const mxString& defaultValue,
									mxString& value);

public:
	// Callback
	typedef int (*fsReceiverUICallbackFunction)(const char*, void*, void*);
	mxBool				setUICallbackFunction(fsReceiverUICallbackFunction func, void* data);

public:
	// Cast
	template<class T> T*		cast()			{ return static_cast<T*>(this); }
	template<class T> const T*	cast() const	{ return static_cast<const T*>(this); }

protected:
	mxBool				createFilters();
	void				deleteFilters();

	void				initializeFilters();

	mxBool				buildGraph();
	void				clearGraph();
	mxBool				buildDVInterleavedGraph();
	mxBool				buildDirectVideoGraph();
	mxBool				buildFocusNoneVideoGraph();
	mxBool				buildFocusPartialZoomVideoGraph();
	mxBool				buildFocusNonLinearZoomVideoGraph();
	mxBool				buildFocusImageCompoundVideoGraph();
	mxBool				buildAudioGraph();

protected:
	// Graph Event
	void				graphEvent(mxInt eventCode, void* param1, void* param2);
	static void			graphEventProc(mxInt eventCode, 
										void *param1, void* param2, void *data);

	// Process Intermediate
	void				processIntermediate(const fsFilterPtr* filters, mxUInt numFilters);
	static void			processIntermediateProc(const fsFilterPtr* filters, 
										mxUInt numFilters, void* data);

	// Status
	void				status(mxInt changed, const fsStatus& status);
	static void			statusProc(mxInt changed, const fsStatus& status, void *data);

	// Mediator Event
	void				mediatorEvent(fsValueMediator::EventTypeEnum eventType,
										void* eventParam);
	static void			mediatorEventProc(fsValueMediator::EventTypeEnum eventType,
										void* eventParam, void* data);

private:
	void				showMessageBox(LPSTR sz, ...);

	void				createPointerReceivers(mxUInt num);

	void				pointerDefaultSetup();
	void				pointerAutomaticSetup(mxUShort portNumber);
	void				updatePointer();

protected:
	HWND							_windowHandle;

	mxCriticalSection				_graphCS;
	fsGraphManagerPtr				_graph;
	fsStatusReceiverPtr				_statusReceiver;
	fsValueMediatorPtr				_portMediator;
	fsMulticastDetectorsPtr			_multicastDetectors;

	mxBool							_isValid;
	mxBool							_isStarted;
	mxULong							_statusTime;
	fsErrorMessage					_errorMessage;

	mxBool							_freezeFlag;
	mxBool							_quitFlag;

	fsReceiverUICallbackFunction	_UICallbackFunction;
	void*							_UICallbackData;

	// Parameters
	mxBool							_synchronizeSenderAddressOn;

	mxInt							_focusDisplay;
	mxBool							_directVideoModeOn;
	mxBool							_dvInterleavedModeOn;
	mxInt							_pointerCoordinate;

	mxBool							_pointerAutomaticSetupOn;
	mxBool							_raiseOn;
	mxBool							_drawLocusOn;
	mxInt							_generalPointer;
	mxInt							_raisePointer;
	mxInt							_locusPointer;
	mxInt							_pointers[NUM_POINTERDETAILENTRY];
	mxString						_pointerPhrases[NUM_POINTERDETAILENTRY];
	mxByte							_pointerColors[NUM_POINTERDETAILENTRY][3];

	mxInt							_videoWindowPolicy;
	mxBool							_keepVideoAspect;
	mxBool							_fitWindowToVideo;

	// Renderer Filters
	fsFilterPtr						_videoRender;
	fsFilterPtr						_audioRender;

	// Multicast Receiver Filters
	fsMulticastReceiverFilterPtr	_videoReceiverMain;
	fsMulticastReceiverFilterPtr	_videoReceiverSub;
	fsMulticastReceiverFilterPtr	_audioReceiver;
	fsMulticastReceiverFilterPtr	_pointerFocusReceiver;
	mxArray<fsMulticastReceiverFilterPtr>	_pointerReceivers;

	// Multicast Sender Filters
	fsMulticastSenderFilterPtr		_pointerSender;

	// Other Filters
	fsFormatTransFilterPtr			_formatTransform1;
	fsFormatTransFilterPtr			_formatTransform2;
	fsVideoOverlayMixerFilterPtr	_videoOverlayMixer;
	fsNonLinearZoomFilterPtr		_nonLinearZoom;
	fsPointerMixerFilterPtr			_pointerMixer;
	fsPointerPositionFilterPtr		_pointerPosition;
	fsFilterPtr						_dvSplitter;
	fsVolumeIndicatorFilterPtr		_volumeIndicator;
	fsDrawCaptionFilterPtr			_drawCaption;
};
//------------------------------------------------------------------------------
#endif // __fsReceiver_h__
