/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsReceiver/fsReceiver.h>
//------------------------------------------------------------------------------
mxBool
fsReceiver::buildGraph()
{
	assert(_graph->getNumFilters() == 0);

	mxBool ret = true;

	// Direct Video Mode
	if (_directVideoModeOn)
	{
		if (_dvInterleavedModeOn)
		{
			// Video & Audio (DV only)
			ret &= buildDVInterleavedGraph();
		}
		else
		{
			// Video
			ret &= buildDirectVideoGraph();
			// Audio
			ret &= buildAudioGraph();
		}
	}
	// Focus Disiplay Mode
	else
	{
		// Video
		switch(_focusDisplay)
		{
		case FSFD_NONE:
			ret &= buildFocusNoneVideoGraph();
			break;
		case FSFD_PARTIAL_ZOOM:
			ret &= buildFocusPartialZoomVideoGraph();
			break;
		case FSFD_NON_LINEAR_ZOOM:
			ret &= buildFocusNonLinearZoomVideoGraph();
			break;
		case FSFD_IMAGE_COMPOUND:
			ret &= buildFocusImageCompoundVideoGraph();
			break;
		}

		// Audio
		ret &= buildAudioGraph();
	}

	// Window Setting
	_graph->setVideoWindowOwner(_windowHandle);
	_graph->setVideoWindowVisible(true);
	if (_windowHandle != NULL)
		_graph->setVideoWindowStyle(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);

	// 
	if (3000 < GetTickCount() - _statusTime)
		ret = true;

	_isValid = ret;

	return ret;
}
//------------------------------------------------------------------------------
void
fsReceiver::clearGraph()
{
	_isValid = true;

	// Window Setting
	_graph->setVideoWindowVisible(false);
	_graph->setVideoWindowOwner(NULL);

	// Remove All Filters
	_graph->removeAllFilters();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#define AddFilter(__filter)			\
	_graph->addFilter(*(__filter));
//------------------------------------------------------------------------------
#define ConnectDirect(__filterOut, __outPin, __filterIn, __inPin, __errType, __errMess)		\
if (!_graph->connectFilterDirect(*(__filterOut), (__outPin), *(__filterIn), (__inPin)))		\
{																							\
	_errorMessage.Id ++;																	\
	_errorMessage.Type = fsErrorMessage::__errType;											\
	_errorMessage.Message = __errMess;														\
																							\
	return false;																			\
}
//------------------------------------------------------------------------------
#define ConnectIndirect(__filterOut, __outPin, __filterIn, __inPin, __errType, __errMess)	\
if (!_graph->connectFilterIndirect(*(__filterOut), (__outPin), *(__filterIn), (__inPin)))	\
{																							\
	_errorMessage.Id ++;																	\
	_errorMessage.Type = fsErrorMessage::__errType;											\
	_errorMessage.Message = __errMess;														\
																							\
	return false;																			\
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsReceiver::buildDVInterleavedGraph()
{
	//
	// Connection
	//
	// Video
	AddFilter(_videoReceiverMain);
	AddFilter(_dvSplitter);
	AddFilter(_pointerMixer);
	ConnectDirect(_videoReceiverMain, 0, _dvSplitter, 0,
		T_UNEXPECTED, "Connect: VideoReceiverMain->DVSplitter");
	ConnectIndirect(_dvSplitter, 0, _pointerMixer, 0,
		T_NOTFOUND_DECODER, "DV VideoDecoder");

	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
	{
		AddFilter(_pointerReceivers[i]);
		ConnectDirect(_pointerReceivers[i], 0, _pointerMixer, i + 1,
			T_UNEXPECTED, "Connect: PointerReceiver[" + mxString((mxInt)i) + "]->PointerMixer");
	}

	AddFilter(_drawCaption);
	ConnectDirect(_pointerMixer, 0, _drawCaption, 0,
		T_UNEXPECTED, "Connect: PointerMixer->DrawCaption");

	AddFilter(_videoRender);
	ConnectIndirect(_drawCaption, 0, _videoRender, 0,
		T_WRONG_MEDIATYPE, "Connect: DrawCaption->>VideoRenderer");

	AddFilter(_pointerPosition);
	AddFilter(_pointerSender);
	ConnectDirect(_pointerPosition, 0, _pointerSender, 0,
		T_UNEXPECTED, "Connect: PointerPosition->PointerSender");

	// Audio
	if (_audioRender != NULL)
	{
		AddFilter(_audioRender);
		ConnectIndirect(_dvSplitter, 1, _audioRender, 0,
			T_UNEXPECTED, "Connect: DVSplitter->>AudioRender");
	}

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::buildDirectVideoGraph()
{
	//
	// Connection
	//
	AddFilter(_videoReceiverMain);
	AddFilter(_pointerMixer);
	ConnectIndirect(_videoReceiverMain, 0, _pointerMixer, 0,
		T_NOTFOUND_DECODER, "Video");

	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
	{
		AddFilter(_pointerReceivers[i]);
		ConnectDirect(_pointerReceivers[i], 0, _pointerMixer, i + 1,
			T_UNEXPECTED, "Connect: PointerReceiver[" + mxString((mxInt)i) + "]->PointerMixer");
			
	}

	AddFilter(_drawCaption);
	ConnectDirect(_pointerMixer, 0, _drawCaption, 0,
		T_UNEXPECTED, "Connect: PointerMixer->DrawCaption");

	AddFilter(_videoRender);
	ConnectIndirect(_drawCaption, 0, _videoRender, 0,
		T_WRONG_MEDIATYPE, "Connect: DrawCaption->>VideoRenderer");

	AddFilter(_pointerPosition);
	AddFilter(_pointerSender);
	ConnectDirect(_pointerPosition, 0, _pointerSender, 0,
		T_UNEXPECTED, "Connect: PointerPosition->PointerSender");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::buildFocusNoneVideoGraph()
{
	//
	// Connection
	//
	AddFilter(_videoReceiverMain);
	AddFilter(_pointerMixer);
	ConnectIndirect(_videoReceiverMain, 0, _pointerMixer, 0,
		T_NOTFOUND_DECODER, "Video");

	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
	{
		AddFilter(_pointerReceivers[i]);
		ConnectDirect(_pointerReceivers[i], 0, _pointerMixer, i + 1,
			T_UNEXPECTED, "Connect: PointerReceiver[" + mxString((mxInt)i) + "]->PointerMixer");
	}

	AddFilter(_drawCaption);
	ConnectDirect(_pointerMixer, 0, _drawCaption, 0,
		T_UNEXPECTED, "Connect: PointerMixer->DrawCaption");

	AddFilter(_videoRender);
	ConnectIndirect(_drawCaption, 0, _videoRender, 0,
		T_WRONG_MEDIATYPE, "Connect: DrawCaption->>VideoRenderer");

	AddFilter(_pointerPosition);
	AddFilter(_pointerSender);
	ConnectDirect(_pointerPosition, 0, _pointerSender, 0,
		T_UNEXPECTED, "Connect: PointerPosition->PointerSender");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::buildFocusPartialZoomVideoGraph()
{
	//
	// Connection
	//
	AddFilter(_videoReceiverMain);
	AddFilter(_formatTransform1);
	ConnectIndirect(_videoReceiverMain, 0, _formatTransform1, 0,
		T_NOTFOUND_DECODER, "Video");

	AddFilter(_pointerMixer);
	ConnectDirect(_formatTransform1, 0, _pointerMixer, 0,
		T_UNEXPECTED, "Connect: FormatTransform1->PointerMixer");

	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
	{
		AddFilter(_pointerReceivers[i]);
		ConnectDirect(_pointerReceivers[i], 0, _pointerMixer, i + 1,
			T_UNEXPECTED, "Connect: PointerReceiver[" + mxString((mxInt)i) + "]->PointerMixer");
	}

	AddFilter(_drawCaption);
	ConnectDirect(_pointerMixer, 0, _drawCaption, 0,
		T_UNEXPECTED, "Connect: PointerMixer->DrawCaption");

	AddFilter(_videoRender);
	ConnectIndirect(_drawCaption, 0, _videoRender, 0,
		T_WRONG_MEDIATYPE, "Connect: DrawCaption->>VideoRenderer");

	AddFilter(_pointerPosition);
	AddFilter(_pointerSender);
	ConnectDirect(_pointerPosition, 0, _pointerSender, 0,
		T_UNEXPECTED, "Connect: PointerPosition->PointerSender");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::buildFocusNonLinearZoomVideoGraph()
{
	//
	// Connection
	//
	AddFilter(_videoReceiverMain);
	AddFilter(_nonLinearZoom);
	ConnectIndirect(_videoReceiverMain, 0, _nonLinearZoom, 0,
		T_NOTFOUND_DECODER, "Video");

	AddFilter(_pointerFocusReceiver);
	ConnectDirect(_pointerFocusReceiver, 0, _nonLinearZoom, 1,
		T_UNEXPECTED, "Connect: PointerFocusReceiver->NonLinearZoom");

	AddFilter(_pointerMixer);
	ConnectDirect(_nonLinearZoom, 0, _pointerMixer,	0,
		T_UNEXPECTED, "Connect: NonLinearZoom->PointerMixer");

	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
	{
		AddFilter(_pointerReceivers[i]);
		ConnectDirect(_pointerReceivers[i], 0, _pointerMixer, i + 1,
			T_UNEXPECTED, "Connect: PointerReceiver[" + mxString((mxInt)i) + "]->PointerMixer");
	}

	AddFilter(_drawCaption);
	ConnectDirect(_pointerMixer, 0, _drawCaption, 0,
		T_UNEXPECTED, "Connect: PointerMixer->DrawCaption");

	AddFilter(_videoRender);
	ConnectIndirect(_drawCaption, 0, _videoRender, 0,
		T_WRONG_MEDIATYPE, "Connect: DrawCaption->>VideoRenderer");

	AddFilter(_pointerPosition);
	AddFilter(_pointerSender);
	ConnectDirect(_pointerPosition,	0, _pointerSender,	0,
		T_UNEXPECTED, "Connect: PointerPosition->PointerSender");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::buildFocusImageCompoundVideoGraph()
{
	//
	// Connection
	//
	AddFilter(_videoReceiverMain);
	AddFilter(_formatTransform2);
	ConnectIndirect(_videoReceiverMain, 0, _formatTransform2, 0,
		T_NOTFOUND_DECODER, "Video Main");

	AddFilter(_videoReceiverSub);
	AddFilter(_videoOverlayMixer);
	ConnectIndirect(_videoReceiverSub, 0, _videoOverlayMixer, 1,
		T_NOTFOUND_DECODER, "Video Context");
	ConnectDirect(_formatTransform2, 0, _videoOverlayMixer, 0,
		T_UNEXPECTED, "Connect: FormatTransform2->VideoOverlayMixer");

	AddFilter(_pointerFocusReceiver);
	ConnectDirect(_pointerFocusReceiver, 0, _videoOverlayMixer, 2,
		T_UNEXPECTED, "Connect: PointerFocusReceiver->VideoOverlayMixer");

	AddFilter(_pointerMixer);
	ConnectDirect(_videoOverlayMixer, 0, _pointerMixer, 0,
		T_UNEXPECTED, "Connect: VideoOverlayMixer->PointerMixer");

	for(mxUInt i = 0; i < _pointerReceivers.getSize(); i++)
	{
		AddFilter(_pointerReceivers[i]);
		ConnectDirect(_pointerReceivers[i], 0, _pointerMixer, i + 1,
			T_UNEXPECTED, "Connect: PointerReceiver[" + mxString((mxInt)i) + "]->PointerMixer");
	}

	AddFilter(_drawCaption);
	ConnectDirect(_pointerMixer, 0, _drawCaption, 0,
		T_UNEXPECTED, "Connect: PointerMixer->DrawCaption");

	AddFilter(_videoRender);
	ConnectIndirect(_drawCaption, 0, _videoRender, 0,
		T_WRONG_MEDIATYPE, "Connect: DrawCaption->>VideoRenderer");

	AddFilter(_pointerPosition);
	AddFilter(_pointerSender);
	ConnectDirect(_pointerPosition,	0, _pointerSender,	0,
		T_UNEXPECTED, "Connect: PointerPosition->PointerSender");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsReceiver::buildAudioGraph()
{
	if (_audioRender == NULL)
		return true;

	AddFilter(_audioReceiver);
	AddFilter(_volumeIndicator);
	ConnectIndirect(_audioReceiver, 0, _volumeIndicator, 0,
		T_NOTFOUND_DECODER, "Audio");

	AddFilter(_audioRender);
	ConnectDirect(_volumeIndicator, 0, _audioRender, 0, 
		T_UNEXPECTED, "Connect: VolumeIndicator->AudioRender");

	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#undef AddFilter
#undef ConnectDirect
#undef ConnectIndirect
//------------------------------------------------------------------------------
