/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsReceiverIf_h__
#define __fsReceiverIf_h__
//------------------------------------------------------------------------------
#include <mx/mxCore/mxString.h>
#include <mx/mxCore/mxCore.h>
//------------------------------------------------------------------------------
class __declspec(novtable) fsReceiverIf
{
public:
	//
	// Streaming Control
	//
	virtual mxBool			getValid() const = 0;
	virtual void			getErrorMessage(mxInt& id, mxInt& type, mxString& message) const = 0;
	virtual void			setStart(mxBool flag) = 0;
	virtual mxBool			getStart() const = 0;
	virtual void			setPointerTransmissionOn(mxBool flag) = 0;
	virtual mxBool			getPointerTransmissionOn() const = 0;
	virtual mxBool			receiveSenderData() const = 0;
	virtual mxBool			receiveVideo() const = 0;
	virtual mxBool			receiveAudio() const = 0;
	virtual mxBool			receivePointerFocus() const = 0;


	//
	// Multicast Settings
	//
	virtual void			setAddress(const mxString& address) = 0;
	virtual const mxString&	getAddress() const = 0;
	virtual void			setTimeToLive(mxULong ttl) = 0;
	virtual mxULong			getTimeToLive() const = 0;
	virtual void			setSynchronizeSenderAddressOn(mxBool flag) = 0;
	virtual mxBool			getSynchronizeSenderAddressOn() const = 0;

	// Create Address
	virtual mxString		createIPv4Address() = 0;
	virtual mxString		createIPv6Address() = 0;

	// Rejection Address
	virtual const mxString&	getCurrentReceiveAddress() const = 0;
	virtual mxUInt			getNumRejectionAddress() const = 0;
	virtual mxString		getRejectionAddress(mxUInt index) const = 0;
	virtual void			addRejectionAddress(const mxString& address) = 0;
    virtual void			removeRejectionAddress(mxUInt index) = 0;

	// VideoReceiver (Main)
	virtual mxInt			getVideoReceiverMainReceiveRate() const = 0;
	virtual mxInt			getVideoReceiverMainFrameRate() const = 0;
	virtual mxInt			getVideoReceiverMainNumDropPackets() const = 0;
	virtual mxInt			getVideoReceiverMainNumDropFrames() const = 0;
	virtual mxInt			getVideoReceiverMainPacketSize() const = 0;

	// VideoReceiver (Sub)
	virtual mxInt			getVideoReceiverSubReceiveRate() const = 0;
	virtual mxInt			getVideoReceiverSubFrameRate() const = 0;
	virtual mxInt			getVideoReceiverSubNumDropPackets() const = 0;
	virtual mxInt			getVideoReceiverSubNumDropFrames() const = 0;
	virtual mxInt			getVideoReceiverSubPacketSize() const = 0;

	// AudioReceiver
	virtual mxInt			getAudioReceiverReceiveRate() const = 0;
	virtual mxInt			getAudioReceiverFrameRate() const = 0;
	virtual mxInt			getAudioReceiverNumDropPackets() const = 0;
	virtual mxInt			getAudioReceiverNumDropFrames() const = 0;
	virtual mxInt			getAudioReceiverPacketSize() const = 0;

	// PointerReceiver (Focus)
	virtual mxInt			getPointerFocusReceiverReceiveRate() const = 0;
	virtual mxInt			getPointerFocusReceiverFrameRate() const = 0;
	virtual mxInt			getPointerFocusReceiverNumDropPackets() const = 0;
	virtual mxInt			getPointerFocusReceiverNumDropFrames() const = 0;
	virtual mxInt			getPointerFocusReceiverPacketSize() const = 0;

	// PointerReceiver[s]
	virtual mxInt			getPointerReceiverReceiveRate() const = 0;
	virtual mxInt			getPointerReceiverFrameRate() const = 0;
	virtual mxInt			getPointerReceiverNumDropPackets() const = 0;
	virtual mxInt			getPointerReceiverNumDropFrames() const = 0;
	virtual mxInt			getPointerReceiverPacketSize() const = 0;

	// PointerSender
	virtual void			setPointerSenderMaxTransmissionRate(mxInt rate) = 0;
	virtual mxInt			getPointerSenderMaxTransmissionRate() const = 0;
	virtual void			setPointerSenderTransmissionCount(mxInt count) = 0;
	virtual mxInt			getPointerSenderTransmissionCount() const = 0;
	virtual void			setPointerSenderPacketSize(mxInt packetSize) = 0;
	virtual mxInt			getPointerSenderPacketSize() const = 0;


	//
	// Multicast Detector Settings
	//
	virtual mxBool			addMulticastDetector(const mxString& address) = 0;
	virtual mxBool			removeMulticastDetector(const mxString& address) = 0;
	virtual mxBool			wasDetectedMulticast(const mxString& address) = 0;


	//
	// Pointer Settings
	//
	virtual void			setPointerAutomaticSetupOn(mxBool flag) = 0;
	virtual mxBool			getPointerAutomaticSetupOn() const = 0;

	// Basic Settings
	virtual void			setGeneralPointer(mxInt pointer) = 0;
	virtual mxInt			getGeneralPointer() const = 0;
	virtual void			setRaiseOn(mxBool flag) = 0;
	virtual mxBool			getRaiseOn() const = 0;
	virtual void			setRaisePointer(mxInt pointer) = 0;
	virtual mxInt			getRaisePointer() const = 0;
	virtual void			setDrawLocusOn(mxBool flag) = 0;
	virtual mxBool			getDrawLocusOn() const = 0;
	virtual void			setLocusPointer(mxInt pointer) = 0;
	virtual mxInt			getLocusPointer() const = 0;

	// Detail Settings
	virtual void			setPointer(mxInt entryNo, mxInt pointer) = 0;
	virtual mxInt			getPointer(mxInt entryNo) const = 0;
	virtual void			setPointerPhrase(mxInt entryNo, const mxString& phrase) = 0;
	virtual const mxString&	getPointerPhrase(mxInt entryNo) const = 0;
	virtual void			setPointerColor(mxInt entryNo, const mxByte* color) = 0;
	virtual void			getPointerColor(mxInt entryNo, mxByte* color) const = 0;

	// Other Settings
	virtual void			setPointerCoordinate(mxInt coordinate) = 0;
	virtual mxInt			getPointerCoordinate() const = 0;
	virtual void			computePointerRegion() = 0;


	//
	// Caption Settings
	//
	virtual void			updateCaption() = 0;


	//
	// Video Window Settings
	//
	virtual void			setVideoWindowPolicy(mxInt policy) = 0;
	virtual mxInt			getVideoWindowPolicy() const = 0;
	virtual void			setKeepVideoAspect(mxBool flag) = 0;
	virtual mxBool			getKeepVideoAspect() const = 0;
	virtual void			setFitWindowToVideo(mxBool flag) = 0;
	virtual mxBool			getFitWindowToVideo() const = 0;
	virtual void			computeVideoWindow() = 0;


	//
	// Volume Settings
	//
	virtual void			getVolume(mxInt& left, mxInt& right) const = 0;
	virtual void			setVolumeEvaluationTime(mxInt evalTime) = 0;
	virtual mxInt			getVolumeEvaluationTime() const = 0;


	//
	// Other
	//
	virtual void			quit() = 0;
	virtual void			freeze(mxBool flag) = 0;
};
//------------------------------------------------------------------------------
#endif //__fsReceiverIf_h__
