/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsReceiverRMI/fsReceiverSkel.h>
#include <fs/fsReceiverRMI/fsReceiverIf.h>
#include <fs/fsCommon/fsCommon.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------
fsReceiverSkel::fsReceiverSkel(fsReceiverIf* receiver) :
mxObject()
{
	_receiver = receiver;

	_rObject = new fsRemoteObject(invokeProc, this);

	fsRMIPropertyPtr property = new fsRMIProperty();
	property->setAddress(DEFAULT_ADDRESS);
	property->setPort(fsRMIProperty::PID_TRANSMIT, PORT_RECEIVER_REMOTEACK);
	property->setPort(fsRMIProperty::PID_RECEIVE, PORT_RECEIVER_REMOTE);
	property->setTimeToLive(1);
	property->setPacketSize(1024);
	property->setMaximumTransmissionRate(10 * 1024 * 1024);
	property->setTransmissionCount(2);
	_rObject->setProperty(property);
}
//------------------------------------------------------------------------------
fsReceiverSkel::~fsReceiverSkel()
{
	_rObject = NULL;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsReceiverSkel::open()
{
	return _rObject->open();
}
//------------------------------------------------------------------------------
mxBool
fsReceiverSkel::close()
{
	return _rObject->close();
}
//------------------------------------------------------------------------------
void
fsReceiverSkel::setRMIProperty(const fsRMIPropertyPtr& property)
{
	_rObject->setProperty(property);
}
//------------------------------------------------------------------------------
const fsRMIPropertyPtr&
fsReceiverSkel::getRMIProperty() const
{
	return _rObject->getProperty();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
const fsReceiverIf*
fsReceiverSkel::getReceiver() const
{
	return _receiver;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsReceiverSkel::invoke(const mxString& method,
							 fsInvokeData& param,
							 fsInvokeData& retVal)
{

	//
	// Streaming Control
	//
	if (method == "getValid")
	{
		retVal.writeBool(_receiver->getValid());
	}
	else if (method == "getErrorMessage")
	{
		mxInt id;
		mxInt type;
		mxString message;
		_receiver->getErrorMessage(id, type, message);
		retVal.writeInt(id);
		retVal.writeInt(type);
		retVal.writeString(message);
	}
	else if (method == "setStart")
	{
		_receiver->setStart(param.readBool());
	}
	else if (method == "getStart")
	{
		retVal.writeBool(_receiver->getStart());
	}
	else if (method == "setPointerTransmissionOn")
	{
		_receiver->setPointerTransmissionOn(param.readBool());
	}
	else if (method == "getPointerTransmissionOn")
	{
		retVal.writeBool(_receiver->getPointerTransmissionOn());
	}
	else if (method == "receiveSenderData")
	{
		retVal.writeBool(_receiver->receiveSenderData());
	}
	else if (method == "receiveVideo")
	{
		retVal.writeBool(_receiver->receiveVideo());
	}
	else if (method == "receiveAudio")
	{
		retVal.writeBool(_receiver->receiveAudio());
	}
	else if (method == "receivePointerFocus")
	{
		retVal.writeBool(_receiver->receivePointerFocus());
	}

	//
	// Multicast Settings
	//
	else if (method == "setAddress")
	{
		mxString address = param.readString();

		_receiver->setAddress(address);

		const fsRMIPropertyPtr& property = _rObject->getProperty();
		property->setAddress(address);
		_rObject->setProperty(property);
	}
	else if (method == "getAddress")
	{
		retVal.writeString(_receiver->getAddress());
	}
	else if (method == "setTimeToLive")
	{
		mxULong ttl = (mxULong)param.readInt();

		_receiver->setTimeToLive(ttl);

		const fsRMIPropertyPtr& property = _rObject->getProperty();
		property->setTimeToLive(ttl);
		_rObject->setProperty(property);
	}
	else if (method == "getTimeToLive")
	{
		retVal.writeInt((mxInt)_receiver->getTimeToLive());
	}
	else if (method == "setSynchronizeSenderAddressOn")
	{
		_receiver->setSynchronizeSenderAddressOn(param.readBool());
	}
	else if (method == "getSynchronizeSenderAddressOn")
	{
		retVal.writeBool(_receiver->getSynchronizeSenderAddressOn());
	}

	// Create Address
	else if (method == "createIPv4Address")
	{
		retVal.writeString(_receiver->createIPv4Address());
	}
	else if (method == "createIPv6Address")
	{
		retVal.writeString(_receiver->createIPv6Address());
	}

	// Rejection Address
	else if (method == "getCurrentReceiveAddress")
	{
		retVal.writeString(_receiver->getCurrentReceiveAddress());
	}
	else if (method == "getNumRejectionAddress") 
	{
		retVal.writeInt((mxInt)_receiver->getNumRejectionAddress());
	}
	else if (method == "getRejectionAddress")
	{
		retVal.writeString(_receiver->getRejectionAddress((mxUInt)param.readInt()));
	}
	else if (method == "addRejectionAddress")
	{
		_receiver->addRejectionAddress(param.readString());
	}
	else if (method == "removeRejectionAddress")
	{
		_receiver->removeRejectionAddress((mxUInt)param.readInt());
	}

	// VideoReceiver (Main)
	else if (method == "getVideoReceiverMainReceiveRate")
	{
		retVal.writeInt(_receiver->getVideoReceiverMainReceiveRate());
	}
	else if (method == "getVideoReceiverMainFrameRate")
	{
		retVal.writeInt(_receiver->getVideoReceiverMainFrameRate());
	}
	else if (method == "getVideoReceiverMainNumDropPackets")
	{
		retVal.writeInt(_receiver->getVideoReceiverMainNumDropPackets());
	}
	else if (method == "getVideoReceiverMainNumDropFrames")
	{
		retVal.writeInt(_receiver->getVideoReceiverMainNumDropFrames());
	}
	else if (method == "getVideoReceiverMainPacketSize")
	{
		retVal.writeInt(_receiver->getVideoReceiverMainPacketSize());
	}

	// VideoReceiver (Sub)
	else if (method == "getVideoReceiverSubReceiveRate")
	{
		retVal.writeInt(_receiver->getVideoReceiverSubReceiveRate());
	}
	else if (method == "getVideoReceiverSubFrameRate")
	{
		retVal.writeInt(_receiver->getVideoReceiverSubFrameRate());
	}
	else if (method == "getVideoReceiverSubNumDropPackets")
	{
		retVal.writeInt(_receiver->getVideoReceiverSubNumDropPackets());
	}
	else if (method == "getVideoReceiverSubNumDropFrames")
	{
		retVal.writeInt(_receiver->getVideoReceiverSubNumDropFrames());
	}
	else if (method == "getVideoReceiverSubPacketSize")
	{
		retVal.writeInt(_receiver->getVideoReceiverSubPacketSize());
	}

	// AudioReceiver
	else if (method == "getAudioReceiverReceiveRate")
	{
		retVal.writeInt(_receiver->getAudioReceiverReceiveRate());
	}
	else if (method == "getAudioReceiverFrameRate")
	{
		retVal.writeInt(_receiver->getAudioReceiverFrameRate());
	}
	else if (method == "getAudioReceiverNumDropPackets")
	{
		retVal.writeInt(_receiver->getAudioReceiverNumDropPackets());
	}
	else if (method == "getAudioReceiverNumDropFrames")
	{
		retVal.writeInt(_receiver->getAudioReceiverNumDropFrames());
	}
	else if (method == "getAudioReceiverPacketSize")
	{
		retVal.writeInt(_receiver->getAudioReceiverPacketSize());
	}

	// PointerReceiver (Focus)
	else if (method == "getPointerFocusReceiverReceiveRate")
	{
		retVal.writeInt(_receiver->getPointerFocusReceiverReceiveRate());
	}
	else if (method == "getPointerFocusReceiverFrameRate")
	{
		retVal.writeInt(_receiver->getPointerFocusReceiverFrameRate());
	}
	else if (method == "getPointerFocusReceiverNumDropPackets")
	{
		retVal.writeInt(_receiver->getPointerFocusReceiverNumDropPackets());
	}
	else if (method == "getPointerFocusReceiverNumDropFrames")
	{
		retVal.writeInt(_receiver->getPointerFocusReceiverNumDropFrames());
	}
	else if (method == "getPointerFocusReceiverPacketSize")
	{
		retVal.writeInt(_receiver->getPointerFocusReceiverPacketSize());
	}

	// PointerReceiver[s]
	else if (method == "getPointerReceiverReceiveRate")
	{
		retVal.writeInt(_receiver->getPointerReceiverReceiveRate());
	}
	else if (method == "getPointerReceiverFrameRate")
	{
		retVal.writeInt(_receiver->getPointerReceiverFrameRate());
	}
	else if (method == "getPointerReceiverNumDropPackets")
	{
		retVal.writeInt(_receiver->getPointerReceiverNumDropPackets());
	}
	else if (method == "getPointerReceiverNumDropFrames")
	{
		retVal.writeInt(_receiver->getPointerReceiverNumDropFrames());
	}
	else if (method == "getPointerReceiverPacketSize")
	{
		retVal.writeInt(_receiver->getPointerReceiverPacketSize());
	}

	// PointerSender
	else if (method == "setPointerSenderMaxTransmissionRate")
	{
		_receiver->setPointerSenderMaxTransmissionRate(param.readInt());
	}
	else if (method == "getPointerSenderMaxTransmissionRate")
	{
		retVal.writeInt(_receiver->getPointerSenderMaxTransmissionRate());
	}
	else if (method == "setPointerSenderTransmissionCount")
	{
		_receiver->setPointerSenderTransmissionCount(param.readInt());
	}
	else if (method == "getPointerSenderTransmissionCount")
	{
		retVal.writeInt(_receiver->getPointerSenderTransmissionCount());
	}
	else if (method == "setPointerSenderPacketSize")
	{
		_receiver->setPointerSenderPacketSize(param.readInt());
	}
	else if (method == "getPointerSenderPacketSize")
	{
		retVal.writeInt(_receiver->getPointerSenderPacketSize());
	}


	//
	// Multicast Detector Settings
	//
	else if (method == "addMulticastDetector")
	{
		retVal.writeBool(_receiver->addMulticastDetector(param.readString()));
	}
	else if (method == "removeMulticastDetector")
	{
		retVal.writeBool(_receiver->removeMulticastDetector(param.readString()));
	}
	else if (method == "wasDetectedMulticast")
	{
		retVal.writeBool(_receiver->wasDetectedMulticast(param.readString()));
	}


	//
	// Pointer Settings
	//
	else if (method == "setPointerAutomaticSetupOn")
	{
		_receiver->setPointerAutomaticSetupOn(param.readBool());
	}
	else if (method == "getPointerAutomaticSetupOn")
	{
		retVal.writeBool(_receiver->getPointerAutomaticSetupOn());
	}

	// Basic Settings
	else if (method == "setGeneralPointer")
	{
		_receiver->setGeneralPointer(param.readInt());
	}
	else if (method == "getGeneralPointer")
	{
		retVal.writeInt(_receiver->getGeneralPointer());
	}
	else if (method == "setRaiseOn")
	{
		_receiver->setRaiseOn(param.readBool());
	}
	else if (method == "getRaiseOn")
	{
		retVal.writeBool(_receiver->getRaiseOn());
	}
	else if (method == "setRaisePointer")
	{
		_receiver->setRaisePointer(param.readInt());
	}
	else if (method == "getRaisePointer")
	{
		retVal.writeInt(_receiver->getRaisePointer());
	}
	else if (method == "setDrawLocusOn")
	{
		_receiver->setDrawLocusOn(param.readBool());
	}
	else if (method == "getDrawLocusOn")
	{
		retVal.writeBool(_receiver->getDrawLocusOn());
	}
	else if (method == "setLocusPointer")
	{
		_receiver->setLocusPointer(param.readInt());
	}
	else if (method == "getLocusPointer")
	{
		retVal.writeInt(_receiver->getLocusPointer());
	}

	// Detail Settings
	else if (method == "setPointer")
	{
		mxInt entryNo = param.readInt();
		mxInt pointer = param.readInt();
		_receiver->setPointer(entryNo, pointer);
	}
	else if (method == "getPointer")
	{
		retVal.writeInt(_receiver->getPointer(param.readInt()));
	}
	else if (method == "setPointerPhrase")
	{
		mxInt entryNo = param.readInt();
		mxString phrase = param.readString();
		_receiver->setPointerPhrase(entryNo, phrase);
	}
	else if (method == "getPointerPhrase")
	{
		retVal.writeString(_receiver->getPointerPhrase(param.readInt()));
	}
	else if (method == "setPointerColor")
	{
		mxInt entryNo = param.readInt();
		mxByte color[3];
		param.readByte(color, 3);
		_receiver->setPointerColor(entryNo, color);
	}
	else if (method == "getPointerColor")
	{
		mxByte color[3];
		_receiver->getPointerColor(param.readInt(), color);
		retVal.writeByte(color, 3);
	}

	// Other Settings
	else if (method == "setPointerCoordinate")
	{
		_receiver->setPointerCoordinate(param.readInt());
	}
	else if (method == "getPointerCoordinate")
	{
		retVal.writeInt(_receiver->getPointerCoordinate());
	}
	else if (method == "computePointerRegion")
	{
		_receiver->computePointerRegion();
	}


	//
	// Caption Settings
	//
	else if (method == "updateCaption")
	{
		_receiver->updateCaption();
	}


	//
	// Video Window Settings
	//
	else if (method == "setVideoWindowPolicy")
	{
		_receiver->setVideoWindowPolicy(param.readInt());
	}
	else if (method == "getVideoWindowPolicy")
	{
		retVal.writeInt(_receiver->getVideoWindowPolicy());
	}
	else if (method == "setKeepVideoAspect")
	{
		_receiver->setKeepVideoAspect(param.readBool());
	}
	else if (method == "getKeepVideoAspect")
	{
		retVal.writeBool(_receiver->getKeepVideoAspect());
	}
	else if (method == "setFitWindowToVideo")
	{
		_receiver->setFitWindowToVideo(param.readBool());
	}
	else if (method == "getFitWindowToVideo")
	{
		retVal.writeBool(_receiver->getFitWindowToVideo());
	}
	else if (method == "computeVideoWindow")
	{
		_receiver->computeVideoWindow();
	}


	//
	// Volume Settings
	//
	else if (method == "getVolume")
	{
		mxInt lr[2];
		_receiver->getVolume(lr[0], lr[1]);
		retVal.writeInt(lr, 2);
	}
	else if (method == "setVolumeEvaluationTime")
	{
		_receiver->setVolumeEvaluationTime(param.readInt());
	}
	else if (method == "getVolumeEvaluationTime")
	{
		retVal.writeInt(_receiver->getVolumeEvaluationTime());
	}


	//
	// Other
	//
	else if (method == "quit")
	{
		_receiver->quit();
	}
	else if (method == "freeze")
	{
		_receiver->freeze(param.readBool());
	}
}
//------------------------------------------------------------------------------
void
fsReceiverSkel::invokeProc(const mxString& method,
								 fsInvokeData& param,
								 fsInvokeData& retVal,
								 void *data)
{
	fsReceiverSkel *remoteObject = (fsReceiverSkel *)data;
	remoteObject->invoke(method, param, retVal);
}
//------------------------------------------------------------------------------
