/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsReceiverRMI/fsReceiverStub.h>
#include <fs/fsCommon/fsCommon.h>
//------------------------------------------------------------------------------
fsReceiverStub::fsReceiverStub() :
mxObject()
{
	_rInterface = new fsRemoteInterface(invokeNotifyProc, this);

	fsRMIPropertyPtr property = new fsRMIProperty();
	property->setAddress(DEFAULT_ADDRESS);
	property->setPort(fsRMIProperty::PID_TRANSMIT, PORT_RECEIVER_REMOTE);
	property->setPort(fsRMIProperty::PID_RECEIVE, PORT_RECEIVER_REMOTEACK);
	property->setTimeToLive(1);
	property->setPacketSize(1024);
	property->setMaximumTransmissionRate(10 * 1024 * 1024);
	property->setTransmissionCount(2);
	property->setInvokeTimeout(100);
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
fsReceiverStub::~fsReceiverStub()
{
	_rInterface = NULL;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsReceiverStub::open()
{
	return _rInterface->open();
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::close()
{
	return _rInterface->close();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setRMIProperty(const fsRMIPropertyPtr& property)
{
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
const fsRMIPropertyPtr&
fsReceiverStub::getRMIProperty() const
{
	return _rInterface->getProperty();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Implements fsReceiverIf
//
//------------------------------------------------------------------------------

fsInvokeData param;
fsInvokeData ret;
mxString str;

//------------------------------------------------------------------------------
//
// Streaming Control
//
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::getValid() const
{
	param.reset();
	_rInterface->invoke("getValid", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::getErrorMessage(mxInt& id, mxInt& type, mxString& message) const
{
	param.reset();
	_rInterface->invoke("getErrorMessage", param, ret);
	id = ret.readInt();
	type = ret.readInt();
	message = ret.readString();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setStart(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setStart", param);
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::getStart() const
{
	param.reset();
	_rInterface->invoke("getStart", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setPointerTransmissionOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setPointerTransmissionOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::getPointerTransmissionOn() const
{
	param.reset();
	_rInterface->invoke("getPointerTransmissionOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::receiveSenderData() const
{
	param.reset();
	_rInterface->invoke("receiveSenderData", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::receiveVideo() const
{
	param.reset();
	_rInterface->invoke("receiveVideo", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::receiveAudio() const
{
	param.reset();
	_rInterface->invoke("receiveAudio", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::receivePointerFocus() const
{
	param.reset();
	_rInterface->invoke("receivePointerFocus", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Multicast Settings
//
//------------------------------------------------------------------------------
void
fsReceiverStub::setAddress(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("setAddress", param);

	const fsRMIPropertyPtr& property = _rInterface->getProperty();
	property->setAddress(address);
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
const mxString&
fsReceiverStub::getAddress() const
{
	param.reset();
	_rInterface->invoke("getAddress", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setTimeToLive(mxULong ttl)
{
	param.reset();
	param.writeInt((mxInt)ttl);
	_rInterface->invoke("setTimeToLive", param);

	const fsRMIPropertyPtr& property = _rInterface->getProperty();
	property->setTimeToLive(ttl);
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
mxULong
fsReceiverStub::getTimeToLive() const
{
	param.reset();
	_rInterface->invoke("getTimeToLive", param, ret);
	return (mxULong)ret.readInt();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setSynchronizeSenderAddressOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setSynchronizeSenderAddressOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::getSynchronizeSenderAddressOn() const
{
	param.reset();
	_rInterface->invoke("getSynchronizeSenderAddressOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
// Create Address
mxString
fsReceiverStub::createIPv4Address()
{
	param.reset();
	_rInterface->invoke("createIPv4Address", param, ret);
	return ret.readString();
}
//------------------------------------------------------------------------------
mxString
fsReceiverStub::createIPv6Address()
{
	param.reset();
	_rInterface->invoke("createIPv6Address", param, ret);
	return ret.readString();
}
//------------------------------------------------------------------------------
// Rejection Address
const mxString&
fsReceiverStub::getCurrentReceiveAddress() const
{
	param.reset();
	_rInterface->invoke("getCurrentReceiveAddress", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
mxUInt
fsReceiverStub::getNumRejectionAddress() const
{
	param.reset();
	_rInterface->invoke("getNumRejectionAddress", param, ret);
	return (mxUInt)ret.readInt();
}
//------------------------------------------------------------------------------
mxString
fsReceiverStub::getRejectionAddress(mxUInt index) const
{
	param.reset();
	param.writeInt((mxInt)index);
	_rInterface->invoke("getRejectionAddress", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
void
fsReceiverStub::addRejectionAddress(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("addRejectionAddress", param);
}
//------------------------------------------------------------------------------
void
fsReceiverStub::removeRejectionAddress(mxUInt index)
{
	param.reset();
	param.writeInt((mxInt)index);
	_rInterface->invoke("removeRejectionAddress", param);
}
//------------------------------------------------------------------------------
// VideoReceiver (Main)
mxInt
fsReceiverStub::getVideoReceiverMainReceiveRate() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverMainReceiveRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVideoReceiverMainFrameRate() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverMainFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVideoReceiverMainNumDropPackets() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverMainNumDropPackets", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVideoReceiverMainNumDropFrames() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverMainNumDropFrames", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVideoReceiverMainPacketSize() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverMainPacketSize", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// VideoReceiver (Sub)
mxInt
fsReceiverStub::getVideoReceiverSubReceiveRate() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverSubReceiveRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVideoReceiverSubFrameRate() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverSubFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVideoReceiverSubNumDropPackets() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverSubNumDropPackets", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVideoReceiverSubNumDropFrames() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverSubNumDropFrames", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVideoReceiverSubPacketSize() const
{
	param.reset();
	_rInterface->invoke("getVideoReceiverSubPacketSize", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// AudioReceiver
mxInt
fsReceiverStub::getAudioReceiverReceiveRate() const
{
	param.reset();
	_rInterface->invoke("getAudioReceiverReceiveRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getAudioReceiverFrameRate() const
{
	param.reset();
	_rInterface->invoke("getAudioReceiverFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getAudioReceiverNumDropPackets() const
{
	param.reset();
	_rInterface->invoke("getAudioReceiverNumDropPackets", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getAudioReceiverNumDropFrames() const
{
	param.reset();
	_rInterface->invoke("getAudioReceiverNumDropFrames", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getAudioReceiverPacketSize() const
{
	param.reset();
	_rInterface->invoke("getAudioReceiverPacketSize", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// PointerReceiver (Focus)
mxInt
fsReceiverStub::getPointerFocusReceiverReceiveRate() const
{
	param.reset();
	_rInterface->invoke("getPointerFocusReceiverReceiveRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerFocusReceiverFrameRate() const
{
	param.reset();
	_rInterface->invoke("getPointerFocusReceiverFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerFocusReceiverNumDropPackets() const
{
	param.reset();
	_rInterface->invoke("getPointerFocusReceiverNumDropPackets", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerFocusReceiverNumDropFrames() const
{
	param.reset();
	_rInterface->invoke("getPointerFocusReceiverNumDropFrames", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerFocusReceiverPacketSize() const
{
	param.reset();
	_rInterface->invoke("getPointerFocusReceiverPacketSize", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// PointerReceiver[s]
mxInt
fsReceiverStub::getPointerReceiverReceiveRate() const
{
	param.reset();
	_rInterface->invoke("getPointerReceiverReceiveRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerReceiverFrameRate() const
{
	param.reset();
	_rInterface->invoke("getPointerReceiverFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerReceiverNumDropPackets() const
{
	param.reset();
	_rInterface->invoke("getPointerReceiverNumDropPackets", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerReceiverNumDropFrames() const
{
	param.reset();
	_rInterface->invoke("getPointerReceiverNumDropFrames", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerReceiverPacketSize() const
{
	param.reset();
	_rInterface->invoke("getPointerReceiverPacketSize", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// PointerSender
void
fsReceiverStub::setPointerSenderMaxTransmissionRate(mxInt rate)
{
	param.reset();
	param.writeInt(rate);
	_rInterface->invoke("setPointerSenderMaxTransmissionRate", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerSenderMaxTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getPointerSenderMaxTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setPointerSenderTransmissionCount(mxInt count)
{
	param.reset();
	param.writeInt(count);
	_rInterface->invoke("setPointerSenderTransmissionCount", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerSenderTransmissionCount() const
{
	param.reset();
	_rInterface->invoke("getPointerSenderTransmissionCount", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setPointerSenderPacketSize(mxInt packetSize)
{
	param.reset();
	param.writeInt(packetSize);
	_rInterface->invoke("setPointerSenderPacketSize", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerSenderPacketSize() const
{
	param.reset();
	_rInterface->invoke("getPointerSenderPacketSize", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Multicast Detector Settings
//
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::addMulticastDetector(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("addMulticastDetector", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::removeMulticastDetector(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("removeMulticastDetector", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::wasDetectedMulticast(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("wasDetectedMulticast", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Pointer Settings
//
//------------------------------------------------------------------------------
void
fsReceiverStub::setPointerAutomaticSetupOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setPointerAutomaticSetupOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::getPointerAutomaticSetupOn() const
{
	param.reset();
	_rInterface->invoke("getPointerAutomaticSetupOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
// Basic Settings
void
fsReceiverStub::setGeneralPointer(mxInt pointer)
{
	param.reset();
	param.writeInt(pointer);
	_rInterface->invoke("setGeneralPointer", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getGeneralPointer() const
{
	param.reset();
	_rInterface->invoke("getGeneralPointer", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setRaiseOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setRaiseOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::getRaiseOn() const
{
	param.reset();
	_rInterface->invoke("getRaiseOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setRaisePointer(mxInt pointer)
{
	param.reset();
	param.writeInt(pointer);
	_rInterface->invoke("setRaisePointer", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getRaisePointer() const
{
	param.reset();
	_rInterface->invoke("getRaisePointer", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setDrawLocusOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setDrawLocusOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::getDrawLocusOn() const
{
	param.reset();
	_rInterface->invoke("getDrawLocusOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setLocusPointer(mxInt pointer)
{
	param.reset();
	param.writeInt(pointer);
	_rInterface->invoke("setLocusPointer", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getLocusPointer() const
{
	param.reset();
	_rInterface->invoke("getLocusPointer", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// Detail Settings
void
fsReceiverStub::setPointer(mxInt entryNo, mxInt pointer)
{
	param.reset();
	param.writeInt(entryNo);
	param.writeInt(pointer);
	_rInterface->invoke("setPointer", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointer(mxInt entryNo) const
{
	param.reset();
	param.writeInt(entryNo);
	_rInterface->invoke("getPointer", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setPointerPhrase(mxInt entryNo, const mxString& phrase)
{
	param.reset();
	param.writeInt(entryNo);
	param.writeString(phrase);
	_rInterface->invoke("setPointerPhrase", param);
}
//------------------------------------------------------------------------------
const mxString&
fsReceiverStub::getPointerPhrase(mxInt entryNo) const
{
	param.reset();
	param.writeInt(entryNo);
	_rInterface->invoke("getPointerPhrase", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setPointerColor(mxInt entryNo, const mxByte* color)
{
	param.reset();
	param.writeInt(entryNo);
	param.writeByte(color, 3);
	_rInterface->invoke("setPointerColor", param);
}
//------------------------------------------------------------------------------
void
fsReceiverStub::getPointerColor(mxInt entryNo, mxByte* color) const
{
	param.reset();
	param.writeInt(entryNo);
	_rInterface->invoke("getPointerColor", param, ret);
	ret.readByte(color, 3);
}
//------------------------------------------------------------------------------
// Other Settings
void
fsReceiverStub::setPointerCoordinate(mxInt coordinate)
{
	param.reset();
	param.writeInt(coordinate);
	_rInterface->invoke("setPointerCoordinate", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getPointerCoordinate() const
{
	param.reset();
	_rInterface->invoke("getPointerCoordinate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::computePointerRegion()
{
	_rInterface->invoke("computePointerRegion");
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Caption Settings
//
//------------------------------------------------------------------------------
void
fsReceiverStub::updateCaption()
{
	_rInterface->invoke("updateCaption");
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Video Window Settings
//
//------------------------------------------------------------------------------
void
fsReceiverStub::setVideoWindowPolicy(mxInt policy)
{
	param.reset();
	param.writeInt(policy);
	_rInterface->invoke("setVideoWindowPolicy", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVideoWindowPolicy() const
{
	param.reset();
	_rInterface->invoke("getVideoWindowPolicy", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setKeepVideoAspect(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setKeepVideoAspect", param);
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::getKeepVideoAspect() const
{
	param.reset();
	_rInterface->invoke("getKeepVideoAspect", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setFitWindowToVideo(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setFitWindowToVideo", param);
}
//------------------------------------------------------------------------------
mxBool
fsReceiverStub::getFitWindowToVideo() const
{
	param.reset();
	_rInterface->invoke("getFitWindowToVideo", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsReceiverStub::computeVideoWindow()
{
	_rInterface->invoke("computeVideoWindow");
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Volume Settings
//
//------------------------------------------------------------------------------
void
fsReceiverStub::getVolume(mxInt& left, mxInt& right) const
{
	param.reset();
	_rInterface->invoke("getVolume", param, ret);
	mxInt lr[2];
	ret.readInt(lr, 2);
	left = lr[0];
	right = lr[1];
}
//------------------------------------------------------------------------------
void
fsReceiverStub::setVolumeEvaluationTime(mxInt evalTime)
{
	param.reset();
	param.writeInt(evalTime);
	_rInterface->invoke("setVolumeEvaluationTime", param);
}
//------------------------------------------------------------------------------
mxInt
fsReceiverStub::getVolumeEvaluationTime() const
{
	param.reset();
	_rInterface->invoke("getVolumeEvaluationTime", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Other
//
//------------------------------------------------------------------------------
void
fsReceiverStub::quit()
{
	_rInterface->invoke("quit");
}
//------------------------------------------------------------------------------
void
fsReceiverStub::freeze(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("freeze", param);
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
void
fsReceiverStub::invokeNotifyProc(const mxString& method, mxBool result, void *data)
{
	fsReceiverStub *receiver = (fsReceiverStub *)data;
	receiver->invokeNotify(method, result);
}
//------------------------------------------------------------------------------
