/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsReceiverStub_h__
#define __fsReceiverStub_h__
//------------------------------------------------------------------------------
#include <fs/fsReceiverRMI/fsReceiverIf.h>
#include <fs/fsReceiverRMI/fsReceiverRMI.h>
#include <fs/fsMulticastTools/fsRemoteInterface.h>
#include <mx/mxCore/mxObject.h>
//------------------------------------------------------------------------------
class fsReceiverStub;
typedef mxObjectPtr<fsReceiverStub>	fsReceiverStubPtr;
//------------------------------------------------------------------------------
class FSRECEIVERRMICLASS fsReceiverStub : public mxObject, public fsReceiverIf
{
	MXOBJECTHEADER(fsReceiverStub, mxObject);

public:
	fsReceiverStub();

protected:
	virtual ~fsReceiverStub();

public:
	mxBool				open();
	mxBool				close();
	void				setRMIProperty(const fsRMIPropertyPtr& property);
	const fsRMIPropertyPtr&	getRMIProperty() const;

public:
	//
	// Implements fsReceiverIf
	//
	//
	// Streaming Control
	//
	mxBool				getValid() const;
	void				getErrorMessage(mxInt& id, mxInt& type, mxString& message) const;
	void				setStart(mxBool flag);
	mxBool				getStart() const;
	void				setPointerTransmissionOn(mxBool flag);
	mxBool				getPointerTransmissionOn() const;
	mxBool				receiveSenderData() const;
	mxBool				receiveVideo() const;
	mxBool				receiveAudio() const;
	mxBool				receivePointerFocus() const;


	//
	// Multicast Settings
	//
	void				setAddress(const mxString& address);
	const mxString&		getAddress() const;
	void				setTimeToLive(mxULong ttl);
	mxULong				getTimeToLive() const;
	void				setSynchronizeSenderAddressOn(mxBool flag);
	mxBool				getSynchronizeSenderAddressOn() const;

	// Create Address
	mxString			createIPv4Address();
	mxString			createIPv6Address();

	// Rejection Address
	const mxString&		getCurrentReceiveAddress() const;
	mxUInt				getNumRejectionAddress() const;
	mxString			getRejectionAddress(mxUInt index) const;
	void				addRejectionAddress(const mxString& address);
    void				removeRejectionAddress(mxUInt index);

	// VideoReceiver (Main)
	mxInt				getVideoReceiverMainReceiveRate() const;
	mxInt				getVideoReceiverMainFrameRate() const;
	mxInt				getVideoReceiverMainNumDropPackets() const;
	mxInt				getVideoReceiverMainNumDropFrames() const;
	mxInt				getVideoReceiverMainPacketSize() const;

	// VideoReceiver (Sub)
	mxInt				getVideoReceiverSubReceiveRate() const;
	mxInt				getVideoReceiverSubFrameRate() const;
	mxInt				getVideoReceiverSubNumDropPackets() const;
	mxInt				getVideoReceiverSubNumDropFrames() const;
	mxInt				getVideoReceiverSubPacketSize() const;

	// AudioReceiver
	mxInt				getAudioReceiverReceiveRate() const;
	mxInt				getAudioReceiverFrameRate() const;
	mxInt				getAudioReceiverNumDropPackets() const;
	mxInt				getAudioReceiverNumDropFrames() const;
	mxInt				getAudioReceiverPacketSize() const;

	// PointerReceiver (Focus)
	mxInt				getPointerFocusReceiverReceiveRate() const;
	mxInt				getPointerFocusReceiverFrameRate() const;
	mxInt				getPointerFocusReceiverNumDropPackets() const;
	mxInt				getPointerFocusReceiverNumDropFrames() const;
	mxInt				getPointerFocusReceiverPacketSize() const;

	// PointerReceiver[s]
	mxInt				getPointerReceiverReceiveRate() const;
	mxInt				getPointerReceiverFrameRate() const;
	mxInt				getPointerReceiverNumDropPackets() const;
	mxInt				getPointerReceiverNumDropFrames() const;
	mxInt				getPointerReceiverPacketSize() const;

	// PointerSender
	void				setPointerSenderMaxTransmissionRate(mxInt rate);
	mxInt				getPointerSenderMaxTransmissionRate() const;
	void				setPointerSenderTransmissionCount(mxInt count);
	mxInt				getPointerSenderTransmissionCount() const;
	void				setPointerSenderPacketSize(mxInt packetSize);
	mxInt				getPointerSenderPacketSize() const;


	//
	// Multicast Detector Settings
	//
	mxBool				addMulticastDetector(const mxString& address);
	mxBool				removeMulticastDetector(const mxString& address);
	mxBool				wasDetectedMulticast(const mxString& address);


	//
	// Pointer Settings
	//
	void				setPointerAutomaticSetupOn(mxBool flag);
	mxBool				getPointerAutomaticSetupOn() const;

	// Basic Settings
	void				setGeneralPointer(mxInt pointer);
	mxInt				getGeneralPointer() const;
	void				setRaiseOn(mxBool flag);
	mxBool				getRaiseOn() const;
	void				setRaisePointer(mxInt pointer);
	mxInt				getRaisePointer() const;
	void				setDrawLocusOn(mxBool flag);
	mxBool				getDrawLocusOn() const;
	void				setLocusPointer(mxInt pointer);
	mxInt				getLocusPointer() const;

	// Detail Settings
	void				setPointer(mxInt entryNo, mxInt pointer);
	mxInt				getPointer(mxInt entryNo) const;
	void				setPointerPhrase(mxInt entryNo, const mxString& phrase);
	const mxString&		getPointerPhrase(mxInt entryNo) const;
	void				setPointerColor(mxInt entryNo, const mxByte* color);
	void				getPointerColor(mxInt entryNo, mxByte* color) const;

	// Other Settings
	void				setPointerCoordinate(mxInt coordinate);
	mxInt				getPointerCoordinate() const;
	void				computePointerRegion();


	//
	// Caption Settings
	//
	void				updateCaption();


	//
	// Video Window Settings
	//
	void				setVideoWindowPolicy(mxInt policy);
	mxInt				getVideoWindowPolicy() const;
	void				setKeepVideoAspect(mxBool flag);
	mxBool				getKeepVideoAspect() const;
	void				setFitWindowToVideo(mxBool flag);
	mxBool				getFitWindowToVideo() const;
	void				computeVideoWindow();


	//
	// Volume Settings
	//
	void				getVolume(mxInt& left, mxInt& right) const;
	void				setVolumeEvaluationTime(mxInt evalTime);
	mxInt				getVolumeEvaluationTime() const;


	//
	// Other
	//
	void				quit();
	void				freeze(mxBool flag);

protected:
	virtual void		invokeNotify(const mxString& method, mxBool result) = 0;
	static void			invokeNotifyProc(const mxString& method, mxBool result, void *data);

protected:
	fsRemoteInterfacePtr		_rInterface;
};
//------------------------------------------------------------------------------
#endif //__fsReceiverStub_h__
