/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsReceiverRemote/fsReceiverRemote.h>
#include <fs/fsCommon/fsCommon.h>
#include <mx/mxCore/mxRegistry.h>
#include <mx/mxCore/mxIniFile.h>
//------------------------------------------------------------------------------
#pragma comment(lib, "fsReceiverRMI.lib")
#pragma comment(lib, "mxCore.lib")
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
BOOL APIENTRY
DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
    return TRUE;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsReceiverRemote::fsReceiverRemote() :
fsReceiverStub()
{
	mxRegistry::setCompanyName(FS_REGISTRY_COMPANY_NAME);
	mxRegistry::setProductName(FS_REGISTRY_PRODUCT_NAME);

	_UICallbackFunction		= NULL;
	_UICallbackData			= NULL;
}
//------------------------------------------------------------------------------
fsReceiverRemote::~fsReceiverRemote()
{
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Callback
mxBool
fsReceiverRemote::setUICallbackFunction(fsReceiverUICallbackFunction func, void* data)
{
	_UICallbackFunction = func;
	_UICallbackData = data;
	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// IniFile
mxBool
fsReceiverRemote::writeIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "a");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);
	mxString str;

	//
	// Multicast Settings
	//
	iniFile.writeString("Receiver", "MulticastAddress", getAddress());
	iniFile.writeUInt("Receiver", "MulticastTimeToLive", (mxUInt)getTimeToLive());

	str = getSynchronizeSenderAddressOn() ? "true" : "false";
	iniFile.writeString("Receiver", "SynchronizeSenderAddressOn", str);

	// Rejection Address
	mxUInt numReject = getNumRejectionAddress();
	iniFile.writeUInt("Receiver", "NumberOfRejectionAddress", numReject);
	for(mxUInt i = 0; i < numReject; i++)
		iniFile.writeString("Receiver", "RejectionAddress" + mxString((mxInt)i), getRejectionAddress(i));

	//
	// Pointer Sender
	//
	iniFile.writeUInt("Receiver", "PointerSenderMaxTransmissionRate", (mxUInt)getPointerSenderMaxTransmissionRate());
	iniFile.writeUInt("Receiver", "PointerSenderTransmissionCount", (mxUInt)getPointerSenderTransmissionCount());
	iniFile.writeUInt("Receiver", "PointerSenderPacketSize", (mxUInt)getPointerSenderPacketSize());

	//
	// Pointer Settings
	//
	FS_GENERALPOINTER2STR(getGeneralPointer(), str);
	iniFile.writeString("Receiver", "PointerGeneralPointer", str);

	str = getRaiseOn() ? "true" : "false";
	iniFile.writeString("Receiver", "PointerRaiseOn", str);

	FS_RAISEPOINTER2STR(getRaisePointer(), str);
	iniFile.writeString("Receiver", "PointerRaisePointer", str);

	str = getDrawLocusOn() ? "true" : "false";
	iniFile.writeString("Receiver", "PointerDrawLocusOn", str);

	FS_LOCUSPOINTER2STR(getLocusPointer(), str);
	iniFile.writeString("Receiver", "PointerLocusPointer", str);

	mxString DetailEntry [NUM_POINTERDETAILENTRY] = {
		"General", "Locus", "Raise1", "Raise2", "Raise3", "Raise4", "Raise5" };
	for(mxUInt i = 0; i < NUM_POINTERDETAILENTRY; i ++)
	{
		str = mxString(getPointer(i));
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"Pointer", str);

		str = getPointerPhrase(i);
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerPhrase", str);

		mxByte color[3];
		getPointerColor(i, color);
		
		str = mxString((mxInt)color[0]);
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorR", str);

		str = mxString((mxInt)color[1]);
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorG", str);

		str = mxString((mxInt)color[2]);
		iniFile.writeString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorB", str);
	}

	str = getPointerAutomaticSetupOn() ? "true" : "false";
	iniFile.writeString("Receiver", "PointerAutomaticSetupOn", str);

	FS_COORDINATE2STR(getPointerCoordinate(), str);
	iniFile.writeString("Receiver", "PointerCoordinate", str);

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsReceiverRemote::readIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "r");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);

	mxString str, def;
	mxUInt val;
	mxInt index;

	// Stop Graph and StatusReceiver
	freeze(true);

	//
	// Multicast Settins
	//
	str = iniFile.readString("Receiver", "MulticastAddress", getAddress());
	val = iniFile.readUInt("Receiver", "MulticastTimeToLive", getTimeToLive());

	setAddress(str);
	setTimeToLive((mxULong)val);

	def = getSynchronizeSenderAddressOn() ? "true" : "false";
	str = iniFile.readString("Receiver", "SynchronizeSenderAddressOn", def);
	str.toLower();
	setSynchronizeSenderAddressOn((str == "true"));

	// Rejection Address
	mxUInt numReject = getNumRejectionAddress();
	for(mxUInt i = 0; i < numReject; i++)
		removeRejectionAddress(0);
	numReject = iniFile.readUInt("Receiver", "NumberOfRejectionAddress", 0);
	for(mxUInt i = 0; i < numReject; i++)
	{
		str = iniFile.readString("Receiver", "RejectionAddress" + mxString((mxInt)i), "");
		if (str.length() != 0)
			addRejectionAddress(str);
	}

	//
	// Pointer Sender
	//
	val = iniFile.readUInt("Receiver", "PointerSenderMaxTransmissionRate", (mxUInt)getPointerSenderMaxTransmissionRate());
	setPointerSenderMaxTransmissionRate((mxInt)val);
	val = iniFile.readUInt("Receiver", "PointerSenderTransmissionCount", (mxUInt)getPointerSenderTransmissionCount());
	setPointerSenderTransmissionCount((mxInt)val);
	val = iniFile.readUInt("Receiver", "PointerSenderPacketSize", (mxUInt)getPointerSenderPacketSize());
	setPointerSenderPacketSize((mxInt)val);

	//
	// Pointer Settings
	//
	FS_GENERALPOINTER2STR(getGeneralPointer(), def);
	str = iniFile.readString("Receiver", "PointerGeneralPointer", def);
	FS_STR2GENERALPOINTER(str, index);
	setGeneralPointer(index);

	def = getRaiseOn() ? "true" : "false";
	str = iniFile.readString("Receiver", "PointerRaiseOn", def);
	str.toLower();
	setRaiseOn((str == "true"));

	FS_RAISEPOINTER2STR(getRaisePointer(), def);
	str = iniFile.readString("Receiver", "PointerRaisePointer", def);
	FS_STR2RAISEPOINTER(str, index);
	setRaisePointer(index);

	def = getDrawLocusOn() ? "true" : "false";
	str = iniFile.readString("Receiver", "PointerDrawLocusOn", def);
	str.toLower();
	setDrawLocusOn((str == "true"));

	FS_LOCUSPOINTER2STR(getLocusPointer(), def);
	str = iniFile.readString("Receiver", "PointerLocusPointer", def);
	FS_STR2LOCUSPOINTER(str, index);
	setLocusPointer(index);

	mxString DetailEntry [NUM_POINTERDETAILENTRY] = {
		"General", "Locus", "Raise1", "Raise2", "Raise3", "Raise4", "Raise5" };
	for(mxUInt i = 0; i < NUM_POINTERDETAILENTRY; i ++)
	{
		def = mxString(getPointer(i));
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"Pointer", def);
		if (str.length() != 0)
			setPointer(i, atoi(str.c_str()));
		else
			setPointer(i, 0);

		def = getPointerPhrase(i);
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerPhrase", def);
		setPointerPhrase(i, str);

		mxByte color[3];
		getPointerColor(i, color);
		
		def = mxString((mxInt)color[0]);
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorR", def);
		if (str.length() != 0)
			color[0] = atoi(str.c_str());
		else
			color[0] = 0;

		def = mxString((mxInt)color[1]);
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorG", def);
		if (str.length() != 0)
			color[1] = atoi(str.c_str());
		else
			color[1] = 0;

		def = mxString((mxInt)color[2]);
		str = iniFile.readString("Receiver", "PointerDetail"+DetailEntry[i]+"PointerColorB", def);
		if (str.length() != 0)
			color[2] = atoi(str.c_str());
		else
			color[2] = 0;

		setPointerColor(i, color);
	}

	def = getPointerAutomaticSetupOn() ? "true" : "false";
	str = iniFile.readString("Receiver", "PointerAutomaticSetupOn", def);
	str.toLower();
	setPointerAutomaticSetupOn((str == "true"));

	FS_COORDINATE2STR(getPointerCoordinate(), def);
	str = iniFile.readString("Receiver", "PointerCoordinate", def);
	FS_STR2COORDINATE(str, index);
	setPointerCoordinate(index);

	// Start Graph and StatusReceiver
	freeze(false);

	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Registry
void
fsReceiverRemote::writeRegistry(const mxString& section,
									   const mxString& entry,
									   const mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	registry.writeString(section, entry, value);
	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsReceiverRemote::readRegistry(const mxString& section,
									  const mxString& entry,
									  const mxString& defaultValue,
									  mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	value = registry.readString(section, entry, defaultValue);
	registry.closeKey();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsReceiverRemote::invokeNotify(const mxString& method, mxBool result)
{
	if (_UICallbackFunction == NULL)
		return ;

	const mxChar* cstr = method.c_str();
	_UICallbackFunction(result ? "InvokeSucceeded" : "InvokeFailed", (void *)cstr, _UICallbackData);
}
//------------------------------------------------------------------------------
