/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "StdAfx.h"
#include <fs/fsReceiverRemote/fsReceiverRemoteInterface.h>
#include <fs/fsReceiverRemote/fsReceiverRemote.h>
#include <crtdbg.h>
//------------------------------------------------------------------------------

static fsReceiverRemotePtr	g_receiver	= NULL;

//------------------------------------------------------------------------------

mxChar str[1024];

#define MXSTR2STR(__str, __mxstr)			\
	mxString mxstr = __mxstr;				\
	if (mxstr.length() != 0)				\
		strcpy(__str, mxstr.c_str());		\
	else									\
		__str[0] = 0;

//------------------------------------------------------------------------------
//
// Initialize / Shutdown
//
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverInitialize()
{
	mxCore::initialize();
	g_receiver = new fsReceiverRemote();
	return g_receiver->open();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverShutdown()
{
	assert(g_receiver != NULL);
	g_receiver->close();
	g_receiver = NULL;

	mxCore::shutdown();

#ifdef _DEBUG
	_CrtDumpMemoryLeaks();
#endif // _DEBUG

	return true;
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverIsInitialized()
{
	return (g_receiver != NULL);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Streaming Control
//
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverGetValid()
{
	assert(g_receiver != NULL);
	return g_receiver->getValid();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverGetErrorMessage(int* id, int* type, char *message)
{
	assert(g_receiver != NULL);

	mxString mes;
	g_receiver->getErrorMessage(*id, *type, mes);

	MXSTR2STR(message, mes);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetStart(bool flag)
{
	assert(g_receiver != NULL);
	g_receiver->setStart(flag);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverGetStart()
{
	assert(g_receiver != NULL);
	return g_receiver->getStart();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetPointerTransmissionOn(bool flag)
{
	assert(g_receiver != NULL);
	g_receiver->setPointerTransmissionOn(flag);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverGetPointerTransmissionOn()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerTransmissionOn();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverReceiveSenderData()
{
	assert(g_receiver != NULL);
	return g_receiver->receiveSenderData();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverReceiveVideo()
{
	assert(g_receiver != NULL);
	return g_receiver->receiveVideo();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverReceiveAudio()
{
	assert(g_receiver != NULL);
	return g_receiver->receiveAudio();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverReceivePointerFocus()
{
	assert(g_receiver != NULL);
	return g_receiver->receivePointerFocus();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multicast Settings
//
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetAddress(const char* address)
{
	assert(g_receiver != NULL);
	g_receiver->setAddress(address);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC const char* MVSTD
fsReceiverGetAddress()
{
	assert(g_receiver != NULL);
	MXSTR2STR(str, g_receiver->getAddress());
	return str;
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetTimeToLive(int ttl)
{
	assert(g_receiver != NULL);
	g_receiver->setTimeToLive(ttl);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetTimeToLive()
{
	assert(g_receiver != NULL);
	return g_receiver->getTimeToLive();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetSynchronizeSenderAddressOn(bool flag)
{
	assert(g_receiver != NULL);
	g_receiver->setSynchronizeSenderAddressOn(flag);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverGetSynchronizeSenderAddressOn()
{
	assert(g_receiver != NULL);
	return g_receiver->getSynchronizeSenderAddressOn();
}
//------------------------------------------------------------------------------
// Create Address
FSRECEIVERREMOTEPROC const char* MVSTD
fsReceiverCreateIPv4Address()
{
	assert(g_receiver != NULL);
	MXSTR2STR(str, g_receiver->createIPv4Address());
	return str;
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC const char* MVSTD
fsReceiverCreateIPv6Address()
{
	assert(g_receiver != NULL);
	MXSTR2STR(str, g_receiver->createIPv6Address());
	return str;
}
//------------------------------------------------------------------------------
// Rejection Address
FSRECEIVERREMOTEPROC const char* MVSTD
fsReceiverGetCurrentReceiveAddress()
{
	assert(g_receiver != NULL);
	MXSTR2STR(str, g_receiver->getCurrentReceiveAddress());
	return str;
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetNumRejectionAddress()
{
	assert(g_receiver != NULL);
	return (mxInt)g_receiver->getNumRejectionAddress();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC const char* MVSTD
fsReceiverGetRejectionAddress(int index)
{
	assert(g_receiver != NULL);
	MXSTR2STR(str, g_receiver->getRejectionAddress((mxUInt)index));
	return str;
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverAddRejectionAddress(const char* address)
{
	assert(g_receiver != NULL);
	g_receiver->addRejectionAddress(address);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverRemoveRejectionAddress(unsigned int index)
{
	assert(g_receiver != NULL);
	g_receiver->removeRejectionAddress(index);
}
//------------------------------------------------------------------------------
// VideoReceiver (Main)
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverMainReceiveRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverMainReceiveRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverMainFrameRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverMainFrameRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverMainNumDropPackets()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverMainNumDropPackets();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverMainNumDropFrames()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverMainNumDropFrames();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverMainPacketSize()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverMainPacketSize();
}
//------------------------------------------------------------------------------
// VideoReceiver (Sub)
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverSubReceiveRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverSubReceiveRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverSubFrameRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverSubFrameRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverSubNumDropPackets()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverSubNumDropPackets();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverSubNumDropFrames()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverSubNumDropFrames();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoReceiverSubPacketSize()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoReceiverSubPacketSize();
}
//------------------------------------------------------------------------------
// AudioReceiver
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetAudioReceiverReceiveRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getAudioReceiverReceiveRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetAudioReceiverFrameRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getAudioReceiverFrameRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetAudioReceiverNumDropPackets()
{
	assert(g_receiver != NULL);
	return g_receiver->getAudioReceiverNumDropPackets();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetAudioReceiverNumDropFrames()
{
	assert(g_receiver != NULL);
	return g_receiver->getAudioReceiverNumDropFrames();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetAudioReceiverPacketSize()
{
	assert(g_receiver != NULL);
	return g_receiver->getAudioReceiverPacketSize();
}
//------------------------------------------------------------------------------
// PointerReceiver (Focus)
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerFocusReceiverReceiveRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerFocusReceiverReceiveRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerFocusReceiverFrameRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerFocusReceiverFrameRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerFocusReceiverNumDropPackets()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerFocusReceiverNumDropPackets();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerFocusReceiverNumDropFrames()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerFocusReceiverNumDropFrames();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerFocusReceiverPacketSize()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerFocusReceiverPacketSize();
}
//------------------------------------------------------------------------------
// PointerReceiver[s]
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerReceiverReceiveRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerReceiverReceiveRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerReceiverFrameRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerReceiverFrameRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerReceiverNumDropPackets()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerReceiverNumDropPackets();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerReceiverNumDropFrames()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerReceiverNumDropFrames();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerReceiverPacketSize()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerReceiverPacketSize();
}
//------------------------------------------------------------------------------
// Pointer Sender
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetPointerSenderMaxTransmissionRate(int rate)
{
	assert(g_receiver != NULL);
	g_receiver->setPointerSenderMaxTransmissionRate(rate);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerSenderMaxTransmissionRate()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerSenderMaxTransmissionRate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetPointerSenderTransmissionCount(int count)
{
	assert(g_receiver != NULL);
	g_receiver->setPointerSenderTransmissionCount(count);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerSenderTransmissionCount()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerSenderTransmissionCount();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetPointerSenderPacketSize(int packetSize)
{
	assert(g_receiver != NULL);
	g_receiver->setPointerSenderPacketSize(packetSize);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerSenderPacketSize()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerSenderPacketSize();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multicast Detector Settings
//
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverAddMulticastDetector(const char* address)
{
	assert(g_receiver != NULL);
	return g_receiver->addMulticastDetector(address);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverRemoveMulticastDetector(const char* address)
{
	assert(g_receiver != NULL);
	return g_receiver->removeMulticastDetector(address);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverWasDetectedMulticast(const char* address)
{
	assert(g_receiver != NULL);
	return g_receiver->wasDetectedMulticast(address);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Pointer Settings
//
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetPointerAutomaticSetupOn(bool flag)
{
	assert(g_receiver != NULL);
	g_receiver->setPointerAutomaticSetupOn(flag);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverGetPointerAutomaticSetupOn()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerAutomaticSetupOn();
}
//------------------------------------------------------------------------------
// Basic Settings
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetGeneralPointer(int pointer)
{
	assert(g_receiver != NULL);
	g_receiver->setGeneralPointer(pointer);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetGeneralPointer()
{
	assert(g_receiver != NULL);
	return g_receiver->getGeneralPointer();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetRaiseOn(bool flag)
{
	assert(g_receiver != NULL);
	g_receiver->setRaiseOn(flag);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverGetRaiseOn()
{
	assert(g_receiver != NULL);
	return g_receiver->getRaiseOn();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetRaisePointer(int pointer)
{
	assert(g_receiver != NULL);
	g_receiver->setRaisePointer(pointer);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetRaisePointer()
{
	assert(g_receiver != NULL);
	return g_receiver->getRaisePointer();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetDrawLocusOn(bool flag)
{
	assert(g_receiver != NULL);
	g_receiver->setDrawLocusOn(flag);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverGetDrawLocusOn()
{
	assert(g_receiver != NULL);
	return g_receiver->getDrawLocusOn();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetLocusPointer(int pointer)
{
	assert(g_receiver != NULL);
	g_receiver->setLocusPointer(pointer);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetLocusPointer()
{
	assert(g_receiver != NULL);
	return g_receiver->getLocusPointer();
}
//------------------------------------------------------------------------------
// Detail Settings
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetPointer(int entryNo, int pointer)
{
	assert(g_receiver != NULL);
	g_receiver->setPointer(entryNo, pointer);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointer(int entryNo)
{
	assert(g_receiver != NULL);
	return g_receiver->getPointer(entryNo);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetPointerPhrase(int entryNo, const char* phrase)
{
	assert(g_receiver != NULL);
	g_receiver->setPointerPhrase(entryNo, phrase);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC const char* MVSTD
fsReceiverGetPointerPhrase(int entryNo)
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerPhrase(entryNo).c_str();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetPointerColor(int entryNo, const unsigned char* color)
{
	assert(g_receiver != NULL);
	g_receiver->setPointerColor(entryNo, color);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverGetPointerColor(int entryNo, unsigned char* color)
{
	assert(g_receiver != NULL);
	g_receiver->getPointerColor(entryNo, color);
}
//------------------------------------------------------------------------------
// Other Settings
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetPointerCoordinate(int coordinate)
{
	assert(g_receiver != NULL);
	g_receiver->setPointerCoordinate(coordinate);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetPointerCoordinate()
{
	assert(g_receiver != NULL);
	return g_receiver->getPointerCoordinate();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverComputePointerRegion()
{
	assert(g_receiver != NULL);
	g_receiver->computePointerRegion();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Caption Settings
//
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverUpdateCaption()
{
	assert(g_receiver != NULL);
	g_receiver->updateCaption();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Video Window Settings
//
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetVideoWindowPolicy(int policy)
{
	assert(g_receiver != NULL);
	g_receiver->setVideoWindowPolicy(policy);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVideoWindowPolicy()
{
	assert(g_receiver != NULL);
	return g_receiver->getVideoWindowPolicy();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetKeepVideoAspect(bool flag)
{
	assert(g_receiver != NULL);
	g_receiver->setKeepVideoAspect(flag);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverGetKeepVideoAspect()
{
	assert(g_receiver != NULL);
	return g_receiver->getKeepVideoAspect();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetFitWindowToVideo(bool flag)
{
	assert(g_receiver != NULL);
	g_receiver->setFitWindowToVideo(flag);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverGetFitWindowToVideo()
{
	assert(g_receiver != NULL);
	return g_receiver->getFitWindowToVideo();
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverComputeVideoWindow()
{
	assert(g_receiver != NULL);
	g_receiver->computeVideoWindow();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Volume Settings
//
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverGetVolume(int* left, int* right)
{
	assert(g_receiver != NULL);
	g_receiver->getVolume(*left, *right);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverSetVolumeEvaluationTime(int evalTime)
{
	assert(g_receiver != NULL);
	g_receiver->setVolumeEvaluationTime(evalTime);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC int MVSTD
fsReceiverGetVolumeEvaluationTime()
{
	assert(g_receiver != NULL);
	return g_receiver->getVolumeEvaluationTime();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Other
//
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverQuit()
{
	assert(g_receiver != NULL);
	g_receiver->quit();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// IniFile
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverWriteIniFile(const char* filename)
{
	assert(g_receiver != NULL);
	return g_receiver->writeIniFile(filename);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverReadIniFile(const char* filename)
{
	assert(g_receiver != NULL);
	return g_receiver->readIniFile(filename);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Registry
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverReadRegistry(const char* section, const char* entry, const char* defaultValue, char* value)
{
	assert(g_receiver != NULL);
	mxString valueStr;
	g_receiver->readRegistry(section, entry, defaultValue, valueStr);
	MXSTR2STR(value, valueStr);
}
//------------------------------------------------------------------------------
FSRECEIVERREMOTEPROC void MVSTD
fsReceiverWriteRegistry(const char* section, const char* entry, const char* value)
{
	assert(g_receiver != NULL);
	g_receiver->writeRegistry(section, entry, value);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Callback
FSRECEIVERREMOTEPROC bool MVSTD
fsReceiverSetCallbackFunction(fsReceiverUICallbackFunction function, void* data)
{
    assert(g_receiver != NULL);
    return g_receiver->setUICallbackFunction(function, data);
}
//------------------------------------------------------------------------------
