/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsSender/fsSender.h>
#include <fs/fsCommon/fsFilterEnumerator.h>
#include <fs/fsMulticastTools/fsMulticastAddress.h>
#include <mx/mxCore/mxFile.h>
#include <mx/mxCore/mxIniFile.h>
#include <mx/mxCore/mxRegistry.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
#include <olectl.h>
#if (1100 > _MSC_VER)
#include <olectlid.h>
#endif
//------------------------------------------------------------------------------
#pragma comment(lib, "fsCommon.lib")
#pragma comment(lib, "fsStatus.lib")
#pragma comment(lib, "fsMulticastTools.lib")
#pragma comment(lib, "fsSenderRMI.lib")
#pragma comment(lib, "mxCore.lib")
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
BOOL APIENTRY 
DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
    return TRUE;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#define ClearGraph				\
	if (!_freezeFlag)			\
	{							\
		_graph->stop();			\
		clearGraph();			\
	}
#define BuildGraph											\
	if (!_freezeFlag)										\
	{														\
		if (buildGraph())									\
			_isStarted ? _graph->start() : _graph->stop();	\
		else												\
			_graph->stop();									\
	}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsSender::fsSender() :
mxObject()
{
	mxRegistry::setCompanyName(FS_REGISTRY_COMPANY_NAME);
	mxRegistry::setProductName(FS_REGISTRY_PRODUCT_NAME);

	_videoSources.setGrowSize(16);
	_audioSources.setGrowSize(16);
	_videoEncodersMain.setGrowSize(16);
	_videoEncodersSub.setGrowSize(16);
	_audioEncoders.setGrowSize(16);
}
//------------------------------------------------------------------------------
fsSender::~fsSender()
{
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Initialize / Shutdown
mxBool
fsSender::initialize(HWND windowHandle)
{
	// Initialize Variables
	_windowHandle			= windowHandle;
	_isValid				= false;
	_isStarted				= true;

	_freezeFlag				= false;
	_quitFlag				= false;

	_UICallbackFunction		= NULL;
	_UICallbackData			= NULL;

	_videoSource			= 0;
	_audioSource			= 0;
	_playVideoClipOn		= false;
	_playSoundFileOn		= false;
	_videoEncoder			= 0;
	_audioEncoder			= 0;
	_focusDisplay			= FSFD_NONE;
	_focusCoordinate		= FSC_VIDEOWINDOW;//FSC_DESKTOPWINDOW;
	_focusFrameVisible		= true;
	_directVideoModeOn		= false;
	_dvInterleavedModeOn	= false;
	_captionOn				= true;
	_caption				= "";
	_videoWindowPolicy		= FSVWP_ORIGINALSIZE;
	_keepVideoAspect		= true;
	_fitWindowToVideo		= false;

	// Initialize COM
	HRESULT hr = CoInitialize(NULL);
	if (FAILED(hr))
		return false;

	// Create Graph manager
	_graph = new fsGraphManager(graphEventProc, this, processIntermediateProc, this);
	if (!_graph->initialize(true))
	{
		_graph = NULL;
		return false;
	}

	// Open Status sender
	_statusSender = new fsStatusSender();
	_statusSender->setAddress(DEFAULT_ADDRESS);
	_statusSender->setTimeToLive(DEFAULT_TIMETOLIVE);
	_statusSender->setPort(PORT_STATUS);
	_statusSender->setMaxTransmissionRate(DEFAULT_MAXTRANSMISSIONRATE);
	_statusSender->setTransmissionCount(DEFAULT_TRANSMISSIONCOUNT);
	_statusSender->setPacketSize(DEFAULT_PACKETSIZE);
	_statusSender->open();

	// Create Multicast Detector
	_multicastDetectors = new fsMulticastDetectors();

	// Load EncoderPriorityList
	const mxString& modulePath = mxFile::getModulePath();
	_videoEncoderPriorities = new fsNamePriorityList();
	_audioEncoderPriorities = new fsNamePriorityList();
	_videoEncoderPriorities->load(modulePath + "\\videoEncoderPriorities.txt");
	_audioEncoderPriorities->load(modulePath + "\\audioEncoderPriorities.txt");

	// Create Filters
	if (!createFilters())
	{
		deleteFilters();

		_statusSender->close();
		_statusSender = NULL;

		_graph->shutdown();
		_graph = NULL;

		return false;
	}

	// Initialize Filters
	initializeFilters();

	// Initialize Status
	initializeStatus();

	// Read Registry
	readRegistry();

	// Start Graph
	_statusSender->send(_status);
	BuildGraph;

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSender::shutdown()
{
	if (_graph == NULL)
		return true;

	// Stop Graph
	_graph->stop();
	clearGraph();

	// Write Registry
	writeRegistry();

	// Delete Filters
	deleteFilters();

	// Load EncoderPriorityList
	const mxString& modulePath = mxFile::getModulePath();
	_videoEncoderPriorities->save(modulePath + "\\videoEncoderPriorities.txt");
	_audioEncoderPriorities->save(modulePath + "\\audioEncoderPriorities.txt");
	_videoEncoderPriorities = NULL;
	_audioEncoderPriorities = NULL;

	// Delete Multicast Detector
	_multicastDetectors	= NULL;

	// Delete Status sender
	_statusSender->close();
	_statusSender		= NULL;

	// Delete Graph manager
	_graph->shutdown();
	_graph				= NULL;

	// Shutdown COM
	CoUninitialize();

	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Implements fsSenderIf
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Streaming Control
//
//------------------------------------------------------------------------------
mxBool
fsSender::getValid() const
{
	return _isValid;
}
//------------------------------------------------------------------------------
void
fsSender::getErrorMessage(mxInt& id, mxInt& type, mxString& message) const
{
	id		= _errorMessage.Id;
	type	= _errorMessage.Type;
	message	= _errorMessage.Message;
}
//------------------------------------------------------------------------------
void
fsSender::setStart(mxBool flag)
{
	assert(_graph != NULL);

	if (_isStarted == flag)
		return ;

	_isStarted = flag;
	if (_isValid)
		_isStarted ? _graph->start() : _graph->stop();
	else
		_graph->stop();
}
//------------------------------------------------------------------------------
mxBool
fsSender::getStart() const
{
	return _isStarted;
}
//------------------------------------------------------------------------------
void
fsSender::setVideoTransmissionOn(mxBool flag)
{
	_videoSenderMain->setEnableTransmission(flag);
	_videoSenderSub->setEnableTransmission(flag);
}
//------------------------------------------------------------------------------
mxBool
fsSender::getVideoTransmissionOn() const
{
	return _videoSenderMain->getEnableTransmission();
}
//------------------------------------------------------------------------------
void
fsSender::setAudioTransmissionOn(mxBool flag)
{
	_audioSender->setEnableTransmission(flag);
}
//------------------------------------------------------------------------------
mxBool
fsSender::getAudioTransmissionOn() const
{
	return _audioSender->getEnableTransmission();
}
//------------------------------------------------------------------------------
void
fsSender::setFocusTransmissionOn(mxBool flag)
{
	_pointerSender->setEnableTransmission(flag);
}
//------------------------------------------------------------------------------
mxBool
fsSender::getFocusTransmissionOn() const
{
	return (_focusDisplay != FSFD_NONE && _pointerSender->getEnableTransmission());
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Multicast Settings
//
//------------------------------------------------------------------------------
void
fsSender::setAddress(const mxString& address)
{
	_status.MulticastAddress = address;
	_statusSender->send(_status);

	_videoSenderMain->setAddress(address);
	_videoSenderSub->setAddress(address);
	_audioSender->setAddress(address);
	_pointerSender->setAddress(address);

	_pointerReceiver1->setAddress(address);
	_pointerReceiver2->setAddress(address);

	_statusSender->setAddress(address);
}
//------------------------------------------------------------------------------
const mxString&
fsSender::getAddress() const
{
	return _videoSenderMain->getAddress();
}
//------------------------------------------------------------------------------
void
fsSender::setTimeToLive(mxULong ttl)
{
	_videoSenderMain->setTimeToLive(ttl);
	_videoSenderSub->setTimeToLive(ttl);
	_audioSender->setTimeToLive(ttl);
	_pointerSender->setTimeToLive(ttl);

	_pointerReceiver1->setTimeToLive(ttl);
	_pointerReceiver2->setTimeToLive(ttl);

	_statusSender->setTimeToLive(ttl);
}
//------------------------------------------------------------------------------
mxULong
fsSender::getTimeToLive() const
{
	return _videoSenderMain->getTimeToLive();
}
//------------------------------------------------------------------------------
void
fsSender::setTransmissionCount(mxUInt count)
{
	_videoSenderMain->setTransmissionCount(count);
	_videoSenderSub->setTransmissionCount(count);
	_audioSender->setTransmissionCount(count);
	_pointerSender->setTransmissionCount(count);

	_statusSender->setTransmissionCount(count);
}
//------------------------------------------------------------------------------
mxUInt
fsSender::getTransmissionCount() const
{
	return _videoSenderMain->getTransmissionCount();
}
//------------------------------------------------------------------------------
void
fsSender::setPacketSize(mxUInt size)
{
	_videoSenderMain->setMaximumPacketSize(size);
	_videoSenderSub->setMaximumPacketSize(size);
	_audioSender->setMaximumPacketSize(size);
	_pointerSender->setMaximumPacketSize(size);

	_statusSender->setPacketSize(size);
}
//------------------------------------------------------------------------------
mxUInt
fsSender::getPacketSize() const
{
	return _videoSenderMain->getMaximumPacketSize();
}
//------------------------------------------------------------------------------
// Create Address
mxString
fsSender::createIPv4Address()
{
	return fsMulticastAddress::createAddress(fsMulticastAddress::AF_IPv4);
}
//------------------------------------------------------------------------------
mxString
fsSender::createIPv6Address()
{
	return fsMulticastAddress::createAddress(fsMulticastAddress::AF_IPv6);
}
//------------------------------------------------------------------------------
// Video Sender (Main)
void
fsSender::setVideoSenderMainMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	_videoSenderMain->setMaximumTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoSenderMainMaxTransmissionRate() const
{
	return _videoSenderMain->getMaximumTransmissionRate();
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoSenderMainTransmissionRate() const
{
	return _videoSenderMain->getTransmissionRate();
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoSenderMainFrameRate() const
{
	return _videoSenderMain->getFrameRate();
}
//------------------------------------------------------------------------------
// Video Sender (Sub)
void
fsSender::setVideoSenderSubMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	_videoSenderSub->setMaximumTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoSenderSubMaxTransmissionRate() const
{
	return _videoSenderSub->getMaximumTransmissionRate();
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoSenderSubTransmissionRate() const
{
	return _videoSenderSub->getTransmissionRate();
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoSenderSubFrameRate() const
{
	return _videoSenderSub->getFrameRate();
}
//------------------------------------------------------------------------------
// Audio Sender
void
fsSender::setAudioSenderMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	_audioSender->setMaximumTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getAudioSenderMaxTransmissionRate() const
{
	return _audioSender->getMaximumTransmissionRate();
}
//------------------------------------------------------------------------------
mxInt
fsSender::getAudioSenderTransmissionRate() const
{
	return _audioSender->getTransmissionRate();
}
//------------------------------------------------------------------------------
mxInt
fsSender::getAudioSenderFrameRate() const
{
	return _audioSender->getFrameRate();
}
//------------------------------------------------------------------------------
// Pointer Sender
void
fsSender::setPointerSenderMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	_pointerSender->setMaximumTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getPointerSenderMaxTransmissionRate() const
{
	return _pointerSender->getMaximumTransmissionRate();
}
//------------------------------------------------------------------------------
mxInt
fsSender::getPointerSenderTransmissionRate() const
{
	return _pointerSender->getTransmissionRate();
}
//------------------------------------------------------------------------------
mxInt
fsSender::getPointerSenderFrameRate() const
{
	return _pointerSender->getFrameRate();
}
//------------------------------------------------------------------------------
// Status Sender
void
fsSender::setStatusSenderMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	_statusSender->setMaxTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getStatusSenderMaxTransmissionRate() const
{
	return _statusSender->getMaxTransmissionRate();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multicast Detector Settings
//
//------------------------------------------------------------------------------
mxBool
fsSender::addMulticastDetector(const mxString& address)
{
	const mxString& curAddress = _videoSenderMain->getAddress();
	if (curAddress == address)
		return false;

	return _multicastDetectors->add(address, PORT_STATUS);
}
//------------------------------------------------------------------------------
mxBool
fsSender::removeMulticastDetector(const mxString& address)
{
	return _multicastDetectors->remove(address, PORT_STATUS);
}
//------------------------------------------------------------------------------
mxBool
fsSender::wasDetectedMulticast(const mxString& address)
{
	const mxString& curAddress = _videoSenderMain->getAddress();
	if (curAddress == address)
		return false;

	return _multicastDetectors->detected(address, PORT_STATUS);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
const mxString INDEXOUTOFBOUNDS = "#Index out-of-bounds#";
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Source Settings
//
//------------------------------------------------------------------------------
// Video Source
mxInt
fsSender::getNumVideoSources() const
{
	return _videoSources.getSize();
}
//------------------------------------------------------------------------------
void
fsSender::setVideoSource(mxInt index)
{
	if (!_playVideoClipOn && _videoSource == index)
		return;

	ClearGraph;

	_playVideoClipOn = false;
	_playSoundFileOn = false;

	_videoSource = index;

	BuildGraph;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoSource() const
{
	return _videoSource;
}
//------------------------------------------------------------------------------
const mxString&
fsSender::getVideoSourceFriendlyName(mxInt index)
{
	if ((mxInt)_videoSources.getSize() <= index)
		return INDEXOUTOFBOUNDS;
	return _videoSources[index]->cast<fsSourceFilter>()->getFriendlyName();
}
//------------------------------------------------------------------------------
// Audio Source
mxInt
fsSender::getNumAudioSources() const
{
	return _audioSources.getSize();
}
//------------------------------------------------------------------------------
void
fsSender::setAudioSource(mxInt index)
{
	if (!_playVideoClipOn && _audioSource == index)
		return;

	ClearGraph;

	_playVideoClipOn = false;
	_playSoundFileOn = false;

	_audioSource = index;

	BuildGraph;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getAudioSource() const
{
	return _audioSource;
}
//------------------------------------------------------------------------------
const mxString&
fsSender::getAudioSourceFriendlyName(mxInt index)
{
	if ((mxInt)_audioSources.getSize() <= index)
		return INDEXOUTOFBOUNDS;
	return _audioSources[index]->cast<fsSourceFilter>()->getFriendlyName();
}
//------------------------------------------------------------------------------
// Video Source Property Page
mxInt
fsSender::getNumVideoSourcePropertyPages() const
{
	fsFilterPtr videoSource = _videoSources[_videoSource];
	assert(videoSource != NULL);

	return videoSource->cast<fsSourceFilter>()->getNumPropertyPages();
}
//------------------------------------------------------------------------------
const mxString&
fsSender::getVideoSourcePropertyPageName(mxInt index)
{
	fsFilterPtr videoSource = _videoSources[_videoSource];
	assert(videoSource != NULL);

	return videoSource->cast<fsSourceFilter>()->getPropertyPageName(index);
}
//------------------------------------------------------------------------------
void
fsSender::showVideoSourcePropertyPage(HWND windowHandle, mxInt index)
{
	fsFilterPtr videoSource = _videoSources[_videoSource];
	assert(videoSource != NULL);

	ClearGraph;

	if (videoSource->cast<fsSourceFilter>()->showPropertyPage(windowHandle, index))
	{
		if (_focusDisplay == FSFD_IMAGE_COMPOUND)
		{
			_status.ImageCompoundOutputVideoResolution
				= _formatTransformMain->getActualOutputVideoResolution();
			_statusSender->send(_status);
		}
	}

	BuildGraph;
}
//------------------------------------------------------------------------------
// Audio Source Property Page
mxInt
fsSender::getNumAudioSourcePropertyPages() const
{
	fsFilterPtr audioSource = _audioSources[_audioSource];
	assert(audioSource != NULL);

	return audioSource->cast<fsSourceFilter>()->getNumPropertyPages();
}
//------------------------------------------------------------------------------
const mxString&
fsSender::getAudioSourcePropertyPageName(mxInt index)
{
	fsFilterPtr audioSource = _audioSources[_audioSource];
	assert(audioSource != NULL);

	return audioSource->cast<fsSourceFilter>()->getPropertyPageName(index);
}
//------------------------------------------------------------------------------
void
fsSender::showAudioSourcePropertyPage(HWND windowHandle, mxInt index)
{
	fsFilterPtr audioSource = _audioSources[_audioSource];
	assert(audioSource != NULL);

	ClearGraph;
	audioSource->cast<fsSourceFilter>()->showPropertyPage(windowHandle, index);
	BuildGraph;
}
//------------------------------------------------------------------------------
const mxString NONAME = "";
//------------------------------------------------------------------------------
// Video Clip
mxBool
fsSender::isPlayVideoClip() const
{
	return _playVideoClipOn;
}
//------------------------------------------------------------------------------
mxBool
fsSender::playVideoClip(const mxString& filename)
{
	_playVideoClipOn = true;
	_playSoundFileOn = false;

	ClearGraph;
	_asyncReader = new fsAsyncReaderFilter(filename);
	_asyncReader->cast<fsAsyncReaderFilter>()->load(filename);
	BuildGraph;

	if (!_isValid)
	{
		ClearGraph;
		_playVideoClipOn = false;
		BuildGraph;
		return false;
	}

	return true;
}
//------------------------------------------------------------------------------
// Sound File
mxBool
fsSender::isPlaySoundFile() const
{
	return _audioMixer->isMixing();
}
//------------------------------------------------------------------------------
mxBool
fsSender::playSoundFile(const mxString& filename)
{
	if (_playVideoClipOn)
		return false;

	_playSoundFileOn = true;

	ClearGraph;
	_asyncReader = new fsAsyncReaderFilter(filename);
	_asyncReader->cast<fsAsyncReaderFilter>()->load(filename);
	BuildGraph;

	_playSoundFileOn = false;

	if (!_isValid)
	{
		ClearGraph;
		BuildGraph;
		return false;
	}

	return true;
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Encoder Settings
//
//------------------------------------------------------------------------------
// Video Encoder
mxInt
fsSender::getNumVideoEncoder() const
{
	return _videoEncodersMain.getSize();
}
//------------------------------------------------------------------------------
void
fsSender::setVideoEncoder(mxInt index)
{
	if (_videoEncoder == (mxUInt)index)
		return ;
#ifdef _DEBUG
if (index != 0)
mxNotify::post(mxNotify::INFO, "Video Encoder = %s", _videoEncodersMain[index]->cast<fsEncoderFilter>()->getFriendlyName());
#endif

	ClearGraph;
	_videoEncoder = index;
	BuildGraph;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoEncoder() const
{
	return _videoEncoder;
}
//------------------------------------------------------------------------------
const mxString NOT_USE = "<Not Use>";
//------------------------------------------------------------------------------
const mxString&
fsSender::getVideoEncoderFriendlyName(mxInt index) const
{
	if ((mxInt)_videoEncodersMain.getSize() <= index)
		return INDEXOUTOFBOUNDS;

	fsFilterPtr filter = _videoEncodersMain[index];
	if (filter != NULL)
		return filter->cast<fsEncoderFilter>()->getFriendlyName();
	else
		return NOT_USE;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoEncoderPriority(mxInt index) const
{
	if ((mxInt)_videoEncodersMain.getSize() <= index)
		return fsNamePriorityList::P_RECOMMENDED;

	fsFilterPtr filter = _videoEncodersMain[index];
	if (filter == NULL)
		return fsNamePriorityList::P_RECOMMENDED;
	return _videoEncoderPriorities->getPriority(filter->cast<fsEncoderFilter>()->getFriendlyName());
}
//------------------------------------------------------------------------------
// Audio Encoder
mxInt
fsSender::getNumAudioEncoder() const
{
	return _audioEncoders.getSize();
}
//------------------------------------------------------------------------------
void
fsSender::setAudioEncoder(mxInt index)
{
	if (_audioEncoder == (mxUInt)index)
		return ;
#ifdef _DEBUG
if (index != 0)
mxNotify::post(mxNotify::INFO, "Audio Encoder = %s", _audioEncoders[index]->cast<fsEncoderFilter>()->getFriendlyName());
#endif

	ClearGraph;
	_audioEncoder = index;
	BuildGraph;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getAudioEncoder() const
{
	return _audioEncoder;
}
//------------------------------------------------------------------------------
const mxString&
fsSender::getAudioEncoderFriendlyName(mxInt index) const
{
	if ((mxInt)_audioEncoders.getSize() <= index)
		return INDEXOUTOFBOUNDS;

	fsFilterPtr filter = _audioEncoders[index];
	if (filter != NULL)
		return filter->cast<fsEncoderFilter>()->getFriendlyName();
	else
		return NOT_USE;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getAudioEncoderPriority(mxInt index) const
{
	if ((mxInt)_audioEncoders.getSize() <= index)
		return fsNamePriorityList::P_RECOMMENDED;

	fsFilterPtr filter = _audioEncoders[index];
	if (filter == NULL)
		return fsNamePriorityList::P_RECOMMENDED;
	return _audioEncoderPriorities->getPriority(filter->cast<fsEncoderFilter>()->getFriendlyName());
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Focus Display Settings
//
//------------------------------------------------------------------------------
void
fsSender::setFocusDisplay(mxInt focusDisplay)
{
	if (_focusDisplay == focusDisplay)
		return ;

	ClearGraph;
	_focusDisplay = focusDisplay;
	BuildGraph;

	if (_focusDisplay == FSFD_IMAGE_COMPOUND)
	{
		_status.ImageCompoundOutputVideoResolution
				= _formatTransformMain->getActualOutputVideoResolution();
	}
	_status.FocusDisplay = _focusDisplay;
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getFocusDisplay() const
{
	return _focusDisplay;
}
//------------------------------------------------------------------------------
void
fsSender::setFocusFrameVisible(mxBool flag)
{
	if (_focusFrameVisible == flag)
		return ;

	ClearGraph;
	_focusFrameVisible = flag;
	BuildGraph;
}
//------------------------------------------------------------------------------
mxBool
fsSender::getFocusFrameVisible() const
{
	return _focusFrameVisible;
}
//------------------------------------------------------------------------------
// Video Resolution
void
fsSender::setVideoResolution(mxInt videoResolution)
{
	if (_formatTransformMain->getOutputVideoResolution() == videoResolution)
		return ;

	ClearGraph;
	_formatTransformMain->setOutputVideoResolution(videoResolution);
	BuildGraph;

	if (_focusDisplay == FSFD_IMAGE_COMPOUND)
	{
		_status.ImageCompoundOutputVideoResolution
			= _formatTransformMain->getActualOutputVideoResolution();
		_statusSender->send(_status);
	}
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoResolution() const
{
	return _formatTransformMain->getOutputVideoResolution();
}
//------------------------------------------------------------------------------
// Partial Zoom
void
fsSender::setPartialZoomSourceVideoResolution(mxInt videoResolution)
{
	if (_videoExtract1->getOutputVideoResolution() == videoResolution)
		return ;

	ClearGraph;

	_videoExtract1->setOutputVideoResolution(videoResolution);

	mxUInt width, height;
	FS_RESOLUTION2SIZE(videoResolution, width, height);
	_drawFrame1->setSize(width, height);

	BuildGraph;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getPartialZoomSourceVideoResolution() const
{
	return _videoExtract1->getOutputVideoResolution();
}
//------------------------------------------------------------------------------
void
fsSender::setPartialZoomOutputVideoResolution(mxInt videoResolution)
{
	_status.PartialZoomOutputVideoResolution = videoResolution;
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getPartialZoomOutputVideoResolution() const
{
	return _status.PartialZoomOutputVideoResolution;
}
//------------------------------------------------------------------------------
void
fsSender::setNonLinearZoomOutputVideoResolution(mxInt videoResolution)
{
	_status.NonLinearZoomOutputVideoResolution = videoResolution;
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getNonLinearZoomOutputVideoResolution() const
{
	return _status.NonLinearZoomOutputVideoResolution;
}
//------------------------------------------------------------------------------
void
fsSender::setNonLinearZoomMaskType(mxInt maskType)
{
	_drawFrame2->setFrameType(maskType);
	_status.NonLinearZoomMaskType = maskType;
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getNonLinearZoomMaskType() const
{
	return _status.NonLinearZoomMaskType;
}
//------------------------------------------------------------------------------
void
fsSender::setNonLinearZoomMaskSize(mxInt width, mxInt height)
{
	_drawFrame2->setSize(width, height);
	_status.NonLinearZoomMaskWidth = width;
	_status.NonLinearZoomMaskHeight = height;
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------
void
fsSender::getNonLinearZoomMaskSize(mxInt& width, mxInt& height) const
{
	width = _status.NonLinearZoomMaskWidth;
	height = _status.NonLinearZoomMaskHeight;
}
//------------------------------------------------------------------------------
void
fsSender::setImageCompoundBaseVideoResolution(mxInt videoResolution)
{
	if (_formatTransformSub->getOutputVideoResolution() == videoResolution)
		return ;

	ClearGraph;
	_formatTransformSub->setOutputVideoResolution(videoResolution);
	BuildGraph;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getImageCompoundBaseVideoResolution() const
{
	return _formatTransformSub->getOutputVideoResolution();
}
//------------------------------------------------------------------------------
void
fsSender::setImageCompoundAdditionalVideoResolution(mxInt videoResolution)
{
	if (_videoExtract2->getOutputVideoResolution() == videoResolution)
		return ;

	ClearGraph;

	_videoExtract2->setOutputVideoResolution(videoResolution);

	mxUInt width, height;
	FS_RESOLUTION2SIZE(videoResolution, width, height);
	_drawFrame3->setSize(width, height);

	BuildGraph;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getImageCompoundAdditionalVideoResolution() const
{
	return _videoExtract2->getOutputVideoResolution();
}
//------------------------------------------------------------------------------
void
fsSender::setImageCompoundMaskType(mxInt maskType)
{
	mxInt ft;
	switch(maskType)
	{
	case FSMT_RECTANGLE:
	case FSMT_CHECKERS:
		ft = IDrawFrame::DFFT_RECTANGLE;
		break;
	case FSMT_OVAL:
		ft = IDrawFrame::DFFT_OVAL;
		break;
	}
	_drawFrame3->setFrameType(ft);

	_status.ImageCompoundMaskType = maskType;
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getImageCompoundMaskType() const
{
	return 	_status.ImageCompoundMaskType;
}
//------------------------------------------------------------------------------
void
fsSender::setImageCompoundBlendingType(mxInt blendingType)
{
	_status.ImageCompoundBlendingType = blendingType;
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getImageCompoundBlendingType() const
{
	return _status.ImageCompoundBlendingType;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Other Settings
void
fsSender::setFocusCoordinate(mxInt coordinate)
{
	_focusCoordinate = coordinate;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getFocusCoordinate() const
{
	return _focusCoordinate;
}
//------------------------------------------------------------------------------
void
fsSender::computeFocusRegion()
{
	if (_focusDisplay == FSFD_NONE)
		return ;

	switch(_focusCoordinate)
	{
	case FSC_DESKTOPWINDOW:
	default:
		{
			_pointerPosition->setNormalizedOrigin(0.0f, 0.0f);
			_pointerPosition->setNormalizedSize(1.0f, 1.0f);
		}
		break;

	case FSC_VIDEOWINDOW:
		{
			mxInt x, y;
			mxUInt w, h;

			_graph->getVideoWindowPosition(x, y);
			_graph->getVideoWindowSize(w, h);

			RECT rect;
			GetWindowRect(GetDesktopWindow(), &rect);

			mxUInt dw, dh;
			dw = rect.right - rect.left;
			dh = rect.bottom - rect.top;

			_pointerPosition->setNormalizedOrigin(-mxFloat(x) / mxFloat(w), 
												  -mxFloat(y) / mxFloat(h));
			_pointerPosition->setNormalizedSize(mxFloat(dw) / mxFloat(w),
												mxFloat(dh) / mxFloat(h));
			//TODO

		}
		break;
	}
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Direct Video Mode Settings
//
//------------------------------------------------------------------------------
void
fsSender::setDirectVideoModeOn(mxBool flag)
{
	if (_directVideoModeOn == flag)
		return ;

	ClearGraph;
	_directVideoModeOn = flag;
	BuildGraph;

	_status.DirectVideoModeOn = _directVideoModeOn;
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------
mxBool
fsSender::getDirectVideoModeOn() const
{
	return _directVideoModeOn;
}
//------------------------------------------------------------------------------
void
fsSender::setDVInterleavedModeOn(mxBool flag)
{
	if (_dvInterleavedModeOn == flag)
		return ;

	ClearGraph;
	_dvInterleavedModeOn = flag;
	BuildGraph;

	_status.DvInterleavedModeOn = _dvInterleavedModeOn;
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------
mxBool
fsSender::getDVInterleavedModeOn() const
{
	return _dvInterleavedModeOn;
}
//------------------------------------------------------------------------------
mxBool
fsSender::isValidDVInterleavedMode() const
{
	assert(_videoSource < _videoSources.getSize());

	fsFilterPtr videoSource = _videoSources[_videoSource];
	assert(videoSource != NULL);

	if (videoSource->getClassType() != fsCaptureFilter::getStaticClassType())
		return false;

	return videoSource->cast<fsCaptureFilter>()->isDigitalVideo();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Caption Settings
//
//------------------------------------------------------------------------------
void
fsSender::setCaptionOn(mxBool flag)
{
	_captionOn = flag;
	setupCaption();
}
//------------------------------------------------------------------------------
mxBool
fsSender::getCaptionOn() const
{
	return _captionOn;
}
//------------------------------------------------------------------------------
void
fsSender::setCaption(const mxString& caption)
{
	_caption = caption;
	setupCaption();
}
//------------------------------------------------------------------------------
const mxString&
fsSender::getCaption() const
{
	return _caption;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Video Window Settings
//
//------------------------------------------------------------------------------
void
fsSender::setVideoWindowPolicy(mxInt policy)
{
	_videoWindowPolicy = policy;
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVideoWindowPolicy() const
{
	return _videoWindowPolicy;
}
//------------------------------------------------------------------------------
void
fsSender::setKeepVideoAspect(mxBool flag)
{
	_keepVideoAspect = flag;
}
//------------------------------------------------------------------------------
mxBool
fsSender::getKeepVideoAspect() const
{
	return _keepVideoAspect;
}
//------------------------------------------------------------------------------
void
fsSender::setFitWindowToVideo(mxBool flag)
{
	_fitWindowToVideo = flag;
}
//------------------------------------------------------------------------------
mxBool
fsSender::getFitWindowToVideo() const
{
	return _fitWindowToVideo;
}
//------------------------------------------------------------------------------
void
fsSender::computeVideoWindow()
{
	if (_windowHandle == NULL)
		return ;

	// Get ClientWindow size
	mxUInt cWidth, cHeight;
	RECT rect;
	GetWindowRect(_windowHandle, &rect);
	cWidth = (mxUInt)(rect.right - rect.left);
	cHeight = (mxUInt)(rect.bottom - rect.top);

	// Get Video Renderer Resolution
	mxUInt videoWidth, videoHeight;
	_graph->getVideoWindowResolution(videoWidth, videoHeight);


	mxInt x, y;
	mxUInt width, height;

	x = 0;
	y = 0;
	width = 1;
	height= 1;

	switch(_videoWindowPolicy)
	{
	case FSVWP_FITTOWINDOW:
		{
			if (_keepVideoAspect)
			{
				mxFloat videoAspect, cAspect;

				if (videoHeight == 0)
					break;
				videoAspect = mxFloat(videoWidth) / mxFloat(videoHeight);

				if (cHeight == 0)
					break;
				cAspect = mxFloat(cWidth) / mxFloat(cHeight);

				if (cAspect < videoAspect)
				{
					width = cWidth;
					height = (mxUInt)(mxFloat(width) / videoAspect);

					x = 0;
					y = (cHeight - height) / 2;
				}
				else
				{
					height = cHeight;
					width = (mxUInt)(mxFloat(height) * videoAspect);

					y = 0;
					x = (cWidth - width) / 2;
				}
			}
			else
			{
				x = 0;
				y = 0;

				width = cWidth;
				height = cHeight;
			}
		}
		break;
	case FSVWP_ORIGINALSIZE:
		{
			if (videoWidth == 0 || videoHeight == 0)
				break;

			if (_fitWindowToVideo)
			{
				const mxInt videoSize[2] = { videoWidth, videoHeight };

				if (_UICallbackFunction != NULL)
					_UICallbackFunction("ResizeForm", (void *)videoSize, _UICallbackData);

				GetWindowRect(_windowHandle, &rect);
				cWidth = (mxUInt)(rect.right - rect.left);
				cHeight = (mxUInt)(rect.bottom - rect.top);
			}

			x = mxInt((cWidth < videoWidth) ? 0 : ((cWidth - videoWidth) / 2));
			y = mxInt((cHeight < videoHeight) ? 0 : ((cHeight - videoHeight) / 2));

			width = videoWidth;
			height = videoHeight;
		}
		break;
	}

	_graph->setVideoWindowSize(width, height);
	_graph->setVideoWindowPosition(x, y);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Volume
//
//------------------------------------------------------------------------------
void
fsSender::getVolume(mxInt& left, mxInt& right) const
{
	_volumeIndicator->getVolume(left, right);
}
//------------------------------------------------------------------------------
void
fsSender::setVolumeEvaluationTime(mxInt evalTime)
{
	_volumeIndicator->setEvaluationTime((mxULong)evalTime);
}
//------------------------------------------------------------------------------
mxInt
fsSender::getVolumeEvaluationTime() const
{
	return (mxInt)_volumeIndicator->getEvaluationTime();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Other
//
//------------------------------------------------------------------------------
void
fsSender::quit()
{
	_quitFlag = true;
}
//------------------------------------------------------------------------------
void
fsSender::freeze(mxBool flag)
{
	if (_freezeFlag && flag || !_freezeFlag && !flag)
		return ;

	if (flag)
	{
		_statusSender->setTransmissionOn(false);
		ClearGraph;
		_freezeFlag = true;
	}
	else
	{
		_freezeFlag = false;
		BuildGraph;
		_statusSender->setTransmissionOn(true);
	}
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// Quit
mxBool
fsSender::checkQuit()
{
	return _quitFlag;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Status
void
fsSender::sendStatus()
{
	_statusSender->send(_status);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// IniFile
mxBool
fsSender::writeIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "a");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);

	mxString str;
	mxUInt index;

	//
	// Stream Control
	//
	str = getVideoTransmissionOn() ? "true" : "false";
	iniFile.writeString("Sender", "VideoTransmissionOn", str);

	str = getAudioTransmissionOn() ? "true" : "false";
	iniFile.writeString("Sender", "AudioTransmissionOn", str);

	str = getFocusTransmissionOn() ? "true" : "false";
	iniFile.writeString("Sender", "FocusTransmissionOn", str);


	//
	// Multicast Settings
	//
	iniFile.writeString("Sender", "MulticastAddress", getAddress());
	iniFile.writeUInt("Sender", "MulticastTimeToLive", (mxUInt)getTimeToLive());
	iniFile.writeUInt("Sender", "MulticastTransmissionCount", getTransmissionCount());
	iniFile.writeUInt("Sender", "MulticastPacketSize", getPacketSize());

	// Video Sender (Main)
	iniFile.writeUInt("Sender", "VideoSenderMainMaxTransmissionRate",
							getVideoSenderMainMaxTransmissionRate());

	// Video Sender (Sub)
	iniFile.writeUInt("Sender", "VideoSenderSubMaxTransmissionRate",
							getVideoSenderSubMaxTransmissionRate());

	// Audio Sender
	iniFile.writeUInt("Sender", "AudioSenderMaxTransmissionRate",
							getAudioSenderMaxTransmissionRate());

	// Pointer Sender / Receiver
	iniFile.writeUInt("Sender", "PointerSenderReceiverMaxTransmissionRate",
							getPointerSenderMaxTransmissionRate());

	// Status Sender
	iniFile.writeUInt("Sender", "StatusSenderMaxTransmissionRate",
							getStatusSenderMaxTransmissionRate());


	//
	// Source
	//
	// Video Source
	index = getVideoSource();
	iniFile.writeString("Sender", "VideoSource", getVideoSourceFriendlyName(index));

	// Audio Source
	index = getAudioSource();
	iniFile.writeString("Sender", "AudioSource", getAudioSourceFriendlyName(index));


	//
	// Encoder
	//
	// Video Encoder
	index = getVideoEncoder();
	iniFile.writeString("Sender", "VideoEncoder", getVideoEncoderFriendlyName(index));

	// Audio Encoder
	index = getAudioEncoder();
	iniFile.writeString("Sender", "AudioEncoder", getAudioEncoderFriendlyName(index));


	//
	// Focus Display
	//
	FS_FOCUSDISPLAY2STR(getFocusDisplay(), str);
	iniFile.writeString("Sender", "FocusDisplay", str);

	str = getFocusFrameVisible() ? "true" : "false";
	iniFile.writeString("Sender", "FocusDisplayFocusFrameVisible", str);

	// VideoResolution
	FS_RESOLUTION2STR(getVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayVideoResolution", str);

	// PartialZoom
	FS_RESOLUTION2STR(getPartialZoomSourceVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayPartialZoomSourceVideoResolution", str);

	FS_RESOLUTION2STR(getPartialZoomOutputVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayPartialZoomOutputVideoResolution", str);

	// NonLinearZoom Format
	FS_RESOLUTION2STR(getNonLinearZoomOutputVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayNonLinearZoomVideoResolution", str);

	// NonLinearZoom Mask Type
	FS_MASKTYPE2STR(getNonLinearZoomMaskType(), str);
	iniFile.writeString("Sender", "FocusDisplayNonLinearZoomMaskType", str);

	// NonLinearZoom Mask Size
	mxInt width, height;
	getNonLinearZoomMaskSize(width, height);
	iniFile.writeUInt("Sender", "FocusDisplayNonLinearZoomMaskWidth", (mxUInt)width);
	iniFile.writeUInt("Sender", "FocusDisplayNonLinearZoomMaskHeight", (mxUInt)height);

	// ImageCompound Base Video Format
	FS_RESOLUTION2STR(getImageCompoundBaseVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayImageCompoundBaseVideoResolution", str);

	// ImageCompound Additional Video Format
	FS_RESOLUTION2STR(getImageCompoundAdditionalVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayImageCompoundAdditionalVideoResolution", str);

	// ImageCompound Mask Type
	FS_MASKTYPE2STR(getImageCompoundMaskType(), str);
	iniFile.writeString("Sender", "FocusDisplayImageCompoundMaskType",str);

	// ImageCompound Blending Type
	FS_BLENDINGTYPE2STR(getImageCompoundBlendingType(), str);
	iniFile.writeString("Sender", "FocusDisplayImageCompoundBlendingType", str);

	// FocusCoordinate
	FS_COORDINATE2STR(getFocusCoordinate(), str);
	iniFile.writeString("Sender", "FocusCoordinate", str);


	//
	// Direct Video Mode
	//
	iniFile.writeString("Sender", "DirectVideoMode",
										getDirectVideoModeOn() ? "true" : "false");

	iniFile.writeString("Sender", "DVInterleavedMode",
										getDVInterleavedModeOn() ? "true" : "false");


	//
	// Caption Settings
	//
	iniFile.writeString("Sender", "CaptionOn", getCaptionOn() ? "true" : "false");
	iniFile.writeString("Sender", "Caption", getCaption());


	//
	// Still Picture
	//
	str = _stillPicture->cast<fsStillPictureFilter>()->getFilename();
	iniFile.writeString("Sender", "StillPictureFilePath", str);


	//
	// Digital Clock
	//
	fsDigitalClockFilter* digitalClock = _digitalClock->cast<fsDigitalClockFilter>();
	str = digitalClock->getHeaderString();
	iniFile.writeString("Sender", "DigitalClockHeaderString", str);

	str = digitalClock->getFooterString();
	iniFile.writeString("Sender", "DigitalClockFooterString", str);

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSender::readIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "r");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);

	mxString def, str;
	mxUInt num;
	mxInt index;

	// Stop Graph and StatusSender
	freeze(true);

	//
	// Stream Control
	//
	def = getVideoTransmissionOn() ? "true" : "false";
	str = iniFile.readString("Sender", "VideoTransmissionOn", def);
	str.toLower();
	setVideoTransmissionOn((str == "true"));

	def = getAudioTransmissionOn() ? "true" : "false";
	str = iniFile.readString("Sender", "AudioTransmissionOn", str);
	str.toLower();
	setAudioTransmissionOn((str == "true"));

	def = getFocusTransmissionOn() ? "true" : "false";
	str = iniFile.readString("Sender", "FocusTransmissionOn", str);
	str.toLower();
	setFocusTransmissionOn((str == "true"));


	//
	// Multicast Settings
	//
	mxString address = iniFile.readString("Sender", "MulticastAddress", getAddress());
	mxULong ttl = iniFile.readUInt("Sender", "MulticastTimeToLive", getTimeToLive());
	mxUInt count = iniFile.readUInt("Sender", "MulticastTransmissionCount", getTransmissionCount());
	mxUInt size = iniFile.readUInt("Sender", "MulticastPacketSize", getPacketSize());
	setAddress(address);
	setTimeToLive(ttl);
	setTransmissionCount(count);
	setPacketSize(size);

	// Video Sender (Main)
	mxUInt val;
	val = iniFile.readUInt("Sender", "VideoSenderMainMaxTransmissionRate",
						getVideoSenderMainMaxTransmissionRate());
	setVideoSenderMainMaxTransmissionRate(val);

	// Video Sender (Sub)
	val = iniFile.readUInt("Sender", "VideoSenderSubMaxTransmissionRate",
						getVideoSenderSubMaxTransmissionRate());
	setVideoSenderSubMaxTransmissionRate(val);

	// Audio Sender
	val = iniFile.readUInt("Sender", "AudioSenderMaxTransmissionRate",
						getAudioSenderMaxTransmissionRate());
	setAudioSenderMaxTransmissionRate(val);

	// Pointer Sender / Receiver
	val = iniFile.readUInt("Sender", "PointerSenderReceiverMaxTransmissionRate",
						getPointerSenderMaxTransmissionRate());
	setPointerSenderMaxTransmissionRate(val);

	// Status Sender
	val = iniFile.readUInt("Sender", "StatusSenderMaxTransmissionRate",
						getStatusSenderMaxTransmissionRate());
	setStatusSenderMaxTransmissionRate(val);


	//
	// Source
	//
	// Video Source
	index = getVideoSource();
	num = getNumVideoSources();
	assert(index < (mxInt)num);
	def = getVideoSourceFriendlyName(index);
	str = iniFile.readString("Sender", "VideoSource", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getVideoSourceFriendlyName(i))
		{
			setVideoSource(i);
			break;
		}
	}

	// Audio Source
	index = getAudioSource();
	num = getNumAudioSources();
	assert(index < (mxInt)num);
	def = getAudioSourceFriendlyName(index);
	str = iniFile.readString("Sender", "AudioSource", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getAudioSourceFriendlyName(i))
		{
			setAudioSource(i);
			break;
		}
	}


	//
	// Encoder
	//
	// Video Encoder
	index = getVideoEncoder();
	num = getNumVideoEncoder();
	assert(index < (mxInt)num);
	def = getVideoEncoderFriendlyName(index);
	str = iniFile.readString("Sender", "VideoEncoder", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getVideoEncoderFriendlyName(i))
		{
			setVideoEncoder(i);
			break;
		}
	}

	// Audio Encoder
	index = getAudioEncoder();
	num = getNumAudioEncoder();
	assert(index < (mxInt)num);
	def = getAudioEncoderFriendlyName(index);
	str = iniFile.readString("Sender", "AudioEncoder", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getAudioEncoderFriendlyName(i))
		{
			setAudioEncoder(i);
			break;
		}
	}


	//
	// Focus Display
	//
	FS_FOCUSDISPLAY2STR(getFocusDisplay(), def);
	str = iniFile.readString("Sender", "FocusDisplay", def);
	FS_STR2FOCUSDISPLAY(str, index);
	setFocusDisplay(index);

	def = getFocusFrameVisible() ? "true" : "false";
	str = iniFile.readString("Sender", "FocusDisplayFocusFrameVisible", def);
	str.toLower();
	setFocusFrameVisible((str == "true"));

	// VideoResolution
	FS_RESOLUTION2STR(getVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setVideoResolution(index);

	// PartialZoom
	FS_RESOLUTION2STR(getPartialZoomSourceVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayPartialZoomSourceVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setPartialZoomSourceVideoResolution(index);

	FS_RESOLUTION2STR(getPartialZoomOutputVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayPartialZoomOutputVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setPartialZoomOutputVideoResolution(index);

	// NonLinearZoom Format
	FS_RESOLUTION2STR(getNonLinearZoomOutputVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayNonLinearZoomVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setNonLinearZoomOutputVideoResolution(index);

	// NonLinearZoom Mask Type
	FS_MASKTYPE2STR(getNonLinearZoomMaskType(), def);
	str = iniFile.readString("Sender", "FocusDisplayNonLinearZoomMaskType", def);
	FS_STR2MASKTYPE(str, index);
	setNonLinearZoomMaskType(index);

	// NonLinearZoom Mask Size
	mxInt width, height;
	getNonLinearZoomMaskSize(width, height);
	width = (mxInt)iniFile.readUInt("Sender", "FocusDisplayNonLinearZoomMaskWidth", (mxUInt)width);
	height = (mxInt)iniFile.readUInt("Sender", "FocusDisplayNonLinearZoomMaskHeight", (mxUInt)height);
	setNonLinearZoomMaskSize(width, height);

	// ImageCompound Base Video Format
	FS_RESOLUTION2STR(getImageCompoundBaseVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayImageCompoundBaseVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setImageCompoundBaseVideoResolution(index);

	// ImageCompound Additional Video Format
	FS_RESOLUTION2STR(getImageCompoundAdditionalVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayImageCompoundAdditionalVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setImageCompoundAdditionalVideoResolution(index);

	// ImageCompound Mask Type
	FS_MASKTYPE2STR(getImageCompoundMaskType(), def);
	str = iniFile.readString("Sender", "FocusDisplayImageCompoundMaskType", def);
	FS_STR2MASKTYPE(str, index);
	setImageCompoundMaskType(index);

	// ImageCompound Blending Type
	FS_BLENDINGTYPE2STR(getImageCompoundBlendingType(), def);
	str = iniFile.readString("Sender", "FocusDisplayImageCompoundBlendingType", def);
	FS_STR2BLENDINGTYPE(str, index);
	setImageCompoundBlendingType(index);

	// FocusCoordinate
	FS_COORDINATE2STR(getFocusCoordinate(), def);
	str = iniFile.readString("Sender", "FocusCoordinate", def);
	FS_STR2COORDINATE(str, index);
	setFocusCoordinate(index);

	//
	// Direct Video Mode
	//
	def = getDirectVideoModeOn() ? "true" : "false";
	str = iniFile.readString("Sender", "DirectVideoMode", def);
	str.toLower();
	setDirectVideoModeOn((str == "true"));

	def = getDVInterleavedModeOn() ? "true" : "false";
	str = iniFile.readString("Sender", "DVInterleavedMode", def);
	str.toLower();
	setDVInterleavedModeOn((str == "true"));


	//
	// Caption Settings
	//
	def = getCaptionOn() ? "true" : "false";
	str = iniFile.readString("Sender", "CaptionOn", def);
	str.toLower();
	setCaptionOn((str == "true"));

	def = getCaption();
	str = iniFile.readString("Sender", "Caption", def);
	setCaption(str);


	//
	// Still Picture
	//
	def = _stillPicture->cast<fsStillPictureFilter>()->getFilename();
	str = iniFile.readString("Sender", "StillPictureFilePath", def);
	_stillPicture->cast<fsStillPictureFilter>()->setFilename(str);


	//
	// Digital Clock
	//
	fsDigitalClockFilter* digitalClock = _digitalClock->cast<fsDigitalClockFilter>();
	def = digitalClock->getHeaderString();
	str = iniFile.readString("Sender", "DigitalClockHeaderString", def);
	digitalClock->setHeaderString(str);

	def = digitalClock->getFooterString();
	str = iniFile.readString("Sender", "DigitalClockFooterString", def);
	digitalClock->setFooterString(str);


	// Start Graph and StatusSender
	freeze(false);


	if (getFocusDisplay() == FSFD_IMAGE_COMPOUND)
		_status.ImageCompoundOutputVideoResolution = 
				_formatTransformMain->getActualOutputVideoResolution();
	_statusSender->send(_status);


	return true;
}
//------------------------------------------------------------------------------
// Registry
void
fsSender::writeRegistry()
{
	//// CURRENT_USER
	mxRegistry	registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);

	mxString str;
	mxUInt num;
	mxUInt index;

	//
	// Multicast Settings
	//
	// Video Sender (Main)
	registry.writeUInt("Sender", "VideoSenderMainMaxTransmissionRate",
							getVideoSenderMainMaxTransmissionRate());

	// Video Sender (Sub)
	registry.writeUInt("Sender", "VideoSenderSubMaxTransmissionRate",
							getVideoSenderSubMaxTransmissionRate());

	// Audio Sender
	registry.writeUInt("Sender", "AudioSenderMaxTransmissionRate",
							getAudioSenderMaxTransmissionRate());

	// Pointer Sender / Receiver
	registry.writeUInt("Sender", "PointerSenderReceiverMaxTransmissionRate",
							getPointerSenderMaxTransmissionRate());

	// Status Sender
	registry.writeUInt("Sender", "StatusSenderMaxTransmissionRate",
							getStatusSenderMaxTransmissionRate());


	//
	// Source
	//
	// Video Source
	index = getVideoSource();
	num = getNumVideoSources();
	assert(index < num);
	registry.writeString("Sender", "VideoSource", getVideoSourceFriendlyName(index));

	// Audio Source
	index = getAudioSource();
	num = getNumAudioSources();
	assert(index < num);
	registry.writeString("Sender", "AudioSource", getAudioSourceFriendlyName(index));


	//
	// Encoder
	//
	// Video Encoder
	index = getVideoEncoder();
	num = getNumVideoEncoder();
	assert(index < num);
	registry.writeString("Sender", "VideoEncoder", getVideoEncoderFriendlyName(index));

	// Audio Encoder
	index = getAudioEncoder();
	num = getNumAudioEncoder();
	assert(index < num);
	registry.writeString("Sender", "AudioEncoder", getAudioEncoderFriendlyName(index));


	//
	// Video Window Settings
	//
	// Video Window Policy
	FS_VIDEOWINDOWPOLICY2STR(getVideoWindowPolicy(), str);
	registry.writeString("Sender", "VideoWindowPolicy", str);

	// Keep Video Aspect
	str = (getKeepVideoAspect() ? "True" : "False");
	registry.writeString("Sender", "KeepVideoAspect", str);
	
	// Fit Window to Video
	str = (getFitWindowToVideo() ? "True" : "False");
	registry.writeString("Sender", "FitWindowToVideo", str);


	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsSender::readRegistry()
{
	//// CURRENT_USER
	mxRegistry	registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);

	mxString def, str;
	mxUInt num;
	mxInt index;


	// Stop Graph and SenderStatus
	_statusSender->setTransmissionOn(false);
	_freezeFlag = true;


	//
	// Multicast Settings
	//
	// Video Sender (Main)
	mxUInt val;
	val = registry.readUInt("Sender", "VideoSenderMainMaxTransmissionRate",
						getVideoSenderMainMaxTransmissionRate());
	setVideoSenderMainMaxTransmissionRate(val);

	// Video Sender (Sub)
	val = registry.readUInt("Sender", "VideoSenderSubMaxTransmissionRate",
						getVideoSenderSubMaxTransmissionRate());
	setVideoSenderSubMaxTransmissionRate(val);

	// Audio Sender
	val = registry.readUInt("Sender", "AudioSenderMaxTransmissionRate",
						getAudioSenderMaxTransmissionRate());
	setAudioSenderMaxTransmissionRate(val);

	// Pointer Sender / Receiver
	val = registry.readUInt("Sender", "PointerSenderReceiverMaxTransmissionRate",
						getPointerSenderMaxTransmissionRate());
	setPointerSenderMaxTransmissionRate(val);

	// Status Sender
	val = registry.readUInt("Sender", "StatusSenderMaxTransmissionRate",
						getStatusSenderMaxTransmissionRate());
	setStatusSenderMaxTransmissionRate(val);


	//
	// Source
	//
	// Video Source
	index = getVideoSource();
	num = getNumVideoSources();
	assert(index < (mxInt)num);
	def = getVideoSourceFriendlyName(index);
	str = registry.readString("Sender", "VideoSource", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getVideoSourceFriendlyName(i))
		{
			setVideoSource(i);
			break;
		}
	}

	// Audio Source
	index = getAudioSource();
	num = getNumAudioSources();
	assert(index < (mxInt)num);
	def = getAudioSourceFriendlyName(index);
	str = registry.readString("Sender", "AudioSource", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getAudioSourceFriendlyName(i))
		{
			setAudioSource(i);
			break;
		}
	}
	

	//
	// Encoder
	//
	// Video Encoder
	index = getVideoEncoder();
	num = getNumVideoEncoder();
	assert(index < (mxInt)num);
	def = getVideoEncoderFriendlyName(index);
	str = registry.readString("Sender", "VideoEncoder", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getVideoEncoderFriendlyName(i))
		{
			setVideoEncoder(i);
			break;
		}
	}

	// Audio Encoder
	index = getAudioEncoder();
	num = getNumAudioEncoder();
	assert(index < (mxInt)num);
	def = getAudioEncoderFriendlyName(index);
	str = registry.readString("Sender", "AudioEncoder", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getAudioEncoderFriendlyName(i))
		{
			setAudioEncoder(i);
			break;
		}
	}


	//
	// Video Window Settings
	//
	// Video Window Policy
	FS_VIDEOWINDOWPOLICY2STR(getVideoWindowPolicy(), def);
	str = registry.readString("Sender", "VideoWindowPolicy", def);
	FS_STR2VIDEOWINDOWPOLICY(str, index);
	setVideoWindowPolicy(index);

	// Keep Video Aspect
	def = (getKeepVideoAspect() ? "True" : "False");
	str = registry.readString("Sender", "KeepVideoAspect", def);
	str.toLower();
	setKeepVideoAspect((str == "true"));
	
	// Fit Window to Video
	def = (getFitWindowToVideo() ? "True" : "False");
	str = registry.readString("Sender", "FitWindowToVideo", def);
	str.toLower();
	setFitWindowToVideo((str == "true"));


	// Start Graph and StatusSender
	_freezeFlag = false;
	_statusSender->setTransmissionOn(true);


	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsSender::writeRegistry(const mxString& section,
						const mxString& entry,
						const mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	registry.writeString(section, entry, value);
	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsSender::readRegistry(const mxString& section,
					   const mxString& entry,
					   const mxString& defaultValue,
					   mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	value = registry.readString(section, entry, defaultValue);
	registry.closeKey();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Callback
mxBool
fsSender::setUICallbackFunction(fsSenderUICallbackFunction func, void* data)
{
	_UICallbackFunction = func;
	_UICallbackData = data;
	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Graph Event
void
fsSender::graphEvent(mxInt eventCode, void* param1, void* param2)
{
	switch(eventCode)
	{
	case fsGraphManager::ET_STREAMCOMPLETED:
#ifdef _DEBUG
		mxNotify::post(mxNotify::INFO, "ET_STREAMCOMPLETED");
#endif

		_playVideoClipOn = false;
		ClearGraph;
		BuildGraph;

		computeVideoWindow();

		break;
	case fsGraphManager::ET_MEDIATYPECHANGED:
#ifdef _DEBUG
		mxNotify::post(mxNotify::INFO, "ET_MEDIATYPECHANGED");
#endif

		break;
	}
}
//------------------------------------------------------------------------------
void
fsSender::graphEventProc(mxInt eventCode, void *param1, void* param2, void *data)
{
	fsSender* sender = (fsSender *)data;
	sender->graphEvent(eventCode, param1, param2);
}
//------------------------------------------------------------------------------
// Process Intermediate
// ̊֐́AconnectFilterIndirect()ĂяoƂɁA
// ԃtB^ꂽꍇɌĂяo܂B
// ԃtB^ɑ΂A炩̃p[^ݒsꍇɎgp܂B
void
fsSender::processIntermediate(const fsFilterPtr* filters, mxUInt numFilters)
{
	for(mxUInt i = 0; i < numFilters; i++)
	{
		IBaseFilter* filterObj = filters[i]->getBaseFilter();

		// DV Decoder Resolution Setting
		IIPDVDec* dvDecoder = NULL;
		HRESULT hr = filterObj->QueryInterface(IID_IIPDVDec, (void **)&dvDecoder);
		if (hr == S_OK)
		{
			dvDecoder->put_IPDisplay(DVDECODERRESOLUTION_720x480);
			dvDecoder->Release();
			dvDecoder = NULL;
		}
	}
}
//------------------------------------------------------------------------------
void
fsSender::processIntermediateProc(const fsFilterPtr* filters, mxUInt numFilters, void* data)
{
	fsSender* sender = (fsSender *)data;
	sender->processIntermediate(filters, numFilters);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsSender::createFilters()
{
	fsFilterPtr filter = NULL;
	fsFilterEnumeratorPtr enumerator;

	//
	// Create Filters
	//

	// Source Filters
	enumerator = new fsFilterEnumerator(fsFilterEnumerator::C_VIDEO_CAPTURE);
	while(enumerator->nextFilter(filter))
		_videoSources.add(filter);

	enumerator = new fsFilterEnumerator(fsFilterEnumerator::C_AUDIO_CAPTURE);
	while(enumerator->nextFilter(filter))
		_audioSources.add(filter);

	// Encoder Filters
	_videoEncodersMain.add(NULL);
	enumerator = new fsFilterEnumerator(fsFilterEnumerator::C_VIDEO_ENCODER);
	while(enumerator->nextFilter(filter))
		_videoEncodersMain.add(filter);

	_videoEncodersSub.add(NULL);
	enumerator = new fsFilterEnumerator(fsFilterEnumerator::C_VIDEO_ENCODER);
	while(enumerator->nextFilter(filter))
		_videoEncodersSub.add(filter);

	_audioEncoders.add(NULL);
	enumerator = new fsFilterEnumerator(fsFilterEnumerator::C_AUDIO_ENCODER);
	while(enumerator->nextFilter(filter))
		_audioEncoders.add(filter);

	// Renderer Filter
	_videoRender			= new fsVideoRenderFilter("VideoRender");

	_audioRender = NULL;
	enumerator = new fsFilterEnumerator(fsFilterEnumerator::C_AUDIO_RENDERER);
	fsFilterPtr tmpAudioRender = NULL;
	while(enumerator->nextFilter(filter))
	{
		tmpAudioRender = filter;

		const mxString& friendlyName = filter->cast<fsAudioRenderFilter>()->getFriendlyName();
		if (friendlyName.subString(0, 7) != "Default" &&
			friendlyName.subString(0, 6) != "Direct")
		{
			_audioRender = filter;
			break;
		}
	}
	if (_audioRender == NULL)
		_audioRender = tmpAudioRender;

	// Multicast Sender Filters
	_videoSenderMain		= new fsMulticastSenderFilter("VideoSender(Main)");
	_videoSenderSub			= new fsMulticastSenderFilter("VideoSender(Sub)");
	_audioSender			= new fsMulticastSenderFilter("AudioSender");
	_pointerSender			= new fsMulticastSenderFilter("PointerSender");

	// Multicast Receiver Filters
	_pointerReceiver1		= new fsMulticastReceiverFilter("PointerReceiver1");
	_pointerReceiver2		= new fsMulticastReceiverFilter("PointerReceiver2");

	// Other Filters
	_formatTransformMain	= new fsFormatTransFilter("FormatTransform(Main)");
	_formatTransformSub		= new fsFormatTransFilter("FormatTransform(Sub)");
	_videoExtract1			= new fsVideoExtractFilter("VideoExtract1");
	_videoExtract2			= new fsVideoExtractFilter("VideoExtract2");
	_pointerPosition		= new fsPointerPositionFilter("PointerPosition");
	_drawFrame1				= new fsDrawFrameFilter("DrawFrame1");
	_drawFrame2				= new fsDrawFrameFilter("DrawFrame2");
	_drawFrame3				= new fsDrawFrameFilter("DrawFrame3");
	_stillPicture			= new fsStillPictureFilter("StillPictureFilter");
	_digitalClock			= new fsDigitalClockFilter("DigitalClockFilter");
	_dvSplitter				= new fsDVSplitterFilter("DVSplitter");
	_volumeIndicator		= new fsVolumeIndicatorFilter("VolumeIndicator");
	_audioMixer				= new fsAudioMixerFilter("AudioMixer");

	// Filters generated each time
	_asyncReader			= new fsAsyncReaderFilter("AsyncReader");
	_smartTee				= new fsSmartTeeFilter("SmartTee");
	_videoInfPinTee			= new fsInfPinTeeFilter("VideoInfinitePinTee");
	_audioInfPinTee			= new fsInfPinTeeFilter("AudioInfinitePinTee");


	//
	// Additional Video Souce
	//
	if (_stillPicture->isValid())
		_videoSources.add(_stillPicture);
	if (_digitalClock->isValid())
		_videoSources.add(_digitalClock);
	
	//
	// Check Filters
	//
	// ȉ̃tB^̐Ɏsꍇ́AAvP[V͑sB
	//  AudioSources
	//  AudioRender
	//  DVSplitter
	//  AsyncReader

	mxBool ret = true;
	mxString message("The following DirectShow Filter was not found.\n\n");

	if (_videoSources.getSize() == 0)
	{
		message += "EVideo Source (AnalogCapture, StillPicture etc..)\n";
		ret &= false;
	}

	if (_audioSources.getSize() == 0)
	{
		message += "EAudio Source (SoundDevice etc..)\n";
	}

	if (!_videoRender->isValid())
	{
		message += "EVideo Renderer\n";
		ret &= false;
	}
	if (!_audioRender->isValid())
	{
		message += "EAudio Renderer\n";
		_audioRender = NULL;
	}
	if (!_videoSenderMain->isValid() ||
		!_videoSenderSub->isValid() ||
		!_audioSender->isValid() ||
		!_pointerSender->isValid())
	{
		message += "EMulticast Sender\n";
		ret &= false;
	}
	if (!_pointerReceiver1->isValid() ||
		!_pointerReceiver2->isValid())
	{
		message += "EMulticast Receiver\n";
		ret &= false;
	}
	if (!_formatTransformMain->isValid() ||
		!_formatTransformSub->isValid())
	{
		message += "EFormat Transform\n";
		ret &= false;
	}
	if (!_videoExtract1->isValid() ||
		!_videoExtract2->isValid())
	{
		message += "EImage Extract\n";
		ret &= false;
	}
	if (!_pointerPosition->isValid())
	{
		message += "EPointer Position\n";
		ret &= false;
	}
	if (!_drawFrame1->isValid() ||
		!_drawFrame2->isValid() ||
		!_drawFrame3->isValid())
	{
		message += "EDraw Frame\n";
		ret &= false;
	}
	if (!_dvSplitter->isValid())
	{
		message += "EDV-Splitter\n";
		_dvSplitter = NULL;
	}
	if (!_volumeIndicator->isValid())
	{
		message += "EVolume Indicator\n";
		ret &= false;
	}
	if (!_audioMixer->isValid())
	{
		message += "EAudio Mixer\n";
		ret &= false;
	}
	if (!_asyncReader->isValid())
	{
		message += "EAsync Reader\n";
		_asyncReader = NULL;
	}
	if (!_smartTee->isValid())
	{
		message += "ESmart Tee\n";
		ret &= false;
	}
	if (!_videoInfPinTee->isValid() ||
		!_audioInfPinTee->isValid())
	{
		message += "EInfinite Pin Tee\n";
		ret &= false;
	}

	if (!ret)
		showMessageBox((LPTSTR)message.c_str());


	//// This is Temporary
	for(mxUInt i = 0; i < _audioSources.getSize(); i++)
	{
		const mxFloat FRAMERATE		= 0.033f;

		const mxInt CHANNELS		= 2;		// Stereo or not
		const mxInt BYTESPERSAMPLE	= 2;		// 16 bit
		const mxInt FREQUENCY		= 44100;

		const mxLong BYTESPERSECOND = (mxLong)(CHANNELS * BYTESPERSAMPLE * FREQUENCY);

		// Set to 30ms worth of data    
		const mxLong BUFFERSIZE = (mxLong)((mxFloat)BYTESPERSECOND * FRAMERATE);

		// Set the buffer size based on selected settings
		mxLong numBuf = 6;
		mxLong bufSize = BUFFERSIZE;
		mxLong bufAlign = BYTESPERSAMPLE * CHANNELS;
		mxLong bufPrefix = 0;

		if (_audioSources[i]->getClassType() == fsCaptureFilter::getStaticClassType() &&
			0 < _audioSources[i]->getNumOutputPins())
			_audioSources[i]->cast<fsCaptureFilter>()->setAllocatorProperties(0, 
										numBuf, bufSize, bufAlign, bufPrefix);
	}
	//// This is Temporary

	return ret;
}
//------------------------------------------------------------------------------
void
fsSender::deleteFilters()
{
	_videoSources.clear();
	_audioSources.clear();
	_videoEncodersMain.clear();
	_videoEncodersSub.clear();
	_audioEncoders.clear();
	_videoRender			= NULL;
	_audioRender			= NULL;
	_videoSenderMain		= NULL;
	_videoSenderSub			= NULL;
	_audioSender			= NULL;
	_pointerSender			= NULL;
	_pointerReceiver1		= NULL;
	_pointerReceiver2		= NULL;
	_formatTransformMain	= NULL;
	_formatTransformSub		= NULL;
	_videoExtract1			= NULL;
	_videoExtract2			= NULL;
	_pointerPosition		= NULL;
	_drawFrame1				= NULL;
	_drawFrame2				= NULL;
	_drawFrame3				= NULL;
	_stillPicture			= NULL;
	_digitalClock			= NULL;
	_dvSplitter				= NULL;
	_volumeIndicator		= NULL;
	_audioMixer				= NULL;

	_asyncReader			= NULL;
	_smartTee				= NULL;
	_videoInfPinTee			= NULL;
	_audioInfPinTee			= NULL;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsSender::initializeFilters()
{
	// VideoSender (Main)
	_videoSenderMain->setAddress(DEFAULT_ADDRESS);
	_videoSenderMain->setPort(PORT_VIDEOMAIN);
	_videoSenderMain->setTimeToLive(DEFAULT_TIMETOLIVE);
	_videoSenderMain->setMaximumTransmissionRate(DEFAULT_MAXTRANSMISSIONRATE);
	_videoSenderMain->setTransmissionCount(DEFAULT_TRANSMISSIONCOUNT);
	_videoSenderMain->setMaximumPacketSize(DEFAULT_PACKETSIZE);
	_videoSenderMain->setUserData((const mxByte *)"", 1);

	// VideoSender (Sub)
	_videoSenderSub->setAddress(DEFAULT_ADDRESS);
	_videoSenderSub->setPort(PORT_VIDEOFOCUS);
	_videoSenderSub->setTimeToLive(DEFAULT_TIMETOLIVE);
	_videoSenderSub->setMaximumTransmissionRate(DEFAULT_MAXTRANSMISSIONRATE);
	_videoSenderSub->setTransmissionCount(DEFAULT_TRANSMISSIONCOUNT);
	_videoSenderSub->setMaximumPacketSize(DEFAULT_PACKETSIZE);
	
	// AudioSender
	_audioSender->setAddress(DEFAULT_ADDRESS);
	_audioSender->setPort(PORT_AUDIO);
	_audioSender->setTimeToLive(DEFAULT_TIMETOLIVE);
	_audioSender->setMaximumTransmissionRate(DEFAULT_MAXTRANSMISSIONRATE);
	_audioSender->setTransmissionCount(DEFAULT_TRANSMISSIONCOUNT);
	_audioSender->setMaximumPacketSize(DEFAULT_PACKETSIZE);

	// PointerSender (Focus)
	_pointerSender->setAddress(DEFAULT_ADDRESS);
	_pointerSender->setPort(PORT_FOCUS_POINTER);
	_pointerSender->setTimeToLive(DEFAULT_TIMETOLIVE);
	_pointerSender->setMaximumTransmissionRate(DEFAULT_MAXTRANSMISSIONRATE);
	_pointerSender->setTransmissionCount(DEFAULT_TRANSMISSIONCOUNT);
	_pointerSender->setMaximumPacketSize(DEFAULT_PACKETSIZE);

	// PointerReceiver (Focus)
	_pointerReceiver1->setAddress(DEFAULT_ADDRESS);
	_pointerReceiver1->setPort(PORT_FOCUS_POINTER);
	_pointerReceiver1->setTimeToLive(DEFAULT_TIMETOLIVE);
	_pointerReceiver1->setEnableConnectReceiveMediaType(false);
	_pointerReceiver1->setMediaType(IMulticastReceiver::MRMT_POINTER);

	_pointerReceiver2->setAddress(DEFAULT_ADDRESS);
	_pointerReceiver2->setPort(PORT_FOCUS_POINTER);
	_pointerReceiver2->setTimeToLive(DEFAULT_TIMETOLIVE);
	_pointerReceiver2->setEnableConnectReceiveMediaType(false);
	_pointerReceiver2->setMediaType(IMulticastReceiver::MRMT_POINTER);

	// FormatTransform (Main)
	_formatTransformMain->setOutputVideoResolution(IFormatTrans::VR_THROUGH);
	_formatTransformMain->setInputVideoColor(IFormatTrans::VC_RGB24);

	// FormatTransform (Sub)
	_formatTransformSub->setOutputVideoResolution(IFormatTrans::VR_SQCIF);
	_formatTransformSub->setInputVideoColor(IFormatTrans::VC_RGB24);

	// VideoExtract
	_videoExtract1->setOutputVideoResolution(IVideoExtract::VR_QCIF);
	_videoExtract2->setOutputVideoResolution(IVideoExtract::VR_QCIF);

	// PointerPosition
	_pointerPosition->setOutputMinimumPeriod(50);

	// StillPicture
	fsStillPictureFilter *sp = _stillPicture->cast<fsStillPictureFilter>();
	sp->setFilename(mxFile::getModulePath() + mxString("\\stillpicture.bmp"));
	sp->setOutputVideoResolution(IStillPicture::SPVR_320x240);
	sp->setOutputMode(IStillPicture::SPOM_KEEPASPECTHV);

	// DigitalClock
	fsDigitalClockFilter *dc = _digitalClock->cast<fsDigitalClockFilter>();
	dc->setHeaderString("fBAJZ^[");
	dc->setFooterString("FocusShare\tgEFA");

	// DrawFrame
	_drawFrame1->setThickness(2);
	_drawFrame1->setColor(0xff, 0x00,0x00);
	_drawFrame1->setSize(176, 144);
	_drawFrame1->setFrameType(IDrawFrame::DFFT_RECTANGLE);

	_drawFrame2->setThickness(2);
	_drawFrame2->setColor(0x00, 0xff,0x00);
	_drawFrame2->setSize(160, 120);
	_drawFrame2->setFrameType(IDrawFrame::DFFT_RECTANGLE);

	_drawFrame3->setThickness(2);
	_drawFrame3->setColor(0x00, 0x00,0xff);
	_drawFrame3->setSize(176, 144);
	_drawFrame3->setFrameType(IDrawFrame::DFFT_RECTANGLE);
/*
	//
	// JnADV Capture ƃI[fBILv`̑gݍ킹ŃOtƂ
	// I[fBILv`f[^oȂsւ̑Ή
	// 
	createAudioGraph();
	start();
	Sleep(100);
	stop();
	clearGraph();
	// 
	//
*/
}
//------------------------------------------------------------------------------
void
fsSender::initializeStatus()
{
	_focusDisplay			= FSFD_NONE;
	_focusFrameVisible		= true;
	_directVideoModeOn		= false;
	_dvInterleavedModeOn	= false;

	// Direct Video mode
	_status.DirectVideoModeOn					= _directVideoModeOn;
	_status.DvInterleavedModeOn					= _dvInterleavedModeOn;

	// Focus display
	_status.FocusDisplay						= _focusDisplay;

	// Partial Zoom parameter
	_status.PartialZoomOutputVideoResolution	= FSVR_320x240;

	// Non Linear Zoom parameter
	_status.NonLinearZoomOutputVideoResolution	= FSVR_320x240;
	_status.NonLinearZoomMaskType				= FSMT_RECTANGLE;
	_status.NonLinearZoomMaskWidth				= 160;
	_status.NonLinearZoomMaskHeight				= 120;

	// Image Compound parameter
	_status.ImageCompoundOutputVideoResolution	= FSVR_SQCIF;
	_status.ImageCompoundMaskType				= FSMT_RECTANGLE;
	_status.ImageCompoundBlendingType			= FSBT_SIMPLE;

	// Multicast Address
	_status.MulticastAddress					= DEFAULT_ADDRESS;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsSender::showMessageBox(LPTSTR sz,...)
{
    static TCHAR tach[2000];
    va_list va;

    va_start(va, sz);
    wvsprintf(tach, sz, va);
    va_end(va);

    MessageBox(_windowHandle, tach, TEXT("Error"), MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL);
}
//------------------------------------------------------------------------------
void
fsSender::setupCaption()
{
	mxUInt length = _caption.length();
	const mxChar *captionStr = _caption.c_str();

	mxChar str[64];
	if (captionStr != NULL && _captionOn)
	{
		strncpy(str, captionStr, 62);
		str[62] = 0;
	}
	else
	{
		str[0] = 0;
	}

	_videoSenderMain->setUserData((const mxByte *)str, 64);
}
//------------------------------------------------------------------------------


/*
void
fsSender::computePointerFocusRegion()
{
	mxFloat x, y;
	mxFloat width, height;

	if (_focusDisplay == FSFD_NONE)
		return ;

	switch(_focusDisplay)
	{
	case FSFD_PARTIAL_ZOOM:
		{
			mxInt pwidth, pheight;
			mxInt dwidth, dheight;
			mxInt resolution;

			resolution = _videoExtract1->getOutputVideoResolution();
			FS_RESOLUTION2SIZE(resolution, pwidth, pheight);

			resolution = _formatTransformMain->getActualOutputVideoResolution();
			FS_RESOLUTION2SIZE(resolution, dwidth, dheight);

			x = 0;
			y = 0;
			width = mxFloat(dwidth - pwidth) / mxFloat(dwidth);
			height = mxFloat(dheight - pheight) / mxFloat(dheight);
		}
		break;
	case FSFD_NON_LINEAR_ZOOM:
		{
			mxInt dwidth, dheight;
			mxInt resolution;

			resolution = _formatTransformMain->getActualOutputVideoResolution();
			FS_RESOLUTION2SIZE(resolution, dwidth, dheight);

			x = 0;
			y = 0;
			width = mxFloat(dwidth - _status.NonLinearZoomMaskWidth) / mxFloat(dwidth);
			height = mxFloat(dheight - _status.NonLinearZoomMaskHeight) / mxFloat(dheight);
		}
		break;
	case FSFD_IMAGE_COMPOUND:
		{
			mxInt pwidth, pheight;
			mxInt dwidth, dheight;
			mxInt resolution;

			resolution = _videoExtract2->getOutputVideoResolution();
			FS_RESOLUTION2SIZE(resolution, pwidth, pheight);

			resolution = _formatTransformMain->getActualOutputVideoResolution();
			FS_RESOLUTION2SIZE(resolution, dwidth, dheight);

			x = - mxFloat(pwidth) / mxFloat(dwidth);
			y = - mxFloat(pheight) / mxFloat(dheight);
			width = mxFloat(dwidth + pwidth) / mxFloat(dwidth);
			height = mxFloat(dheight + pheight) / mxFloat(dheight);
		}
		break;
	}

	if (width < 0.0f)	width = 0.0f;
	if (height < 0.0f)	height = 0.0f;

	_pointerPosition->setNormalizedOrigin(x, y);
	_pointerPosition->setNormalizedSize(width, height);
}
*/
