/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsSender_h__
#define __fsSender_h__
//------------------------------------------------------------------------------
#include <fs/fsSenderRMI/fsSenderIf.h>
#include <fs/fsCommon/fsVideoRenderFilter.h>
#include <fs/fsCommon/fsAudioRenderFilter.h>
#include <fs/fsCommon/fsMulticastSenderFilter.h>
#include <fs/fsCommon/fsMulticastReceiverFilter.h>
#include <fs/fsCommon/fsFormatTransFilter.h>
#include <fs/fsCommon/fsVideoExtractFilter.h>
#include <fs/fsCommon/fsPointerPositionFilter.h>
#include <fs/fsCommon/fsDVSplitterFilter.h>
#include <fs/fsCommon/fsInfPinTeeFilter.h>
#include <fs/fsCommon/fsSmartTeeFilter.h>
#include <fs/fsCommon/fsStillPictureFilter.h>
#include <fs/fsCommon/fsDigitalClockFilter.h>
#include <fs/fsCommon/fsAsyncReaderFilter.h>
#include <fs/fsCommon/fsDrawFrameFilter.h>
#include <fs/fsCommon/fsVolumeIndicatorFilter.h>
#include <fs/fsCommon/fsAudioMixerFilter.h>
#include <fs/fsCommon/fsGraphManager.h>
#include <fs/fsCommon/fsNamePriorityList.h>
#include <fs/fsCommon/fsErrorMessage.h>
#include <fs/fsStatus/fsStatusSender.h>
#include <fs/fsMulticastTools/fsMulticastDetectors.h>
#include <mx/mxCore/mxObject.h>
#include <mx/mxCore/mxCore.h>
//------------------------------------------------------------------------------
class fsSender;
typedef mxObjectPtr<fsSender>	fsSenderPtr;
//------------------------------------------------------------------------------
class fsSender : public mxObject, public fsSenderIf
{
	MXOBJECTHEADER(fsSender, mxObject);

public:
	fsSender();

protected:
	virtual ~fsSender();

public:
	// Initialize / Shutdown
	mxBool				initialize(HWND windowHandle);
	mxBool				shutdown();

public:
	//
	// Implements fsSenderIf
	//

	//
	// Streaming Control
	//
	mxBool				getValid() const;
	void				getErrorMessage(mxInt& id, mxInt& type, mxString& message) const;
	void				setStart(mxBool flag);
	mxBool				getStart() const;
	void				setVideoTransmissionOn(mxBool flag);
	mxBool				getVideoTransmissionOn() const;
	void				setAudioTransmissionOn(mxBool flag);
	mxBool				getAudioTransmissionOn() const;
	void				setFocusTransmissionOn(mxBool flag);
	mxBool				getFocusTransmissionOn() const;

	//
	// Multicast Settings
	//
	void				setAddress(const mxString& address);
	const mxString&		getAddress() const;
	void				setTimeToLive(mxULong ttl);
	mxULong				getTimeToLive() const;
	void				setTransmissionCount(mxUInt count);
	mxUInt				getTransmissionCount() const;
	void				setPacketSize(mxUInt size);
	mxUInt				getPacketSize() const;

	// Create Address
	mxString			createIPv4Address();
	mxString			createIPv6Address();

	// Video Sender (Main)
	void				setVideoSenderMainMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getVideoSenderMainMaxTransmissionRate() const;
	mxInt				getVideoSenderMainTransmissionRate() const;
	mxInt				getVideoSenderMainFrameRate() const;

	// Video Sender (Sub)
	void				setVideoSenderSubMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getVideoSenderSubMaxTransmissionRate() const;
	mxInt				getVideoSenderSubTransmissionRate() const;
	mxInt				getVideoSenderSubFrameRate() const;

	// Audio Sender
	void				setAudioSenderMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getAudioSenderMaxTransmissionRate() const;
	mxInt				getAudioSenderTransmissionRate() const;
	mxInt				getAudioSenderFrameRate() const;

	// Pointer Sender
	void				setPointerSenderMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getPointerSenderMaxTransmissionRate() const;
	mxInt				getPointerSenderTransmissionRate() const;
	mxInt				getPointerSenderFrameRate() const;

	// Status Sender
	void				setStatusSenderMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getStatusSenderMaxTransmissionRate() const;


	//
	// Multicast Detector Settings
	//
	mxBool				addMulticastDetector(const mxString& address);
	mxBool				removeMulticastDetector(const mxString& address);
	mxBool				wasDetectedMulticast(const mxString& address);


	//
	// Source Settings
	//

	// Video Source
	mxInt				getNumVideoSources() const;
	void				setVideoSource(mxInt index);
	mxInt				getVideoSource() const;
	const mxString&		getVideoSourceFriendlyName(mxInt index);

	// Audio Source
	mxInt				getNumAudioSources() const;
	void				setAudioSource(mxInt index);
	mxInt				getAudioSource() const;
	const mxString&		getAudioSourceFriendlyName(mxInt index);

	// Video Source property page
	mxInt				getNumVideoSourcePropertyPages() const;
	const mxString&		getVideoSourcePropertyPageName(mxInt index);
    void				showVideoSourcePropertyPage(HWND windowHandle, mxInt index);

	// Audio Source property page
	mxInt				getNumAudioSourcePropertyPages() const;
	const mxString&		getAudioSourcePropertyPageName(mxInt index);
    void				showAudioSourcePropertyPage(HWND windowHandle, mxInt index);

	// Video Clip
	mxBool				isPlayVideoClip() const;
	mxBool				playVideoClip(const mxString& filename);

	// Sound File
	mxBool				isPlaySoundFile() const;
	mxBool				playSoundFile(const mxString& filename);


	//
	// Encoder Settings
	//

	// Video Encoder
	mxInt				getNumVideoEncoder() const;
	void				setVideoEncoder(mxInt index);
	mxInt				getVideoEncoder() const;
	const mxString&		getVideoEncoderFriendlyName(mxInt index) const;
	mxInt				getVideoEncoderPriority(mxInt index) const;

	// Audio Encoder
	mxInt				getNumAudioEncoder() const;
	void				setAudioEncoder(mxInt index);
	mxInt				getAudioEncoder() const;
	const mxString&		getAudioEncoderFriendlyName(mxInt index) const;
	mxInt				getAudioEncoderPriority(mxInt index) const;


	//
	// Focus Display Settings
	//
	void				setFocusDisplay(mxInt focusDisplay);
	mxInt				getFocusDisplay() const;
	void				setFocusFrameVisible(mxBool flag);
	mxBool				getFocusFrameVisible() const;

	// Video Resolution
	void				setVideoResolution(mxInt videoResolution);
	mxInt				getVideoResolution() const;

	// Partial Zoom
	void				setPartialZoomSourceVideoResolution(mxInt videoResolution);
	mxInt				getPartialZoomSourceVideoResolution() const;
	void				setPartialZoomOutputVideoResolution(mxInt videoResolution);
	mxInt				getPartialZoomOutputVideoResolution() const;

	// Non-Linear Zoom
	void				setNonLinearZoomOutputVideoResolution(mxInt videoResolution);
	mxInt				getNonLinearZoomOutputVideoResolution() const;
	void				setNonLinearZoomMaskType(mxInt maskType);
	mxInt				getNonLinearZoomMaskType() const;
	void				setNonLinearZoomMaskSize(mxInt width, mxInt height);
	void				getNonLinearZoomMaskSize(mxInt& width, mxInt& height) const;

	// Image Compound
	void				setImageCompoundBaseVideoResolution(mxInt videoResolution);
	mxInt				getImageCompoundBaseVideoResolution() const;
	void				setImageCompoundAdditionalVideoResolution(mxInt videoResolution);
	mxInt				getImageCompoundAdditionalVideoResolution() const;
	void				setImageCompoundMaskType(mxInt maskType);
	mxInt				getImageCompoundMaskType() const;
	void				setImageCompoundBlendingType(mxInt blendingType);
	mxInt				getImageCompoundBlendingType() const;

	// Other Settings
	void				setFocusCoordinate(mxInt coordinate);
	mxInt				getFocusCoordinate() const;
	void				computeFocusRegion();


	//
	// Direct Video Mode Settings
	//
	void				setDirectVideoModeOn(mxBool flag);
	mxBool				getDirectVideoModeOn() const;
	void				setDVInterleavedModeOn(mxBool flag);
	mxBool				getDVInterleavedModeOn() const;
	mxBool				isValidDVInterleavedMode() const;


	//
	// Caption Settings
	//
	void				setCaptionOn(mxBool flag);
	mxBool				getCaptionOn() const;
	void				setCaption(const mxString& caption);
	const mxString&		getCaption() const;


	//
	// Video Window Settings
	//
	void				setVideoWindowPolicy(mxInt policy);
	mxInt				getVideoWindowPolicy() const;
	void				setKeepVideoAspect(mxBool flag);
	mxBool				getKeepVideoAspect() const;
	void				setFitWindowToVideo(mxBool flag);
	mxBool				getFitWindowToVideo() const;
	void				computeVideoWindow();


	//
	// Volume Settings
	//
	void				getVolume(mxInt& left, mxInt& right) const;
	void				setVolumeEvaluationTime(mxInt evalTime);
	mxInt				getVolumeEvaluationTime() const;


	//
	// Other
	//
	void				quit();
	void				freeze(mxBool flag);

public:
	// Quit
	mxBool				checkQuit();

	// Status
	void				sendStatus();

	// IniFile
	mxBool				writeIniFile(const mxString& filename);
	mxBool				readIniFile(const mxString& filename);

	// Registry
	void				writeRegistry();
	void				readRegistry();
	void				writeRegistry(const mxString& section,
									const mxString& entry,
									const mxString& value);
	void				readRegistry(const mxString& section,
									const mxString& entry,
									const mxString& defaultValue,
									mxString& value);

public:
	// Callback
	typedef int (*fsSenderUICallbackFunction)(const char*, void*, void*);
	mxBool				setUICallbackFunction(fsSenderUICallbackFunction func, void* data);

public:
	// Cast
	template<class T> T*		cast()			{ return static_cast<T*>(this); }
	template<class T> const T*	cast() const	{ return static_cast<const T*>(this); }

protected:
	// Graph Event
	void				graphEvent(mxInt eventCode, void* param1, void* param2);
	static void			graphEventProc(mxInt eventCode, 
										void *param1, void* param2, void *data);
	
	// Process Intermediate
	void				processIntermediate(const fsFilterPtr* filters, mxUInt numFilters);
	static void			processIntermediateProc(const fsFilterPtr* filters, 
										mxUInt numFilters, void* data);

protected:
	mxBool				createFilters();
	void				deleteFilters();

	void				initializeFilters();
	void				initializeStatus();

	mxBool				buildGraph();
	void				clearGraph();
	mxBool				buildDVInterleavedGraph();
	mxBool				buildDirectVideoGraph();
	mxBool				buildFocusNoneVideoGraph();
	mxBool				buildFocusPartialZoomVideoGraph();
	mxBool				buildFocusNonLinearZoomVideoGraph();
	mxBool				buildFocusImageCompoundVideoGraph();
	mxBool				buildAudioGraph();
	mxBool				buildAudioGraphFromAsyncSource();

private:
	void				showMessageBox(LPTSTR sz, ...);
	void				setupCaption();

protected:
	HWND							_windowHandle;
    
	fsGraphManagerPtr				_graph;
	fsStatusSenderPtr				_statusSender;
	fsStatus						_status;
	fsMulticastDetectorsPtr			_multicastDetectors;
	fsNamePriorityListPtr			_videoEncoderPriorities;
	fsNamePriorityListPtr			_audioEncoderPriorities;

	mxBool							_isValid;
	mxBool							_isStarted;
	fsErrorMessage					_errorMessage;

	mxBool							_freezeFlag;
	mxBool							_quitFlag;

	fsSenderUICallbackFunction		_UICallbackFunction;
	void*							_UICallbackData;

	// Parameters
	mxUInt							_videoSource;
	mxUInt							_audioSource;
	mxBool							_playVideoClipOn;
	mxBool							_playSoundFileOn;
	mxUInt							_videoEncoder;
	mxUInt							_audioEncoder;
	mxInt							_focusDisplay;
	mxBool							_focusFrameVisible;
	mxInt							_focusCoordinate;
	mxBool							_directVideoModeOn;
	mxBool							_dvInterleavedModeOn;
	mxBool							_captionOn;
	mxString						_caption;
	mxInt							_videoWindowPolicy;
	mxBool							_keepVideoAspect;
	mxBool							_fitWindowToVideo;


	// Source Filters
	mxArray<fsFilterPtr>			_videoSources;
	mxArray<fsFilterPtr>			_audioSources;

	// Encoder Filters
	mxArray<fsFilterPtr>			_videoEncodersMain;
	mxArray<fsFilterPtr>			_videoEncodersSub;
	mxArray<fsFilterPtr>			_audioEncoders;

	// Renderer Filters
	fsFilterPtr						_videoRender;
	fsFilterPtr						_audioRender;

	// Multicast Sender Filters
	fsMulticastSenderFilterPtr		_videoSenderMain;
	fsMulticastSenderFilterPtr		_videoSenderSub;
	fsMulticastSenderFilterPtr		_audioSender;
	fsMulticastSenderFilterPtr		_pointerSender;

	// Multicast Receiver Filters
	fsMulticastReceiverFilterPtr	_pointerReceiver1;
	fsMulticastReceiverFilterPtr	_pointerReceiver2;

	// Other Filters
	fsFormatTransFilterPtr			_formatTransformMain;
	fsFormatTransFilterPtr			_formatTransformSub;
	fsVideoExtractFilterPtr			_videoExtract1;
	fsVideoExtractFilterPtr			_videoExtract2;
	fsPointerPositionFilterPtr		_pointerPosition;
	fsDrawFrameFilterPtr			_drawFrame1;
	fsDrawFrameFilterPtr			_drawFrame2;
	fsDrawFrameFilterPtr			_drawFrame3;
	fsFilterPtr						_stillPicture;
	fsFilterPtr						_digitalClock;
	fsDVSplitterFilterPtr			_dvSplitter;
	fsVolumeIndicatorFilterPtr		_volumeIndicator;
	fsAudioMixerFilterPtr			_audioMixer;

	// Filters generated each time
	fsFilterPtr						_asyncReader;
	fsSmartTeeFilterPtr				_smartTee;
	fsInfPinTeeFilterPtr			_videoInfPinTee;
	fsInfPinTeeFilterPtr			_audioInfPinTee;
};
//------------------------------------------------------------------------------
#endif	// __fsSender_h__
