/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsSender/fsSender.h>
#include <fs/fsCommon/fsCaptureFilter.h>
//------------------------------------------------------------------------------
mxBool
fsSender::buildGraph()
{
	mxBool ret = true;

	// Direct Video Mode
	if (_directVideoModeOn)
	{
		if (_dvInterleavedModeOn)
		{
			// Video & Audio (DV only)
			ret &= buildDVInterleavedGraph();
		}
		else
		{
			// Video
			ret &= buildDirectVideoGraph();
			// Audio
			ret &= buildAudioGraph();
		}
	}
	// Focus Disiplay Mode
	else
	{
		// Video
		switch(_focusDisplay)
		{
		case FSFD_NONE:
			ret &= buildFocusNoneVideoGraph();
			break;
		case FSFD_PARTIAL_ZOOM:
			ret &= buildFocusPartialZoomVideoGraph();
			break;
		case FSFD_NON_LINEAR_ZOOM:
			ret &= buildFocusNonLinearZoomVideoGraph();
			break;
		case FSFD_IMAGE_COMPOUND:
			ret &= buildFocusImageCompoundVideoGraph();
			break;
		}

		// Audio
		ret &= buildAudioGraph();
	}

	// Window Setting
	_graph->setVideoWindowOwner(_windowHandle);
	_graph->setVideoWindowVisible(true);
	if (_windowHandle != NULL)
		_graph->setVideoWindowStyle(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);

	_isValid = ret;

	return ret;
}
//------------------------------------------------------------------------------
void
fsSender::clearGraph()
{
	_isValid = true;

	// Window Setting
	_graph->setVideoWindowVisible(false);
	_graph->setVideoWindowOwner(NULL);

	// Remove All Filters from FilterGraph
	_graph->removeAllFilters();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#define AddFilter(__filter)			\
	_graph->addFilter(*(__filter));
//------------------------------------------------------------------------------
#define ConnectDirect(__filterOut, __outPin, __filterIn, __inPin, __errType, __errMess)		\
if (!_graph->connectFilterDirect(*(__filterOut), (__outPin), *(__filterIn), (__inPin)))		\
{																							\
	_errorMessage.Id ++;																	\
	_errorMessage.Type = fsErrorMessage::__errType;											\
	_errorMessage.Message = __errMess;														\
																							\
	return false;																			\
}
//------------------------------------------------------------------------------
#define ConnectIndirect(__filterOut, __outPin, __filterIn, __inPin, __errType, __errMess)	\
if (!_graph->connectFilterIndirect(*(__filterOut), (__outPin), *(__filterIn), (__inPin)))	\
{																							\
	_errorMessage.Id ++;																	\
	_errorMessage.Type = fsErrorMessage::__errType;											\
	_errorMessage.Message = __errMess;														\
																							\
	return false;																			\
}
//------------------------------------------------------------------------------
#define USE_SMART_TEE
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsSender::buildDVInterleavedGraph()
{
	if (_dvSplitter == NULL)
		return false;

	if (_playVideoClipOn)
		return false;

	// Get Video Source
	assert(_videoSource < _videoSources.getSize());
	fsFilterPtr sourceFilter = _videoSources[_videoSource];
	assert(sourceFilter != NULL);

	// Re-create Tee filter
	_videoInfPinTee	= new fsInfPinTeeFilter("Video InfPinTee");

	//
	// Connection
	//
	AddFilter(sourceFilter);
	AddFilter(_videoInfPinTee);
	// Video Capture filter
	if (sourceFilter->getClassType() == fsCaptureFilter::getStaticClassType())
	{
#ifdef USE_SMART_TEE
		_smartTee = new fsSmartTeeFilter("SmartTee");
		AddFilter(_smartTee);
		ConnectDirect(sourceFilter, 1, _smartTee, 0,
			T_WRONG_CAPTURE, "Video Capture");
		ConnectDirect(_smartTee, 1, _videoInfPinTee, 0,
			T_UNEXPECTED, "Connect: SmartTee->VideoInfPinTee");
#else
		ConnectDirect(sourceFilter, 1, _videoInfPinTee, 0,
			T_WRONG_CAPTURE, "Video Capture");
#endif
	}
	// Other Source filter
	else
	{
		assert(false);
	}

	AddFilter(_videoSenderMain);
	AddFilter(_dvSplitter);
	AddFilter(_videoRender);

	ConnectDirect(_videoInfPinTee, 0, _videoSenderMain, 0,
		T_UNEXPECTED, "Connect: VideoInfPinTee->VideoSenderMain");
	ConnectDirect(_videoInfPinTee, 1, _dvSplitter, 0,
		T_UNEXPECTED, "Connect: VideoInfPinTee->DVSplitter");

	ConnectIndirect(_dvSplitter, 0, _videoRender, 0,
		T_UNEXPECTED, "Connect: DVSplitter->>VideoRender");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSender::buildDirectVideoGraph()
{
	if (_playVideoClipOn)
		return false;

	// Get Video Source
	assert(_videoSource < _videoSources.getSize());
	fsFilterPtr sourceFilter = _videoSources[_videoSource];
	assert(sourceFilter != NULL);

	// Re-create Tee filter
	_videoInfPinTee	= new fsInfPinTeeFilter("Video InfPinTee");

	//
	// Connection
	//
	AddFilter(sourceFilter);
	AddFilter(_videoInfPinTee);
	// Video Capture filter
	if (sourceFilter->getClassType() == fsCaptureFilter::getStaticClassType())
	{
		mxInt outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getPreviewPinIndex();
		if (outPinIdx != -1)
		{
			ConnectDirect(sourceFilter, outPinIdx, _videoInfPinTee, 0,
				T_WRONG_CAPTURE, "Video");
		}
		else
		{
			outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getCapturePinIndex();
			if (outPinIdx != -1)
			{
#ifdef USE_SMART_TEE
				_smartTee = new fsSmartTeeFilter("SmartTee");
				AddFilter(_smartTee);
				ConnectDirect(sourceFilter, outPinIdx, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectDirect(_smartTee, 1, _videoInfPinTee, 0,
					T_UNEXPECTED, "Connect: SmartTee->VideoInfPinTee");
#else
				ConnectDirect(sourceFilter, outPinIdx, _videoInfPinTee, 0,
					T_WRONG_CAPTURE, "Video");
#endif
			}
			else
			{
#ifdef USE_SMART_TEE
				_smartTee = new fsSmartTeeFilter("SmartTee");
				AddFilter(_smartTee);
				ConnectDirect(sourceFilter, 0, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectDirect(_smartTee, 1, _videoInfPinTee, 0,
					T_UNEXPECTED, "Connect: SmartTee->VideoInfPinTee");
#else
				ConnectDirect(sourceFilter, outPinIdx, _videoInfPinTee, 0,
					T_WRONG_CAPTURE, "Video");
#endif
			}
		}
	}
	// Other Source filter
	else
	{
		ConnectIndirect(sourceFilter, 0, _videoInfPinTee, 0,
			T_UNEXPECTED, "[Source]->>VideoInfPinTee");
	}

	AddFilter(_videoSenderMain);
	AddFilter(_videoRender);

	ConnectDirect(_videoInfPinTee, 0, _videoSenderMain, 0,
		T_UNEXPECTED, "Connect: VideoInfPinTee->VideoSenderMain");
	ConnectIndirect(_videoInfPinTee, 1, _videoRender, 0,
		T_WRONG_MEDIATYPE, "Connect: VideoInfPinTee->>VideoRenderer");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSender::buildFocusNoneVideoGraph()
{
	// Get Video Source
	fsFilterPtr sourceFilter;
	if (_playVideoClipOn)
	{
		if (_asyncReader == NULL)
			return false;
		sourceFilter = _asyncReader;
	}
	else
	{
		assert(_videoSource < _videoSources.getSize());
		sourceFilter = _videoSources[_videoSource];
		assert(sourceFilter != NULL);
	}

	// Re-create Tee filter
	_videoInfPinTee	= new fsInfPinTeeFilter("Video InfPinTee");
	_smartTee		= new fsSmartTeeFilter("Video SmartTee");

	// Get Video Encoder
	assert(_videoEncoder < _videoEncodersMain.getSize());
	fsFilterPtr encoderFilter = _videoEncodersMain[_videoEncoder];

	//
	// Connection
	//
	AddFilter(sourceFilter);
	AddFilter(_formatTransformMain);
	// Video Capture filter
	if (sourceFilter->getClassType() == fsCaptureFilter::getStaticClassType())
	{
		mxInt outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getPreviewPinIndex();
		if (outPinIdx != -1)
		{
			ConnectIndirect(sourceFilter, outPinIdx, _formatTransformMain, 0,
				T_WRONG_CAPTURE, "Video");
		}
		else
		{
			AddFilter(_smartTee);

			outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getCapturePinIndex();
			if (outPinIdx != -1)
			{
				ConnectDirect(sourceFilter, outPinIdx, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectIndirect(_smartTee, 1, _formatTransformMain, 0,
					T_WRONG_MEDIATYPE, "Connect: SmartTee->>FormatTransformMain");
			}
			else
			{
				ConnectDirect(sourceFilter, 0, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectIndirect(_smartTee, 1, _formatTransformMain, 0,
					T_WRONG_MEDIATYPE, "Connect: SmartTee->>FormatTransformMain");
			}
		}
	}
	// Other Source filter
	else
	{
		ConnectIndirect(sourceFilter, 0, _formatTransformMain, 0,
			T_UNEXPECTED, "[Source]->>FormatTransformMain");
	}

	AddFilter(_videoInfPinTee);
	ConnectDirect(_formatTransformMain, 0, _videoInfPinTee, 0,
		T_UNEXPECTED, "Connect: FormatTransform->VideoInfPinTee");
	
	if (encoderFilter != NULL)
	{
		AddFilter(encoderFilter);
		AddFilter(_videoSenderMain);
		ConnectDirect(_videoInfPinTee, 0, encoderFilter, 0, 
			T_WRONG_ENCODER, "Video");
		ConnectDirect(encoderFilter, 0, _videoSenderMain, 0,
			T_UNEXPECTED, "Connect: [VideoEncoder]->VideoSenderMain");
	}
	else
	{
		AddFilter(_videoSenderMain);
		ConnectDirect(_videoInfPinTee, 0, _videoSenderMain, 0,
			T_UNEXPECTED, "Connect: VideoInfPinTee->VideoSenderMain");
	}

	AddFilter(_videoRender);
	ConnectIndirect(_videoInfPinTee, 1, _videoRender, 0,
		T_WRONG_MEDIATYPE, "Connect: VideoInfPinTee->>VideoRenderer");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSender::buildFocusPartialZoomVideoGraph()
{
	// Get Video Source
	fsFilterPtr sourceFilter;
	if (_playVideoClipOn)
	{
		if (_asyncReader == NULL)
			return false;
		sourceFilter = _asyncReader;
	}
	else
	{
		assert(_videoSource < _videoSources.getSize());
		sourceFilter = _videoSources[_videoSource];
		assert(sourceFilter != NULL);
	}

	// Re-create Tee filter
	_videoInfPinTee	= new fsInfPinTeeFilter("Video InfPinTee");
	_smartTee		= new fsSmartTeeFilter("Video SmartTee");

	// Get Video Encoder
	assert(_videoEncoder < _videoEncodersMain.getSize());
	fsFilterPtr encoderFilter = _videoEncodersMain[_videoEncoder];

	//
	// Connection
	//
	AddFilter(sourceFilter);
	AddFilter(_formatTransformMain);
	// Video Capture filter
	if (sourceFilter->getClassType() == fsCaptureFilter::getStaticClassType())
	{
		mxInt outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getPreviewPinIndex();
		if (outPinIdx != -1)
		{
			ConnectIndirect(sourceFilter, outPinIdx, _formatTransformMain, 0,
				T_WRONG_CAPTURE, "Video");
		}
		else
		{
			AddFilter(_smartTee);

			outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getCapturePinIndex();
			if (outPinIdx != -1)
			{
				ConnectDirect(sourceFilter, outPinIdx, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectIndirect(_smartTee, 1, _formatTransformMain, 0,
					T_WRONG_MEDIATYPE, "Connect: SmartTee->>FormatTransformMain");
			}
			else
			{
				ConnectDirect(sourceFilter, 0, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectIndirect(_smartTee, 1, _formatTransformMain, 0,
					T_WRONG_MEDIATYPE, "Connect: SmartTee->>FormatTransformMain");
			}
		}
	}
	// Other Source filter
	else
	{
		ConnectIndirect(sourceFilter, 0, _formatTransformMain, 0,
			T_UNEXPECTED, "[Source]->>FormatTransformMain");
	}

	AddFilter(_videoInfPinTee);
	AddFilter(_videoExtract1);
	ConnectDirect(_formatTransformMain, 0, _videoInfPinTee, 0,
		T_UNEXPECTED, "Connect: FormatTransform->VideoInfPinTee");
	ConnectDirect(_videoInfPinTee, 0, _videoExtract1, 0,
		T_UNEXPECTED, "Connect: VideoInfPinTee->VideoExtract1");
	
	if (encoderFilter != NULL)
	{
		AddFilter(encoderFilter);
		AddFilter(_videoSenderMain);
		ConnectDirect(_videoExtract1, 0, encoderFilter, 0,
			T_WRONG_ENCODER, "Video");
		ConnectDirect(encoderFilter, 0, _videoSenderMain, 0,
			T_UNEXPECTED, "Connect: [VideoEncoder]->VideoSenderMain");
	}
	else
	{
		AddFilter(_videoSenderMain);
		ConnectDirect(_videoExtract1, 0, _videoSenderMain, 0,
			T_UNEXPECTED, "Connect: VideoExtract1->VideoSenderMain");
	}

	AddFilter(_videoRender);
	if (_focusFrameVisible)
	{
		AddFilter(_drawFrame1);
		ConnectDirect(_videoInfPinTee, 1, _drawFrame1, 0,
			T_UNEXPECTED, "Connect: VideoInfPinTee->DrawFrame1");
		ConnectIndirect(_drawFrame1, 0, _videoRender, 0,
			T_WRONG_MEDIATYPE, "Connect: DrawFrame1->>VideoRenderer");

		AddFilter(_pointerReceiver2);
		ConnectDirect(_pointerReceiver2, 0, _drawFrame1, 1,
			T_UNEXPECTED, "Connect: PointerReceiver2->DrawFrame1");
	}
	else
	{
		ConnectIndirect(_videoInfPinTee, 1, _videoRender, 0,
			T_WRONG_MEDIATYPE, "Connect: VideoInfPinTee->>VideoRenderer");
	}

	AddFilter(_pointerReceiver1);
	ConnectDirect(_pointerReceiver1, 0, _videoExtract1, 1,
		T_UNEXPECTED, "Connect: PointerReceiver1->VideoExtract1");

	AddFilter(_pointerPosition);
	AddFilter(_pointerSender);
	ConnectDirect(_pointerPosition, 0, _pointerSender, 0,
		T_UNEXPECTED, "Connect: PointerPosition->PointerSender");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSender::buildFocusNonLinearZoomVideoGraph()
{
	// Get Video Source
	fsFilterPtr sourceFilter;
	if (_playVideoClipOn)
	{
		if (_asyncReader == NULL)
			return false;
		sourceFilter = _asyncReader;
	}
	else
	{
		assert(_videoSource < _videoSources.getSize());
		sourceFilter = _videoSources[_videoSource];
		assert(sourceFilter != NULL);
	}

	// Re-create Tee filter
	_videoInfPinTee	= new fsInfPinTeeFilter("Video InfPinTee");
	_smartTee		= new fsSmartTeeFilter("Video SmartTee");

	// Get Video Encoder
	assert(_videoEncoder < _videoEncodersMain.getSize());
	fsFilterPtr encoderFilter = _videoEncodersMain[_videoEncoder];

	//
	// Connection
	//
	AddFilter(sourceFilter);
	AddFilter(_formatTransformMain);
	// Video Capture filter
	if (sourceFilter->getClassType() == fsCaptureFilter::getStaticClassType())
	{
		mxInt outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getPreviewPinIndex();
		if (outPinIdx != -1)
		{
			ConnectIndirect(sourceFilter, outPinIdx, _formatTransformMain, 0,
				T_WRONG_CAPTURE, "Video");
		}
		else
		{
			AddFilter(_smartTee);

			outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getCapturePinIndex();
			if (outPinIdx != -1)
			{
				ConnectDirect(sourceFilter, outPinIdx, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectIndirect(_smartTee, 1, _formatTransformMain, 0,
					T_WRONG_MEDIATYPE, "Connect: SmartTee->>FormatTransformMain");
			}
			else
			{
				ConnectDirect(sourceFilter, 0, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectIndirect(_smartTee, 1, _formatTransformMain, 0,
					T_WRONG_MEDIATYPE, "Connect: SmartTee->>FormatTransformMain");
			}
		}
	}
	// Other Source filter
	else
	{
		ConnectIndirect(sourceFilter, 0, _formatTransformMain, 0,
			T_UNEXPECTED, "[Source]->>FormatTransformMain");
	}

	AddFilter(_videoInfPinTee);
	ConnectDirect(_formatTransformMain, 0, _videoInfPinTee, 0,
		T_UNEXPECTED, "Connect: FormatTransform->VideoInfPinTee");
	
	if (encoderFilter != NULL)
	{
		AddFilter(encoderFilter);
		AddFilter(_videoSenderMain);
		ConnectDirect(_videoInfPinTee, 0, encoderFilter, 0, 
			T_WRONG_ENCODER, "Video");
		ConnectDirect(encoderFilter, 0, _videoSenderMain, 0,
			T_UNEXPECTED, "Connect: [VideoEncoder]->VideoSenderMain");
	}
	else
	{
		AddFilter(_videoSenderMain);
		ConnectDirect(_videoInfPinTee, 0, _videoSenderMain, 0,
			T_UNEXPECTED, "Connect: VideoInfPinTee->VideoSenderMain");
	}

	AddFilter(_videoRender);
	if (_focusFrameVisible)
	{
		AddFilter(_drawFrame2);
		ConnectDirect(_videoInfPinTee, 1, _drawFrame2, 0,
			T_UNEXPECTED, "Connect: VideoInfPinTee->DrawFrame2");
		ConnectIndirect(_drawFrame2, 0, _videoRender, 0,
			T_WRONG_MEDIATYPE, "Connect: DrawFrame2->>VideoRenderer");

		AddFilter(_pointerReceiver2);
		ConnectDirect(_pointerReceiver2, 0, _drawFrame2, 1,
			T_UNEXPECTED, "Connect: PointerReceiver2->DrawFrame2");
	}
	else
	{
		ConnectIndirect(_videoInfPinTee, 1, _videoRender, 0,
			T_WRONG_MEDIATYPE, "Connect: VideoInfPinTee->>VideoRenderer");
	}

	AddFilter(_pointerPosition);
	AddFilter(_pointerSender);
	ConnectDirect(_pointerPosition, 0, _pointerSender, 0,
		T_UNEXPECTED, "Connect: PointerPosition->PointerSender");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSender::buildFocusImageCompoundVideoGraph()
{
	// Get Video Source
	fsFilterPtr sourceFilter;
	if (_playVideoClipOn)
	{
		if (_asyncReader == NULL)
			return false;
		sourceFilter = _asyncReader;
	}
	else
	{
		assert(_videoSource < _videoSources.getSize());
		sourceFilter = _videoSources[_videoSource];
		assert(sourceFilter != NULL);
	}

	// Re-create Tee filter
	_videoInfPinTee	= new fsInfPinTeeFilter("Video InfPinTee");
	_smartTee		= new fsSmartTeeFilter("Video SmartTee");

	// Get Video Encoder
	assert(_videoEncoder < _videoEncodersMain.getSize());
	fsFilterPtr encoderFilterMain = _videoEncodersMain[_videoEncoder];
	fsFilterPtr encoderFilterSub = _videoEncodersSub[_videoEncoder];

	//
	// Connection
	//
	AddFilter(sourceFilter);
	AddFilter(_formatTransformMain);
	// Video Capture filter
	if (sourceFilter->getClassType() == fsCaptureFilter::getStaticClassType())
	{
		mxInt outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getPreviewPinIndex();
		if (outPinIdx != -1)
		{
			ConnectIndirect(sourceFilter, outPinIdx, _formatTransformMain, 0,
				T_WRONG_CAPTURE, "Video");
		}
		else
		{
			AddFilter(_smartTee);

			outPinIdx = sourceFilter->cast<fsCaptureFilter>()->getCapturePinIndex();
			if (outPinIdx != -1)
			{
				ConnectDirect(sourceFilter, outPinIdx, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectIndirect(_smartTee, 1, _formatTransformMain, 0,
					T_WRONG_MEDIATYPE, "Connect: SmartTee->>FormatTransformMain");
			}
			else
			{
				ConnectDirect(sourceFilter, 0, _smartTee, 0,
					T_WRONG_CAPTURE, "Video");
				ConnectIndirect(_smartTee, 1, _formatTransformMain, 0,
					T_WRONG_MEDIATYPE, "Connect: SmartTee->>FormatTransformMain");
			}
		}
	}
	// Other Source filter
	else
	{
		ConnectIndirect(sourceFilter, 0, _formatTransformMain, 0,
			T_UNEXPECTED, "[Source]->>FormatTransformMain");
	}

	AddFilter(_videoInfPinTee);
	ConnectDirect(_formatTransformMain, 0, _videoInfPinTee, 0,
		T_UNEXPECTED, "Connect: FormatTransformMain->VideoInfPinTee");
	
	AddFilter(_formatTransformSub);
	ConnectDirect(_videoInfPinTee, 0, _formatTransformSub, 0,
		T_UNEXPECTED, "Connect: VideoInfPinTee->FormatTransformSub");

	AddFilter(_videoExtract2);
	ConnectDirect(_videoInfPinTee, 1, _videoExtract2, 0,
		T_UNEXPECTED, "Connect: VideoInfPinTee->VideoExtract2");

	AddFilter(_videoRender);
	if (_focusFrameVisible)
	{
		AddFilter(_drawFrame3);
		ConnectDirect(_videoInfPinTee, 2, _drawFrame3, 0,
			T_UNEXPECTED, "Connect: VideoInfPinTee->DrawFrame3");
		ConnectIndirect(_drawFrame3, 0, _videoRender, 0,
			T_WRONG_MEDIATYPE, "Connect: DrawFrame3->>VideoRenderer");

		AddFilter(_pointerReceiver2);
		ConnectDirect(_pointerReceiver2, 0, _drawFrame3, 1,
			T_UNEXPECTED, "Connect: PointerReceiver2->DrawFrame3");
	}
	else
	{
		ConnectIndirect(_videoInfPinTee, 2, _videoRender, 0,
			T_WRONG_MEDIATYPE, "Connect: VideoInfPinTee->>VideoRenderer");
	}

	if (encoderFilterMain != NULL)
	{
		AddFilter(encoderFilterMain);
		AddFilter(encoderFilterSub);
		AddFilter(_videoSenderMain);
		AddFilter(_videoSenderSub);
		ConnectDirect(_formatTransformSub, 0, encoderFilterMain, 0, 
			T_WRONG_ENCODER, "Video Main");
		ConnectDirect(encoderFilterMain, 0, _videoSenderMain, 0,
			T_UNEXPECTED, "Connect: [VideoEncoder]Main->VideoSenderMain");

		ConnectDirect(_videoExtract2, 0, encoderFilterSub, 0,
			T_WRONG_ENCODER, "Video Context");
		ConnectDirect(encoderFilterSub, 0, _videoSenderSub, 0,
			T_UNEXPECTED, "Connect: [VideoEncoder]Sub->VideoSenderSub");
	}
	else
	{
		AddFilter(_videoSenderMain);
		AddFilter(_videoSenderSub);
		ConnectDirect(_formatTransformSub, 0, _videoSenderMain, 0, 
			T_UNEXPECTED, "Connect: FormatTransformSub->VideoSenderMain");

		ConnectDirect(_videoExtract2, 0, _videoSenderSub, 0,
			T_UNEXPECTED, "Connect: VideoExtract2->VideoSenderSub");
	}

	AddFilter(_pointerReceiver1);
	ConnectDirect(_pointerReceiver1, 0, _videoExtract2, 1,
		T_UNEXPECTED, "Connect: PointerReceiver1->VideoExtract2");

	AddFilter(_pointerPosition);
	AddFilter(_pointerSender);
	ConnectDirect(_pointerPosition, 0, _pointerSender, 0,
		T_UNEXPECTED, "Connect: PointerPosition->PointerSender");

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSender::buildAudioGraph()
{
	// Try Connect to Async-Source Audio pin
	if (_playVideoClipOn && buildAudioGraphFromAsyncSource())
		return true;
		
	// Get Audio Source
	if (_audioSources.getSize() <= _audioSource)
		return true;
	fsFilterPtr sourceFilter = _audioSources[_audioSource];
	assert(sourceFilter != NULL);

	// Get Audio Encoder
	assert(_audioEncoder < _audioEncoders.getSize());
	fsFilterPtr encoderFilter = _audioEncoders[_audioEncoder];

	// Re-create Tee filter
	_audioInfPinTee	= new fsInfPinTeeFilter("Audio InfPinTee");


	//
	// Connection
	//
	if (_playSoundFileOn)
	{
		AddFilter(sourceFilter);
		AddFilter(_audioMixer);
		ConnectIndirect(sourceFilter, 0, _audioMixer, 0, 
			T_WRONG_CAPTURE, "Audio");
		AddFilter(_volumeIndicator);
		ConnectDirect(_audioMixer, 0, _volumeIndicator, 0,
			T_UNEXPECTED, "Connect: AudioMixer->VolumeIndicator");

		AddFilter(_asyncReader);
		ConnectIndirect(_asyncReader, 0, _audioMixer, 1, 
			T_UNEXPECTED, "Connect: AsyncReader->>AudioMixer");
	}
	else
	{
		AddFilter(sourceFilter);
		AddFilter(_volumeIndicator);
		ConnectIndirect(sourceFilter, 0, _volumeIndicator, 0,
			T_WRONG_CAPTURE, "Audio");
	}

	AddFilter(_audioSender);
	if (encoderFilter != NULL)
	{
		AddFilter(encoderFilter);
		ConnectIndirect(_volumeIndicator, 0, encoderFilter, 0,
			T_WRONG_ENCODER, "Audio");
		ConnectDirect(encoderFilter, 0, _audioSender, 0,
			T_UNEXPECTED, "Connect: [AudioEncoder]->AudioSender");
	}
	else
	{
		ConnectDirect(_volumeIndicator, 0, _audioSender, 0,
			T_UNEXPECTED, "Connect: VolumeIndicator->AudioSender");
	}

	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSender::buildAudioGraphFromAsyncSource()
{
	if (_asyncReader == NULL)
		return false;

	fsFilterPtr downFilter = _asyncReader->getDownStreamFilter(0);
	if (downFilter == NULL)
		return false;

	assert(_audioEncoder < _audioEncoders.getSize());
	fsFilterPtr encoder = _audioEncoders[_audioEncoder];
	if (encoder != NULL)
		AddFilter(encoder);

	AddFilter(_volumeIndicator);
	AddFilter(_audioSender);

	mxUInt numPins = downFilter->getNumOutputPins();
	mxUInt i = 0;
	for(i = 0; i < numPins; i++)
	{
		if (downFilter->isOutputConnected(i))
			continue;

		if (encoder != NULL)
		{
			if (!_graph->connectFilterIndirect(*downFilter, i, _volumeIndicator->cast<fsFilter>(), 0))
				continue;
			if (!_graph->connectFilterDirect(_volumeIndicator->cast<fsFilter>(), 0, *encoder, 0))
				continue;
			if (!_graph->connectFilterDirect(*encoder, 0, _audioSender->cast<fsFilter>(), 0))
				continue;

			break;
		}
		else
		{
			if (!_graph->connectFilterIndirect(*downFilter, i, _volumeIndicator->cast<fsFilter>(), 0))
				continue;
			if (!_graph->connectFilterDirect(_volumeIndicator->cast<fsFilter>(), 0, _audioSender->cast<fsFilter>(), 0))
				continue;

			break;
		}
	}

	if (i == numPins)
	{
		_graph->removeFilter(_audioSender->cast<fsFilter>());
		if (encoder != NULL)
			_graph->removeFilter(*encoder);
		return false;
	}

	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#undef AddFilter
#undef ConnectDirect
#undef ConnectIndirect
//------------------------------------------------------------------------------
