/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsSenderIf_h__
#define __fsSenderIf_h__
//------------------------------------------------------------------------------
#include <mx/mxCore/mxString.h>
#include <mx/mxCore/mxCore.h>
//------------------------------------------------------------------------------
class __declspec(novtable) fsSenderIf
{
public:

	//
	// Streaming Control
	//
	virtual mxBool			getValid() const = 0;
	virtual void			getErrorMessage(mxInt& id, mxInt& type, mxString& message) const = 0;
	virtual void			setStart(mxBool flag) = 0;
	virtual mxBool			getStart() const = 0;
	virtual void			setVideoTransmissionOn(mxBool flag) = 0;
	virtual mxBool			getVideoTransmissionOn() const = 0;
	virtual void			setAudioTransmissionOn(mxBool flag) = 0;
	virtual mxBool			getAudioTransmissionOn() const = 0;
	virtual void			setFocusTransmissionOn(mxBool flag) = 0;
	virtual mxBool			getFocusTransmissionOn() const = 0;


	//
	// Multicast Settings
	//
	virtual void			setAddress(const mxString& address) = 0;
	virtual const mxString&	getAddress() const = 0;
	virtual void			setTimeToLive(mxULong ttl) = 0;
	virtual mxULong			getTimeToLive() const = 0;
	virtual void			setTransmissionCount(mxUInt count) = 0;
	virtual mxUInt			getTransmissionCount() const = 0;
	virtual void			setPacketSize(mxUInt size) = 0;
	virtual mxUInt			getPacketSize() const = 0;

	// Create Address
	virtual mxString		createIPv4Address() = 0;
	virtual mxString		createIPv6Address() = 0;

	// Video Sender (Main)
	virtual void			setVideoSenderMainMaxTransmissionRate(mxInt rateInKBitPerSec) = 0;
	virtual mxInt			getVideoSenderMainMaxTransmissionRate() const = 0;
	virtual mxInt			getVideoSenderMainTransmissionRate() const = 0;
	virtual mxInt			getVideoSenderMainFrameRate() const = 0;

	// Video Sender (Sub)
	virtual void			setVideoSenderSubMaxTransmissionRate(mxInt rateInKBitPerSec) = 0;
	virtual mxInt			getVideoSenderSubMaxTransmissionRate() const = 0;
	virtual mxInt			getVideoSenderSubTransmissionRate() const = 0;
	virtual mxInt			getVideoSenderSubFrameRate() const = 0;

	// Audio Sender
	virtual void			setAudioSenderMaxTransmissionRate(mxInt rateInKBitPerSec) = 0;
	virtual mxInt			getAudioSenderMaxTransmissionRate() const = 0;
	virtual mxInt			getAudioSenderTransmissionRate() const = 0;
	virtual mxInt			getAudioSenderFrameRate() const = 0;

	// Pointer Sender
	virtual void			setPointerSenderMaxTransmissionRate(mxInt rateInKBitPerSec) = 0;
	virtual mxInt			getPointerSenderMaxTransmissionRate() const = 0;
	virtual mxInt			getPointerSenderTransmissionRate() const = 0;
	virtual mxInt			getPointerSenderFrameRate() const = 0;

	// Status Sender
	virtual void			setStatusSenderMaxTransmissionRate(mxInt rateInKBitPerSec) = 0;
	virtual mxInt			getStatusSenderMaxTransmissionRate() const = 0;


	//
	// Multicast Detector Settings
	//
	virtual mxBool			addMulticastDetector(const mxString& address) = 0;
	virtual mxBool			removeMulticastDetector(const mxString& address) = 0;
	virtual mxBool			wasDetectedMulticast(const mxString& address) = 0;


	//
	// Source Settings
	//

	// Video Source
	virtual mxInt			getNumVideoSources() const = 0;
	virtual void			setVideoSource(mxInt index) = 0;
	virtual mxInt			getVideoSource() const = 0;
	virtual const mxString&	getVideoSourceFriendlyName(mxInt index) = 0;

	// Audio Source
	virtual mxInt			getNumAudioSources() const = 0;
	virtual void			setAudioSource(mxInt index) = 0;
	virtual mxInt			getAudioSource() const = 0;
	virtual const mxString&	getAudioSourceFriendlyName(mxInt index) = 0;

	// Video Source property page
	virtual mxInt			getNumVideoSourcePropertyPages() const = 0;
	virtual const mxString&	getVideoSourcePropertyPageName(mxInt index) = 0;
    virtual void			showVideoSourcePropertyPage(HWND windowHandle, mxInt index) = 0;

	// Audio Source property page
	virtual mxInt			getNumAudioSourcePropertyPages() const = 0;
	virtual const mxString&	getAudioSourcePropertyPageName(mxInt index) = 0;
    virtual void			showAudioSourcePropertyPage(HWND windowHandle, mxInt index) = 0;

	// Video Clip
	virtual mxBool			isPlayVideoClip() const = 0;
	virtual mxBool			playVideoClip(const mxString& filename) = 0;

	// Sound File
	virtual mxBool			isPlaySoundFile() const = 0;
	virtual mxBool			playSoundFile(const mxString& filename) = 0;


	//
	// Encoder Settings
	//

	// Video Encoder
	virtual mxInt			getNumVideoEncoder() const = 0;
	virtual void			setVideoEncoder(mxInt index) = 0;
	virtual mxInt			getVideoEncoder() const = 0;
	virtual const mxString&	getVideoEncoderFriendlyName(mxInt index) const = 0;
	virtual mxInt			getVideoEncoderPriority(mxInt index) const = 0;

	// Audio Encoder
	virtual mxInt			getNumAudioEncoder() const = 0;
	virtual void			setAudioEncoder(mxInt index) = 0;
	virtual mxInt			getAudioEncoder() const = 0;
	virtual const mxString&	getAudioEncoderFriendlyName(mxInt index) const = 0;
	virtual mxInt			getAudioEncoderPriority(mxInt index) const = 0;


	//
	// Focus Display Settings
	//
	virtual void			setFocusDisplay(mxInt focusDisplay) = 0;
	virtual mxInt			getFocusDisplay() const = 0;
	virtual void			setFocusFrameVisible(mxBool flag) = 0;
	virtual mxBool			getFocusFrameVisible() const = 0;

	// Video Resolution
	virtual void			setVideoResolution(mxInt videoResolution) = 0;
	virtual mxInt			getVideoResolution() const = 0;

	// Partial Zoom
	virtual void			setPartialZoomSourceVideoResolution(mxInt videoResolution) = 0;
	virtual mxInt			getPartialZoomSourceVideoResolution() const = 0;
	virtual void			setPartialZoomOutputVideoResolution(mxInt videoResolution) = 0;
	virtual mxInt			getPartialZoomOutputVideoResolution() const = 0;

	// Non-Linear Zoom
	virtual void			setNonLinearZoomOutputVideoResolution(mxInt videoResolution) = 0;
	virtual mxInt			getNonLinearZoomOutputVideoResolution() const = 0;
	virtual void			setNonLinearZoomMaskType(mxInt maskType) = 0;
	virtual mxInt			getNonLinearZoomMaskType() const = 0;
	virtual void			setNonLinearZoomMaskSize(mxInt width, mxInt height) = 0;
	virtual void			getNonLinearZoomMaskSize(mxInt& width, mxInt& height) const = 0;

	// Image Compound
	virtual void			setImageCompoundBaseVideoResolution(mxInt videoResolution) = 0;
	virtual mxInt			getImageCompoundBaseVideoResolution() const = 0;
	virtual void			setImageCompoundAdditionalVideoResolution(mxInt videoResolution) = 0;
	virtual mxInt			getImageCompoundAdditionalVideoResolution() const = 0;
	virtual void			setImageCompoundMaskType(mxInt maskType) = 0;
	virtual mxInt			getImageCompoundMaskType() const = 0;
	virtual void			setImageCompoundBlendingType(mxInt blendingType) = 0;
	virtual mxInt			getImageCompoundBlendingType() const = 0;

	// Other Settings
	virtual void			setFocusCoordinate(mxInt coordinate) = 0;
	virtual mxInt			getFocusCoordinate() const = 0;
	virtual void			computeFocusRegion() = 0;


	//
	// Direct Video Mode Settings
	//
	virtual void			setDirectVideoModeOn(mxBool flag) = 0;
	virtual mxBool			getDirectVideoModeOn() const = 0;
	virtual void			setDVInterleavedModeOn(mxBool flag) = 0;
	virtual mxBool			getDVInterleavedModeOn() const = 0;
	virtual mxBool			isValidDVInterleavedMode() const = 0;


	//
	// Caption Settings
	//
	virtual void			setCaptionOn(mxBool flag) = 0;
	virtual mxBool			getCaptionOn() const = 0;
	virtual void			setCaption(const mxString& caption) = 0;
	virtual const mxString&	getCaption() const = 0;


	//
	// Video Window Settings
	//
	virtual void			setVideoWindowPolicy(mxInt policy) = 0;
	virtual mxInt			getVideoWindowPolicy() const = 0;
	virtual void			setKeepVideoAspect(mxBool flag) = 0;
	virtual mxBool			getKeepVideoAspect() const = 0;
	virtual void			setFitWindowToVideo(mxBool flag) = 0;
	virtual mxBool			getFitWindowToVideo() const = 0;
	virtual void			computeVideoWindow() = 0;


	//
	// Volume Settings
	//
	virtual void			getVolume(mxInt& left, mxInt& right) const = 0;
	virtual void			setVolumeEvaluationTime(mxInt evalTime) = 0;
	virtual mxInt			getVolumeEvaluationTime() const = 0;


	//
	// Other
	//
	virtual void			quit() = 0;
	virtual void			freeze(mxBool flag) = 0;
};
//------------------------------------------------------------------------------
#endif // __fsSenderIf_h__
