/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsSenderRMI/fsSenderSkel.h>
#include <fs/fsSenderRMI/fsSenderIf.h>
#include <fs/fsCommon/fsCommon.h>
#ifdef _DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------
fsSenderSkel::fsSenderSkel(fsSenderIf* sender) :
mxObject()
{
	_sender = sender;

	_rObject = new fsRemoteObject(invokeProc, this);

	fsRMIPropertyPtr property = new fsRMIProperty();
	property->setAddress(DEFAULT_ADDRESS);
	property->setPort(fsRMIProperty::PID_TRANSMIT, PORT_SENDER_REMOTEACK);
	property->setPort(fsRMIProperty::PID_RECEIVE, PORT_SENDER_REMOTE);
	property->setTimeToLive(1);
	property->setPacketSize(1024);
	property->setMaximumTransmissionRate(10 * 1024 * 1024);
	property->setTransmissionCount(2);
	_rObject->setProperty(property);
}
//------------------------------------------------------------------------------
fsSenderSkel::~fsSenderSkel()
{
	_rObject = NULL;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsSenderSkel::open()
{
	return _rObject->open();
}
//------------------------------------------------------------------------------
mxBool
fsSenderSkel::close()
{
	return _rObject->close();
}
//------------------------------------------------------------------------------
void
fsSenderSkel::setRMIProperty(const fsRMIPropertyPtr& property)
{
	_rObject->setProperty(property);
}
//------------------------------------------------------------------------------
const fsRMIPropertyPtr&
fsSenderSkel::getRMIProperty() const
{
	return _rObject->getProperty();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
const fsSenderIf*
fsSenderSkel::getSender() const
{
	return _sender;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsSenderSkel::invoke(const mxString& method,
							 fsInvokeData& param,
							 fsInvokeData& retVal)
{
	//
	// Streaming Control
	//
	if (method == "getValid")
	{
		retVal.writeBool(_sender->getValid());
	}
	else if (method == "getErrorMessage")
	{
		mxInt id;
		mxInt type;
		mxString message;
		_sender->getErrorMessage(id, type, message);
		retVal.writeInt(id);
		retVal.writeInt(type);
		retVal.writeString(message);
	}
	else if (method == "setStart")
	{
		_sender->setStart(param.readBool());
	}
	else if (method == "getStart")
	{
		retVal.writeBool(_sender->getStart());
	}
	else if (method == "setVideoTransmissionOn")
	{
		_sender->setVideoTransmissionOn(param.readBool());
	}
	else if (method == "getVideoTransmissionOn")
	{
		retVal.writeBool(_sender->getVideoTransmissionOn());
	}
	else if (method == "setAudioTransmissionOn")
	{
		_sender->setAudioTransmissionOn(param.readBool());
	}
	else if (method == "getAudioTransmissionOn")
	{
		retVal.writeBool(_sender->getAudioTransmissionOn());
	}
	else if (method == "setFocusTransmissionOn")
	{
		_sender->setFocusTransmissionOn(param.readBool());
	}
	else if (method == "getFocusTransmissionOn")
	{
		retVal.writeBool(_sender->getFocusTransmissionOn());
	}


	//
	// Multicast Settings
	//
	else if (method == "setAddress")
	{
		mxString address = param.readString();

		_sender->setAddress(address);

		const fsRMIPropertyPtr& property = _rObject->getProperty();
		property->setAddress(address);
		_rObject->setProperty(property);
	}
	else if (method == "getAddress")
	{
		retVal.writeString(_sender->getAddress());
	}
	else if (method == "setTimeToLive")
	{
		mxULong ttl = (mxULong)param.readInt();

		_sender->setTimeToLive(ttl);

		const fsRMIPropertyPtr& property = _rObject->getProperty();
		property->setTimeToLive(ttl);
		_rObject->setProperty(property);
	}
	else if (method == "getTimeToLive")
	{
		retVal.writeInt((mxInt)_sender->getTimeToLive());
	}
	else if (method == "setTransmissionCount")
	{
		_sender->setTransmissionCount((mxUInt)param.readInt());
	}
	else if (method == "getTransmissionCount")
	{
		retVal.writeInt((mxInt)_sender->getTransmissionCount());
	}
	else if (method == "setPacketSize")
	{
		_sender->setPacketSize((mxUInt)param.readInt());
	}
	else if (method == "getPacketSize")
	{
		retVal.writeInt((mxInt)_sender->getPacketSize());
	}

	// Create Address
	else if (method == "createIPv4Address")
	{
		retVal.writeString(_sender->createIPv4Address());
	}
	else if (method == "createIPv6Address")
	{
		retVal.writeString(_sender->createIPv6Address());
	}

	// Video Sender (Main)
	else if (method == "setVideoSenderMainMaxTransmissionRate")
	{
		_sender->setVideoSenderMainMaxTransmissionRate(param.readInt());
	}
	else if (method == "getVideoSenderMainMaxTransmissionRate")
	{
		retVal.writeInt(_sender->getVideoSenderMainMaxTransmissionRate());
	}
	else if (method == "getVideoSenderMainTransmissionRate")
	{
		retVal.writeInt(_sender->getVideoSenderMainTransmissionRate());
	}
	else if (method == "getVideoSenderMainFrameRate")
	{
		retVal.writeInt(_sender->getVideoSenderMainFrameRate());
	}

	// Video Sender (Sub)
	else if (method == "setVideoSenderSubMaxTransmissionRate")
	{
		_sender->setVideoSenderSubMaxTransmissionRate(param.readInt());
	}
	else if (method == "getVideoSenderSubMaxTransmissionRate")
	{
		retVal.writeInt(_sender->getVideoSenderSubMaxTransmissionRate());
	}
	else if (method == "getVideoSenderSubTransmissionRate")
	{
		retVal.writeInt(_sender->getVideoSenderSubTransmissionRate());
	}
	else if (method == "getVideoSenderSubFrameRate")
	{
		retVal.writeInt(_sender->getVideoSenderSubFrameRate());
	}

	// Audio Sender
	else if (method == "setAudioSenderMaxTransmissionRate")
	{
		_sender->setAudioSenderMaxTransmissionRate(param.readInt());
	}
	else if (method == "getAudioSenderMaxTransmissionRate")
	{
		retVal.writeInt(_sender->getAudioSenderMaxTransmissionRate());
	}
	else if (method == "getAudioSenderTransmissionRate")
	{
		retVal.writeInt(_sender->getAudioSenderTransmissionRate());
	}
	else if (method == "getAudioSenderFrameRate")
	{
		retVal.writeInt(_sender->getAudioSenderFrameRate());
	}

	// Pointer Sender
	else if (method == "setPointerSenderMaxTransmissionRate")
	{
		_sender->setPointerSenderMaxTransmissionRate(param.readInt());
	}
	else if (method == "getPointerSenderMaxTransmissionRate")
	{
		retVal.writeInt(_sender->getPointerSenderMaxTransmissionRate());
	}
	else if (method == "getPointerSenderTransmissionRate")
	{
		retVal.writeInt(_sender->getPointerSenderTransmissionRate());
	}
	else if (method == "getPointerSenderFrameRate")
	{
		retVal.writeInt(_sender->getPointerSenderFrameRate());
	}

	// Status Sender
	else if (method == "setStatusSenderMaxTransmissionRate")
	{
		_sender->setStatusSenderMaxTransmissionRate(param.readInt());
	}
	else if (method == "getStatusSenderMaxTransmissionRate")
	{
		retVal.writeInt(_sender->getStatusSenderMaxTransmissionRate());
	}


	//
	// Multicast Detector Settings
	//
	else if (method == "addMulticastDetector")
	{
		retVal.writeBool(_sender->addMulticastDetector(param.readString()));
	}
	else if (method == "removeMulticastDetector")
	{
		retVal.writeBool(_sender->removeMulticastDetector(param.readString()));
	}
	else if (method == "wasDetectedMulticast")
	{
		retVal.writeBool(_sender->wasDetectedMulticast(param.readString()));
	}


	//
	// Source Settings
	//

	// Video Source
	else if (method == "getNumVideoSources")
	{
		retVal.writeInt(_sender->getNumVideoSources());
	}
	else if (method == "setVideoSource")
	{
		_sender->setVideoSource(param.readInt());
	}
	else if (method == "getVideoSource")
	{
		retVal.writeInt(_sender->getVideoSource());
	}
	else if (method == "getVideoSourceFriendlyName")
	{
		retVal.writeString(_sender->getVideoSourceFriendlyName(param.readInt()));
	}

	// Audio Source
	else if (method == "getNumAudioSources")
	{
		retVal.writeInt(_sender->getNumAudioSources());
	}
	else if (method == "setAudioSource")
	{
		_sender->setAudioSource(param.readInt());
	}
	else if (method == "getAudioSource")
	{
		retVal.writeInt(_sender->getAudioSource());
	}
	else if (method == "getAudioSourceFriendlyName")
	{
		retVal.writeString(_sender->getAudioSourceFriendlyName(param.readInt()));
	}

	// Video Source property page
	else if (method == "getNumVideoSourcePropertyPages")
	{
		retVal.writeInt(_sender->getNumVideoSourcePropertyPages());
	}
	else if (method == "getVideoSourcePropertyPageName")
	{
		retVal.writeString(_sender->getVideoSourcePropertyPageName(param.readInt()));
	}
    else if (method == "showVideoSourcePropertyPage")
	{
		HWND windowHandle = (HWND)param.readInt();
		mxInt index = param.readInt();
		_sender->showVideoSourcePropertyPage(windowHandle, index);
	}

	// Audio Source property page
	else if (method == "getNumAudioSourcePropertyPages")
	{
		retVal.writeInt(_sender->getNumAudioSourcePropertyPages());
	}
	else if (method == "getAudioSourcePropertyPageName")
	{
		retVal.writeString(_sender->getAudioSourcePropertyPageName(param.readInt()));
	}
    else if (method == "showAudioSourcePropertyPage")
	{
		HWND windowHandle = (HWND)param.readInt();
		mxInt index = param.readInt();
		_sender->showAudioSourcePropertyPage(windowHandle, index);
	}

	// Video Clip
	else if (method == "isPlayVideoClip")
	{
		retVal.writeBool(_sender->isPlayVideoClip());
	}
	else if (method == "playVideoClip")
	{
		_sender->playVideoClip(param.readString());
	}

	// Sound File
	else if (method == "isPlaySoundFile")
	{
		retVal.writeBool(_sender->isPlaySoundFile());
	}
	else if (method == "playSoundFile")
	{
		_sender->playSoundFile(param.readString());
	}


	//
	// Encoder Settings
	//

	// Video Encoder
	else if (method == "getNumVideoEncoder")
	{
		retVal.writeInt(_sender->getNumVideoEncoder());
	}
	else if (method == "setVideoEncoder")
	{
		_sender->setVideoEncoder(param.readInt());
	}
	else if (method == "getVideoEncoder")
	{
		retVal.writeInt(_sender->getVideoEncoder());
	}
	else if (method == "getVideoEncoderFriendlyName")
	{
		retVal.writeString(_sender->getVideoEncoderFriendlyName(param.readInt()));
	}
	else if (method == "getVideoEncoderPriority")
	{
		retVal.writeInt(_sender->getVideoEncoderPriority(param.readInt()));
	}

	// Audio Encoder
	else if (method == "getNumAudioEncoder")
	{
		retVal.writeInt(_sender->getNumAudioEncoder());
	}
	else if (method == "setAudioEncoder")
	{
		_sender->setAudioEncoder(param.readInt());
	}
	else if (method == "getAudioEncoder")
	{
		retVal.writeInt(_sender->getAudioEncoder());
	}
	else if (method == "getAudioEncoderFriendlyName")
	{
		retVal.writeString(_sender->getAudioEncoderFriendlyName(param.readInt()));
	}
	else if (method == "getAudioEncoderPriority")
	{
		retVal.writeInt(_sender->getAudioEncoderPriority(param.readInt()));
	}


	//
	// Focus Display Settings
	//
	else if (method == "setFocusDisplay")
	{
		_sender->setFocusDisplay(param.readInt());
	}
	else if (method == "getFocusDisplay")
	{
		retVal.writeInt(_sender->getFocusDisplay());
	}
	else if (method == "setFocusFrameVisible")
	{
		_sender->setFocusFrameVisible(param.readBool());
	}
	else if (method == "getFocusFrameVisible")
	{
		retVal.writeBool(_sender->getFocusFrameVisible());
	}

	// Video Resolution
	else if (method == "setVideoResolution")
	{
		_sender->setVideoResolution(param.readInt());
	}
	else if (method == "getVideoResolution")
	{
		retVal.writeInt(_sender->getVideoResolution());
	}

	// Partial Zoom
	else if (method == "setPartialZoomSourceVideoResolution")
	{
		_sender->setPartialZoomSourceVideoResolution(param.readInt());
	}
	else if (method == "getPartialZoomSourceVideoResolution")
	{
		retVal.writeInt(_sender->getPartialZoomSourceVideoResolution());
	}
	else if (method == "setPartialZoomOutputVideoResolution")
	{
		_sender->setPartialZoomOutputVideoResolution(param.readInt());
	}
	else if (method == "getPartialZoomOutputVideoResolution")
	{
		retVal.writeInt(_sender->getPartialZoomOutputVideoResolution());
	}

	// Non-Linear Zoom
	else if (method == "setNonLinearZoomOutputVideoResolution")
	{
		_sender->setNonLinearZoomOutputVideoResolution(param.readInt());
	}
	else if (method == "getNonLinearZoomOutputVideoResolution")
	{
		retVal.writeInt(_sender->getNonLinearZoomOutputVideoResolution());
	}
	else if (method == "setNonLinearZoomMaskType")
	{
		_sender->setNonLinearZoomMaskType(param.readInt());
	}
	else if (method == "getNonLinearZoomMaskType")
	{
		retVal.writeInt(_sender->getNonLinearZoomMaskType());
	}
	else if (method == "setNonLinearZoomMaskSize")
	{
		mxInt size[2];
		param.readInt(size, 2);
		_sender->setNonLinearZoomMaskSize(size[0], size[1]);
	}
	else if (method == "getNonLinearZoomMaskSize")
	{
		mxInt size[2];
		_sender->getNonLinearZoomMaskSize(size[0], size[1]);
		retVal.writeInt(size, 2);
	}

	// Image Compound
	else if (method == "setImageCompoundBaseVideoResolution")
	{
		_sender->setImageCompoundBaseVideoResolution(param.readInt());
	}
	else if (method == "getImageCompoundBaseVideoResolution")
	{
		retVal.writeInt(_sender->getImageCompoundBaseVideoResolution());
	}
	else if (method == "setImageCompoundAdditionalVideoResolution")
	{
		_sender->setImageCompoundAdditionalVideoResolution(param.readInt());
	}
	else if (method == "getImageCompoundAdditionalVideoResolution")
	{
		retVal.writeInt(_sender->getImageCompoundAdditionalVideoResolution());
	}
	else if (method == "setImageCompoundMaskType")
	{
		_sender->setImageCompoundMaskType(param.readInt());
	}
	else if (method == "getImageCompoundMaskType")
	{
		retVal.writeInt(_sender->getImageCompoundMaskType());
	}
	else if (method == "setImageCompoundBlendingType")
	{
		_sender->setImageCompoundBlendingType(param.readInt());
	}
	else if (method == "getImageCompoundBlendingType")
	{
		retVal.writeInt(_sender->getImageCompoundBlendingType());
	}

	// Other Settings
	else if (method == "setFocusCoordinate")
	{
		_sender->setFocusCoordinate(param.readInt());
	}
	else if (method == "getFocusCoordinate")
	{
		retVal.writeInt(_sender->getFocusCoordinate());
	}
	else if (method == "computeFocusRegion")
	{
		_sender->computeFocusRegion();
	}


	//
	// Direct Video Mode Settings
	//
	else if (method == "setDirectVideoModeOn")
	{
		_sender->setDirectVideoModeOn(param.readBool());
	}
	else if (method == "getDirectVideoModeOn")
	{
		retVal.writeBool(_sender->getDirectVideoModeOn());
	}
	else if (method == "setDVInterleavedModeOn")
	{
		_sender->setDVInterleavedModeOn(param.readBool());
	}
	else if (method == "getDVInterleavedModeOn")
	{
		retVal.writeBool(_sender->getDVInterleavedModeOn());
	}
	else if (method == "isValidDVInterleavedMode")
	{
		retVal.writeBool(_sender->isValidDVInterleavedMode());
	}


	//
	// Caption Settings
	//
	else if (method == "setCaptionOn")
	{
		_sender->setCaptionOn(param.readBool());
	}
	else if (method == "getCaptionOn")
	{
		retVal.writeBool(_sender->getCaptionOn());
	}
	else if (method == "setCaption")
	{
		_sender->setCaption(param.readString());
	}
	else if (method == "getCaption")
	{
		retVal.writeString(_sender->getCaption());
	}


	//
	// Video Window Settings
	//
	else if (method == "setVideoWindowPolicy")
	{
		_sender->setVideoWindowPolicy(param.readInt());
	}
	else if (method == "getVideoWindowPolicy")
	{
		retVal.writeInt(_sender->getVideoWindowPolicy());
	}
	else if (method == "setKeepVideoAspect")
	{
		_sender->setKeepVideoAspect(param.readBool());
	}
	else if (method == "getKeepVideoAspect")
	{
		retVal.writeBool(_sender->getKeepVideoAspect());
	}
	else if (method == "setFitWindowToVideo")
	{
		_sender->setFitWindowToVideo(param.readBool());
	}
	else if (method == "getFitWindowToVideo")
	{
		retVal.writeBool(_sender->getFitWindowToVideo());
	}
	else if (method == "computeVideoWindow")
	{
		_sender->computeVideoWindow();
	}


	//
	// Volume Settings
	//
	else if (method == "getVolume")
	{
		mxInt lr[2];
		_sender->getVolume(lr[0], lr[1]);
		retVal.writeInt(lr, 2);
	}
	else if (method == "setVolumeEvaluationTime")
	{
		_sender->setVolumeEvaluationTime(param.readInt());
	}
	else if (method == "getVolumeEvaluationTime")
	{
		retVal.writeInt(_sender->getVolumeEvaluationTime());
	}


	//
	// Other
	//
	else if (method == "quit")
	{
		_sender->quit();
	}
	else if (method == "freeze")
	{
		_sender->freeze(param.readBool());
	}
}
//------------------------------------------------------------------------------
void
fsSenderSkel::invokeProc(const mxString& method,
								 fsInvokeData& param,
								 fsInvokeData& retVal,
								 void *data)
{
	fsSenderSkel *remoteObject = (fsSenderSkel *)data;
	remoteObject->invoke(method, param, retVal);
}
//------------------------------------------------------------------------------
