/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <fs/fsSenderRMI/fsSenderStub.h>
#include <fs/fsCommon/fsCommon.h>
//------------------------------------------------------------------------------
fsSenderStub::fsSenderStub() :
mxObject()
{
	_rInterface = new fsRemoteInterface(invokeNotifyProc, this);

	fsRMIPropertyPtr property = new fsRMIProperty();
	property->setAddress(DEFAULT_ADDRESS);
	property->setPort(fsRMIProperty::PID_TRANSMIT, PORT_SENDER_REMOTE);
	property->setPort(fsRMIProperty::PID_RECEIVE, PORT_SENDER_REMOTEACK);
	property->setTimeToLive(1);
	property->setPacketSize(1024);
	property->setMaximumTransmissionRate(10 * 1024 * 1024);
	property->setTransmissionCount(2);
	property->setInvokeTimeout(100);
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
fsSenderStub::~fsSenderStub()
{
	_rInterface = NULL;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
mxBool
fsSenderStub::open()
{
	return _rInterface->open();
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::close()
{
	return _rInterface->close();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setRMIProperty(const fsRMIPropertyPtr& property)
{
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
const fsRMIPropertyPtr&
fsSenderStub::getRMIProperty() const
{
	return _rInterface->getProperty();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Implements fsSenderIf
//
//------------------------------------------------------------------------------

fsInvokeData param;
fsInvokeData ret;
mxString str;

//------------------------------------------------------------------------------
//
// Streaming Control
//
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getValid() const
{
	param.reset();
	_rInterface->invoke("getValid", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsSenderStub::getErrorMessage(mxInt& id, mxInt& type, mxString& message) const
{
	param.reset();
	_rInterface->invoke("getErrorMessage", param, ret);
	id = ret.readInt();
	type = ret.readInt();
	message = ret.readString();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setStart(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setStart", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getStart() const
{
	param.reset();
	_rInterface->invoke("getStart", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setVideoTransmissionOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setVideoTransmissionOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getVideoTransmissionOn() const
{
	param.reset();
	_rInterface->invoke("getVideoTransmissionOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setAudioTransmissionOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setAudioTransmissionOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getAudioTransmissionOn() const
{
	param.reset();
	_rInterface->invoke("getAudioTransmissionOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setFocusTransmissionOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setFocusTransmissionOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getFocusTransmissionOn() const
{
	param.reset();
	_rInterface->invoke("getFocusTransmissionOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Multicast Settings
//
//------------------------------------------------------------------------------
void
fsSenderStub::setAddress(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("setAddress", param);

	const fsRMIPropertyPtr& property = _rInterface->getProperty();
	property->setAddress(address);
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
const mxString&
fsSenderStub::getAddress() const
{
	param.reset();
	_rInterface->invoke("getAddress", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
void
fsSenderStub::setTimeToLive(mxULong ttl)
{
	param.reset();
	param.writeInt((mxInt)ttl);
	_rInterface->invoke("setTimeToLive", param);

	const fsRMIPropertyPtr& property = _rInterface->getProperty();
	property->setTimeToLive(ttl);
	_rInterface->setProperty(property);
}
//------------------------------------------------------------------------------
mxULong
fsSenderStub::getTimeToLive() const
{
	param.reset();
	_rInterface->invoke("getTimeToLive", param, ret);
	return (mxULong)ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setTransmissionCount(mxUInt count)
{
	param.reset();
	param.writeInt((mxInt)count);
	_rInterface->invoke("setTransmissionCount", param);
}
//------------------------------------------------------------------------------
mxUInt
fsSenderStub::getTransmissionCount() const
{
	param.reset();
	_rInterface->invoke("getTransmissionCount", param, ret);
	return (mxUInt)ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setPacketSize(mxUInt size)
{
	param.reset();
	param.writeInt((mxInt)size);
	_rInterface->invoke("setPacketSize", param);
}
//------------------------------------------------------------------------------
mxUInt
fsSenderStub::getPacketSize() const
{
	param.reset();
	_rInterface->invoke("getPacketSize", param, ret);
	return (mxUInt)ret.readInt();
}
//------------------------------------------------------------------------------
// Create Address
mxString
fsSenderStub::createIPv4Address()
{
	param.reset();
	_rInterface->invoke("createIPv4Address", param, ret);
	return ret.readString();
}
//------------------------------------------------------------------------------
mxString
fsSenderStub::createIPv6Address()
{
	param.reset();
	_rInterface->invoke("createIPv6Address", param, ret);
	return ret.readString();
}
//------------------------------------------------------------------------------
// Video Sender (Main)
void
fsSenderStub::setVideoSenderMainMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	param.reset();
	param.writeInt(rateInKBitPerSec);
	_rInterface->invoke("setVideoSenderMainMaxTransmissionRate", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoSenderMainMaxTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getVideoSenderMainMaxTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoSenderMainTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getVideoSenderMainTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoSenderMainFrameRate() const
{
	param.reset();
	_rInterface->invoke("getVideoSenderMainFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// Video Sender (Sub)
void
fsSenderStub::setVideoSenderSubMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	param.reset();
	param.writeInt(rateInKBitPerSec);
	_rInterface->invoke("setVideoSenderSubMaxTransmissionRate", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoSenderSubMaxTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getVideoSenderSubMaxTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoSenderSubTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getVideoSenderSubTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoSenderSubFrameRate() const
{
	param.reset();
	_rInterface->invoke("getVideoSenderSubFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// Audio Sender
void
fsSenderStub::setAudioSenderMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	param.reset();
	param.writeInt(rateInKBitPerSec);
	_rInterface->invoke("setAudioSenderMaxTransmissionRate", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getAudioSenderMaxTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getAudioSenderMaxTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getAudioSenderTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getAudioSenderTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getAudioSenderFrameRate() const
{
	param.reset();
	_rInterface->invoke("getAudioSenderFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// Pointer Sender
void
fsSenderStub::setPointerSenderMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	param.reset();
	param.writeInt(rateInKBitPerSec);
	_rInterface->invoke("setPointerSenderMaxTransmissionRate", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getPointerSenderMaxTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getPointerSenderMaxTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getPointerSenderTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getPointerSenderTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getPointerSenderFrameRate() const
{
	param.reset();
	_rInterface->invoke("getPointerSenderFrameRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// Status Sender
void
fsSenderStub::setStatusSenderMaxTransmissionRate(mxInt rateInKBitPerSec)
{
	param.reset();
	param.writeInt(rateInKBitPerSec);
	_rInterface->invoke("setStatusSenderMaxTransmissionRate", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getStatusSenderMaxTransmissionRate() const
{
	param.reset();
	_rInterface->invoke("getStatusSenderMaxTransmissionRate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multicast Detector Settings
//
//------------------------------------------------------------------------------
mxBool
fsSenderStub::addMulticastDetector(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("addMulticastDetector", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::removeMulticastDetector(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("removeMulticastDetector", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::wasDetectedMulticast(const mxString& address)
{
	param.reset();
	param.writeString(address);
	_rInterface->invoke("wasDetectedMulticast", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Source Settings
//
//------------------------------------------------------------------------------
// Video Source
mxInt
fsSenderStub::getNumVideoSources() const
{
	param.reset();
	_rInterface->invoke("getNumVideoSources", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setVideoSource(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("setVideoSource", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoSource() const
{
	param.reset();
	_rInterface->invoke("getVideoSource", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
const mxString&
fsSenderStub::getVideoSourceFriendlyName(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getVideoSourceFriendlyName", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
// Audio Source
mxInt
fsSenderStub::getNumAudioSources() const
{
	param.reset();
	_rInterface->invoke("getNumAudioSources", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setAudioSource(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("setAudioSource", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getAudioSource() const
{
	param.reset();
	_rInterface->invoke("getAudioSource", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
const mxString&
fsSenderStub::getAudioSourceFriendlyName(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getAudioSourceFriendlyName", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
// Video Source Property Page
mxInt
fsSenderStub::getNumVideoSourcePropertyPages() const
{
	param.reset();
	_rInterface->invoke("getNumVideoSourcePropertyPages", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
const mxString&
fsSenderStub::getVideoSourcePropertyPageName(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getVideoSourcePropertyPageName", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
void
fsSenderStub::showVideoSourcePropertyPage(HWND windowHandle, mxInt index)
{
	param.reset();
	param.writeInt((mxInt)windowHandle);
	param.writeInt(index);
	_rInterface->invoke("showVideoSourcePropertyPage", param);
}
//------------------------------------------------------------------------------
// Audio Source Property Page
mxInt
fsSenderStub::getNumAudioSourcePropertyPages() const
{
	param.reset();
	_rInterface->invoke("getNumAudioSourcePropertyPages", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
const mxString&
fsSenderStub::getAudioSourcePropertyPageName(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getAudioSourcePropertyPageName", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
void
fsSenderStub::showAudioSourcePropertyPage(HWND windowHandle, mxInt index)
{
	param.reset();
	param.writeInt((mxInt)windowHandle);
	param.writeInt(index);
	_rInterface->invoke("showAudioSourcePropertyPage", param);
}
//------------------------------------------------------------------------------
// Video Clip
mxBool
fsSenderStub::isPlayVideoClip() const
{
	param.reset();
	_rInterface->invoke("isPlayVideoClip", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::playVideoClip(const mxString& filename)
{
	param.reset();
	param.writeString(filename);
	_rInterface->invoke("playVideoClip", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
// Sound File
mxBool
fsSenderStub::isPlaySoundFile() const
{
	param.reset();
	_rInterface->invoke("isPlaySoundFile", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::playSoundFile(const mxString& filename)
{
	param.reset();
	param.writeString(filename);
	_rInterface->invoke("playSoundFile", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
// Encoder Settings
//
//------------------------------------------------------------------------------
// Video Encoder
mxInt
fsSenderStub::getNumVideoEncoder() const
{
	param.reset();
	_rInterface->invoke("getNumVideoEncoder", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setVideoEncoder(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("setVideoEncoder", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoEncoder() const
{
	param.reset();
	_rInterface->invoke("getVideoEncoder", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
const mxString&
fsSenderStub::getVideoEncoderFriendlyName(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getVideoEncoderFriendlyName", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoEncoderPriority(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getVideoEncoderPriority", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// Audio Encoder
mxInt
fsSenderStub::getNumAudioEncoder() const
{
	param.reset();
	_rInterface->invoke("getNumAudioEncoder", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setAudioEncoder(mxInt index)
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("setAudioEncoder", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getAudioEncoder() const
{
	param.reset();
	_rInterface->invoke("getAudioEncoder", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
const mxString&
fsSenderStub::getAudioEncoderFriendlyName(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getAudioEncoderFriendlyName", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getAudioEncoderPriority(mxInt index) const
{
	param.reset();
	param.writeInt(index);
	_rInterface->invoke("getAudioEncoderPriority", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Focus Display Settings
//
//------------------------------------------------------------------------------
void
fsSenderStub::setFocusDisplay(mxInt focusDisplay)
{
	param.reset();
	param.writeInt(focusDisplay);
	_rInterface->invoke("setFocusDisplay", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getFocusDisplay() const
{
	param.reset();
	_rInterface->invoke("getFocusDisplay", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setFocusFrameVisible(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setFocusFrameVisible", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getFocusFrameVisible() const
{
	param.reset();
	_rInterface->invoke("getFocusFrameVisible", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
// Video Resolution
void
fsSenderStub::setVideoResolution(mxInt videoResolution)
{
	param.reset();
	param.writeInt(videoResolution);
	_rInterface->invoke("setVideoResolution", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoResolution() const
{
	param.reset();
	_rInterface->invoke("getVideoResolution", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
// Partial Zoom
void
fsSenderStub::setPartialZoomSourceVideoResolution(mxInt videoResolution)
{
	param.reset();
	param.writeInt(videoResolution);
	_rInterface->invoke("setPartialZoomSourceVideoResolution", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getPartialZoomSourceVideoResolution() const
{
	param.reset();
	_rInterface->invoke("getPartialZoomSourceVideoResolution", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setPartialZoomOutputVideoResolution(mxInt videoResolution)
{
	param.reset();
	param.writeInt(videoResolution);
	_rInterface->invoke("setPartialZoomOutputVideoResolution", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getPartialZoomOutputVideoResolution() const
{
	param.reset();
	_rInterface->invoke("getPartialZoomOutputVideoResolution", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setNonLinearZoomOutputVideoResolution(mxInt videoResolution)
{
	param.reset();
	param.writeInt(videoResolution);
	_rInterface->invoke("setNonLinearZoomOutputVideoResolution", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getNonLinearZoomOutputVideoResolution() const
{
	param.reset();
	_rInterface->invoke("getNonLinearZoomOutputVideoResolution", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setNonLinearZoomMaskType(mxInt maskType)
{
	param.reset();
	param.writeInt(maskType);
	_rInterface->invoke("setNonLinearZoomMaskType", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getNonLinearZoomMaskType() const
{
	param.reset();
	_rInterface->invoke("getNonLinearZoomMaskType", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setNonLinearZoomMaskSize(mxInt width, mxInt height)
{
	mxInt size[2];
	size[0] = width;
	size[1] = height;
	param.reset();
	param.writeInt(size, 2);
	_rInterface->invoke("setNonLinearZoomMaskSize", param);
}
//------------------------------------------------------------------------------
void
fsSenderStub::getNonLinearZoomMaskSize(mxInt& width, mxInt& height) const
{
	param.reset();
	_rInterface->invoke("getNonLinearZoomMaskSize", param, ret);
	mxInt size[2];
	ret.readInt(size, 2);
	width = size[0];
	height = size[1];
}
//------------------------------------------------------------------------------
void
fsSenderStub::setImageCompoundBaseVideoResolution(mxInt videoResolution)
{
	param.reset();
	param.writeInt(videoResolution);
	_rInterface->invoke("setImageCompoundBaseVideoResolution", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getImageCompoundBaseVideoResolution() const
{
	param.reset();
	_rInterface->invoke("getImageCompoundBaseVideoResolution", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setImageCompoundAdditionalVideoResolution(mxInt videoResolution)
{
	param.reset();
	param.writeInt(videoResolution);
	_rInterface->invoke("setImageCompoundAdditionalVideoResolution", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getImageCompoundAdditionalVideoResolution() const
{
	param.reset();
	_rInterface->invoke("getImageCompoundAdditionalVideoResolution", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setImageCompoundMaskType(mxInt maskType)
{
	param.reset();
	param.writeInt(maskType);
	_rInterface->invoke("setImageCompoundMaskType", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getImageCompoundMaskType() const
{
	param.reset();
	_rInterface->invoke("getImageCompoundMaskType", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setImageCompoundBlendingType(mxInt blendingType)
{
	param.reset();
	param.writeInt(blendingType);
	_rInterface->invoke("setImageCompoundBlendingType", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getImageCompoundBlendingType() const
{
	param.reset();
	_rInterface->invoke("getImageCompoundBlendingType", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setFocusCoordinate(mxInt coordinate)
{
	param.reset();
	param.writeInt(coordinate);
	_rInterface->invoke("setFocusCoordinate", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getFocusCoordinate() const
{
	param.reset();
	_rInterface->invoke("getFocusCoordinate", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::computeFocusRegion()
{
	_rInterface->invoke("computeFocusRegion");
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Direct Video Mode Settings
//
//------------------------------------------------------------------------------
void
fsSenderStub::setDirectVideoModeOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setDirectVideoModeOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getDirectVideoModeOn() const
{
	param.reset();
	_rInterface->invoke("getDirectVideoModeOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setDVInterleavedModeOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setDVInterleavedModeOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getDVInterleavedModeOn() const
{
	param.reset();
	_rInterface->invoke("getDVInterleavedModeOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::isValidDVInterleavedMode() const
{
	param.reset();
	_rInterface->invoke("isValidDVInterleavedMode", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Caption Settings
//
//------------------------------------------------------------------------------
void
fsSenderStub::setCaptionOn(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setCaptionOn", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getCaptionOn() const
{
	param.reset();
	_rInterface->invoke("getCaptionOn", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setCaption(const mxString& caption)
{
	param.reset();
	param.writeString(caption);
	_rInterface->invoke("setCaption", param);
}
//------------------------------------------------------------------------------
mxString captionStr;
//------------------------------------------------------------------------------
const mxString&
fsSenderStub::getCaption() const
{
	param.reset();
	_rInterface->invoke("getCaption", param, ret);
	str = ret.readString();
	return str;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Video Window Settings
//
//------------------------------------------------------------------------------
void
fsSenderStub::setVideoWindowPolicy(mxInt policy)
{
	param.reset();
	param.writeInt(policy);
	_rInterface->invoke("setVideoWindowPolicy", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVideoWindowPolicy() const
{
	param.reset();
	_rInterface->invoke("getVideoWindowPolicy", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setKeepVideoAspect(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setKeepVideoAspect", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getKeepVideoAspect() const
{
	param.reset();
	_rInterface->invoke("getKeepVideoAspect", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsSenderStub::setFitWindowToVideo(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("setFitWindowToVideo", param);
}
//------------------------------------------------------------------------------
mxBool
fsSenderStub::getFitWindowToVideo() const
{
	param.reset();
	_rInterface->invoke("getFitWindowToVideo", param, ret);
	return ret.readBool();
}
//------------------------------------------------------------------------------
void
fsSenderStub::computeVideoWindow()
{
	_rInterface->invoke("computeVideoWindow");
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Volume Settings
//
//------------------------------------------------------------------------------
void
fsSenderStub::getVolume(mxInt& left, mxInt& right) const
{
	param.reset();
	_rInterface->invoke("getVolume", param, ret);
	mxInt lr[2];
	ret.readInt(lr, 2);
	left = lr[0];
	right = lr[1];
}
//------------------------------------------------------------------------------
void
fsSenderStub::setVolumeEvaluationTime(mxInt evalTime)
{
	param.reset();
	param.writeInt(evalTime);
	_rInterface->invoke("setVolumeEvaluationTime", param);
}
//------------------------------------------------------------------------------
mxInt
fsSenderStub::getVolumeEvaluationTime() const
{
	param.reset();
	_rInterface->invoke("getVolumeEvaluationTime", param, ret);
	return ret.readInt();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Other
//
//------------------------------------------------------------------------------
void
fsSenderStub::quit()
{
	_rInterface->invoke("quit");
}
//------------------------------------------------------------------------------
void
fsSenderStub::freeze(mxBool flag)
{
	param.reset();
	param.writeBool(flag);
	_rInterface->invoke("freeze", param);
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
void
fsSenderStub::invokeNotifyProc(const mxString& method, mxBool result, void *data)
{
	fsSenderStub *sender = (fsSenderStub *)data;
	sender->invokeNotify(method, result);
}
//------------------------------------------------------------------------------
