/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef __fsSenderStub_h__
#define __fsSenderStub_h__
//------------------------------------------------------------------------------
#include <fs/fsSenderRMI/fsSenderIf.h>
#include <fs/fsSenderRMI/fsSenderRMI.h>
#include <fs/fsMulticastTools/fsRemoteInterface.h>
#include <mx/mxCore/mxObject.h>
//------------------------------------------------------------------------------
class fsSenderStub;
typedef mxObjectPtr<fsSenderStub>	fsSenderStubPtr;
//------------------------------------------------------------------------------
class FSSENDERRMICLASS fsSenderStub : public mxObject, public fsSenderIf
{
	MXOBJECTHEADER(fsSenderStub, mxObject);

public:
	fsSenderStub();

protected:
	virtual ~fsSenderStub();

public:
	mxBool				open();
	mxBool				close();
	void				setRMIProperty(const fsRMIPropertyPtr& property);
	const fsRMIPropertyPtr&	getRMIProperty() const;

public:
	//
	// Implements fsSenderIf
	//

	//
	// Streaming Control
	//
	mxBool				getValid() const;
	void				getErrorMessage(mxInt& id, mxInt& type, mxString& message) const;
	void				setStart(mxBool flag);
	mxBool				getStart() const;
	void				setVideoTransmissionOn(mxBool flag);
	mxBool				getVideoTransmissionOn() const;
	void				setAudioTransmissionOn(mxBool flag);
	mxBool				getAudioTransmissionOn() const;
	void				setFocusTransmissionOn(mxBool flag);
	mxBool				getFocusTransmissionOn() const;

	//
	// Multicast Settings
	//
	void				setAddress(const mxString& address);
	const mxString&		getAddress() const;
	void				setTimeToLive(mxULong ttl);
	mxULong				getTimeToLive() const;
	void				setTransmissionCount(mxUInt count);
	mxUInt				getTransmissionCount() const;
	void				setPacketSize(mxUInt size);
	mxUInt				getPacketSize() const;

	// Create Address
	mxString			createIPv4Address();
	mxString			createIPv6Address();

	// Video Sender (Main)
	void				setVideoSenderMainMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getVideoSenderMainMaxTransmissionRate() const;
	mxInt				getVideoSenderMainTransmissionRate() const;
	mxInt				getVideoSenderMainFrameRate() const;

	// Video Sender (Sub)
	void				setVideoSenderSubMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getVideoSenderSubMaxTransmissionRate() const;
	mxInt				getVideoSenderSubTransmissionRate() const;
	mxInt				getVideoSenderSubFrameRate() const;

	// Audio Sender
	void				setAudioSenderMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getAudioSenderMaxTransmissionRate() const;
	mxInt				getAudioSenderTransmissionRate() const;
	mxInt				getAudioSenderFrameRate() const;

	// Pointer Sender
	void				setPointerSenderMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getPointerSenderMaxTransmissionRate() const;
	mxInt				getPointerSenderTransmissionRate() const;
	mxInt				getPointerSenderFrameRate() const;

	// Status Sender
	void				setStatusSenderMaxTransmissionRate(mxInt rateInKBitPerSec);
	mxInt				getStatusSenderMaxTransmissionRate() const;


	//
	// Multicast Detector Settings
	//
	mxBool				addMulticastDetector(const mxString& address);
	mxBool				removeMulticastDetector(const mxString& address);
	mxBool				wasDetectedMulticast(const mxString& address);


	//
	// Source Settings
	//

	// Video Source
	mxInt				getNumVideoSources() const;
	void				setVideoSource(mxInt index);
	mxInt				getVideoSource() const;
	const mxString&		getVideoSourceFriendlyName(mxInt index);

	// Audio Source
	mxInt				getNumAudioSources() const;
	void				setAudioSource(mxInt index);
	mxInt				getAudioSource() const;
	const mxString&		getAudioSourceFriendlyName(mxInt index);

	// Video Source property page
	mxInt				getNumVideoSourcePropertyPages() const;
	const mxString&		getVideoSourcePropertyPageName(mxInt index);
    void				showVideoSourcePropertyPage(HWND windowHandle, mxInt index);

	// Audio Source property page
	mxInt				getNumAudioSourcePropertyPages() const;
	const mxString&		getAudioSourcePropertyPageName(mxInt index);
    void				showAudioSourcePropertyPage(HWND windowHandle, mxInt index);

	// Video Clip
	mxBool				isPlayVideoClip() const;
	mxBool				playVideoClip(const mxString& filename);

	// Sound File
	mxBool				isPlaySoundFile() const;
	mxBool				playSoundFile(const mxString& filename);


	//
	// Encoder Settings
	//

	// Video Encoder
	mxInt				getNumVideoEncoder() const;
	void				setVideoEncoder(mxInt index);
	mxInt				getVideoEncoder() const;
	const mxString&		getVideoEncoderFriendlyName(mxInt index) const;
	mxInt				getVideoEncoderPriority(mxInt index) const;

	// Audio Encoder
	mxInt				getNumAudioEncoder() const;
	void				setAudioEncoder(mxInt index);
	mxInt				getAudioEncoder() const;
	const mxString&		getAudioEncoderFriendlyName(mxInt index) const;
	mxInt				getAudioEncoderPriority(mxInt index) const;


	//
	// Focus Display Settings
	//
	void				setFocusDisplay(mxInt focusDisplay);
	mxInt				getFocusDisplay() const;
	void				setFocusFrameVisible(mxBool flag);
	mxBool				getFocusFrameVisible() const;

	// Video Resolution
	void				setVideoResolution(mxInt videoResolution);
	mxInt				getVideoResolution() const;

	// Partial Zoom
	void				setPartialZoomSourceVideoResolution(mxInt videoResolution);
	mxInt				getPartialZoomSourceVideoResolution() const;
	void				setPartialZoomOutputVideoResolution(mxInt videoResolution);
	mxInt				getPartialZoomOutputVideoResolution() const;

	// Non-Linear Zoom
	void				setNonLinearZoomOutputVideoResolution(mxInt videoResolution);
	mxInt				getNonLinearZoomOutputVideoResolution() const;
	void				setNonLinearZoomMaskType(mxInt maskType);
	mxInt				getNonLinearZoomMaskType() const;
	void				setNonLinearZoomMaskSize(mxInt width, mxInt height);
	void				getNonLinearZoomMaskSize(mxInt& width, mxInt& height) const;

	// Image Compound
	void				setImageCompoundBaseVideoResolution(mxInt videoResolution);
	mxInt				getImageCompoundBaseVideoResolution() const;
	void				setImageCompoundAdditionalVideoResolution(mxInt videoResolution);
	mxInt				getImageCompoundAdditionalVideoResolution() const;
	void				setImageCompoundMaskType(mxInt maskType);
	mxInt				getImageCompoundMaskType() const;
	void				setImageCompoundBlendingType(mxInt blendingType);
	mxInt				getImageCompoundBlendingType() const;

	// Other Settings
	void				setFocusCoordinate(mxInt coordinate);
	mxInt				getFocusCoordinate() const;
	void				computeFocusRegion();


	//
	// Direct Video Mode Settings
	//
	void				setDirectVideoModeOn(mxBool flag);
	mxBool				getDirectVideoModeOn() const;
	void				setDVInterleavedModeOn(mxBool flag);
	mxBool				getDVInterleavedModeOn() const;
	mxBool				isValidDVInterleavedMode() const;


	//
	// Caption Settings
	//
	void				setCaptionOn(mxBool flag);
	mxBool				getCaptionOn() const;
	void				setCaption(const mxString& caption);
	const mxString&		getCaption() const;


	//
	// Video Window Settings
	//
	void				setVideoWindowPolicy(mxInt policy);
	mxInt				getVideoWindowPolicy() const;
	void				setKeepVideoAspect(mxBool flag);
	mxBool				getKeepVideoAspect() const;
	void				setFitWindowToVideo(mxBool flag);
	mxBool				getFitWindowToVideo() const;
	void				computeVideoWindow();


	//
	// Volume Settings
	//
	void				getVolume(mxInt& left, mxInt& right) const;
	void				setVolumeEvaluationTime(mxInt evalTime);
	mxInt				getVolumeEvaluationTime() const;


	//
	// Other
	//
	void				quit();
	void				freeze(mxBool flag);

protected:
	virtual void		invokeNotify(const mxString& method, mxBool result) = 0;
	static void			invokeNotifyProc(const mxString& method, mxBool result, void *data);

protected:
	fsRemoteInterfacePtr		_rInterface;
};
//------------------------------------------------------------------------------
#endif //__fsSenderStub_h__
