/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsSenderRemote/fsSenderRemote.h>
#include <fs/fsCommon/fsCommon.h>
#include <mx/mxCore/mxRegistry.h>
#include <mx/mxCore/mxIniFile.h>
//------------------------------------------------------------------------------
#pragma comment(lib, "fsSenderRMI.lib")
#pragma comment(lib, "mxCore.lib")
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
BOOL APIENTRY
DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
    return TRUE;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
fsSenderRemote::fsSenderRemote() :
fsSenderStub()
{
	mxRegistry::setCompanyName(FS_REGISTRY_COMPANY_NAME);
	mxRegistry::setProductName(FS_REGISTRY_PRODUCT_NAME);

	_UICallbackFunction		= NULL;
	_UICallbackData			= NULL;
}
//------------------------------------------------------------------------------
fsSenderRemote::~fsSenderRemote()
{
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Callback
mxBool
fsSenderRemote::setUICallbackFunction(fsSenderUICallbackFunction func, void* data)
{
	_UICallbackFunction = func;
	_UICallbackData = data;
	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// IniFile
mxBool
fsSenderRemote::writeIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "a");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);

	mxString str;
	mxUInt index;


	//
	// Stream Control
	//
	str = getVideoTransmissionOn() ? "true" : "false";
	iniFile.writeString("Sender", "VideoTransmissionOn", str);

	str = getAudioTransmissionOn() ? "true" : "false";
	iniFile.writeString("Sender", "AudioTransmissionOn", str);

	str = getFocusTransmissionOn() ? "true" : "false";
	iniFile.writeString("Sender", "FocusTransmissionOn", str);


	//
	// Multicast Settings
	//
	iniFile.writeString("Sender", "MulticastAddress", getAddress());
	iniFile.writeUInt("Sender", "MulticastTimeToLive", (mxUInt)getTimeToLive());
	iniFile.writeUInt("Sender", "MulticastTransmissionCount", getTransmissionCount());
	iniFile.writeUInt("Sender", "MulticastPacketSize", getPacketSize());

	// Video Sender (Main)
	iniFile.writeUInt("Sender", "VideoSenderMainMaxTransmissionRate",
							getVideoSenderMainMaxTransmissionRate());

	// Video Sender (Sub)
	iniFile.writeUInt("Sender", "VideoSenderSubMaxTransmissionRate",
							getVideoSenderSubMaxTransmissionRate());

	// Audio Sender
	iniFile.writeUInt("Sender", "AudioSenderMaxTransmissionRate",
							getAudioSenderMaxTransmissionRate());

	// Pointer Sender / Receiver
	iniFile.writeUInt("Sender", "PointerSenderReceiverMaxTransmissionRate",
							getPointerSenderMaxTransmissionRate());

	// Status Sender
	iniFile.writeUInt("Sender", "StatusSenderMaxTransmissionRate",
							getStatusSenderMaxTransmissionRate());


	//
	// Source
	//
	// Video Source
	index = getVideoSource();
	iniFile.writeString("Sender", "VideoSource", getVideoSourceFriendlyName(index));

	// Audio Source
	index = getAudioSource();
	iniFile.writeString("Sender", "AudioSource", getAudioSourceFriendlyName(index));


	//
	// Encoder
	//
	// Video Encoder
	index = getVideoEncoder();
	iniFile.writeString("Sender", "VideoEncoder", getVideoEncoderFriendlyName(index));

	// Audio Encoder
	index = getAudioEncoder();
	iniFile.writeString("Sender", "AudioEncoder", getAudioEncoderFriendlyName(index));


	//
	// Focus Display
	//
	FS_FOCUSDISPLAY2STR(getFocusDisplay(), str);
	iniFile.writeString("Sender", "FocusDisplay", str);

	str = getFocusFrameVisible() ? "true" : "false";
	iniFile.writeString("Sender", "FocusDisplayFocusFrameVisible", str);

	// VideoResolution
	FS_RESOLUTION2STR(getVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayVideoResolution", str);

	// PartialZoom
	FS_RESOLUTION2STR(getPartialZoomSourceVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayPartialZoomSourceVideoResolution", str);

	FS_RESOLUTION2STR(getPartialZoomOutputVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayPartialZoomOutputVideoResolution", str);

	// NonLinearZoom Format
	FS_RESOLUTION2STR(getNonLinearZoomOutputVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayNonLinearZoomVideoResolution", str);

	// NonLinearZoom Mask Type
	FS_MASKTYPE2STR(getNonLinearZoomMaskType(), str);
	iniFile.writeString("Sender", "FocusDisplayNonLinearZoomMaskType", str);

	// NonLinearZoom Mask Size
	mxInt width, height;
	getNonLinearZoomMaskSize(width, height);
	iniFile.writeUInt("Sender", "FocusDisplayNonLinearZoomMaskWidth", (mxUInt)width);
	iniFile.writeUInt("Sender", "FocusDisplayNonLinearZoomMaskHeight", (mxUInt)height);

	// ImageCompound Base Video Format
	FS_RESOLUTION2STR(getImageCompoundBaseVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayImageCompoundBaseVideoResolution", str);

	// ImageCompound Additional Video Format
	FS_RESOLUTION2STR(getImageCompoundAdditionalVideoResolution(), str);
	iniFile.writeString("Sender", "FocusDisplayImageCompoundAdditionalVideoResolution", str);

	// ImageCompound Mask Type
	FS_MASKTYPE2STR(getImageCompoundMaskType(), str);
	iniFile.writeString("Sender", "FocusDisplayImageCompoundMaskType",str);

	// ImageCompound Blending Type
	FS_BLENDINGTYPE2STR(getImageCompoundBlendingType(), str);
	iniFile.writeString("Sender", "FocusDisplayImageCompoundBlendingType", str);

	// FocusCoordinate
	FS_COORDINATE2STR(getFocusCoordinate(), str);
	iniFile.writeString("Sender", "FocusCoordinate", str);


	//
	// Direct Video Mode
	//
	iniFile.writeString("Sender", "DirectVideoMode",
										getDirectVideoModeOn() ? "true" : "false");

	iniFile.writeString("Sender", "DVInterleavedMode",
										getDVInterleavedModeOn() ? "true" : "false");


	//
	// Caption Settings
	//
	iniFile.writeString("Sender", "CaptionOn", getCaptionOn() ? "true" : "false");
	iniFile.writeString("Sender", "Caption", getCaption());


	return true;
}
//------------------------------------------------------------------------------
mxBool
fsSenderRemote::readIniFile(const mxString& filename)
{
	if (filename.length() == 0)
		return false;

	FILE *fp = fopen(filename.c_str(), "r");
	if (fp == NULL)
		return false;
	fclose(fp);

	mxIniFile iniFile(filename);

	mxString def, str;
	mxUInt num;
	mxInt index;

	// Stop Graph and StatusSender
	freeze(true);

	//
	// Stream Control
	//
	def = getVideoTransmissionOn() ? "true" : "false";
	str = iniFile.readString("Sender", "VideoTransmissionOn", def);
	str.toLower();
	setVideoTransmissionOn((str == "true"));

	def = getAudioTransmissionOn() ? "true" : "false";
	str = iniFile.readString("Sender", "AudioTransmissionOn", str);
	str.toLower();
	setAudioTransmissionOn((str == "true"));

	def = getFocusTransmissionOn() ? "true" : "false";
	str = iniFile.readString("Sender", "FocusTransmissionOn", str);
	str.toLower();
	setFocusTransmissionOn((str == "true"));


	//
	// Multicast Settings
	//
	mxString address = iniFile.readString("Sender", "MulticastAddress", getAddress());
	mxULong ttl = iniFile.readUInt("Sender", "MulticastTimeToLive", getTimeToLive());
	mxUInt count = iniFile.readUInt("Sender", "MulticastTransmissionCount", getTransmissionCount());
	mxUInt size = iniFile.readUInt("Sender", "MulticastPacketSize", getPacketSize());
	setAddress(address);
	setTimeToLive(ttl);
	setTransmissionCount(count);
	setPacketSize(size);

	// Video Sender (Main)
	mxUInt val;
	val = iniFile.readUInt("Sender", "VideoSenderMainMaxTransmissionRate",
						getVideoSenderMainMaxTransmissionRate());
	setVideoSenderMainMaxTransmissionRate(val);

	// Video Sender (Sub)
	val = iniFile.readUInt("Sender", "VideoSenderSubMaxTransmissionRate",
						getVideoSenderSubMaxTransmissionRate());
	setVideoSenderSubMaxTransmissionRate(val);

	// Audio Sender
	val = iniFile.readUInt("Sender", "AudioSenderMaxTransmissionRate",
						getAudioSenderMaxTransmissionRate());
	setAudioSenderMaxTransmissionRate(val);

	// Pointer Sender / Receiver
	val = iniFile.readUInt("Sender", "PointerSenderReceiverMaxTransmissionRate",
						getPointerSenderMaxTransmissionRate());
	setPointerSenderMaxTransmissionRate(val);

	// Status Sender
	val = iniFile.readUInt("Sender", "StatusSenderMaxTransmissionRate",
						getStatusSenderMaxTransmissionRate());
	setStatusSenderMaxTransmissionRate(val);


	//
	// Source
	//
	// Video Source
	index = getVideoSource();
	num = getNumVideoSources();
	def = getVideoSourceFriendlyName(index);
	str = iniFile.readString("Sender", "VideoSource", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getVideoSourceFriendlyName(i))
		{
			setVideoSource(i);
			break;
		}
	}

	// Audio Source
	index = getAudioSource();
	num = getNumAudioSources();
	def = getAudioSourceFriendlyName(index);
	str = iniFile.readString("Sender", "AudioSource", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getAudioSourceFriendlyName(i))
		{
			setAudioSource(i);
			break;
		}
	}


	//
	// Encoder
	//
	// Video Encoder
	index = getVideoEncoder();
	num = getNumVideoEncoder();
	def = getVideoEncoderFriendlyName(index);
	str = iniFile.readString("Sender", "VideoEncoder", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getVideoEncoderFriendlyName(i))
		{
			setVideoEncoder(i);
			break;
		}
	}

	// Audio Encoder
	index = getAudioEncoder();
	num = getNumVideoEncoder();
	def = getAudioEncoderFriendlyName(index);
	str = iniFile.readString("Sender", "AudioEncoder", def);
	for(mxUInt i = 0; i < num; i++)
	{
		if (str == getAudioEncoderFriendlyName(i))
		{
			setAudioEncoder(i);
			break;
		}
	}


	//
	// Focus Display
	//
	FS_FOCUSDISPLAY2STR(getFocusDisplay(), def);
	str = iniFile.readString("Sender", "FocusDisplay", def);
	FS_STR2FOCUSDISPLAY(str, index);
	setFocusDisplay(index);

	def = getFocusFrameVisible() ? "true" : "false";
	str = iniFile.readString("Sender", "FocusDisplayFocusFrameVisible", def);
	str.toLower();
	setFocusFrameVisible((str == "true"));

	// VideoResolution
	FS_RESOLUTION2STR(getVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setVideoResolution(index);

	// PartialZoom
	FS_RESOLUTION2STR(getPartialZoomSourceVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayPartialZoomSourceVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setPartialZoomSourceVideoResolution(index);

	FS_RESOLUTION2STR(getPartialZoomOutputVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayPartialZoomOutputVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setPartialZoomOutputVideoResolution(index);

	// NonLinearZoom Format
	FS_RESOLUTION2STR(getNonLinearZoomOutputVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayNonLinearZoomVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setNonLinearZoomOutputVideoResolution(index);

	// NonLinearZoom Mask Type
	FS_MASKTYPE2STR(getNonLinearZoomMaskType(), def);
	str = iniFile.readString("Sender", "FocusDisplayNonLinearZoomMaskType", def);
	FS_STR2MASKTYPE(str, index);
	setNonLinearZoomMaskType(index);

	// NonLinearZoom Mask Size
	mxInt width, height;
	getNonLinearZoomMaskSize(width, height);
	width = (mxInt)iniFile.readUInt("Sender", "FocusDisplayNonLinearZoomMaskWidth", (mxUInt)width);
	height = (mxInt)iniFile.readUInt("Sender", "FocusDisplayNonLinearZoomMaskHeight", (mxUInt)height);
	setNonLinearZoomMaskSize(width, height);

	// ImageCompound Base Video Format
	FS_RESOLUTION2STR(getImageCompoundBaseVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayImageCompoundBaseVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setImageCompoundBaseVideoResolution(index);

	// ImageCompound Additional Video Format
	FS_RESOLUTION2STR(getImageCompoundAdditionalVideoResolution(), def);
	str = iniFile.readString("Sender", "FocusDisplayImageCompoundAdditionalVideoResolution", def);
	FS_STR2RESOLUTION(str, index);
	setImageCompoundAdditionalVideoResolution(index);

	// ImageCompound Mask Type
	FS_MASKTYPE2STR(getImageCompoundMaskType(), def);
	str = iniFile.readString("Sender", "FocusDisplayImageCompoundMaskType", def);
	FS_STR2MASKTYPE(str, index);
	setImageCompoundMaskType(index);

	// ImageCompound Blending Type
	FS_BLENDINGTYPE2STR(getImageCompoundBlendingType(), def);
	str = iniFile.readString("Sender", "FocusDisplayImageCompoundBlendingType", def);
	FS_STR2BLENDINGTYPE(str, index);
	setImageCompoundBlendingType(index);

	// FocusCoordinate
	FS_COORDINATE2STR(getFocusCoordinate(), def);
	str = iniFile.readString("Sender", "FocusCoordinate", def);
	FS_STR2COORDINATE(str, index);
	setFocusCoordinate(index);

	//
	// Direct Video Mode
	//
	def = getDirectVideoModeOn() ? "true" : "false";
	str = iniFile.readString("Sender", "DirectVideoMode", def);
	str.toLower();
	setDirectVideoModeOn((str == "true"));

	def = getDVInterleavedModeOn() ? "true" : "false";
	str = iniFile.readString("Sender", "DVInterleavedMode", def);
	str.toLower();
	setDVInterleavedModeOn((str == "true"));


	//
	// Caption Settings
	//
	def = getCaptionOn() ? "true" : "false";
	str = iniFile.readString("Sender", "CaptionOn", def);
	str.toLower();
	setCaptionOn((str == "true"));

	def = getCaption();
	str = iniFile.readString("Sender", "Caption", def);
	setCaption(str);


	// Start Graph and StatusSender
	freeze(false);


	return true;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Registry
void
fsSenderRemote::writeRegistry(const mxString& section,
									   const mxString& entry,
									   const mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	registry.writeString(section, entry, value);
	registry.closeKey();
}
//------------------------------------------------------------------------------
void
fsSenderRemote::readRegistry(const mxString& section,
									  const mxString& entry,
									  const mxString& defaultValue,
									  mxString& value)
{
	assert(section.length() != 0);
	assert(entry.length() != 0);

	mxRegistry registry;
	registry.createKey(VERSION_STRING, mxRegistry::RK_CURRENT_USER);
	value = registry.readString(section, entry, defaultValue);
	registry.closeKey();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
fsSenderRemote::invokeNotify(const mxString& method, mxBool result)
{
	if (_UICallbackFunction == NULL)
		return ;

	const mxChar* cstr = method.c_str();
	_UICallbackFunction(result ? "InvokeSucceeded" : "InvokeFailed", (void *)cstr, _UICallbackData);
}
//------------------------------------------------------------------------------
