/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#include "StdAfx.h"
#include <fs/fsSenderRemote/fsSenderRemoteInterface.h>
#include <fs/fsSenderRemote/fsSenderRemote.h>
#include <crtdbg.h>
//------------------------------------------------------------------------------

static fsSenderRemotePtr g_sender			= NULL;

//------------------------------------------------------------------------------

mxChar str[1024];

#define MXSTR2STR(__str, __mxstr)			\
	mxString mxstr = __mxstr;				\
	if (mxstr.length() != 0)				\
		strcpy(__str, mxstr.c_str());		\
	else									\
		__str[0] = 0;

//------------------------------------------------------------------------------
//
// Initialize / Shutdown
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderInitialize()
{
	mxCore::initialize();
	g_sender = new fsSenderRemote();
	return g_sender->open();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderShutdown()
{
	assert(g_sender != NULL);
	g_sender->close();
	g_sender = NULL;
	mxCore::shutdown();

#ifdef _DEBUG
	_CrtDumpMemoryLeaks();
#endif // _DEBUG

	return true;
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderIsInitialized()
{
	return (g_sender != NULL);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Streaming Control
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetValid()
{
	assert(g_sender != NULL);
	return g_sender->getValid();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderGetErrorMessage(int* id, int* type, char* message)
{
	assert(g_sender != NULL);

	mxString mes;
	g_sender->getErrorMessage(*id, *type, mes);

	MXSTR2STR(message, mes);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetStart(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setStart(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetStart()
{
	assert(g_sender != NULL);
	return g_sender->getStart();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetVideoTransmissionOn(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setVideoTransmissionOn(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetVideoTransmissionOn()
{
	assert(g_sender != NULL);
	return g_sender->getVideoTransmissionOn();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetAudioTransmissionOn(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setAudioTransmissionOn(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetAudioTransmissionOn()
{
	assert(g_sender != NULL);
	return g_sender->getAudioTransmissionOn();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetFocusTransmissionOn(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setFocusTransmissionOn(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetFocusTransmissionOn()
{
	assert(g_sender != NULL);
	return g_sender->getFocusTransmissionOn();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multicast Settings
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetAddress(const char* address)
{
	assert(g_sender != NULL);
	g_sender->setAddress(address);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC const char* MVSTD
fsSenderGetAddress()
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->getAddress());
	return str;
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetTimeToLive(int ttl)
{
	assert(g_sender != NULL);
	g_sender->setTimeToLive((mxULong)ttl);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetTimeToLive()
{
	assert(g_sender != NULL);
	return (int)g_sender->getTimeToLive();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetTransmissionCount(int count)
{
	assert(g_sender != NULL);
	g_sender->setTransmissionCount((mxUInt)count);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetTransmissionCount()
{
	assert(g_sender != NULL);
	return (int)g_sender->getTransmissionCount();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetPacketSize(int size)
{
	assert(g_sender != NULL);
	g_sender->setPacketSize((mxUInt)size);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetPacketSize()
{
	assert(g_sender != NULL);
	return (int)g_sender->getPacketSize();
}
//------------------------------------------------------------------------------
// Create Address
FSSENDERREMOTEPROC const char* MVSTD
fsSenderCreateIPv4Address()
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->createIPv4Address());
	return str;
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC const char* MVSTD
fsSenderCreateIPv6Address()
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->createIPv6Address());
	return str;
}
//------------------------------------------------------------------------------
// Video Sender (Main)
FSSENDERREMOTEPROC void MVSTD
fsSenderSetVideoSenderMainMaxTransmissionRate(int rateInKBitPerSec)
{
	assert(g_sender != NULL);
	g_sender->setVideoSenderMainMaxTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoSenderMainMaxTransmissionRate()
{
	assert(g_sender != NULL);
	return g_sender->getVideoSenderMainMaxTransmissionRate();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoSenderMainTransmissionRate()
{
	assert(g_sender != NULL);
	return g_sender->getVideoSenderMainTransmissionRate();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoSenderMainFrameRate()
{
	assert(g_sender != NULL);
	return g_sender->getVideoSenderMainFrameRate();
}
//------------------------------------------------------------------------------
// Video Sender (Sub)
FSSENDERREMOTEPROC void MVSTD
fsSenderSetVideoSenderSubMaxTransmissionRate(int rateInKBitPerSec)
{
	assert(g_sender != NULL);
	g_sender->setVideoSenderSubMaxTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoSenderSubMaxTransmissionRate()
{
	assert(g_sender != NULL);
	return g_sender->getVideoSenderSubMaxTransmissionRate();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoSenderSubTransmissionRate()
{
	assert(g_sender != NULL);
	return g_sender->getVideoSenderSubTransmissionRate();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoSenderSubFrameRate()
{
	assert(g_sender != NULL);
	return g_sender->getVideoSenderSubFrameRate();
}
//------------------------------------------------------------------------------
// Audio Sender
FSSENDERREMOTEPROC void MVSTD
fsSenderSetAudioSenderMaxTransmissionRate(int rateInKBitPerSec)
{
	assert(g_sender != NULL);
	g_sender->setAudioSenderMaxTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetAudioSenderMaxTransmissionRate()
{
	assert(g_sender != NULL);
	return g_sender->getAudioSenderMaxTransmissionRate();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetAudioSenderTransmissionRate()
{
	assert(g_sender != NULL);
	return g_sender->getAudioSenderTransmissionRate();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetAudioSenderFrameRate()
{
	assert(g_sender != NULL);
	return g_sender->getAudioSenderFrameRate();
}
//------------------------------------------------------------------------------
// Pointer Sender
FSSENDERREMOTEPROC void MVSTD
fsSenderSetPointerSenderMaxTransmissionRate(int rateInKBitPerSec)
{
	assert(g_sender != NULL);
	g_sender->setPointerSenderMaxTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetPointerSenderMaxTransmissionRate()
{
	assert(g_sender != NULL);
	return g_sender->getPointerSenderMaxTransmissionRate();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetPointerSenderTransmissionRate()
{
	assert(g_sender != NULL);
	return g_sender->getPointerSenderTransmissionRate();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetPointerSenderFrameRate()
{
	assert(g_sender != NULL);
	return g_sender->getPointerSenderFrameRate();
}
//------------------------------------------------------------------------------
// Status Sender
FSSENDERREMOTEPROC void MVSTD
fsSenderSetStatusSenderMaxTransmissionRate(int rateInKBitPerSec)
{
	assert(g_sender != NULL);
	g_sender->setStatusSenderMaxTransmissionRate(rateInKBitPerSec);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetStatusSenderMaxTransmissionRate()
{
	assert(g_sender != NULL);
	return g_sender->getStatusSenderMaxTransmissionRate();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Multicast Detector Settings
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderAddMulticastDetector(const char* address)
{
	assert(g_sender != NULL);
	return g_sender->addMulticastDetector(address);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderRemoveMulticastDetector(const char* address)
{
	assert(g_sender != NULL);
	return g_sender->removeMulticastDetector(address);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderWasDetectedMulticast(const char* address)
{
	assert(g_sender != NULL);
	return g_sender->wasDetectedMulticast(address);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Source Settings
//
//------------------------------------------------------------------------------
// Video Source
FSSENDERREMOTEPROC int MVSTD
fsSenderGetNumVideoSources()
{
	assert(g_sender != NULL);
	return g_sender->getNumVideoSources();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetVideoSource(int index)
{
	assert(g_sender != NULL);
	g_sender->setVideoSource(index);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoSource()
{
	assert(g_sender != NULL);
	return g_sender->getVideoSource();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC const char* MVSTD
fsSenderGetVideoSourceFriendlyName(int index)
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->getVideoSourceFriendlyName(index));
	return str;
}
//------------------------------------------------------------------------------
// Audio Source
FSSENDERREMOTEPROC int MVSTD
fsSenderGetNumAudioSources()
{
	assert(g_sender != NULL);
	return g_sender->getNumAudioSources();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetAudioSource(int index)
{
	assert(g_sender != NULL);
	g_sender->setAudioSource(index);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetAudioSource()
{
	assert(g_sender != NULL);
	return g_sender->getAudioSource();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC const char* MVSTD
fsSenderGetAudioSourceFriendlyName(int index)
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->getAudioSourceFriendlyName(index));
	return str;
}
//------------------------------------------------------------------------------
// Video Source Property Page
FSSENDERREMOTEPROC int MVSTD
fsSenderGetNumVideoSourcePropertyPages()
{
	assert(g_sender != NULL);
	return g_sender->getNumVideoSourcePropertyPages();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC const char* MVSTD
fsSenderGetVideoSourcePropertyPageName(int index)
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->getVideoSourcePropertyPageName(index));
	return str;
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderShowVideoSourcePropertyPage(HWND windowHandle, int index)
{
	assert(g_sender != NULL);
	g_sender->showVideoSourcePropertyPage(windowHandle, index);
}
//------------------------------------------------------------------------------
// Audio Source Property Page
FSSENDERREMOTEPROC int MVSTD
fsSenderGetNumAudioSourcePropertyPages()
{
	assert(g_sender != NULL);
	return g_sender->getNumAudioSourcePropertyPages();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC const char* MVSTD
fsSenderGetAudioSourcePropertyPageName(int index)
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->getAudioSourcePropertyPageName(index));
	return str;
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderShowAudioSourcePropertyPage(HWND windowHandle, int index)
{
	assert(g_sender != NULL);
	g_sender->showAudioSourcePropertyPage(windowHandle, index);
}
//------------------------------------------------------------------------------
// Video Clip
FSSENDERREMOTEPROC bool MVSTD
fsSenderIsPlayVideoClip()
{
	assert(g_sender != NULL);
	return g_sender->isPlayVideoClip();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderPlayVideoClip(const char* filename)
{
	assert(g_sender != NULL);
	return g_sender->playVideoClip(filename);
}
//------------------------------------------------------------------------------
// Sound File
FSSENDERREMOTEPROC bool MVSTD
fsSenderIsPlaySoundFile()
{
	assert(g_sender != NULL);
	return g_sender->isPlaySoundFile();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderPlaySoundFile(const char* filename)
{
	assert(g_sender != NULL);
	return g_sender->playSoundFile(filename);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Encoder Settings
//
//------------------------------------------------------------------------------
// Video Encoder
FSSENDERREMOTEPROC int MVSTD
fsSenderGetNumVideoEncoder()
{
	assert(g_sender != NULL);
	return g_sender->getNumVideoEncoder();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetVideoEncoder(int index)
{
	assert(g_sender != NULL);
	g_sender->setVideoEncoder(index);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoEncoder()
{
	assert(g_sender != NULL);
	return g_sender->getVideoEncoder();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC const char* MVSTD
fsSenderGetVideoEncoderFriendlyName(int index)
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->getVideoEncoderFriendlyName(index));
	return str;
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoEncoderPriority(int index)
{
	assert(g_sender != NULL);
	return g_sender->getVideoEncoderPriority(index);
}
//------------------------------------------------------------------------------
// Audio Encoder
FSSENDERREMOTEPROC int MVSTD
fsSenderGetNumAudioEncoder()
{
	assert(g_sender != NULL);
	return g_sender->getNumAudioEncoder();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetAudioEncoder(int index)
{
	assert(g_sender != NULL);
	g_sender->setAudioEncoder(index);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetAudioEncoder()
{
	assert(g_sender != NULL);
	return g_sender->getAudioEncoder();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC const char* MVSTD
fsSenderGetAudioEncoderFriendlyName(int index)
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->getAudioEncoderFriendlyName(index));
	return str;
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetAudioEncoderPriority(int index)
{
	assert(g_sender != NULL);
	return g_sender->getAudioEncoderPriority(index);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Focus Display Settings
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetFocusDisplay(int focusDisplay)
{
	assert(g_sender != NULL);
	g_sender->setFocusDisplay(focusDisplay);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetFocusDisplay()
{
	assert(g_sender != NULL);
	return g_sender->getFocusDisplay();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetFocusFrameVisible(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setFocusFrameVisible(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetFocusFrameVisible()
{
	assert(g_sender != NULL);
	return g_sender->getFocusFrameVisible();
}
//------------------------------------------------------------------------------
// Video Resolution
FSSENDERREMOTEPROC void MVSTD
fsSenderSetVideoResolution(int videoResolution)
{
	assert(g_sender != NULL);
	g_sender->setVideoResolution(videoResolution);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoResolution()
{
	assert(g_sender != NULL);
	return g_sender->getVideoResolution();
}
//------------------------------------------------------------------------------
// Partial Zoom
FSSENDERREMOTEPROC void MVSTD
fsSenderSetPartialZoomSourceVideoResolution(int videoResolution)
{
	assert(g_sender != NULL);
	g_sender->setPartialZoomSourceVideoResolution(videoResolution);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetPartialZoomSourceVideoResolution()
{
	assert(g_sender != NULL);
	return g_sender->getPartialZoomSourceVideoResolution();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetPartialZoomOutputVideoResolution(int videoResolution)
{
	assert(g_sender != NULL);
	g_sender->setPartialZoomOutputVideoResolution(videoResolution);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetPartialZoomOutputVideoResolution()
{
	assert(g_sender != NULL);
	return g_sender->getPartialZoomOutputVideoResolution();
}
//------------------------------------------------------------------------------
// Non-Linear Zoom
FSSENDERREMOTEPROC void MVSTD
fsSenderSetNonLinearZoomOutputVideoResolution(int videoResolution)
{
	assert(g_sender != NULL);
	g_sender->setNonLinearZoomOutputVideoResolution(videoResolution);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetNonLinearZoomOutputVideoResolution()
{
	assert(g_sender != NULL);
	return g_sender->getNonLinearZoomOutputVideoResolution();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetNonLinearZoomMaskType(int maskType)
{
	assert(g_sender != NULL);
	g_sender->setNonLinearZoomMaskType(maskType);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetNonLinearZoomMaskType()
{
	assert(g_sender != NULL);
	return g_sender->getNonLinearZoomMaskType();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetNonLinearZoomMaskSize(int width, int height)
{
	assert(g_sender != NULL);
	g_sender->setNonLinearZoomMaskSize(width, height);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderGetNonLinearZoomMaskSize(int* width, int* height)
{
	assert(g_sender != NULL);
	g_sender->getNonLinearZoomMaskSize(*width, *height);
}
//------------------------------------------------------------------------------
// Image Compound
FSSENDERREMOTEPROC void MVSTD
fsSenderSetImageCompoundBaseVideoResolution(int videoResolution)
{
	assert(g_sender != NULL);
	g_sender->setImageCompoundBaseVideoResolution(videoResolution);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetImageCompoundBaseVideoResolution()
{
	assert(g_sender != NULL);
	return g_sender->getImageCompoundBaseVideoResolution();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetImageCompoundAdditionalVideoResolution(int videoResolution)
{
	assert(g_sender != NULL);
	g_sender->setImageCompoundAdditionalVideoResolution(videoResolution);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetImageCompoundAdditionalVideoResolution()
{
	assert(g_sender != NULL);
	return g_sender->getImageCompoundAdditionalVideoResolution();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetImageCompoundMaskType(int maskType)
{
	assert(g_sender != NULL);
	g_sender->setImageCompoundMaskType(maskType);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetImageCompoundMaskType()
{
	assert(g_sender != NULL);
	return g_sender->getImageCompoundMaskType();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetImageCompoundBlendingType(int blendingType)
{
	assert(g_sender != NULL);
	g_sender->setImageCompoundBlendingType(blendingType);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetImageCompoundBlendingType()
{
	assert(g_sender != NULL);
	return g_sender->getImageCompoundBlendingType();
}
//------------------------------------------------------------------------------
// Other Settings
FSSENDERREMOTEPROC void MVSTD
fsSenderSetFocusCoordinate(int coordinate)
{
	assert(g_sender != NULL);
	g_sender->setFocusCoordinate(coordinate);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetFocusCoordinate()
{
	assert(g_sender != NULL);
	return g_sender->getFocusCoordinate();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderComputeFocusRegion()
{
	assert(g_sender != NULL);
	g_sender->computeFocusRegion();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Direct Video Mode Settings
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetDirectVideoModeOn(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setDirectVideoModeOn(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetDirectVideoModeOn()
{
	assert(g_sender != NULL);
	return g_sender->getDirectVideoModeOn();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetDVInterleavedModeOn(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setDVInterleavedModeOn(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetDVInterleavedModeOn()
{
	assert(g_sender != NULL);
	return g_sender->getDVInterleavedModeOn();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderIsValidDVInterleavedMode()
{
	assert(g_sender != NULL);
	return g_sender->isValidDVInterleavedMode();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Caption Settings
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetCaptionOn(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setCaptionOn(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetCaptionOn()
{
	assert(g_sender != NULL);
	return g_sender->getCaptionOn();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetCaption(const char* caption)
{
	assert(g_sender != NULL);
	g_sender->setCaption(caption);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC const char* MVSTD
fsSenderGetCaption()
{
	assert(g_sender != NULL);
	MXSTR2STR(str, g_sender->getCaption());
	return str;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Video Window Settings
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetVideoWindowPolicy(int policy)
{
	assert(g_sender != NULL);
	g_sender->setVideoWindowPolicy(policy);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVideoWindowPolicy()
{
	assert(g_sender != NULL);
	return g_sender->getVideoWindowPolicy();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetKeepVideoAspect(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setKeepVideoAspect(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetKeepVideoAspect()
{
	assert(g_sender != NULL);
	return g_sender->getKeepVideoAspect();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetFitWindowToVideo(bool flag)
{
	assert(g_sender != NULL);
	g_sender->setFitWindowToVideo(flag);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderGetFitWindowToVideo()
{
	assert(g_sender != NULL);
	return g_sender->getFitWindowToVideo();
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderComputeVideoWindow()
{
	assert(g_sender != NULL);
	g_sender->computeVideoWindow();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Volume Settings
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderGetVolume(int* left, int* right)
{
	assert(g_sender != NULL);
	g_sender->getVolume(*left, *right);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderSetVolumeEvaluationTime(int evalTime)
{
	assert(g_sender != NULL);
	g_sender->setVolumeEvaluationTime(evalTime);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC int MVSTD
fsSenderGetVolumeEvaluationTime()
{
	assert(g_sender != NULL);
	return g_sender->getVolumeEvaluationTime();
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Other
//
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderQuit()
{
	assert(g_sender != NULL);
	g_sender->quit();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// Ini File
FSSENDERREMOTEPROC bool MVSTD
fsSenderWriteIniFile(const char* filename)
{
	assert(g_sender != NULL);
	return g_sender->writeIniFile(filename);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC bool MVSTD
fsSenderReadIniFile(const char* filename)
{
	assert(g_sender != NULL);
	return g_sender->readIniFile(filename);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderReadRegistry(const char* section, const char* entry, const char* defaultValue, char* value)
{
	assert(g_sender != NULL);
	mxString valueStr;
	g_sender->readRegistry(section, entry, defaultValue, valueStr);
	MXSTR2STR(value, valueStr);
}
//------------------------------------------------------------------------------
FSSENDERREMOTEPROC void MVSTD
fsSenderWriteRegistry(const char* section, const char* entry, const char* value)
{
	assert(g_sender != NULL);
	g_sender->writeRegistry(section, entry, value);
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Callback
FSSENDERREMOTEPROC bool MVSTD
fsSenderSetUICallbackFunction(fsSenderUICallbackFunction func, void* data)
{
	assert(g_sender != NULL);
	return g_sender->setUICallbackFunction(func, data);
}
//------------------------------------------------------------------------------
