/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsStatus/fsStatusReceiver.h>
#include <flBase/flString.h>
//------------------------------------------------------------------------------
fsStatusReceiver::fsStatusReceiver(StatusProc statusProc, void *statusData) :
mxObject()
{
	_receiveOn			= true;
	_isOpen				= false;

	flNetwork::initialize();
	_receiver				= new flFMTReceiver(FSSTATUS_UNIQUEID);

	_statusProc			= statusProc;
	_statusData			= statusData;

	_threadRun			= true;
	_thread				= new mxThread(threadLoopProc, this);
}
//------------------------------------------------------------------------------
fsStatusReceiver::~fsStatusReceiver()
{
	if (_isOpen)
		close();

	_threadRun = false;
	_thread->waitFor();
	delete _thread;

	delete _receiver;
	flNetwork::shutdown();
}
//------------------------------------------------------------------------------
void
fsStatusReceiver::setAddress(const mxString& address)
{
	if (!strcmp(_receiver->getAddress().c_str(), address.c_str()))
		return ;

	_receiver->close();
	_receiver->setAddress(flString(address.c_str()));
	if (_isOpen)
		_receiver->open();
}
//------------------------------------------------------------------------------
mxString
fsStatusReceiver::getAddress() const
{
	return mxString(_receiver->getAddress().c_str());
}
//------------------------------------------------------------------------------
void
fsStatusReceiver::setPort(mxUShort port)
{
	if (_receiver->getPort() == port)
		return ;

	_receiver->close();
	_receiver->setPort(port);
	if (_isOpen)
		_receiver->open();
}
//------------------------------------------------------------------------------
mxUShort
fsStatusReceiver::getPort() const
{
	return _receiver->getPort();
}
//------------------------------------------------------------------------------
void
fsStatusReceiver::setTimeToLive(mxULong ttl)
{
	if (_receiver->getTimeToLive() == ttl)
		return ;

	_receiver->close();
	_receiver->setTimeToLive(ttl);
	if (_isOpen)
		_receiver->open();
}
//------------------------------------------------------------------------------
mxULong
fsStatusReceiver::getTimeToLive() const
{
	return _receiver->getTimeToLive();
}
//------------------------------------------------------------------------------
mxUInt
fsStatusReceiver::getNumRejectionAddress() const
{
	return _receiver->getNumRejectionAddress();
}
//------------------------------------------------------------------------------
mxString
fsStatusReceiver::getRejectionAddress(mxUInt index) const
{
	return mxString(_receiver->getRejectionAddress(index).c_str());
}
//------------------------------------------------------------------------------
void
fsStatusReceiver::addRejectionAddress(const mxString& address)
{
	_receiver->addRejectionAddress(flString(address.c_str()));
}
//------------------------------------------------------------------------------
void
fsStatusReceiver::removeRejectionAddress(mxUInt index)
{
	_receiver->removeRejectionAddress(index);
}
//------------------------------------------------------------------------------
void
fsStatusReceiver::removeAllRejectionAddress()
{
	_receiver->removeAllRejectionAddress();
}
//------------------------------------------------------------------------------
void
fsStatusReceiver::setReceiveOn(mxBool flag)
{
	_receiveOn = flag;
}
//------------------------------------------------------------------------------
mxBool
fsStatusReceiver::getReceiveOn() const
{
	return _receiveOn;
}
//------------------------------------------------------------------------------
mxBool
fsStatusReceiver::open()
{
	if (_isOpen)
		return true;
	_isOpen = true;
	return _receiver->open();
}
//------------------------------------------------------------------------------
mxBool
fsStatusReceiver::close()
{
	if (!_isOpen)
		return true;
	_isOpen = false;
	return _receiver->close();
}
//------------------------------------------------------------------------------
mxBool
fsStatusReceiver::isOpen() const
{
	return _receiver->isOpen();
}
//------------------------------------------------------------------------------
void
fsStatusReceiver::receive(flChar *data, flUInt dataSize)
{
	if (!_receiveOn)
		return ;

	// Decode Status
	fsStatus status;

	_iStream.set(data, dataSize);
	_iStream.readBool(status.DirectVideoModeOn);
	_iStream.readBool(status.DvInterleavedModeOn);
	_iStream.readInt(status.FocusDisplay);
	_iStream.readInt(status.PartialZoomOutputVideoResolution);
	_iStream.readInt(status.NonLinearZoomOutputVideoResolution);
	_iStream.readInt(status.NonLinearZoomMaskType);
	_iStream.readUInt(status.NonLinearZoomMaskWidth);
	_iStream.readUInt(status.NonLinearZoomMaskHeight);
	_iStream.readInt(status.ImageCompoundOutputVideoResolution);
	_iStream.readInt(status.ImageCompoundMaskType);
	_iStream.readInt(status.ImageCompoundBlendingType);
	mxUInt length;
	_iStream.readUInt(length);
	if (length != 0)
	{
		mxChar *str = new mxChar[length + 1];
		_iStream.readByte((flByte *)str, length);
		str[length] = '\0';
		status.MulticastAddress = mxString(str);
		delete str;
	}
	else
	{
		status.MulticastAddress = "";
	}


	// Check Change
	mxInt changed = 0;

	if (_status.DirectVideoModeOn != status.DirectVideoModeOn)
	{
		_status.DirectVideoModeOn = status.DirectVideoModeOn;
		changed |= S_DIRECTVIDEOMODEON;
	}
	if (_status.DvInterleavedModeOn != status.DvInterleavedModeOn)
	{
		_status.DvInterleavedModeOn = status.DvInterleavedModeOn;
		changed |= S_DVINTERLEAVEDMODEON;
	}
	if (_status.FocusDisplay != status.FocusDisplay)
	{
		_status.FocusDisplay = status.FocusDisplay;
		changed |= S_FOCUSDISPLAY;
	}
	if (_status.PartialZoomOutputVideoResolution != status.PartialZoomOutputVideoResolution)
	{
		_status.PartialZoomOutputVideoResolution = status.PartialZoomOutputVideoResolution;
		changed |= S_PARTIALZOOMOUTPUTVIDEORESOLUTION;
	}
	if (_status.NonLinearZoomOutputVideoResolution != status.NonLinearZoomOutputVideoResolution)
	{
		_status.NonLinearZoomOutputVideoResolution = status.NonLinearZoomOutputVideoResolution;
		changed |= S_NONLINEARZOOMOUTPUTVIDEORESOLUTION;
	}
	if (_status.NonLinearZoomMaskType != status.NonLinearZoomMaskType)
	{
		_status.NonLinearZoomMaskType = status.NonLinearZoomMaskType;
		changed |= S_NONLINEARZOOMMASKTYPE;
	}
	if (_status.NonLinearZoomMaskWidth != status.NonLinearZoomMaskWidth)
	{
		_status.NonLinearZoomMaskWidth = status.NonLinearZoomMaskWidth;
		changed |= S_NONLINEARZOOMMASKWIDTH;
	}
	if (_status.NonLinearZoomMaskHeight != status.NonLinearZoomMaskHeight)
	{
		_status.NonLinearZoomMaskHeight = status.NonLinearZoomMaskHeight;
		changed |= S_NONLINEARZOOMMASKHEIGHT;
	}
	if (_status.ImageCompoundOutputVideoResolution != status.ImageCompoundOutputVideoResolution)
	{
		_status.ImageCompoundOutputVideoResolution = status.ImageCompoundOutputVideoResolution;
		changed |= S_IMAGECOMPOUNDOUTPUTVIDEORESOLUTION;
	}
	if (_status.ImageCompoundMaskType != status.ImageCompoundMaskType)
	{
		_status.ImageCompoundMaskType = status.ImageCompoundMaskType;
		changed |= S_IMAGECOMPOUNDMASKTYPE;
	}
	if (_status.ImageCompoundBlendingType != status.ImageCompoundBlendingType)
	{
		_status.ImageCompoundBlendingType = status.ImageCompoundBlendingType;
		changed |= S_IMAGECOMPOUNDBLENDINGTYPE;
	}
	if (_status.MulticastAddress != status.MulticastAddress)
	{
		_status.MulticastAddress = status.MulticastAddress;
		changed |= S_MULTICASTADDRESS;
	}

	if (_statusProc != NULL)
		_statusProc(changed, _status, _statusData);
}
//------------------------------------------------------------------------------
mxUInt
fsStatusReceiver::threadLoop()
{
	while(_threadRun)
	{
		flChar *data;
		flUInt dataSize;

		if (_receiver->receive(data, dataSize))
			receive(data, dataSize);
		else
			Sleep(10);

		Sleep(0);
	}
	return 0;
}
//------------------------------------------------------------------------------
mxUInt __stdcall
fsStatusReceiver::threadLoopProc(void* data)
{
	fsStatusReceiver *receiver = (fsStatusReceiver *)data;
	return receiver->threadLoop();
}
//------------------------------------------------------------------------------
