/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "Stdafx.h"
#include <fs/fsStatus/fsStatusSender.h>
//------------------------------------------------------------------------------
fsStatusSender::fsStatusSender() :
mxObject()
{
	flNetwork::initialize();
	_isOpen = false;
	_transmissionOn = true;
	_sender = new flFMTSender(FSSTATUS_UNIQUEID);
}
//------------------------------------------------------------------------------
fsStatusSender::~fsStatusSender()
{
	_sender->close();
	delete _sender;
	flNetwork::shutdown();
}
//------------------------------------------------------------------------------
void
fsStatusSender::setAddress(const mxString& address)
{
	if (!strcmp(_sender->getAddress().c_str(), address.c_str()))
		return ;

	_sender->close();
	_sender->setAddress(flString(address.c_str()));
	if (_isOpen)
		_sender->open();
}
//------------------------------------------------------------------------------
mxString
fsStatusSender::getAddress() const
{
	return mxString(_sender->getAddress().c_str());
}
//------------------------------------------------------------------------------
void
fsStatusSender::setPort(mxUShort port)
{
	if (_sender->getPort() == port)
		return ;

	_sender->close();
	_sender->setPort(port);
	if (_isOpen)
		_sender->open();
}
//------------------------------------------------------------------------------
mxUShort
fsStatusSender::getPort() const
{
	return _sender->getPort();
}
//------------------------------------------------------------------------------
void
fsStatusSender::setTimeToLive(mxULong ttl)
{
	if (_sender->getTimeToLive() == ttl)
		return ;

	_sender->close();
	_sender->setTimeToLive(ttl);
	if (_isOpen)
		_sender->open();
}
//------------------------------------------------------------------------------
mxULong
fsStatusSender::getTimeToLive() const
{
	return _sender->getTimeToLive();
}
//------------------------------------------------------------------------------
void
fsStatusSender::setMaxTransmissionRate(mxUInt rate)
{
	if (_sender->getMaximumTransmissionRate() == rate)
		return ;

	_sender->close();
	_sender->setMaximumTransmissionRate(rate);
	if (_isOpen)
		_sender->open();
}
//------------------------------------------------------------------------------
mxUInt
fsStatusSender::getMaxTransmissionRate() const
{
	return _sender->getMaximumTransmissionRate();
}
//------------------------------------------------------------------------------
void
fsStatusSender::setTransmissionCount(mxUInt count)
{
	if (_sender->getTransmissionCount() == count)
		return ;

	_sender->close();
	_sender->setTransmissionCount(count);
	if (_isOpen)
		_sender->open();
}
//------------------------------------------------------------------------------
mxUInt
fsStatusSender::getTransmissionCount() const
{
	return _sender->getTransmissionCount();
}
//------------------------------------------------------------------------------
void
fsStatusSender::setPacketSize(mxUInt size)
{
	if (_sender->getPacketSize() == size)
		return ;
	_sender->setPacketSize(size);
}
//------------------------------------------------------------------------------
mxUInt
fsStatusSender::getPacketSize() const
{
	return _sender->getPacketSize();
}
//------------------------------------------------------------------------------
void
fsStatusSender::setTransmissionOn(mxBool flag)
{
	_transmissionOn = flag;
}
//------------------------------------------------------------------------------
mxBool
fsStatusSender::getTransmissionOn() const
{
	return _transmissionOn;
}
//------------------------------------------------------------------------------
mxBool
fsStatusSender::open()
{
	if (_isOpen)
		return true;
	_isOpen = true;
	return _sender->open();
}
//------------------------------------------------------------------------------
mxBool
fsStatusSender::close()
{
	if (!_isOpen)
		return true;
	_isOpen = false;
	return _sender->close();
}
//------------------------------------------------------------------------------
mxBool
fsStatusSender::isOpen() const
{
	return _sender->isOpen();
}
//------------------------------------------------------------------------------
void
fsStatusSender::send(const fsStatus& status)
{
	if (!_transmissionOn)
		return ;

	// Encode Status
	_oStream.reset();
	_oStream.writeBool(status.DirectVideoModeOn);
	_oStream.writeBool(status.DvInterleavedModeOn);
	_oStream.writeInt(status.FocusDisplay);
	_oStream.writeInt(status.PartialZoomOutputVideoResolution);
	_oStream.writeInt(status.NonLinearZoomOutputVideoResolution);
	_oStream.writeInt(status.NonLinearZoomMaskType);
	_oStream.writeUInt(status.NonLinearZoomMaskWidth);
	_oStream.writeUInt(status.NonLinearZoomMaskHeight);
	_oStream.writeInt(status.ImageCompoundOutputVideoResolution);
	_oStream.writeInt(status.ImageCompoundMaskType);
	_oStream.writeInt(status.ImageCompoundBlendingType);
	flUInt length = status.MulticastAddress.length();
	_oStream.writeUInt(length);
	_oStream.writeByte((const flByte *)status.MulticastAddress.c_str(), length);

	flChar *data;
	flUInt dataSize;
	_oStream.get(data, dataSize);

	// Send Status
	_sender->send(data ,dataSize);
}
//------------------------------------------------------------------------------
