/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef	__mxArray_h__
#define	__mxArray_h__
//------------------------------------------------------------------------------
#include <mx/mxCore/mxFuncs.h>
//------------------------------------------------------------------------------
template<class T,
		 class LF=mxLessFunc<T>,
		 class GF=mxGreaterFunc<T>,
		 class CPF=mxCopyFunc<T>,
		 class CMF=mxCompFunc<T> >
class mxArray
{
public:
	static const mxUInt	DEFAULT_GROWSIZE = 16;
	typedef mxInt		(*SortFunc)(const T*, const T*);

	mxArray();
	mxArray(const mxArray& array);
	~mxArray();

public:
	void			copy(const mxArray& array);
	mxBool			equal(const mxArray& array) const;

	void			clear();
	void			fit();

	void			setBufferSize(mxUInt size);
	mxUInt			getBufferSize() const;

	void			setGrowSize(mxUInt size);
	mxUInt			getGrowSize() const;

	void			setSize(mxUInt size);
	mxUInt			getSize() const;

	void			set(mxUInt index, const T& value);
	T&				get(mxUInt index);
	const T&		get(mxUInt index) const;
	const T*		getPointer() const;

	void			add(const T& value);
	void			add(const T* vlaues, mxUInt numValues);
	void			insert(mxUInt index, const T& value);
	mxBool			find(const T& value) const;
	mxBool			find(const T& value, mxUInt& index) const;
	void			remove(const T& value);
	void			removeIndex(mxUInt index);
	void			removeAll(const T& value);
	void			replace(const T& oldValue, const T& newValue);

	void			sortLess();
	void			sortGreater();
	void			sort(SortFunc sortFunc);

public:
	mxBool			operator ==(const mxArray& array) const;
	mxBool			operator !=(const mxArray& array) const;

	mxArray&		operator =(const mxArray& array);

	T&				operator [](mxUInt i);
	const T&		operator [](mxUInt i) const;
	
private:
	static mxInt	lessFunc(const T* value1, const T* value2);
	static mxInt	greaterFunc(const T* value1, const T* value2);

private:
	T*				_values;
	mxUInt			_bufferSize;
	mxUInt			_size;
	mxUInt			_growSize;
};
//------------------------------------------------------------------------------
#include <mx/mxCore/mxArray.inl>
//------------------------------------------------------------------------------
typedef mxArray<mxBool>			mxBoolArray;
typedef mxArray<mxChar>			mxCharArray;
typedef mxArray<mxByte>			mxByteArray;
typedef mxArray<mxShort>		mxShortArray;
typedef mxArray<mxUShort>		mxUShortArray;
typedef mxArray<mxLong>			mxLongArray;
typedef mxArray<mxULong>		mxULongArray;
typedef mxArray<mxInt>			mxIntArray;
typedef mxArray<mxUInt>			mxUIntArray;
typedef mxArray<mxInt32>		mxInt32Array;
typedef mxArray<mxUInt32>		mxUInt32Array;
typedef mxArray<mxInt64>		mxInt64Array;
typedef mxArray<mxUInt64>		mxUInt64Array;
typedef mxArray<mxFloat>		mxFloatArray;
typedef mxArray<mxDouble>		mxDoubleArray;
//------------------------------------------------------------------------------
#endif	// !__mxArray_h__
