/*
**	Copyright (c) 2003 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef	__mxArray_inl__
#define	__mxArray_inl__
//------------------------------------------------------------------------------
#include <mx/mxCore/mxQuickSort.h>
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
mxArray<T, LF, GF, CPF, CMF>::mxArray() :
_values(NULL),
_bufferSize(0),
_size(0),
_growSize(DEFAULT_GROWSIZE)
{
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
mxArray<T, LF, GF, CPF, CMF>::mxArray(const mxArray& array) :
_values(NULL),
_bufferSize(0),
_size(0),
_growSize(DEFAULT_GROWSIZE)
{
	setBufferSize(array._bufferSize);
	_size		= array._size;
	_growSize	= array._growSize;

	if (_size)
	{
		CPF copyFunc;
		copyFunc(_values, array._values, array._size);
	}
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
mxArray<T, LF, GF, CPF, CMF>::~mxArray()
{
	delete [] _values;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::copy(const mxArray& array)
{
	if (this == &array)
		return;

	clear();

	setBufferSize(array._bufferSize);
	_size		= array._size;
	_growSize	= array._growSize;

	if (_size)
	{
		CPF copyFunc;
		copyFunc(_values, array._values, array._size);
	}
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
mxBool
mxArray<T, LF, GF, CPF, CMF>::equal(const mxArray& array) const
{
	if (this == &array)
		return true;
	if (_size != array._size)
		return false;
	if (_size == 0)
		return false;

	CMF compFunc;
	return compFunc(_values, array._values, _size);
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::clear()
{
	delete [] _values;
	_values		= NULL;
	_size		= 0;
	_bufferSize	= 0;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::fit()
{
	if (_bufferSize > _size)
		setBufferSize(_size);
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::setBufferSize(mxUInt size)
{
	if (_bufferSize == size)
		return;

	if (size == 0)
	{
		clear();
		return;
	}

	T* p = new T[size];

	if (_values)
	{
		mxUInt num = (_size < size) ? _size : size;
		if (num)
		{
			CPF copyFunc;
			copyFunc(p, _values, num);
		}
		delete [] _values;
	}

	_values = p;
	if (_size > size)
		_size = size;
	_bufferSize = size;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline mxUInt
mxArray<T, LF, GF, CPF, CMF>::getBufferSize() const
{
	return _bufferSize;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline void
mxArray<T, LF, GF, CPF, CMF>::setGrowSize(mxUInt size)
{
	_growSize = (size > 0) ? size : DEFAULT_GROWSIZE;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline mxUInt
mxArray<T, LF, GF, CPF, CMF>::getGrowSize() const
{
	return _growSize;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::setSize(mxUInt size)
{
	while (_bufferSize <= size)
		setBufferSize(_bufferSize + _growSize);
	_size = size;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline mxUInt
mxArray<T, LF, GF, CPF, CMF>::getSize() const
{
	return _size;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::set(mxUInt index, const T& value)
{
	assert(index < _size);
	_values[index] = value;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline T&
mxArray<T, LF, GF, CPF, CMF>::get(mxUInt index)
{
	assert(index < _size);
	return _values[index];
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline const T&
mxArray<T, LF, GF, CPF, CMF>::get(mxUInt index) const
{
	assert(index < _size);
	return _values[index];
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline const T*
mxArray<T, LF, GF, CPF, CMF>::getPointer() const
{
	return _values;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::add(const T& value)
{
	if (_size == _bufferSize)
		setBufferSize(_bufferSize + _growSize);
	_values[_size++] = value;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::add(const T* values, mxUInt numValues)
{
	while (_size + numValues >= _bufferSize)
		setBufferSize(_bufferSize + _growSize);

	CPF copyFunc;
	copyFunc(&_values[_size], values, numValues);
	_size += numValues;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::insert(mxUInt index, const T& value)
{
	assert(_size != 0 && index < _size);

	T* oldValues = _values;

	if (_size == _bufferSize)
		_bufferSize += _growSize;
	_values = new T[_bufferSize];

	CPF copyFunc;
	if (index)
		copyFunc(_values, oldValues, index);
	_values[index] = value;
	copyFunc(&_values[index+1], &oldValues[index], _size-index);

	delete [] oldValues;

	++_size;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
mxBool
mxArray<T, LF, GF, CPF, CMF>::find(const T& value) const
{
	for (mxUInt i = 0; i < _size; ++i)
	{
		if (_values[i] == value)
			return true;
	}
	return false;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
mxBool
mxArray<T, LF, GF, CPF, CMF>::find(const T& value, mxUInt& index) const
{
	for (mxUInt i = 0; i < _size; ++i)
	{
		if (_values[i] == value)
		{
			index = i;
			return true;
		}
	}
	return false;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::remove(const T& value)
{
	mxUInt index;
	if (!find(value, index))
		return;

	removeIndex(index);
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::removeIndex(mxUInt index)
{
	assert(index < _size);

	if (index == _size-1)
	{
		--_size;
		return;
	}

	T* oldValues = _values;
	_values = new T[_bufferSize];

	CPF copyFunc;
	if (index)
		copyFunc(_values, oldValues, index);
	copyFunc(&_values[index], &oldValues[index+1], _size-index-1);

	delete [] oldValues;

	--_size;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::removeAll(const T& value)
{
	for (mxUInt i = 0; i < _size; ++i)
	{
		if (_values[i] == value)
		{
			removeIndex(i);
			--i;
		}
	}
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::replace(const T& oldValue, const T& newValue)
{
	for (mxUInt i = 0; i < _size; ++i)
	{
		if (_values[i] == oldValue)
			_values[i] = newValue;
	}
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::sortLess()
{
	if (_size > 1)
	{
		mxQuickSort::sort((void*)_values, _size,
			sizeof(T), (mxQuickSort::SortFunc)lessFunc);
	}
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::sortGreater()
{
	if (_size > 1)
	{
		mxQuickSort::sort((void*)_values, _size,
			sizeof(T), (mxQuickSort::SortFunc)greaterFunc);
	}
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
void
mxArray<T, LF, GF, CPF, CMF>::sort(SortFunc sortFunc)
{
	if (_size > 1)
	{
		mxQuickSort::sort((void*)_values, _size,
			sizeof(T), (mxQuickSort::SortFunc)sortFunc);
	}
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline mxBool
mxArray<T, LF, GF, CPF, CMF>::operator ==(const mxArray& array) const
{
	return equal(array);
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline mxBool
mxArray<T, LF, GF, CPF, CMF>::operator !=(const mxArray& array) const
{
	return !equal(array);
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline mxArray<T, LF, GF, CPF, CMF>&
mxArray<T, LF, GF, CPF, CMF>::operator =(const mxArray& array)
{
	copy(array);
	return *this;
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline T&
mxArray<T, LF, GF, CPF, CMF>::operator [](mxUInt i)
{
	return _values[i];
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline const T&
mxArray<T, LF, GF, CPF, CMF>::operator [](mxUInt i) const
{
	return _values[i];
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline mxInt
mxArray<T, LF, GF, CPF, CMF>::lessFunc(const T* value1, const T* value2)
{
	LF lessFunc;
	return lessFunc(value1, value2);
}
//------------------------------------------------------------------------------
template<class T, class LF, class GF, class CPF, class CMF>
inline mxInt
mxArray<T, LF, GF, CPF, CMF>::greaterFunc(const T* value1, const T* value2)
{
	GF greaterFunc;
	return greaterFunc(value1, value2);
}
//------------------------------------------------------------------------------
#endif	// !__mxArray_inl__
