/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "StdAfx.h"
#include <mx/mxCore/mxFile.h>
//------------------------------------------------------------------------------
mxFile::mxFile() :
_fp(NULL),
_size(0)
{
}
//------------------------------------------------------------------------------
mxFile::~mxFile()
{
	if (isOpen())
		close();
}
//------------------------------------------------------------------------------
mxBool
mxFile::open(const mxString& filename, ModeEnum mode)
{
	assert(_fp == NULL);

	HANDLE hFile = CreateFile(
		filename.c_str(),
		GENERIC_READ,
		FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL);

	if (hFile != NULL && hFile != INVALID_HANDLE_VALUE)
	{
		mxULong low, high = 0;
		low = GetFileSize(hFile, &high);
		CloseHandle(hFile);

		if (low == 0xFFFFFFFF)
		{
			if (GetLastError() != NO_ERROR)
				low = high = 0;
		}

		LARGE_INTEGER size;
		size.LowPart = low;
		size.HighPart = high;

		_size = size.QuadPart;
	}
	else
	{
		_size = 0;
	}


	if (mode == READ)
		_fp = fopen(filename.c_str(), "rb");
	else if (mode == WRITE)
		_fp = fopen(filename.c_str(), "wb");
	else if (mode == APPEND)
		_fp = fopen(filename.c_str(), "ab");

	_filename = filename;

	return isOpen();
}
//------------------------------------------------------------------------------
mxBool
mxFile::close()
{
	if (_fp != NULL)
		fclose(_fp);
	_fp = NULL;
	_filename = "";
	_size = 0;

	return true;
}
//------------------------------------------------------------------------------
mxBool
mxFile::isOpen() const
{
	return _fp != NULL;
}
//------------------------------------------------------------------------------
mxBool
mxFile::isEndOfFile() const
{
	assert(_fp != NULL);

	return feof(_fp) != 0;
}
//------------------------------------------------------------------------------
mxBool
mxFile::getPosition(mxInt64& pos) const
{
	assert(_fp != NULL);

	fpos_t p;
	mxBool ret = fgetpos(_fp, &p) == 0;
	if (ret)
		pos = p;

	return ret;
}
//------------------------------------------------------------------------------
mxBool
mxFile::setPosition(const mxInt64& pos) const
{
	assert(_fp != NULL);

	fpos_t p = pos;
	return fsetpos(_fp, &p) == 0;
}
//------------------------------------------------------------------------------
mxBool
mxFile::flush() const
{
	assert(_fp != NULL);

	return fflush(_fp) == 0;
}
//------------------------------------------------------------------------------
mxBool
mxFile::push()
{
	assert(_fp != NULL);

	mxInt64 pos;
	getPosition(pos);

	return pushQueue(pos);
}
//------------------------------------------------------------------------------
mxBool
mxFile::pop()
{
	assert(_fp != NULL);

	mxInt64 pos;
	if (popQueue(pos))
	{
		setPosition(pos);
		return true;
	}
	return false;
}
//------------------------------------------------------------------------------
mxInt64
mxFile::getSize() const
{
	assert(_fp != NULL);

	return _size;
}
//------------------------------------------------------------------------------
mxUInt
mxFile::read(void* buffer, mxUInt size, mxUInt count) const
{
	assert(_fp != NULL);

	return (mxUInt)fread(buffer, size, count, _fp);
}
//------------------------------------------------------------------------------
mxUInt
mxFile::write(void* buffer, mxUInt size, mxUInt count) const
{
	assert(_fp != NULL);

	return (mxUInt)fwrite(buffer, size, count, _fp);
}
//------------------------------------------------------------------------------
const mxString
mxFile::getModuleFilename()
{
	mxChar	name[MAX_PATH+1];
	memset(name, 0, MAX_PATH+1);
	GetModuleFileName(NULL, name, MAX_PATH);

	return mxString(name);
}
//------------------------------------------------------------------------------
const mxString
mxFile::getModulePath()
{
	return getPathWithoutFilename(getModuleFilename());
}
//------------------------------------------------------------------------------
const mxString
mxFile::getTemporaryPath()
{
	mxChar	name[MAX_PATH+1];
	memset(name, 0, MAX_PATH+1);
	GetTempPath(MAX_PATH+1, name);

	return mxString(name);
}
//------------------------------------------------------------------------------
const mxString
mxFile::getTemporaryFilename(const mxString& prefix)
{
	mxChar	name[MAX_PATH+1];
	GetTempFileName(getTemporaryPath().c_str(), prefix.c_str(), 0, name);

	return mxString(name);
}
//------------------------------------------------------------------------------
const mxString
mxFile::getFilenameWithoutPath(const mxString& name)
{
	mxChar	file[MAX_PATH];
	if (strrchr(name.c_str(), '\\'))
		strcpy(file, strrchr(name.c_str(), '\\') + 1);
	else if (strrchr(name.c_str(), '/'))
		strcpy(file, strrchr(name.c_str(), '/') + 1);
	else
		strcpy(file, name.c_str());

	return mxString(file);
}
//------------------------------------------------------------------------------
const mxString
mxFile::getPathWithoutFilename(const mxString& name)
{
	mxChar	path[MAX_PATH];
	strcpy(path, name.c_str());

	if (strrchr(name.c_str(), '\\'))
		path[ name.length() - strlen(strrchr(name.c_str(), '\\')) ] = 0;
	else if (strrchr(name.c_str(), '/'))
		path[ name.length() - strlen(strrchr(name.c_str(), '/')) ] = 0;
	else
		strcpy(path, "");

	return mxString(path);
}
//------------------------------------------------------------------------------
const mxString
mxFile::getExtension(const mxString& name)
{
	mxChar	ext[MAX_PATH];
	if (strrchr(name.c_str(), '.'))
	{
		strcpy(ext, strrchr(name.c_str(), '.') + 1);
		return mxString(ext);
	}

	return "";
}
//------------------------------------------------------------------------------
const mxStringArray
mxFile::findFilenames(const mxString& filename)
{
	mxStringArray	foundFilenames;
	mxString			path = getPathWithoutFilename(filename);

	WIN32_FIND_DATA		data;
	HANDLE				hFF = FindFirstFile(filename.c_str(), &data);

	if (hFF != INVALID_HANDLE_VALUE)
	{
		mxChar name[MAX_PATH];
		do
		{
			strcpy(name, data.cFileName);
			foundFilenames.add(path + "\\" + name);
		} while (FindNextFile(hFF, &data));

		FindClose(hFF);
	}

	return foundFilenames;
}
//------------------------------------------------------------------------------
const mxStringArray
mxFile::findFilenames(const mxStringArray& searchPath,
					  const mxString& filename)
{
	mxStringArray	foundFilenames, temp;
	mxString			name = getFilenameWithoutPath(filename);

	for (mxUInt i = 0, j; i < searchPath.getSize(); ++i)
	{
		temp = findFilenames(searchPath[i] + "\\" + name);
		for (j = 0; j < temp.getSize(); ++j)
			foundFilenames.add(temp[j]);
	}

	return foundFilenames;
}
//------------------------------------------------------------------------------
mxBool
mxFile::pushQueue(const mxInt64& pos)
{
	_posQueue.add(pos);

	return true;
}
//------------------------------------------------------------------------------
mxBool
mxFile::popQueue(mxInt64& pos)
{
	mxUInt s = _posQueue.getSize();
	if (s == 0)
		return false;

	pos = _posQueue[s-1];
	_posQueue.setSize(s-1);

	return true;
}
//------------------------------------------------------------------------------
mxBool
mxFile::clearQueue()
{
	_posQueue.setSize(0);

	return true;
}
//------------------------------------------------------------------------------
