/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef	__mxFuncs_h__
#define	__mxFuncs_h__
//------------------------------------------------------------------------------
#include <mx/mxCore/mxCore.h>
//------------------------------------------------------------------------------
template<class T>
struct mxLessFunc
{
	mxInt	operator ()(const T* data1, const T* data2) const
	{
		if (*data1 < *data2) return -1;
		else if (*data1 > *data2) return 1;
		return 0;
	}
};
//------------------------------------------------------------------------------
template<class T>
struct mxGreaterFunc
{
	mxInt	operator ()(const T* data1, const T* data2) const
	{
		if (*data1 < *data2) return 1;
		else if (*data1 > *data2) return -1;
		return 0;
	}
};
//------------------------------------------------------------------------------
template<class T>
struct mxCopyFunc
{
	void	operator ()(T* dst, const T* src, mxUInt num) const
	{ memcpy(dst, src, sizeof(T)*num); }
};
//------------------------------------------------------------------------------
template<class T>
struct mxCompFunc
{
	mxBool	operator ()(const T* dst, const T* src, mxUInt num) const
	{ return memcmp(dst, src, sizeof(T)*num) == 0; }
};
//------------------------------------------------------------------------------
template<class T>
struct mxKeyHashFunc
{
	mxUInt	operator ()(const T& key) const { return mxUInt(key); }
};
//------------------------------------------------------------------------------
template<class T>
struct mxKeyCompFunc
{
	mxBool	operator ()(const T& key1, const T& key2) const
	{ return key1 == key2; }
};
//------------------------------------------------------------------------------
#define	DECLARE_SAFE_COPYFUNC(type)											\
template<>																	\
struct mxCopyFunc<type>														\
{																			\
	void	operator ()(type* dst, const type* src, mxUInt num) const		\
	{ for (mxUInt i = 0; i < num; ++i) dst[i] = src[i]; }					\
}
//------------------------------------------------------------------------------
#define	DECLARE_SAFE_COMPFUNC(type)											\
template<>																	\
struct mxCompFunc<type>														\
{																			\
	mxBool	operator ()(const type* dst, const type* src, mxUInt num) const	\
	{																		\
		for (mxUInt i = 0; i < num; ++i)									\
		{																	\
			if (dst[i] != src[i])											\
				return false;												\
		}																	\
		return true;														\
	}																		\
}
//------------------------------------------------------------------------------
#endif	// !__mxFuncs_h__
