/*
**	Copyright (c) 2003 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef	__mxHash_inl__
#define	__mxHash_inl__
//------------------------------------------------------------------------------
#ifdef	_DEBUG
#include <mx/mxCore/mxNotify.h>
#endif
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxHashMap<K, V, HF, CF>::mxHashMap(mxUInt tableSize) :
_tableSize(tableSize), _keyTable(NULL), _valueTable(NULL)
{
	if (_tableSize == 0)
		_tableSize = DEFAULT_TABLESIZE;
	_keyTable	= new mxArray<K>[_tableSize];
	_valueTable	= new mxArray<V>[_tableSize];
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxHashMap<K, V, HF, CF>::mxHashMap(const mxHashMap& hashMap)
{
	_tableSize	= hashMap._tableSize;
	_keyTable	= new mxArray<K>[_tableSize];
	_valueTable	= new mxArray<V>[_tableSize];

	for (mxUInt i = 0; i < _tableSize; ++i)
	{
		_keyTable[i]	= hashMap._keyTable[i];
		_valueTable[i]	= hashMap._valueTable[i];
	}
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxHashMap<K, V, HF, CF>::~mxHashMap()
{
	delete [] _keyTable;
	delete [] _valueTable;
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
void
mxHashMap<K, V, HF, CF>::copy(const mxHashMap& hashMap)
{
	if (this == &hashMap)
		return;

	clear();
	delete [] _keyTable;
	delete [] _valueTable;

	_tableSize	= hashMap._tableSize;
	_keyTable	= new mxArray<K>[_tableSize];
	_valueTable	= new mxArray<V>[_tableSize];

	for (mxUInt i = 0; i < _tableSize; ++i)
	{
		_keyTable[i].copy(hashMap._keyTable[i]);
		_valueTable[i].copy(hashMap._valueTable[i]);
	}
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxBool
mxHashMap<K, V, HF, CF>::equal(const mxHashMap& hashMap) const
{
	if (this == &hashMap)
		return true;

	if (_tableSize != hashMap._tableSize)
		return false;

	for (mxUInt i = 0; i < _tableSize; ++i)
	{
		if (_keyTable[i] != hashMap._keyTable[i])
			return false;
		if (_valueTable[i] != hashMap._valueTable[i])
			return false;
	}

	return true;
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
void
mxHashMap<K, V, HF, CF>::clear()
{
	for (mxUInt i = 0; i < _tableSize; ++i)
	{
		_keyTable[i].clear();
		_valueTable[i].clear();
	}
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxUInt
mxHashMap<K, V, HF, CF>::getSize() const
{
	mxUInt size = 0;
	for (mxUInt i = 0; i < _tableSize; ++i)
		size += _keyTable[i].getSize();
	return size;
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
inline mxUInt
mxHashMap<K, V, HF, CF>::getTableSize() const
{
	return _tableSize;
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxBool
mxHashMap<K, V, HF, CF>::resizeTable(mxUInt newTableSize)
{
	assert(newTableSize > 0);

	mxUInt		oldTableSize	= _tableSize;
	mxArray<K>*	oldKeyTable		= _keyTable;
	mxArray<V>*	oldValueTable	= _valueTable;

	_tableSize	= newTableSize;
	_keyTable	= new mxArray<K>[_tableSize];
	_valueTable	= new mxArray<V>[_tableSize];

	for (mxUInt i = 0, j; i < oldTableSize; ++i)
	{
		mxArray<K>& karray = oldKeyTable[i];
		mxArray<V>& varray = oldValueTable[i];

		for (j = 0; j < karray.getSize(); ++j)
			insert(karray[j], varray[j]);
		
		karray.clear();
		varray.clear();
	}

	delete [] oldKeyTable;
	delete [] oldValueTable;

	return true;
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
void
mxHashMap<K, V, HF, CF>::insert(const K& key, const V& value)
{
	HF hashFunc;
	mxUInt hashValue = hashFunc(key) % _tableSize;
	_keyTable[hashValue].add(key);
	_valueTable[hashValue].add(value);
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxBool
mxHashMap<K, V, HF, CF>::get(const K& key, V& value) const
{
	HF hashFunc;
	CF compFunc;
	mxUInt hashValue = hashFunc(key) % _tableSize;
	const mxArray<K>& karray = _keyTable[hashValue];
	const mxArray<V>& varray = _valueTable[hashValue];
	for (mxUInt i = 0; i < karray.getSize(); ++i)
	{
		if (compFunc(key, karray[i]))
		{
			value = varray[i];
			return true;
		}
	}
	return false;
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxBool
mxHashMap<K, V, HF, CF>::find(const K& key) const
{
	HF hashFunc;
	CF compFunc;
	mxUInt hashValue = hashFunc(key) % _tableSize;
	const mxArray<K>& karray = _keyTable[hashValue];
	for (mxUInt i = 0; i < karray.getSize(); ++i)
	{
		if (compFunc(key, karray[i]))
			return true;
	}
	return false;
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
void
mxHashMap<K, V, HF, CF>::remove(const K& key)
{
	HF hashFunc;
	CF compFunc;
	mxUInt hashValue = hashFunc(key) % _tableSize;
	mxArray<K>& karray = _keyTable[hashValue];
	mxArray<V>& varray = _valueTable[hashValue];
	for (mxUInt i = 0; i < karray.getSize(); ++i)
	{
		if (compFunc(key, karray[i]))
		{
			karray.removeIndex(i);
			varray.removeIndex(i);
			break;
		}
	}
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
void
mxHashMap<K, V, HF, CF>::replace(const K& key, const V& newValue)
{
	HF hashFunc;
	CF compFunc;
	mxUInt hashValue = hashFunc(key) % _tableSize;
	mxArray<K>& karray = _keyTable[hashValue];
	mxArray<V>& varray = _valueTable[hashValue];
	for (mxUInt i = 0; i < karray.getSize(); ++i)
	{
		if (compFunc(key, karray[i]))
		{
			varray[i] = newValue;
			break;
		}
	}
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
const K&
mxHashMap<K, V, HF, CF>::getKey(mxUInt index) const
{
	assert(index < getSize());

	for (mxUInt i = 0; i < _tableSize; ++i)
	{
		if (index < _keyTable[i].getSize())
			break;
		index -= _keyTable[i].getSize();
	}
	assert(i < _tableSize);

	return _keyTable[i][index];
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
const V&
mxHashMap<K, V, HF, CF>::getValue(mxUInt index) const
{
	assert(index < getSize());

	for (mxUInt i = 0; i < _tableSize; ++i)
	{
		if (index < _valueTable[i].getSize())
			break;
		index -= _valueTable[i].getSize();
	}
	assert(i < _tableSize);

	return _valueTable[i][index];
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxBool
mxHashMap<K, V, HF, CF>::operator ==(const mxHashMap& hashMap) const
{
	return equal(hashMap);
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxBool
mxHashMap<K, V, HF, CF>::operator !=(const mxHashMap& hashMap) const
{
	return !equal(hashMap);
}
//------------------------------------------------------------------------------
template<class K, class V, class HF, class CF>
mxHashMap<K, V, HF, CF>&
mxHashMap<K, V, HF, CF>::operator =(const mxHashMap& hashMap)
{
	copy(hashMap);
	return *this;
}
//------------------------------------------------------------------------------
#ifdef	_DEBUG
template<class K, class V, class HF, class CF>
void
mxHashMap<K, V, HF, CF>::postInformation() const
{
	for (mxUInt i = 0; i < _tableSize; ++i)
	{
		mxNotify::post(mxNotify::DEBUG, "table:%d size:%d",
			i, _keyTable[i].getSize());
	}
}
#endif
//------------------------------------------------------------------------------
#endif	// !__mxHash_inl__
