/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "StdAfx.h"
#include <mx/mxCore/mxIniFile.h>
//------------------------------------------------------------------------------
mxIniFile::mxIniFile()
{
}
//------------------------------------------------------------------------------
mxIniFile::mxIniFile(const mxString& filename) :
_filename(filename)
{
}
//------------------------------------------------------------------------------
void
mxIniFile::setFilename(const mxString& filename)
{
	_filename = filename;
}
//------------------------------------------------------------------------------
const mxString&
mxIniFile::getFilename() const
{
	return _filename;
}
//------------------------------------------------------------------------------
mxUInt
mxIniFile::readUInt(const mxString& section,
					const mxString& key,
					mxUInt defaultValue)
{
	assert(_filename.length() > 0);

	mxUInt value = GetPrivateProfileInt(
		section.c_str(),
		key.c_str(),
		defaultValue,
		_filename.c_str());
	return value;
}
//------------------------------------------------------------------------------
mxBool
mxIniFile::writeUInt(const mxString& section,
					 const mxString& key,
					 mxUInt value)
{
	assert(_filename.length() > 0);

	mxString svalue = mxString((mxInt)value);

	return writeString(section, key, svalue);
}
//------------------------------------------------------------------------------
mxFloat
mxIniFile::readFloat(const mxString& section,
					 const mxString& key,
					 mxFloat defaultValue)
{
	assert(_filename.length() > 0);

	mxString svalue = readString(section, key, mxString(mxFloat(defaultValue)));
	mxFloat value;
	sscanf(svalue.c_str(), "%f", &value);

	return value;
}
//------------------------------------------------------------------------------
mxBool
mxIniFile::writeFloat(const mxString& section,
					  const mxString& key,
					  mxFloat value)
{
	assert(_filename.length() > 0);

	mxString svalue = mxString((mxFloat)value);

	return writeString(section, key, svalue);
}
//------------------------------------------------------------------------------
mxString
mxIniFile::readString(const mxString& section,
					  const mxString& key,
					  const mxString& defaultValue)
{
	assert(_filename.length() > 0);

	mxChar buf[MAX_PATH];
	GetPrivateProfileString(
		section.c_str(),
		key.c_str(),
		defaultValue.c_str(),
		buf,
		MAX_PATH,
		_filename.c_str());

	return buf;
}
//------------------------------------------------------------------------------
mxBool
mxIniFile::writeString(const mxString& section,
					   const mxString& key,
					   const mxString& value)
{
	assert(_filename.length() > 0);

	return WritePrivateProfileString(
		section.c_str(),
		key.c_str(),
		value.c_str(),
		_filename.c_str()) != 0;
}
//------------------------------------------------------------------------------
