/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "StdAfx.h"
#include <mx/mxCore/mxNotify.h>
#include <cstdio>
//------------------------------------------------------------------------------
static const mxChar* LEVEL_STR[] =
{
	"[Error]",
	"[Warn]",
	"[Notice]",
	"[Info]",
	"[Fatal]",
	"[Debug]",
	NULL
};
static const mxChar OUTPUT_FILENAME[] =
{
	"errorlog.txt"
};
//------------------------------------------------------------------------------
mxNotify::OutputEnum	mxNotify::_output =	mxNotify::DEFAULT_OUTPUT;
//------------------------------------------------------------------------------
void
mxNotify::post(LevelEnum level, const mxChar* format...)
{
	mxChar msg[256], buf[256];

	if (level >= ERR && level <= DEBUG)
		sprintf(msg, "%s : ", LEVEL_STR[level]);
	else
		strcpy(msg, "[Unknown] : ");

	va_list	args;
	va_start(args, format);
	vsprintf(buf, format, args);
	va_end(args);
	strcat(msg, buf);
	strcat(msg, "\n");

	switch (_output)
	{
	case TO_CONSOLE:
#ifndef _WIN32
	case TO_DEBUGWINDOW:
#endif
		fprintf(stderr, "%s", msg);
		break;

	case TO_FILE:
		{
			FILE* fp = fopen(OUTPUT_FILENAME, "a");
			if (fp)
			{
				fprintf(fp, "%s", msg);
				fclose(fp);
			}
		}
		break;

#ifdef _WIN32
	case TO_DEBUGWINDOW:
		OutputDebugString(msg);
		break;
#endif
	}
}
//------------------------------------------------------------------------------
void
mxNotify::post(LevelEnum level, const mxString& str)
{
	mxString msg;

	if (level >= ERR && level <= DEBUG)
	{
		msg = LEVEL_STR[level];
		msg += " : ";
	}
	else
		msg = "[Unknown] : ";

	msg += str;
	msg += "\n";

	switch (_output) {
	case TO_CONSOLE:
#ifndef _WIN32
	case TO_DEBUGWINDOW:
#endif
		fprintf(stderr, "%s", msg.c_str());
		break;

	case TO_FILE:
		{
			FILE* fp = fopen(OUTPUT_FILENAME, "a");
			if (fp)
			{
				fprintf(fp, "%s", msg.c_str());
				fclose(fp);
			}
		}
		break;

#ifdef _WIN32
	case TO_DEBUGWINDOW:
		OutputDebugString(msg.c_str());
		break;
#endif
	}
}
//------------------------------------------------------------------------------
