/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//----------------------------------------------------------------------------
#ifndef	__mxObject_h__
#define	__mxObject_h__
//----------------------------------------------------------------------------
#include <mx/mxCore/mxCore.h>
//----------------------------------------------------------------------------
#pragma warning(disable: 4183)
//----------------------------------------------------------------------------
class MXCORECLASS mxObject
{
protected:
	mxObject();
	virtual ~mxObject();

public:
	// ref()	-> increment a reference counter.
	// unref()	-> decrement a reference counter but not deleted this class.
	mxUInt			ref();
	mxUInt			unref();
	mxUInt			getRef() const;
	// destroy() ->	if a reference counter is 0, it is called _destroy(),
	//				else a reference counter is decremented.
	void			destroy();	

protected:
	// really delete this class.
	// (when you define a derivation class of this class
	//  in other DLL, you must overwrite this function.)
	virtual void	_destroy();

public:
	mxClassType		getClassType() const;
	const mxChar*	getClassName() const;

	mxBool			isExactType(mxClassType type) const;
	virtual mxBool	isDerivedFrom(mxClassType type) const;

	static mxClassType		getStaticClassType();
	static const mxChar*	getStaticClassName();

#ifdef _DEBUG
public:
	void			postInformation();

private:
	friend class mxCore;
	static mxUInt	_newCount;
	static mxUInt	_destroyCount;
#endif

private:
	mxUInt			_referenceCount;
};
//----------------------------------------------------------------------------
#define	SAFE_REF(o)					\
	{ if (o != NULL) o->ref(); }
#define	SAFE_DESTROY(o)				\
	{ if (o != NULL) { o->destroy(); o = NULL; } }
#define	SAFE_UNREF_DESTROY(o)		\
	{ if (o != NULL) { if (o->unref() == 0) { o->destroy(); o = NULL; } } }
//----------------------------------------------------------------------------
#define	MXOBJECTHEADER(ownType, parentType)							\
public:																\
	static mxClassType		getStaticClassType()					\
	{ return typeid(ownType); }										\
	static const mxChar*	getStaticClassName()					\
	{ return typeid(ownType).name(); }								\
public:																\
	virtual mxBool			isDerivedFrom(mxClassType type) const	\
	{																\
		if (type == typeid(ownType))								\
			return true;											\
		return parentType::isDerivedFrom(type);						\
	}																\
private:															\
	typedef parentType	inherited
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
template<class T>
class mxObjectPtr
{
public:
	mxObjectPtr(T* ptr = NULL);
	mxObjectPtr(const mxObjectPtr& objectPtr);
	~mxObjectPtr();

public:
	mxBool			operator !() const;
					operator const mxObject*() const;

	mxObjectPtr&	operator =(const mxObjectPtr& objectPtr);

	T&				operator *() const;
	T*				operator ->() const;

private:
	T*				_ptr;
};
//------------------------------------------------------------------------------
#include <mx/mxCore/mxObject.inl>
//------------------------------------------------------------------------------
#endif	// !__mxObject_h__
