/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "StdAfx.h"
#include <mx/mxCore/mxRegistry.h>
//------------------------------------------------------------------------------
mxString	mxRegistry::_companyName("Company");
mxString	mxRegistry::_productName("Product");
//------------------------------------------------------------------------------
mxRegistry::mxRegistry() :
_keyHandle(NULL)
{
}
//------------------------------------------------------------------------------
mxRegistry::~mxRegistry()
{
	if (_keyHandle != NULL)
		closeKey();
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::createKey(const mxString& key, RootKeyEnum rootKey)
{
	assert(_keyHandle == NULL);

	mxString openKey;
	
	openKey  = "Software\\";
	openKey += _companyName;
	openKey += "\\";
	openKey += _productName;
	openKey += "\\";
	openKey += key;

	HKEY	hRootKey;
	switch (rootKey)
	{
	case RK_CLASSES_ROOT:
		hRootKey = HKEY_CLASSES_ROOT;
		break;
	case RK_CURRENT_USER:
		hRootKey = HKEY_CURRENT_USER;
		break;
	case RK_LOCAL_MACHINE:
		hRootKey = HKEY_LOCAL_MACHINE;
		break;
	case RK_USERS:
		hRootKey = HKEY_USERS;
		break;
	case RK_CURRENT_CONFIG:
		hRootKey = HKEY_CURRENT_CONFIG;
		break;
	default:
		return false;
	}

	_keyHandle = _createKey(openKey, hRootKey);

	return _keyHandle != NULL;
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::closeKey()
{
	assert(_keyHandle != NULL);

	if (RegCloseKey(_keyHandle) == ERROR_SUCCESS)
	{
		_keyHandle = NULL;
		return true;
	}

	return false;
}
//------------------------------------------------------------------------------
mxUInt
mxRegistry::readUInt(const mxString& section,
					 const mxString& entry,
					 mxUInt defaultValue)
{
	assert(_keyHandle != NULL);

	mxUInt result = defaultValue;

	HKEY hOpenKey = _createKey(section, _keyHandle);
	assert(hOpenKey != NULL);

	if (!_readUInt(entry, result, hOpenKey))
		result = defaultValue;

	return result;
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::writeUInt(const mxString& section,
					  const mxString& entry,
					  mxUInt value)
{
	assert(_keyHandle != NULL);

	HKEY hOpenKey = _createKey(section, _keyHandle);
	assert(hOpenKey != NULL);

	return _writeUInt(entry, value, hOpenKey);
}
//------------------------------------------------------------------------------
mxString
mxRegistry::readString(const mxString& section,
					   const mxString& entry,
					   const mxString& defaultValue)
{
	assert(_keyHandle != NULL);

	mxString result = defaultValue;

	HKEY hOpenKey = _createKey(section, _keyHandle);
	assert(hOpenKey != NULL);

	if (!_readString(entry, result, hOpenKey))
		result = defaultValue;

	return result;
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::writeString(const mxString& section,
						const mxString& entry,
						const mxString& value)
{
	assert(_keyHandle != NULL);

	HKEY hOpenKey = _createKey(section, _keyHandle);
	assert(hOpenKey != NULL);

	return _writeString(entry, value, hOpenKey);
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::deleteSection(const mxString& section)
{
	assert(_keyHandle != NULL);

	return RegDeleteKey(_keyHandle, section.c_str()) == ERROR_SUCCESS;
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::deleteValue(const mxString& section, const mxString& entry)
{
	assert(_keyHandle != NULL);

	HKEY hOpenKey = _createKey(section, _keyHandle);
	assert(hOpenKey != NULL);

	return RegDeleteValue(hOpenKey, entry.c_str()) == ERROR_SUCCESS;
}
//------------------------------------------------------------------------------
void
mxRegistry::setCompanyName(const mxString& name)
{
	_companyName = name;
}
//------------------------------------------------------------------------------
const mxString&
mxRegistry::getCompanyName()
{
	return _companyName;
}
//------------------------------------------------------------------------------
void
mxRegistry::setProductName(const mxString& name)
{
	_productName = name;
}
//------------------------------------------------------------------------------
const mxString&
mxRegistry::getProductName()
{
	return _productName;
}
//------------------------------------------------------------------------------
HKEY
mxRegistry::_createKey(const mxString& key, HKEY hKey)
{
	DWORD	disposition;
	HKEY	hOpenedKey;

	if (RegCreateKeyEx(hKey, key.c_str(), 0, "",
			REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
			&hOpenedKey, &disposition) != ERROR_SUCCESS)
		hOpenedKey = NULL;

	return hOpenedKey;
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::_readUInt(const mxString& valueName, mxUInt& value, HKEY hKey)
{
	DWORD	type;
	DWORD	size = sizeof(DWORD);

	if (RegQueryValueEx(hKey, valueName.c_str(), 0, &type,
			(LPBYTE)&value, &size) != ERROR_SUCCESS)
		return false;

	return true;
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::_writeUInt(const mxString& valueName, mxUInt value, HKEY hKey)
{
	DWORD	buf = (DWORD)value;

	if (RegSetValueEx(hKey, valueName.c_str(), 0, REG_DWORD,
			(LPBYTE)&buf, sizeof(DWORD)) != ERROR_SUCCESS)
		return false;

	return true;
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::_readString(const mxString& valueName, mxString& value, HKEY hKey)
{
	const DWORD	MAX_SIZE = 256;
	mxChar	buf[MAX_SIZE];
	DWORD	size = MAX_SIZE;
	DWORD	type;

	memset(buf, 0, MAX_SIZE);

	if (RegQueryValueEx(hKey, valueName.c_str(), 0, &type,
			(LPBYTE)buf, &size) != ERROR_SUCCESS)
		return false;

	value = buf;

	return true;
}
//------------------------------------------------------------------------------
mxBool
mxRegistry::_writeString(const mxString& valueName,
						 const mxString& value, HKEY hKey)
{
	if (RegSetValueEx(hKey, valueName.c_str(), 0, REG_SZ,
			(LPBYTE)value.c_str(), value.length()) != ERROR_SUCCESS)
		return false;

	return true;
}
//------------------------------------------------------------------------------
