/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "StdAfx.h"
#include <mx/mxCore/mxString.h>
#include <cstdio>
//------------------------------------------------------------------------------
mxString::mxString(mxInt intString) :
_string(NULL)
{
	mxChar buf[32];
	sprintf(buf, "%d", intString);
	_string = new mxChar[strlen(buf)+1];
	strcpy(_string, buf);
}
//------------------------------------------------------------------------------
mxString::mxString(mxFloat floatString) :
_string(NULL)
{
	mxChar buf[32];
	sprintf(buf, "%f", floatString);
	_string = new mxChar[strlen(buf)+1];
	strcpy(_string, buf);
}
//------------------------------------------------------------------------------
mxString::mxString(mxDouble doubleString) :
_string(NULL)
{
	mxChar buf[32];
	sprintf(buf, "%lf", doubleString);
	_string = new mxChar[strlen(buf)+1];
	strcpy(_string, buf);
}
//------------------------------------------------------------------------------
mxString::~mxString()
{
	makeEmpty();
}
//------------------------------------------------------------------------------
mxUInt
mxString::length() const
{
	return (_string != NULL) ? (mxUInt)strlen(_string) : 0;
}
//------------------------------------------------------------------------------
void
mxString::length(mxUInt len)
{
	if (len == 0)
	{
		makeEmpty();
	}
	else
	{
		mxUInt	aLen = length();
		mxChar*	newStr = new mxChar[len+1];
		memset(newStr, 0, len+1);
		if (_string != NULL)
			memcpy(newStr, _string, (aLen<len) ? aLen : len);
		makeEmpty();
		_string = newStr;
	}
}
//------------------------------------------------------------------------------
mxBool
mxString::isEmpty() const
{
	return (_string == NULL) || (*_string == '\0');
}
//------------------------------------------------------------------------------
void
mxString::makeEmpty()
{
	delete [] _string;
	_string = NULL;
}
//------------------------------------------------------------------------------
mxString
mxString::lowerCase() const
{
	mxString	str = *this;
	str.toLower();
	return str;
}
//------------------------------------------------------------------------------
mxString
mxString::upperCase() const
{
	mxString	str = *this;
	str.toUpper();
	return str;
}
//------------------------------------------------------------------------------
void
mxString::toLower()
{
	if (_string != NULL)
	{
		for (mxChar* str = _string; *str; str++)
			*str = (mxChar)tolower(*str);
	}
}
//------------------------------------------------------------------------------
void
mxString::toUpper()
{
	if (_string != NULL)
	{
		for (mxChar* str = _string; *str; str++)
			*str = (mxChar)toupper(*str);
	}
}
//------------------------------------------------------------------------------
mxInt
mxString::toInt(mxInt defaultValue) const
{
	if (_string != NULL)
	{
		mxInt intString = defaultValue;
		sscanf(_string, "%d", &intString);
		return intString;
	}
	return defaultValue;
}
//------------------------------------------------------------------------------
mxFloat
mxString::toFloat(mxFloat defaultValue) const
{
	if (_string != NULL)
	{
		mxFloat floatString = defaultValue;
		sscanf(_string, "%f", &floatString);
		return floatString;
	}
	return defaultValue;
}
//------------------------------------------------------------------------------
mxDouble
mxString::toDouble(mxDouble defaultValue) const
{
	if (_string != NULL)
	{
		mxDouble doubleString = defaultValue;
		sscanf(_string, "%lf", &doubleString);
		return doubleString;
	}
	return defaultValue;
}
//------------------------------------------------------------------------------
const mxString
mxString::subString(mxUInt index) const
{
	mxUInt aLen = length();
	if (aLen > index)
	{
		mxString str(_string+index);
		str.length(aLen-index);
		return str;
	}
	return "";
}
//------------------------------------------------------------------------------
const mxString
mxString::subString(mxUInt index, mxUInt count) const
{
	mxUInt aLen = length();
	if (aLen >= index + count)
	{
		mxString str(_string+index);
		str.length(count);
		return str;
	}
	return "";
}
//------------------------------------------------------------------------------
mxString&
mxString::insert(const mxString& str, mxUInt index)
{
	mxUInt aLen = length();
	mxUInt bLen = str.length();
	if (aLen > index)
	{
		mxChar* newStr = new mxChar[aLen+bLen+1];
		if (index > 0)
			memcpy(newStr, _string, index);
		newStr[index] = '\0';
		strcat(newStr, str._string);
		strcat(newStr, _string+index);
		makeEmpty();
		_string = newStr;
	}
	return *this;
}
//------------------------------------------------------------------------------
mxString&
mxString::format(const mxChar* format...)
{
	mxChar str[512];
	va_list	args;
	va_start(args, format);
	vsprintf(str, format, args);
	va_end(args);
    *this = str;
	return *this;
}
//------------------------------------------------------------------------------
mxString&
mxString::replace(const mxString& oldString, const mxString newString)
{
	mxChar* sstr = strstr(_string, oldString._string);
	if (sstr != NULL)
	{
		mxUInt aLen = length();
		mxUInt oLen = oldString.length();
		mxUInt nLen = newString.length();
		mxUInt sLen = (mxUInt)strlen(sstr);

		mxChar* newStr = new mxChar[aLen-oLen+nLen+1];
		memset(newStr, 0, aLen-oLen+nLen+1);

		if (_string != sstr)
			memcpy(newStr, _string, aLen-sLen);
		strcat(newStr, newString._string);
		if (aLen-sLen+oLen < aLen)
			strcat(newStr, &_string[aLen-sLen+oLen]);

		makeEmpty();
		_string = newStr;
	}
	return *this;
}
//------------------------------------------------------------------------------
mxUInt
mxString::hash() const
{
	mxUInt total = 0, shift = 0;
	if (_string != NULL)
	{
		for (mxChar* s = _string; *s; s++)
		{
			total = total ^ ((*s) << shift);
			shift += 5;
			if (shift > 24)
				shift -= 24;
		}
	}
	return total;
}
//------------------------------------------------------------------------------
mxString&
mxString::operator =(const mxChar* str)
{
	mxUInt aLen = (str != NULL) ? (mxUInt)strlen(str) : 0;
	if (aLen > 0)
	{
		mxUInt bLen = length();
		if (str >= _string && str <= _string + bLen + 1)
		{
			mxString tmp(str);
			*this = tmp._string;
		}
		else
		{
			makeEmpty();
			_string = new mxChar[aLen+1];
			strcpy(_string, str);
		}
	}
	else
	{
		makeEmpty();
	}
	return *this;
}
//------------------------------------------------------------------------------
mxString&
mxString::operator +=(const mxChar* str)
{
	mxUInt aLen = (str != NULL) ? (mxUInt)strlen(str) : 0;
	if (aLen > 0)
	{
		mxUInt bLen = length();
		mxChar* newStr = new mxChar[aLen+bLen+1];
		if (bLen > 0)
			strcpy(newStr, _string);
		else
			newStr[0] = '\0';
		strcat(newStr, str);
		makeEmpty();
		_string = newStr;
	}
	return *this;
}
//------------------------------------------------------------------------------
const mxString
operator +(const mxString& str1, const mxString& str2)
{
	mxString newStr(str1);
	newStr += str2._string;
	return newStr;
}
//------------------------------------------------------------------------------
mxBool
operator ==(const mxString& str1, const mxChar* str2)
{
	return (str1._string != NULL && str2 != NULL) ?
		((str1._string[0] == str2[0]) &&
		(strcmp(str1._string, str2) == 0)) : false;
}
//------------------------------------------------------------------------------
mxBool
operator < (const mxString& str1, const mxChar* str2)
{
	return (str1._string != NULL && str2 != NULL) ?
		strcmp(str1._string, str2) < 0 : false;
}
//------------------------------------------------------------------------------
mxBool
operator > (const mxString& str1, const mxChar* str2)
{
	return (str1._string != NULL && str2 != NULL) ?
		strcmp(str1._string, str2) > 0 : false;
}
//------------------------------------------------------------------------------
mxBool
operator <=(const mxString& str1, const mxChar* str2)
{
	return (str1._string != NULL && str2 != NULL) ?
		strcmp(str1._string, str2) <= 0 : false;
}
//------------------------------------------------------------------------------
mxBool
operator >=(const mxString& str1, const mxChar* str2)
{
	return (str1._string != NULL && str2 != NULL) ?
		strcmp(str1._string, str2) >= 0 : false;
}
//------------------------------------------------------------------------------
