/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#ifndef	__mxString_h__
#define	__mxString_h__
//------------------------------------------------------------------------------
#include <mx/mxCore/mxArray.h>
//------------------------------------------------------------------------------
class mxString;
typedef mxArray<mxString>	mxStringArray;
//------------------------------------------------------------------------------
class MXCORECLASS mxString
{
public:
	mxString();
	mxString(const mxChar* str);
	mxString(const mxString& str);
	mxString(mxInt intString);
	mxString(mxFloat floatString);
	mxString(mxDouble doubleString);
	~mxString();

public:
	const mxChar*	c_str() const;

	mxUInt			length() const;
	void			length(mxUInt len);

	mxBool			isEmpty() const;
	void			makeEmpty();

	mxString		lowerCase() const;
	mxString		upperCase() const;
	void			toLower();
	void			toUpper();

	mxInt			toInt(mxInt defaultValue = 0) const;
	mxFloat			toFloat(mxFloat defaultValue = 0) const;
	mxDouble		toDouble(mxDouble defaultValue = 0) const;

	const mxString	subString(mxUInt index) const;
	const mxString	subString(mxUInt index, mxUInt count) const;
	mxString&		insert(const mxString& str, mxUInt index);
	mxString&		format(const mxChar* format...);
	mxString&		replace(const mxString& oldString, const mxString newString);

	mxUInt			hash() const;

public:
	mxChar&			operator [](const int idx);
	const mxChar&	operator [](const int idx) const;

	mxString&		operator =(const mxChar* str);
	mxString&		operator =(const mxString& str);

	mxString&		operator +=(const mxChar* str);
	mxString&		operator +=(const mxString& str);

	mxBool			operator !() const;

	friend MXCORECLASS const mxString
		operator +(const mxString& str1, const mxString& str2);
	friend MXCORECLASS mxBool
		operator ==(const mxString& str1, const mxChar* str2);
	friend MXCORECLASS mxBool
		operator ==(const mxChar* str1, const mxString& str2);
	friend MXCORECLASS mxBool
		operator ==(const mxString& str1, const mxString& str2);

	friend MXCORECLASS mxBool
		operator !=(const mxString& str1, const mxChar* str2);
	friend MXCORECLASS mxBool
		operator !=(const mxChar* str1, const mxString& str2);
	friend MXCORECLASS mxBool
		operator !=(const mxString& str1, const mxString& str2);

	friend MXCORECLASS mxBool
		operator < (const mxString& str1, const mxChar* str2);
	friend MXCORECLASS mxBool
		operator < (const mxChar* str1, const mxString& str2);
	friend MXCORECLASS mxBool
		operator < (const mxString& str1, const mxString& str2);

	friend MXCORECLASS mxBool
		operator > (const mxString& str1, const mxChar* str2);
	friend MXCORECLASS mxBool
		operator > (const mxChar* str1, const mxString& str2);
	friend MXCORECLASS mxBool
		operator > (const mxString& str1, const mxString& str2);

	friend MXCORECLASS mxBool
		operator <=(const mxString& str1, const mxChar* str2);
	friend MXCORECLASS mxBool
		operator <=(const mxChar* str1, const mxString& str2);
	friend MXCORECLASS mxBool
		operator <=(const mxString& str1, const mxString& str2);

	friend MXCORECLASS mxBool
		operator >=(const mxString& str1, const mxChar* str2);
	friend MXCORECLASS mxBool
		operator >=(const mxChar* str1, const mxString& str2);
	friend MXCORECLASS mxBool
		operator >=(const mxString& str1, const mxString& str2);

private:
	mxChar*		_string;
};
//------------------------------------------------------------------------------
DECLARE_SAFE_COPYFUNC(mxString);
DECLARE_SAFE_COMPFUNC(mxString);
//------------------------------------------------------------------------------
template<>
struct mxKeyHashFunc<mxString>
{
	mxUInt	operator ()(const mxString& key) const
	{ return key.hash(); }
};
//------------------------------------------------------------------------------
#include <mx/mxCore/mxString.inl>
//------------------------------------------------------------------------------
#endif	// !__mxString_h__
