/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//------------------------------------------------------------------------------
#include "StdAfx.h"
#define	_THREADEX
#include <mx/mxCore/mxThread.h>
#ifdef _THREADEX
#include <process.h>
#endif
//------------------------------------------------------------------------------
mxThread::mxThread() :
mxThreadHandle()
{
}
//------------------------------------------------------------------------------
mxThread::mxThread(HANDLE handle) :
mxThreadHandle(handle)
{
}
//------------------------------------------------------------------------------
mxThread::mxThread(mxThreadHandler* handler, mxBool createSuspended) :
mxThreadHandle()
{
	create(handler, createSuspended);
}
//------------------------------------------------------------------------------
mxThread::mxThread(ThreadProc proc, void* data, mxBool createSuspended) :
mxThreadHandle()
{
	create(proc, data, createSuspended);
}
//------------------------------------------------------------------------------
mxThread::~mxThread()
{
	if (isValid())
	{
		const mxUInt msec = 1000;
		if (!waitFor(msec))
			terminate();
	}
}
//------------------------------------------------------------------------------
mxBool
mxThread::create(mxThreadHandler* handler, mxBool createSuspended)
{
	if (isValid())
		return false;

#ifdef _THREADEX
	_handle = (HANDLE)_beginthreadex(
					NULL,
					0,
					(ThreadProc)callThreadHandlerProc,
					(void*)handler,
					createSuspended ? CREATE_SUSPENDED : 0,
					&_threadID);
#else
	_handle = CreateThread(
					NULL,
					0,
					(LPTHREAD_START_ROUTINE)callThreadHandlerProc,
					(void*)handler,
					createSuspended ? CREATE_SUSPENDED : 0,
					(mxULong*)&_threadID);
#endif

	return isValid();
}
//------------------------------------------------------------------------------
mxBool
mxThread::create(ThreadProc proc, void* data, mxBool createSuspended)
{
	if (isValid())
		return false;

#ifdef _THREADEX
	_handle = (HANDLE)_beginthreadex(
					NULL,
					0,
					proc,
					data,
					createSuspended ? CREATE_SUSPENDED : 0,
					&_threadID);
#else
	_handle = CreateThread(
					NULL,
					0,
					(LPTHREAD_START_ROUTINE)proc,
					data,
					createSuspended ? CREATE_SUSPENDED : 0,
					(mxULong*)&_threadID);
#endif

	return isValid();
}
//------------------------------------------------------------------------------
mxBool
mxThread::suspend() const
{
	assert(_handle != INVALID_HANDLE_VALUE);

	return SuspendThread(_handle) != (DWORD)-1;
}
//------------------------------------------------------------------------------
mxBool
mxThread::resume() const
{
	assert(_handle != INVALID_HANDLE_VALUE);

	return ResumeThread(_handle) != (DWORD)-1;
}
//------------------------------------------------------------------------------
void
mxThread::exit(mxUInt exitCode) const
{
#ifdef _THREADEX
	_endthreadex(exitCode);
#else
	ExitThread(exitCode);
#endif
}
//------------------------------------------------------------------------------
mxBool
mxThread::terminate(mxUInt exitCode) const
{
	assert(_handle != INVALID_HANDLE_VALUE);

	return TerminateThread(_handle, exitCode) != 0;
}
//------------------------------------------------------------------------------
mxBool
mxThread::getExitCode(mxUInt& exitCode) const
{
	assert(_handle != INVALID_HANDLE_VALUE);

	return GetExitCodeThread(_handle, (mxULong*)&exitCode) != 0;
}
//------------------------------------------------------------------------------
mxBool
mxThread::setPriority(PriorityEnum priority) const
{
	assert(_handle != INVALID_HANDLE_VALUE);

	mxInt p;

	switch (priority)
	{
	case THREAD_PRIORITY_ABOVE_NORMAL:
		p = THREAD_PRIORITY_ABOVE_NORMAL;
		break;
	case THREAD_PRIORITY_BELOW_NORMAL:
		p = THREAD_PRIORITY_BELOW_NORMAL;
		break;
	case THREAD_PRIORITY_HIGHEST:
		p = THREAD_PRIORITY_HIGHEST;
		break;
	case THREAD_PRIORITY_IDLE:
		p = THREAD_PRIORITY_IDLE;
		break;
	case THREAD_PRIORITY_LOWEST:
		p = THREAD_PRIORITY_LOWEST;
		break;
	case THREAD_PRIORITY_NORMAL:
		p = THREAD_PRIORITY_NORMAL;
		break;

	default:
		return false;
	}

	return SetThreadPriority(_handle, p) != 0;
}
//------------------------------------------------------------------------------
mxThread::PriorityEnum
mxThread::getPriority() const
{
	assert(_handle != INVALID_HANDLE_VALUE);

	mxInt p = GetThreadPriority(_handle);
	switch (p)
	{
	case THREAD_PRIORITY_ABOVE_NORMAL:
		return ABOVE_NORMAL;
	case THREAD_PRIORITY_BELOW_NORMAL:
		return BELOW_NORMAL;
	case THREAD_PRIORITY_HIGHEST:
		return HIGHEST;
	case THREAD_PRIORITY_IDLE:
		return IDLE;
	case THREAD_PRIORITY_LOWEST:
		return LOWEST;
	case THREAD_PRIORITY_NORMAL:
		return NORMAL;
	}

	return UNKNOWN_PRIORITY;
}
//------------------------------------------------------------------------------
mxUInt
mxThread::getThreadID() const
{
	return _threadID;
}
//------------------------------------------------------------------------------
mxUInt WINAPI
mxThread::callThreadHandlerProc(void* data)
{
	mxThreadHandler* handler = static_cast<mxThreadHandler*>(data);
	if (handler)
		return handler->handlerProc();
	return 0;
}
//------------------------------------------------------------------------------
