/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//---------------------------------------------------------------------------
#include "fsCommandHistory.h"
#include <assert.h>
//---------------------------------------------------------------------------
fsCommandHistory::fsCommandHistory(TMenuItem* HistoryBeginMenuItem,
                                   TMenuItem* HistoryEndMenuItem,
                                   int maxHistories)
{
    _historyBeginMenuItem	= HistoryBeginMenuItem;
    _historyEndMenuItem		= HistoryEndMenuItem;

    _historyMenuItem		= _historyBeginMenuItem->Parent;

    assert(_historyMenuItem == _historyEndMenuItem->Parent);

	_prefixIdList			= new TList();

    _maxHistories = maxHistories;
}
//---------------------------------------------------------------------------
fsCommandHistory::~fsCommandHistory()
{
	clear();

    delete _prefixIdList;
}
//---------------------------------------------------------------------------
void
fsCommandHistory::clear()
{
	int beginIndex = _historyMenuItem->IndexOf(_historyBeginMenuItem) + 1;
	int endIndex = _historyMenuItem->IndexOf(_historyEndMenuItem);

    for(int i = beginIndex; i < endIndex; i++)
    {
    	TMenuItem* menuItem = _historyMenuItem->Items[beginIndex];
		_historyMenuItem->Delete(beginIndex);
        delete menuItem;
    }

    _prefixIdList->Clear();
}
//---------------------------------------------------------------------------
void
fsCommandHistory::add(TAction* action, int prefixId)
{
	int beginIndex = _historyMenuItem->IndexOf(_historyBeginMenuItem) + 1;

    // Check Action
    TAction* prevAction = (TAction *)_historyMenuItem->Items[beginIndex]->Action;
    if (prevAction == action)
    	return ;

	// Create New MenuItem
	TMenuItem* newMenuItem = new TMenuItem(_historyMenuItem);
    newMenuItem->Action = action;

    // Insert MenuItem
    _historyMenuItem->Insert(beginIndex, newMenuItem);
	_prefixIdList->Insert(0, (void *)prefixId);
}
//---------------------------------------------------------------------------
void
fsCommandHistory::updateAll()
{
	int beginIndex, endIndex;

    // Clear Old MenuItem
	beginIndex = _historyMenuItem->IndexOf(_historyBeginMenuItem) + 1;
	endIndex = _historyMenuItem->IndexOf(_historyEndMenuItem);

	if (_maxHistories < endIndex - beginIndex)
    {
		for(int i = _maxHistories; i < endIndex - beginIndex; i++)
        {
	    	TMenuItem *oldMenuItem = _historyMenuItem->Items[beginIndex + _maxHistories];

            TAction* oldAction = (TAction *)oldMenuItem->Action;
            if (oldAction->Owner == NULL)
            	delete oldAction;

    	    _historyMenuItem->Delete(beginIndex + _maxHistories);
        	delete oldMenuItem;

	        _prefixIdList->Delete(_maxHistories);
        }
    }

    // Udate MenuItem Status
	beginIndex = _historyMenuItem->IndexOf(_historyBeginMenuItem) + 1;
	endIndex = _historyMenuItem->IndexOf(_historyEndMenuItem);

    for(int i = beginIndex, ii = 0; i < endIndex; i++, ii++)
    {
    	// Uncheck All MenuItem
        _historyMenuItem->Items[i]->Enabled = true;
    	_historyMenuItem->Items[i]->Checked = false;

        // Update Caption
        TAction *action = (TAction *)_historyMenuItem->Items[i]->Action;

        if ((int)_prefixIdList->Items[ii] != -1)
	        _historyMenuItem->Items[i]->Caption =
							getPrefixIdString((int)_prefixIdList->Items[ii]) +
        					" \"" + action->Caption + "\"";
        else
	        _historyMenuItem->Items[i]->Caption = " \"" + action->Caption + "\"";
    }
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void
fsCommandHistory::addOriginal(const AnsiString& caption,
							  TNotifyEvent Handler,
                              void *data,
                              int prefixId)
{
	int beginIndex = _historyMenuItem->IndexOf(_historyBeginMenuItem) + 1;

    // Check Caption
    TAction* prevAction = (TAction *)_historyMenuItem->Items[beginIndex]->Action;
    if (prevAction != NULL &&
    	prevAction->Caption == caption &&
        _prefixIdList->Count != 0 &&
        _prefixIdList->Items[0] == (void *)prefixId)
    	return ;

	// Create New Action
	TAction *newAction = new TAction(NULL);
    newAction->Caption = caption;
    newAction->OnExecute = Handler;
    newAction->Tag = (int)data;

	// Create New MenuItem
    TMenuItem *newMenuItem = new TMenuItem(_historyMenuItem);
    newMenuItem->Action = newAction;

    // Insert MenuItem
    _historyMenuItem->Insert(beginIndex, newMenuItem);
	_prefixIdList->Insert(0, (void *)prefixId);
}
//---------------------------------------------------------------------------

