/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//---------------------------------------------------------------------------
#include "fsLanguage.h"
//---------------------------------------------------------------------------
fsLanguage::fsLanguage(TMenuItem* LanguageMenuItem)
{
	_language = 0;

	//
	// R[h̐ݒ
    //
    _languageCodes = new TStringList();

	// ftHg̐ݒ
    _languageCodes->Add("ja");	// Default
    _languageCodes->Add("en");	// Default

    // ǉR[h̓ǂݍ
    readAdditionalLangCodes();

	//
	// \L̐ݒ
    //
    _notationsList = new TList();

    // ftHg̐ݒ
	rsNotations* notations;
	notations = new rsNotations();
   	_notationsList->Add(notations);
    notations->addNotation("ja", "{(&J)");
    notations->addNotation("en", "&Japanese");

	notations = new rsNotations();
   	_notationsList->Add(notations);
    notations->addNotation("ja", "&English");
    notations->addNotation("en", "&English");

    // ǉ\L̓ǂݍ
    for(int i = 0; i < _languageCodes->Count; i++)
    {
    	if (i <= L_ENGLISH)
        {
        	readNotations((rsNotations *)_notationsList->Items[i], _languageCodes->Strings[i]);
        }
        else
        {
		    notations = new rsNotations();
			readNotations(notations, _languageCodes->Strings[i]);

        	_notationsList->Add(notations);
        }
    }

    //
    // ꃁj[ACe̍쐬
    //
    _languageMenuItems = new TList();
    for(int i = 0; i < _languageCodes->Count; i++)
    {
    	TMenuItem *item = new TMenuItem(LanguageMenuItem);
        item->Caption = "Language: " + AnsiString(i);
        item->OnClick = LanguageMenuItemOnClick;

		LanguageMenuItem->Add(item);
		_languageMenuItems->Add(item);
    }

    // Ǐopt@C|C^
    _fp = NULL;
}
//---------------------------------------------------------------------------
fsLanguage::~fsLanguage()
{
	//
	// ꃁj[ACe̍폜
    //
    for(int i = 0; i < _languageMenuItems->Count; i++)
   	    delete (TMenuItem *)_languageMenuItems->Items[i];
    delete _languageMenuItems;

    //
    // R[hA\L̍폜
    //
    delete _languageCodes;
    for(int i = 0; i < _notationsList->Count; i++)
    	delete (rsNotations *)_notationsList->Items[i];
    delete _notationsList;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
fsLanguage::LanguageMenuItemOnClick(TObject *Sender)
{
    for(int i = 0; i < _languageMenuItems->Count; i++)
    {
    	TMenuItem *item = (TMenuItem *)_languageMenuItems->Items[i];
    	if (item == Sender)
        {
			item->Checked = true;
            _language = i;
        }
        else
        {
        	item->Checked = false;
        }
    }

    _setupLanguage();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int
fsLanguage::getNumLanguageCode() const
{
	return _languageCodes->Count;
}
//---------------------------------------------------------------------------
AnsiString
fsLanguage::getLanguageCode(int index) const
{
	return _languageCodes->Strings[index];
}
//---------------------------------------------------------------------------
void
fsLanguage::setCurrentLanguageCode(const AnsiString& langCode)
{
	AnsiString lc = langCode.LowerCase();

	int index = _languageCodes->IndexOf(lc);
    if (index == -1)
    	return ;
    _language = index;

    TMenuItem *menuItem = (TMenuItem *)_languageMenuItems->Items[_language];
    menuItem->Checked = true;

    _setupLanguage();
}
//---------------------------------------------------------------------------
AnsiString
fsLanguage::getCurrentLanguageCode() const
{
	return _languageCodes->Strings[_language];
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool
fsLanguage::nextEntry(AnsiString& key, AnsiString& val)
{
    key = "";
    val = "";

	if (_fp == NULL)
    	return false;

    char str[1024], str2[1024];

	if (fgets(str, 1024, _fp) == NULL)
    	return false;

	strcpy(str2, str);

    char *tok;
    tok = strtok(str2, " \t\n");
    if (tok == NULL)
		return true;
    key = AnsiString(tok);

    char *ptr;
    ptr = str + strlen(tok);
    if (str + strlen(str) <= ptr)
    	return true;

    while((ptr < str + strlen(str)) && (*ptr == ' ' || *ptr == '\t'))
    	ptr++;

    tok = strtok(ptr, "\n");
    if (tok == NULL)
    	return true;
    val = AnsiString(tok);

	return true;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void
fsLanguage::_setupLanguage()
{
	_setupNotation();

	if (_language == L_JAPANESE)
    	setupDefaultJapanese();
    else
    	setupDefaultEnglish();

    _fp = openLanguageFile(_languageCodes->Strings[_language]);

    setupLanguage();

    fclose(_fp);
}
//---------------------------------------------------------------------------
void
fsLanguage::_setupNotation()
{
    TMenuItem* menuItem;
	rsNotations *notations;
    int numNotations;

    const AnsiString& langCode = _languageCodes->Strings[_language];

	for(int i = 0; i < _languageMenuItems->Count; i++)
    {
       	menuItem = (TMenuItem *)_languageMenuItems->Items[i];

		notations = (rsNotations *)_notationsList->Items[i];
        numNotations = notations->getNumNotations();

		menuItem->Caption = AnsiString("<") + _languageCodes->Strings[i] + AnsiString(">");

        for(int j = 0; j < numNotations; j++)
        	if (langCode == notations->getLanguageCode(j))
            {
            	menuItem->Caption = notations->getNotation(j);
                break;
            }
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void
fsLanguage::readAdditionalLangCodes()
{
	// Get Module file name
    char modulefile[1024];
	GetModuleFileName(NULL, modulefile, 1024);

    // Get actual file name
	char path[1024];
	strcpy(path, modulefile);

	if (strrchr(modulefile, '.'))
		path[ strlen(modulefile) - strlen(strrchr(modulefile, '.')) ] = 0;
	else
    	return ;

    char actFilename[1024];
    sprintf(actFilename, "%s_*.lng", path);

    // Sreach
	WIN32_FIND_DATA FindFileData;
    HANDLE hFind;

    hFind = FindFirstFile(actFilename, &FindFileData);
    if (hFind != INVALID_HANDLE_VALUE)
    {
		do
        {
			char *tok;
            tok = strtok(FindFileData.cFileName, "_.");
            tok = strtok(NULL, "_.");
            if (tok != NULL)
            {
            	if (_languageCodes->IndexOf(AnsiString(tok)) == -1)
	            	_languageCodes->Add(AnsiString(tok));
            }
                
        } while(FindNextFile(hFind, &FindFileData));

        FindClose(hFind);
    }
}
//---------------------------------------------------------------------------
void
fsLanguage::readNotations(rsNotations* notations, const AnsiString& langCode)
{
	_fp = openLanguageFile(langCode);
	if (_fp == NULL)
       	return ;

    char str[1024];
    char key[256];
    char val1[256];
    char val2[256];

	while(fgets(str, 1024, _fp) != NULL)
    {
      	if (sscanf(str, "%s%s%s", key, val1, val2) == 3)
        {
           	if (!strcmp(key, "NotationName"))
            	notations->addNotation(AnsiString(val1), AnsiString(val2));
        }
    }

    fclose(_fp);
}
//---------------------------------------------------------------------------
FILE*
fsLanguage::openLanguageFile(const AnsiString& langCode)
{
	// Get Module file name
    char modulefile[1024];
	GetModuleFileName(NULL, modulefile, 1024);

    // Get actual file name
	char path[1024];
	strcpy(path, modulefile);

	if (strrchr(modulefile, '.'))
		path[ strlen(modulefile) - strlen(strrchr(modulefile, '.')) ] = 0;
	else
    	return NULL;

    char actFilename[1024];
    sprintf(actFilename, "%s_%s.lng", path, langCode.c_str());

	FILE *fp = fopen(actFilename, "r");
	return fp;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
rsNotations::rsNotations()
{
	_languageCodes = new TStringList();
	_notations = new TStringList();
}
//---------------------------------------------------------------------------
rsNotations::~rsNotations()
{
	delete _languageCodes;
    delete _notations;
}
//---------------------------------------------------------------------------
void
rsNotations::addNotation(const AnsiString& langCode, const AnsiString& notation)
{
	int index = _languageCodes->IndexOf(langCode);
    if (index == -1)
    {
		_languageCodes->Add(langCode);
    	_notations->Add(notation);
    }
    else
    {
    	_notations->Strings[index] = notation;
    }
}
//---------------------------------------------------------------------------
void
rsNotations::removeNotation(const AnsiString& langCode)
{
	int index = _languageCodes->IndexOf(langCode);
    if (index != -1)
    {
    	_languageCodes->Delete(index);
        _notations->Delete(index);
    }
}
//---------------------------------------------------------------------------
int
rsNotations::getNumNotations() const
{
	return _languageCodes->Count;
}
//---------------------------------------------------------------------------
AnsiString
rsNotations::getNotation(int index) const
{
	return _notations->Strings[index];
}
//---------------------------------------------------------------------------
AnsiString
rsNotations::getLanguageCode(int index) const
{
	return _languageCodes->Strings[index];
}
//---------------------------------------------------------------------------

