/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "fsVolumeIndicatorForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TVolumeIndicatorForm *VolumeIndicatorForm;
//---------------------------------------------------------------------------
__fastcall TVolumeIndicatorForm::TVolumeIndicatorForm(TComponent* Owner)
	: TForm(Owner)
{
	Timer			= NULL;

   	_displayTime	= 3;
    _evalTime		= 0;
    _drawUpdate		= 1;

    const int MARGIN = 10;

    GraphWindow = new TGraphWindow(this);
    GraphWindow->Parent = this;

    GraphWindow->Top = LeftVolumeIndicator->Top;
    GraphWindow->Left = MARGIN;
    GraphWindow->Width = LeftVolumeIndicator->Left - MARGIN * 2;
    GraphWindow->Height = LeftVolumeIndicator->Height;
    GraphWindow->Visible = true;
    GraphWindow->Anchors << akLeft << akRight << akTop << akBottom;

    GraphWindow->RangeMin = -30;
    GraphWindow->RangeMax = 0;

    GraphWindow->SetNumGraph(2);
    GraphWindow->SetGraphColor(0, (TColor)0x00FFFFFF);
    GraphWindow->SetGraphColor(1, (TColor)0x007F7FFF);
}
//---------------------------------------------------------------------------
__fastcall
TVolumeIndicatorForm::~TVolumeIndicatorForm()
{
	if (Timer != NULL)
    {
    	Timer->Terminate();
        delete Timer;
    }
	delete GraphWindow;
}
//---------------------------------------------------------------------------
void __fastcall
TVolumeIndicatorForm::FormShow(TObject *Sender)
{
    Timer	= new TVolumeIndicatorTimer();
    Timer->VolumePerform = VolumePerform;

	updateForm();
    computeParam();
}
//---------------------------------------------------------------------------
void __fastcall
TVolumeIndicatorForm::FormClose(TObject *Sender, TCloseAction &Action)
{
	Timer->Terminate();
    delete Timer;
    Timer = NULL;
}
//---------------------------------------------------------------------------
void __fastcall
TVolumeIndicatorForm::FormResize(TObject *Sender)
{
	computeParam();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
TVolumeIndicatorForm::DisplayTimeComboBoxChange(TObject *Sender)
{
	_displayTime = DisplayTimeComboBox->ItemIndex;
    computeParam();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
TVolumeIndicatorForm::DisplayPauseCheckBoxClick(TObject *Sender)
{
	Timer->Pause = !Timer->Pause;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
TVolumeIndicatorForm::VolumePerform(TObject *sender)
{
	if (_volumeRequest == NULL)
    	return ;

	int volume[2];
   	_volumeRequest(this, volume, _evalTime);

    volume[0] /= 100;
    volume[1] /= 100;

    GraphWindow->PointGraph(0, volume[0]);
    GraphWindow->PointGraph(1, volume[1]);

	static int drawCount = 0;
	if (drawCount++ % _drawUpdate == 0)
    	GraphWindow->Draw();

    LeftVolumeIndicator->Position = volume[0];
    RightVolumeIndicator->Position = volume[1];
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
TVolumeIndicatorForm::updateForm()
{
	DisplayTimeComboBox->ItemIndex = _displayTime;
}
//---------------------------------------------------------------------------
void
TVolumeIndicatorForm::computeParam()
{
	//								60b  20b 10b   5b   2b   1b
	const int	SleepTimes []	= { 250,  125,   62,   62,   30,   15 };
	const int	EvalTimes []	= { 250,  125,   62,   62,   30,   30 };
	const int	Steps []		= { 240,  160,  161,   81,   67,   64 };
    const int 	DrawUpdates []	= {   1,    1,    1,    1,    2,    5 };

    // Setup Timer Interval
	Timer->Interval = SleepTimes[_displayTime];

    // Setup Volume Evaluation Time
    _evalTime = EvalTimes[_displayTime];

    // Setup Graph Step
    GraphWindow->Step = Steps[_displayTime];
    GraphWindow->ClearGraph();

    // Setup Graph Draw Updates
    _drawUpdate = DrawUpdates[_displayTime];
}
//---------------------------------------------------------------------------

