/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "fsMainDataModule.h"
#include "fsVersionInfoForm.h"
#include "fsMulticastSettingForm.h"
#include "fsAddMediaForm.h"
#include "fsReceiveMonitorForm.h"
#include "fsMultiReceiverMainForm.h"
#include "fsMultiReceiverLanguage.h"
#include "fs/fsMultiReceiverRemote/fsMultiReceiverRemoteInterface.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMultiReceiverMainForm *MultiReceiverMainForm;
//---------------------------------------------------------------------------
__fastcall TMultiReceiverMainForm::TMultiReceiverMainForm(TComponent* Owner)
	: TForm(Owner)
{
	_flowLayout = NULL;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::FormCreate(TObject *Sender)
{
	if (!fsMultiReceiverInitialize())
	{
		exit(0);
		return ;
	}

    // CAEg}l[W
	_flowLayout = new fsFlowLayout;

    // tH[̎蓮
	MainDataModule 			= new TMainDataModule(NULL);
    AddMediaForm            = new TAddMediaForm(NULL);
    ReceiveMonitorForm		= new TReceiveMonitorForm(NULL);
    MulticastSettingForm	= new TMulticastSettingForm(NULL);
    VersionInfoForm			= new TVersionInfoForm(NULL);

    toCenter(this);
    toCenter(AddMediaForm);
    toCenter(ReceiveMonitorForm);
    toCenter(MulticastSettingForm);
	toCenter(VersionInfoForm);


    // ̐
    _language				= new fsMultiReceiverLanguage(MainDataModule->LanguageMenuItem);

    // [gĂяo^CAEgtO
    _remoteInvokeTimeout	= false;

	// R[obN̐ݒ
	extern int StaticCallbackFunction(const char* command, void* param,
									  void* data);
	fsMultiReceiverSetUICallbackFunction(StaticCallbackFunction, this);

    //
    // WXgǍ
    //
    // Language
    char str[256];
    fsMultiReceiverReadRegistry("MultiReceiverRemote", "Language",
    	 _language->getLanguageCode(0).c_str(), str);
    _language->setCurrentLanguageCode(str);

    //
    // StringGrid̐ݒ
    //
    VideoStringGrid->Cells[0][0] = "Address:";
    VideoStringGrid->Cells[1][0] = "Caption:";
    VideoStringGrid->Cells[2][0] = "Status:";
}
//---------------------------------------------------------------------------
// MultiReceiverMainForm
void __fastcall
TMultiReceiverMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    //
	// WXg̏
    //

    // Language
	char str[256];
    fsMultiReceiverWriteRegistry("MultiReceiverRemote", "Language", _language->getCurrentLanguageCode().c_str());

    // ̍폜
    delete _language;

    // tH[̎蓮
    delete VersionInfoForm;
    delete MulticastSettingForm;
    delete ReceiveMonitorForm;
    delete AddMediaForm;
    delete MainDataModule;

    // CAEg}l[W̍폜
	delete _flowLayout;

	fsMultiReceiverShutdown();
}
//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::FormResize(TObject *Sender)
{
	int width = ClientWidth - VideoStringGrid->ColWidths[0] - VideoStringGrid->ColWidths[1];
	if (0 < width)
	    VideoStringGrid->ColWidths[2] = width;
}
//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::VideoStringGridClick(TObject *Sender)
{
	MainDataModule->setSelectedVideoIndex(VideoStringGrid->Selection.Top - 1);
}
//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::VideoStringGridKeyUp(TObject *Sender, WORD &Key, TShiftState Shift)
{
    if (Key == VK_DELETE && MainDataModule->getSelectedVideoIndex() != -1)
    {
	    if (Application->MessageBox(
    	        _language->getDynamicWord(DW_AREYOUOK).c_str(),
        	    _language->getDynamicWord(DW_CONFIRM).c_str(),
				MB_YESNO | MB_ICONQUESTION) == IDYES)
        {
            removeMediaReceiver(MainDataModule->getSelectedVideoIndex());
            MainDataModule->setSelectedVideoIndex(-1);
        }
    }
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Callback
int
StaticCallbackFunction(const char* command, void* param, void* data)
{
	if (MultiReceiverMainForm != NULL)
		return MultiReceiverMainForm->ProcessCallback(command, param);
	return 0;
}
//---------------------------------------------------------------------------
int
TMultiReceiverMainForm::ProcessCallback(const char *command, void *param)
{
	if (!strcmp("InvokeSucceeded", command))
    {
    	_remoteInvokeTimeout = false;
    }
    else if (!strcmp("InvokeFailed", command))
    {
    	_remoteInvokeTimeout = true;
    }
	return 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool
TMultiReceiverMainForm::addMediaReceiver(const AnsiString& address)
{
    int numReceiver = fsMultiReceiverGetNumMediaReceiver();
	for(int i = 0; i < numReceiver; i++)
    {
    	const char *addr = fsMultiReceiverGetMediaReceiverAddress(i);
        if (!strcmp(addr, address.c_str()))
        {
	    	Application->MessageBox(
            	_language->getDynamicWord(DW_RECEIVEREXISTERROR).c_str(),
                _language->getDynamicWord(DW_ERROR).c_str(),
                MB_OK | MB_ICONERROR);
            return false;
        }
    }

    bool result = fsMultiReceiverAddMediaReceiver(address.c_str());
    if (!result)
    {
    	Application->MessageBox(
        	_language->getDynamicWord(DW_ADDRECEIVERERROR).c_str(),
            _language->getDynamicWord(DW_ERROR).c_str(),
            MB_OK | MB_ICONERROR);
    }
    return result;
}
//---------------------------------------------------------------------------
bool
TMultiReceiverMainForm::removeMediaReceiver(int index)
{
    bool result = fsMultiReceiverRemoveMediaReceiver(index);
    if (!result)
    {
        Application->MessageBox(
        		_language->getDynamicWord(DW_REMOVERECEIVERERROR).c_str(),
                _language->getDynamicWord(DW_ERROR).c_str(),
                MB_OK | MB_ICONERROR);
    }
    return result;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void
TMultiReceiverMainForm::toCenter(TForm* form)
{
	HWND hWnd = GetDesktopWindow();
    if (hWnd == NULL)
    	return ;

    RECT rect;
    GetWindowRect(hWnd, &rect);

    int width = form->Width;
    int height = form->Height;

    int dwidth = rect.right - rect.left;
    int dheight = rect.bottom - rect.top;

    form->Left = (dwidth - width) / 2;
    form->Top = (dheight - height) / 2;
}
//---------------------------------------------------------------------------

