/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "fsMainDataModule.h"
#include "fsVersionInfoForm.h"
#include "fsAddMediaForm.h"
#include "fsReceiveMonitorForm.h"
#include "fsMulticastSettingForm.h"
#include "fsVideoSelectionForm.h"
#include "fsMultiReceiverMainForm.h"
#include "fsMultiReceiverLanguage.h"
#include "fs/fsMultiReceiver/fsMultiReceiverInterface.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMultiReceiverMainForm *MultiReceiverMainForm;
//---------------------------------------------------------------------------
__fastcall TMultiReceiverMainForm::TMultiReceiverMainForm(TComponent* Owner)
	: TForm(Owner)
{
	_flowLayout = NULL;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::FormCreate(TObject *Sender)
{
	if (!fsMultiReceiverInitialize())
	{
		exit(0);
		return ;
	}

    // CAEg}l[W
	_flowLayout = new fsFlowLayout;

    // tH[̎蓮
	MainDataModule 			= new TMainDataModule(NULL);
    AddMediaForm            = new TAddMediaForm(NULL);
    VideoSelectionForm		= new TVideoSelectionForm(NULL);
    ReceiveMonitorForm		= new TReceiveMonitorForm(NULL);
	VersionInfoForm			= new TVersionInfoForm(NULL);
    MulticastSettingForm	= new TMulticastSettingForm(NULL);

    toCenter(this);
    toCenter(AddMediaForm);
    toCenter(VideoSelectionForm);
    toCenter(ReceiveMonitorForm);
	toCenter(VersionInfoForm);
    toCenter(MulticastSettingForm);


    // ̐
    _language				= new fsMultiReceiverLanguage(MainDataModule->LanguageMenuItem);

	// Callback
	extern int StaticCallbackFunction(const char* command, void* param,
									  void* data);
	fsMultiReceiverSetUICallbackFunction(StaticCallbackFunction, this);

    //
    // WXgǍ
    //
    // Language
    char str[256];
    fsMultiReceiverReadRegistry("MultiReceiver", "Language",
    	 _language->getLanguageCode(0).c_str(), str);
    _language->setCurrentLanguageCode(str);

}
//---------------------------------------------------------------------------
// MultiReceiverMainForm
void __fastcall
TMultiReceiverMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    //
	// WXg̏
    //

    // Language
	char str[256];
    fsMultiReceiverWriteRegistry("MultiReceiver", "Language", _language->getCurrentLanguageCode().c_str());

    // ̍폜
    delete _language;

    // tH[̎蓮
    delete MulticastSettingForm;
	delete VersionInfoForm;
    delete ReceiveMonitorForm;
    delete VideoSelectionForm;
    delete AddMediaForm;
    delete MainDataModule;

    // CAEg}l[W̍폜
	delete _flowLayout;
}
//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::FormDestroy(TObject *Sender)
{
	fsMultiReceiverShutdown();
}
//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::FormKeyUp(TObject *Sender, WORD &Key, TShiftState Shift)
{
    if (Key == VK_DELETE && MainDataModule->getSelectedVideoIndex() != -1)
    {
	    if (Application->MessageBox(
    	        _language->getDynamicWord(DW_AREYOUOK).c_str(),
        	    _language->getDynamicWord(DW_CONFIRM).c_str(),
				MB_YESNO | MB_ICONQUESTION) == IDYES)
        {
            removeMediaReceiver(MainDataModule->getSelectedVideoIndex());
            MainDataModule->setSelectedVideoIndex(-1);
        }
    }
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// VideoRenderBaseScrollBox
void __fastcall
TMultiReceiverMainForm::VideoRenderBaseScrollBoxResize(TObject *Sender)
{
	_flowLayout->doLayout((TScrollBox *)Sender);
}
//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::VideoRenderBaseScrollBoxClick(TObject *Sender)
{
    int count = VideoRenderBaseScrollBox->ControlCount;
    for(int i = 0;i < count; i++)
    {
        TPanel * panel = (TPanel *)VideoRenderBaseScrollBox->Controls[i];
        panel->Color = clBtnFace;
        for(int j = 0; j < panel->ControlCount; j++)
        {
         	TLabel* label = (TLabel *)panel->Controls[j];
            label->Font->Color = clBlack;
        }
    }
    MainDataModule->setSelectedVideoIndex(-1);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::PanelClick(TObject *Sender)
{
    int count = VideoRenderBaseScrollBox->ControlCount;
    for(int i = 0;i < count; i++)
    {
        TPanel * panel = (TPanel *)VideoRenderBaseScrollBox->Controls[i];
        if (Sender == panel)
        {
            MainDataModule->setSelectedVideoIndex(i);
            panel->Color = clHighlight;
            for(int j = 0; j < panel->ControlCount; j++)
            {
            	TLabel* label = (TLabel *)panel->Controls[j];
                label->Font->Color = clYellow;
            }
        }
        else
        {
            panel->Color = clBtnFace;
            for(int j = 0; j < panel->ControlCount; j++)
            {
            	TLabel* label = (TLabel *)panel->Controls[j];
                label->Font->Color = clBlack;
            }
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall
TMultiReceiverMainForm::LabelClick(TObject *Sender)
{
	TLabel *label = (TLabel *)Sender;
    PanelClick(label->Parent);
}

//---------------------------------------------------------------------------
// Callback
int
StaticCallbackFunction(const char* command, void* param, void* data)
{
	if (MultiReceiverMainForm != NULL)
		return MultiReceiverMainForm->ProcessCallback(command, param);
	return 0;
}
//---------------------------------------------------------------------------
int
TMultiReceiverMainForm::ProcessCallback(const char *command, void *param)
{
	if (!strcmp("TimerStop", command))
    {
    	MainDataModule->MultiReceiverTimer->Enabled = false;
    }
    else if (!strcmp("TimerStart", command))
    {
    	MainDataModule->MultiReceiverTimer->Enabled = true;
    }
	else if (!strcmp("ResizePanel", command))
	{
        resizePanel((int)param);
        _flowLayout->doLayout(VideoRenderBaseScrollBox);
	}
    else if (!strcmp("ResizePanelAll", command))
    {
    	resizePanelAll();
    }
	else if (!strcmp("AddMediaReceiver", command))
    {
		struct _Message
		{
			const char*	address;
			HWND		windowHandle;
		};
        _Message *mes = (_Message *)param;

		TPanel *panel = insertMediaReceiverBasePanel(AnsiString(mes->address));
        mes->windowHandle = panel->Handle;

        _flowLayout->doLayout(VideoRenderBaseScrollBox);
    }
    else if (!strcmp("RemoveMediaReceiver", command))
    {
		int index = (int)param;

        VideoRenderBaseScrollBox->RemoveControl(
                        VideoRenderBaseScrollBox->Controls[index]);

        _flowLayout->doLayout(VideoRenderBaseScrollBox);
    }

	return 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool
TMultiReceiverMainForm::addMediaReceiver(const AnsiString& address)
{
    int numReceiver = fsMultiReceiverGetNumMediaReceiver();
	for(int i = 0; i < numReceiver; i++)
    {
    	const char *addr = fsMultiReceiverGetMediaReceiverAddress(i);
        if (!strcmp(addr, address.c_str()))
        {
	    	Application->MessageBox(
            	_language->getDynamicWord(DW_RECEIVEREXISTERROR).c_str(),
                _language->getDynamicWord(DW_ERROR).c_str(),
                MB_OK | MB_ICONERROR);
            return false;
        }
    }

    bool result = fsMultiReceiverAddMediaReceiver(address.c_str());
    if (!result)
    {
    	Application->MessageBox(
        	_language->getDynamicWord(DW_ADDRECEIVERERROR).c_str(),
            _language->getDynamicWord(DW_ERROR).c_str(),
            MB_OK | MB_ICONERROR);
    }
    return result;
}
//---------------------------------------------------------------------------
bool
TMultiReceiverMainForm::removeMediaReceiver(int index)
{
    bool result = fsMultiReceiverRemoveMediaReceiver(index);
    if (!result)
    {
        Application->MessageBox(
        		_language->getDynamicWord(DW_REMOVERECEIVERERROR).c_str(),
                _language->getDynamicWord(DW_ERROR).c_str(),
                MB_OK | MB_ICONERROR);
    }
    return result;
}
//---------------------------------------------------------------------------
void
TMultiReceiverMainForm::resizePanel(int index)
{
  	fsMultiReceiverSetMediaReceiverVideoPosition(index, 10, 10);

    int width, height;
    fsMultiReceiverGetMediaReceiverVideoSize(index, &width, &height);

    if (width <= 1 || height <= 1)
    {
		width = 80;
        height = 50;
	}

	TControl *control = VideoRenderBaseScrollBox->Controls[index];
   	control->Width = width + 20;
	control->Height = height + 50;
}
//---------------------------------------------------------------------------
void
TMultiReceiverMainForm::resizePanelAll()
{
	int numReceiver = fsMultiReceiverGetNumMediaReceiver();
	for(int i = 0; i < numReceiver; i++)
    	resizePanel(i);

    _flowLayout->doLayout(VideoRenderBaseScrollBox);
}
//---------------------------------------------------------------------------
TPanel*
TMultiReceiverMainForm::insertMediaReceiverBasePanel(const AnsiString& address)
{
    TPanel *panel = new TPanel(VideoRenderBaseScrollBox);
    panel->OnClick = PanelClick;
	panel->Left = 0;
    panel->Top = 0;
    panel->Width = 100;
    panel->Height = 100;
    panel->ParentColor = false;
    panel->Color = clBtnFace;
	panel->Show();
	VideoRenderBaseScrollBox->InsertControl(panel);

    TLabel *label = new TLabel(panel);
    label->Name = AnsiString("StatusLabel");
    label->OnClick = LabelClick;
    label->Align = alBottom;
    label->Caption = "Status:";
    panel->InsertControl(label);

	label = new TLabel(panel);
    label->Name = AnsiString("AddressLabel");
    label->OnClick = LabelClick;
    label->Align = alBottom;
    label->Caption = "Address: " + address;
    panel->InsertControl(label);

    label = new TLabel(panel);
    label->Name = AnsiString("CaptionLabel");
    label->OnClick = LabelClick;
    label->Align = alBottom;
    label->Caption = "Caption:";
    panel->InsertControl(label);

    return panel;
}
//---------------------------------------------------------------------------
void
TMultiReceiverMainForm::scrollAndSelect(int index)
{
    TControl *control = VideoRenderBaseScrollBox->Controls[index];
	VideoRenderBaseScrollBox->ScrollInView(control);

    TPanel *panel;

	for(int i = 0; i < VideoRenderBaseScrollBox->ControlCount; i++)
    {
	    panel = (TPanel *)VideoRenderBaseScrollBox->Controls[i];
    	panel->Color = clBtnFace;
        for(int j = 0; j < panel->ControlCount; j++)
        {
        	TLabel* label = (TLabel *)panel->Controls[j];
            label->Font->Color = clBlack;
        }
    }

    panel = (TPanel *)VideoRenderBaseScrollBox->Controls[index];

    MainDataModule->setSelectedVideoIndex(index);
    panel->Color = clHighlight;
    for(int j = 0; j < panel->ControlCount; j++)
    {
    	TLabel* label = (TLabel *)panel->Controls[j];
        label->Font->Color = clYellow;
    }
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void
TMultiReceiverMainForm::toCenter(TForm* form)
{
	HWND hWnd = GetDesktopWindow();
    if (hWnd == NULL)
    	return ;

    RECT rect;
    GetWindowRect(hWnd, &rect);

    int width = form->Width;
    int height = form->Height;

    int dwidth = rect.right - rect.left;
    int dheight = rect.bottom - rect.top;

    form->Left = (dwidth - width) / 2;
    form->Top = (dheight - height) / 2;
}
//---------------------------------------------------------------------------

