/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "fsReceiverMainForm.h"
#include "fsMainDataModule.h"
#include "fsMulticastSettingForm.h"
#include "fsReceiveMonitorForm.h"
#include "fsPointerSettingForm.h"
#include "fsVersionInfoForm.h"
#include "fsReceiverLanguage.h"
#include <ui/fsCommonUI/fsVolumeIndicatorForm.h>
#include <ui/fsCommonUI/fsErrorMessageForm.h>
#include <fs/fsReceiverRemote/fsReceiverRemoteInterface.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TReceiverMainForm *ReceiverMainForm;
//---------------------------------------------------------------------------
__fastcall TReceiverMainForm::TReceiverMainForm(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall
TReceiverMainForm::FormCreate(TObject *Sender)
{
	if (!fsReceiverInitialize())
	{
		exit(0);
		return ;
	}

    // tH[̎蓮
    ErrorMessageForm		= new TErrorMessageForm(NULL);
	MainDataModule 			= new TMainDataModule(NULL);
    PointerSettingForm		= new TPointerSettingForm(NULL);
    MulticastSettingForm   	= new TMulticastSettingForm(NULL);
    ReceiveMonitorForm		= new TReceiveMonitorForm(NULL);
    VolumeIndicatorForm		= new TVolumeIndicatorForm(NULL);
    VersionInfoForm			= new TVersionInfoForm(NULL);

    toCenter(this);
    toCenter(ErrorMessageForm);
    toCenter(PointerSettingForm);
    toCenter(MulticastSettingForm);
    toCenter(ReceiveMonitorForm);
    toCenter(VolumeIndicatorForm);
    toCenter(VersionInfoForm);

    
    // 쐬
    _language				= new fsReceiverLanguage(MainDataModule->LanguageMenuItem);

	// TCY֎~tO
    _forbidResize			= false;

    // [gĂяo^CAEgtO
    _remoteInvokeTimeout	= false;

	// R[obNݒ
	extern int StaticCallbackFunction(const char* command, void* param,
									  void* data);
	fsReceiverSetCallbackFunction(StaticCallbackFunction, this);

    VolumeIndicatorForm->VolumeRequest = VolumeRequest;

    //
    // WXgǍ
    //

    // Language
    char str[256];
    fsReceiverReadRegistry("ReceiverRemote", "Language",
    	 _language->getLanguageCode(0).c_str(), str);
    _language->setCurrentLanguageCode(str);

    // VolumeIndicator Display Time
    const char* cstr;
    switch(VolumeIndicatorForm->DisplayTime)
    {
    case 0:	cstr = "60s";	break;
    case 1:	cstr = "20s";	break;
    case 2:	cstr = "10s";	break;
    case 3:	cstr = "5s";	break;
    case 4:	cstr = "2s";	break;
    case 5:	cstr = "1s";	break;
    default:cstr = "5s";	break;
    }
    fsReceiverReadRegistry("ReceiverRemote", "VolumeIndicatorDisplayTime", cstr, str);
    if (!strcmpi(str, "60s"))  		VolumeIndicatorForm->DisplayTime = 0;
    else if (!strcmpi(str, "20s"))  VolumeIndicatorForm->DisplayTime = 1;
    else if (!strcmpi(str, "10s"))  VolumeIndicatorForm->DisplayTime = 2;
    else if (!strcmpi(str, "5s"))	VolumeIndicatorForm->DisplayTime = 3;
    else if (!strcmpi(str, "2s"))	VolumeIndicatorForm->DisplayTime = 4;
    else if (!strcmpi(str, "1s"))	VolumeIndicatorForm->DisplayTime = 5;
    else							VolumeIndicatorForm->DisplayTime = 3;
}
//---------------------------------------------------------------------------
void __fastcall
TReceiverMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
	MainDataModule->ReceiverTimer->Enabled = false;

    //
	// WXg̏
    //

    // Language
    fsReceiverWriteRegistry("ReceiverRemote", "Language", _language->getCurrentLanguageCode().c_str());

    // VolumeIndicator Display Time
    const char* cstr;
    switch(VolumeIndicatorForm->DisplayTime)
    {
    case 0:	cstr = "60s";	break;
    case 1:	cstr = "20s";	break;
    case 2:	cstr = "10s";	break;
    case 3:	cstr = "5s";	break;
    case 4:	cstr = "2s";	break;
    case 5:	cstr = "1s";	break;
    default:cstr = "5s";	break;
    }
    fsReceiverWriteRegistry("ReceiverRemote", "VolumeIndicatorDisplayTime", cstr);


    // ̍폜
    delete _language;

    // tH[̎蓮
    delete VersionInfoForm;
    delete VolumeIndicatorForm;
    delete ReceiveMonitorForm;
    delete MulticastSettingForm;
    delete PointerSettingForm;
    delete MainDataModule;
    delete ErrorMessageForm;
}
//---------------------------------------------------------------------------
void __fastcall
TReceiverMainForm::FormDestroy(TObject *Sender)
{
	fsReceiverShutdown();
}
//---------------------------------------------------------------------------
void __fastcall
TReceiverMainForm::FormKeyUp(TObject *Sender, WORD &Key, TShiftState Shift)
{
	if (Shift.Contains(ssCtrl) && Key == 'E')
		ErrorMessageForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall
TReceiverMainForm::VideoRenderPanelResize(TObject *Sender)
{
	if (!fsReceiverIsInitialized())
		return ;

    _forbidResize = true;
    fsReceiverComputeVideoWindow();
    _forbidResize = false;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Callback
int
StaticCallbackFunction(const char* command, void* param, void* data)
{
	if (ReceiverMainForm != NULL)
		return ReceiverMainForm->ProcessCallback(command, param);
	return 0;
}
//---------------------------------------------------------------------------
int __fastcall
TReceiverMainForm::ProcessCallback(const char *command, void *param)
{
	if (!strcmp("InvokeSucceeded", command))
    {
    	_remoteInvokeTimeout = false;
    }
    else if (!strcmp("InvokeFailed", command))
    {
    	_remoteInvokeTimeout = true;
    }
	return 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
TReceiverMainForm::VolumeRequest(TObject *Sender, int *volume, int evalTime)
{
	fsReceiverSetVolumeEvaluationTime(evalTime);
	fsReceiverGetVolume(&volume[0], &volume[1]);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void
TReceiverMainForm::toCenter(TForm* form)
{
	HWND hWnd = GetDesktopWindow();
    if (hWnd == NULL)
    	return ;

    RECT rect;
    GetWindowRect(hWnd, &rect);

    int width = form->Width;
    int height = form->Height;

    int dwidth = rect.right - rect.left;
    int dheight = rect.bottom - rect.top;

    form->Left = (dwidth - width) / 2;
    form->Top = (dheight - height) / 2;
}
//---------------------------------------------------------------------------

