/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//---------------------------------------------------------------------------
#include "fsSenderCommandHistory.h"
#include "fsSenderLanguage.h"
#include "fsSenderMainForm.h"
#include "fsMainDataModule.h"
#include <fs/fsSenderRemote/fsSenderRemoteInterface.h>
//---------------------------------------------------------------------------
fsSenderCommandHistory::fsSenderCommandHistory(
						TMenuItem* HistoryBeginMenuItem,
						TMenuItem* HistoryEndMenuItem) :
fsCommandHistory(HistoryBeginMenuItem, HistoryEndMenuItem, 20)
{
}
//---------------------------------------------------------------------------
fsSenderCommandHistory::~fsSenderCommandHistory()
{
}
//---------------------------------------------------------------------------
AnsiString
fsSenderCommandHistory::getPrefixIdString(int prefixId)
{
	AnsiString str;

	switch(prefixId)
    {
    case P_SHOW:
    	str = SenderMainForm->_language->getDynamicWord(DW_SHOW);
        break;
	case P_FOCUSDISPLAY:
    	str = SenderMainForm->_language->getDynamicWord(DW_FOCUSDISPLAY);
    	break;
    case P_MULTICAST:
    	str = SenderMainForm->_language->getDynamicWord(DW_MULTICAST);
    	break;
	case P_SOURCE:
    	str = SenderMainForm->_language->getDynamicWord(DW_SOURCE);
    	break;
	case P_RESOLUTION:
    	str = SenderMainForm->_language->getDynamicWord(DW_RESOLUTION);
    	break;
	case P_TRANSMISSIONRATE:
    	str = SenderMainForm->_language->getDynamicWord(DW_TRANSMISSIONRATE);
    	break;
	case P_TRANSMISSIONCOUNT:
    	str = SenderMainForm->_language->getDynamicWord(DW_TRANSMISSIONCOUNT);
    	break;
    case P_READINIFILE:
    	str = SenderMainForm->_language->getDynamicWord(DW_READINIFILE);
    	break;
    case P_WRITEINIFILE:
    	str = SenderMainForm->_language->getDynamicWord(DW_WRITEINIFILE);
    	break;
    case P_ENCODER:
    	str = SenderMainForm->_language->getDynamicWord(DW_ENCODER);
        break;
    case P_VIDEOSOURCE:
    	str = SenderMainForm->_language->getDynamicWord(DW_VIDEOSOURCE);
        break;
    case P_AUDIOSOURCE:
    	str = SenderMainForm->_language->getDynamicWord(DW_AUDIOSOURCE);
        break;
    case P_VIDEOENCODER:
    	str = SenderMainForm->_language->getDynamicWord(DW_VIDEOENCODER);
        break;
    case P_AUDIOENCODER:
    	str = SenderMainForm->_language->getDynamicWord(DW_AUDIOENCODER);
        break;
    }

    return str;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void
fsSenderCommandHistory::addReadIniFile(const AnsiString& filename)
{
	addOriginal(filename, ReadIniFileMenuItemOnExecute, NULL, P_READINIFILE);
}
//---------------------------------------------------------------------------
void
fsSenderCommandHistory::addWriteIniFile(const AnsiString& filename)
{
	addOriginal(filename, WriteIniFileMenuItemOnExecute, NULL, P_WRITEINIFILE);
}
//---------------------------------------------------------------------------
void
fsSenderCommandHistory::addVideoSource(int index, const AnsiString& friendlyName)
{
    addOriginal(friendlyName, VideoSourceMenuItemOnExecute, (void *)index, P_VIDEOSOURCE);
}
//---------------------------------------------------------------------------
void
fsSenderCommandHistory::addAudioSource(int index, const AnsiString& friendlyName)
{
    addOriginal(friendlyName, AudioSourceMenuItemOnExecute, (void *)index, P_AUDIOSOURCE);
}
//---------------------------------------------------------------------------
void
fsSenderCommandHistory::addVideoEncoder(int index, const AnsiString& friendlyName)
{
    addOriginal(friendlyName, VideoEncoderMenuItemOnExecute, (void *)index, P_VIDEOENCODER);
}
//---------------------------------------------------------------------------
void
fsSenderCommandHistory::addAudioEncoder(int index, const AnsiString& friendlyName)
{
    addOriginal(friendlyName, AudioEncoderMenuItemOnExecute, (void *)index, P_AUDIOENCODER);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall
fsSenderCommandHistory::ReadIniFileMenuItemOnExecute(TObject *Sender)
{
	TAction *action = (TAction *)Sender;
    if (fsSenderReadIniFile(action->Caption.c_str()))
    {
     	fsSenderComputeVideoWindow();

     	MainDataModule->updateVideoSourceSelection();
     	MainDataModule->updateAudioSourceSelection();
	   	MainDataModule->updateVideoEncoderSelection();
	   	MainDataModule->updateAudioEncoderSelection();
    }

    addReadIniFile(action->Caption);
}
//---------------------------------------------------------------------------
void __fastcall
fsSenderCommandHistory::WriteIniFileMenuItemOnExecute(TObject *Sender)
{
	TAction *action = (TAction *)Sender;
    fsSenderWriteIniFile(action->Caption.c_str());
    
    addWriteIniFile(action->Caption);
}
//---------------------------------------------------------------------------
void __fastcall
fsSenderCommandHistory::VideoSourceMenuItemOnExecute(TObject *Sender)
{
	int index = (int)((TMenuItem *)Sender)->Tag;
    fsSenderSetVideoSource(index);
	fsSenderComputeVideoWindow();
    addVideoSource(index, ((TAction *)Sender)->Caption);
}
//---------------------------------------------------------------------------
void __fastcall
fsSenderCommandHistory::AudioSourceMenuItemOnExecute(TObject *Sender)
{
	int index = (int)((TMenuItem *)Sender)->Tag;
    fsSenderSetAudioSource(index);
	fsSenderComputeVideoWindow();
    addAudioSource(index, ((TAction *)Sender)->Caption);
}
//---------------------------------------------------------------------------
void __fastcall
fsSenderCommandHistory::VideoEncoderMenuItemOnExecute(TObject *Sender)
{
	int index = (int)((TMenuItem *)Sender)->Tag;
    fsSenderSetVideoEncoder(index);
	fsSenderComputeVideoWindow();
    addVideoEncoder(index, ((TAction *)Sender)->Caption);
}
//---------------------------------------------------------------------------
void __fastcall
fsSenderCommandHistory::AudioEncoderMenuItemOnExecute(TObject *Sender)
{
	int index = (int)((TMenuItem *)Sender)->Tag;
    fsSenderSetAudioEncoder(index);
	fsSenderComputeVideoWindow();
    addAudioEncoder(index, ((TAction *)Sender)->Caption);
}
//---------------------------------------------------------------------------

