/*
**	Copyright (c) 2003-2004 National Institute of Multimedia Education. All rights reserved.
**	2-12 Wakaba, Mihama, Chiba, 261-0014 JAPAN
**	http://www.nime.ac.jp/
*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "fsMainDataModule.h"
#include "fsMulticastSettingForm.h"
#include <fs/fsSender/fsSenderInterface.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMulticastSettingForm *MulticastSettingForm;
//---------------------------------------------------------------------------
__fastcall TMulticastSettingForm::TMulticastSettingForm(TComponent* Owner)
    : TForm(Owner)
{
	_formShown	= false;
}
//---------------------------------------------------------------------------
void __fastcall
TMulticastSettingForm::FormShow(TObject *Sender)
{
	updateForm();

    _addressIsDirty		= false;
    _ttlIsDirty			= false;
    _packetSizeIsDirty	= false;

    _inputAddress		= "";

    MainDataModule->MulticastDetectionTimer->Enabled = true;

    _formShown = true;
}
//---------------------------------------------------------------------------
void __fastcall
TMulticastSettingForm::FormHide(TObject *Sender)
{
	_formShown = false;

	MainDataModule->MulticastDetectionTimer->Enabled = false;

	if (_inputAddress.Length() != 0)
    	fsSenderRemoveMulticastDetector(_inputAddress.c_str());
}
//---------------------------------------------------------------------------
void __fastcall
TMulticastSettingForm::AddressEditChange(TObject *Sender)
{
	if (!_formShown)
		return;

	_addressIsDirty = true;

    if (_inputAddress.Length() != 0)
    	fsSenderRemoveMulticastDetector(_inputAddress.c_str());

    _inputAddress = AddressEdit->Text;
    if (_inputAddress.Length() != 0)
	    fsSenderAddMulticastDetector(_inputAddress.c_str());
}
//---------------------------------------------------------------------------
void __fastcall
TMulticastSettingForm::TTLEditChange(TObject *Sender)
{
	_ttlIsDirty = true;
}
//---------------------------------------------------------------------------
void __fastcall
TMulticastSettingForm::PacketSizeEditChange(TObject *Sender)
{
	_packetSizeIsDirty = true;
}
//---------------------------------------------------------------------------
void __fastcall
TMulticastSettingForm::TransmitCountComboBoxChange(TObject *Sender)
{
	fsSenderSetTransmissionCount(getTransmissionCount(TransmitCountComboBox));
}
//---------------------------------------------------------------------------
void __fastcall
TMulticastSettingForm::TargetTransmitRateComboBoxChange(TObject *Sender)
{
	int index = TargetTransmitRateComboBox->ItemIndex;
	int rate;

    switch(index)
    {
    case 0:	rate = fsSenderGetVideoSenderMainMaxTransmissionRate();	break;
    case 1:	rate = fsSenderGetVideoSenderSubMaxTransmissionRate();	break;
    case 2:	rate = fsSenderGetAudioSenderMaxTransmissionRate();  	break;
    case 3:	rate = fsSenderGetPointerSenderMaxTransmissionRate(); 	break;
    case 4:	rate = fsSenderGetStatusSenderMaxTransmissionRate();  	break;
    }

    setupTransmissionRate(TransmitRateComboBox, rate);
}
//---------------------------------------------------------------------------
void __fastcall
TMulticastSettingForm::TransmitRateComboBoxKeyUp(TObject *Sender,
												WORD &Key, TShiftState Shift)
{
	if (Key != VK_RETURN)
    	return ;

	int rate = atoi(TransmitRateComboBox->Text.c_str());
    if (!setupTransmissionRate(TransmitRateComboBox, rate))
    	return ;

   	int index = TargetTransmitRateComboBox->ItemIndex;
    switch(index)
   	{
    case 0:	fsSenderSetVideoSenderMainMaxTransmissionRate(rate);	break;
    case 1:	fsSenderSetVideoSenderSubMaxTransmissionRate(rate);		break;
    case 2:	fsSenderSetAudioSenderMaxTransmissionRate(rate);    	break;
    case 3:	fsSenderSetPointerSenderMaxTransmissionRate(rate);    	break;
    case 4:	fsSenderSetStatusSenderMaxTransmissionRate(rate);    	break;
    }
}
//---------------------------------------------------------------------------
void __fastcall
TMulticastSettingForm::TransmitRateComboBoxClick(TObject *Sender)
{
	int rate = getTransmissionRate(TransmitRateComboBox);

	int index = TargetTransmitRateComboBox->ItemIndex;
    switch(index)
    {
    case 0:	fsSenderSetVideoSenderMainMaxTransmissionRate(rate);	break;
    case 1:	fsSenderSetVideoSenderSubMaxTransmissionRate(rate);		break;
    case 2:	fsSenderSetAudioSenderMaxTransmissionRate(rate);    	break;
    case 3:	fsSenderSetPointerSenderMaxTransmissionRate(rate);    	break;
    case 4:	fsSenderSetStatusSenderMaxTransmissionRate(rate);    	break;
    }
}
//---------------------------------------------------------------------------
bool
TMulticastSettingForm::setupTransmissionRate(TComboBox *comboBox, int rate)
{
    if (rate < 1 * 1024 || 1 * 1024 * 1024 * 1024 < rate)
    	return false;

    bool enabled = comboBox->Enabled;
    comboBox->Enabled = false;
    if (rate == 100 * 1024)
    {
        comboBox->ItemIndex = 0;
    }
    else if (rate == 300 * 1024)
    {
        comboBox->ItemIndex = 1;
    }
    else if (rate == 1 * 1024 * 1024)
    {
        comboBox->ItemIndex = 2;
    }
    else if (rate == 10 * 1024 * 1024)
    {
        comboBox->ItemIndex = 3;
    }
    else if (rate == 30 * 1024 * 1024)
    {
        comboBox->ItemIndex = 4;
    }
    else if (rate == 100 * 1024 * 1024)
    {
        comboBox->ItemIndex = 5;
    }
    else if (rate == 1 * 1024 * 1024 * 1024)
    {
        comboBox->ItemIndex = 6;
    }
    else
    {
    	int index = comboBox->Items->IndexOf(AnsiString(rate) + AnsiString(" bps"));
        if (index != -1)
        {
        	comboBox->ItemIndex = index;
        }
        else
        {
        	comboBox->Items->Add(AnsiString(rate) + AnsiString(" bps"));
            comboBox->ItemIndex = comboBox->Items->Count - 1;
        }
    }
    comboBox->Enabled = enabled;

    return true;
}
//---------------------------------------------------------------------------
void
TMulticastSettingForm::setupTransmissionCount(TComboBox *comboBox, int count)
{
    bool enabled = comboBox->Enabled;

    if (0 < count && count <= 4)
        comboBox->ItemIndex = count - 1;
    else
        comboBox->ItemIndex = 0;

    comboBox->Enabled = enabled;
}
//---------------------------------------------------------------------------
int
TMulticastSettingForm::getTransmissionRate(TComboBox *comboBox)
{
    int rate;

    switch(comboBox->ItemIndex)
    {
    case 0: rate = 100 * 1024;              		break;
    case 1: rate = 300 * 1024;              		break;
    case 2: rate = 1 * 1024 * 1024;         		break;
    case 3: rate = 10 * 1024 * 1024;        		break;
    case 4: rate = 30 * 1024 * 1024;        		break;
    case 5: rate = 100 * 1024 * 1024;       		break;
    case 6: rate = 1 * 1024 * 1024 * 1024;  		break;
    default:rate = atoi(comboBox->Text.c_str());    break;
    }

    return rate;
}
//---------------------------------------------------------------------------
int
TMulticastSettingForm::getTransmissionCount(TComboBox *comboBox)
{
    return comboBox->ItemIndex + 1;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void
TMulticastSettingForm::updateForm()
{
	// Items
    AddressEdit->Text = fsSenderGetAddress();
    TTLEdit->Text = AnsiString(fsSenderGetTimeToLive());
    setupTransmissionCount(TransmitCountComboBox,
                                    fsSenderGetTransmissionCount());
    PacketSizeEdit->Text = fsSenderGetPacketSize();

	TargetTransmitRateComboBox->ItemIndex = 0;
    setupTransmissionRate(TransmitRateComboBox,
    					 fsSenderGetVideoSenderMainMaxTransmissionRate());

    MulticastDetectLabel->Visible = false;

	// Actions

}
//---------------------------------------------------------------------------

